@@ -26,7 +26,6 @@
 #include "selector.h"
 #include "CondorError.h"
 #include "ccb_client.h"
-#include "daemon_core_sock_adapter.h"
 #include "condor_sinful.h"
 #include "shared_port_endpoint.h"
 
@@ -75,7 +74,7 @@ CCBClient::~CCBClient()
 		delete m_ccb_sock;
 	}
 	if( m_deadline_timer != -1 ) {
-		daemonCoreSockAdapter.Cancel_Timer(m_deadline_timer);
+		daemonCore->Cancel_Timer(m_deadline_timer);
 		m_deadline_timer = -1;
 	}
 }
@@ -85,6 +84,11 @@ bool
 CCBClient::ReverseConnect( CondorError *error, bool non_blocking )
 {
 	if( non_blocking ) {
+		// Non-blocking mode requires DaemonCore
+		if ( !daemonCore ) {
+			dprintf( D_ALWAYS, "Can't do non-blocking CCB reverse connection without DaemonCore!\n" );
+			return false;
+		}
 		m_target_sock->enter_reverse_connecting_state();
 		// NOTE: now we _must_ call exit_reverse_connecting_state()
 		// before we are done with m_target_sock.  This state
@@ -108,9 +112,9 @@ CCBClient::myName()
 	// It is who we say we are when talking to the CCB server.
 	MyString name;
 	name = get_mySubSystem()->getName();
-	if( daemonCoreSockAdapter.isEnabled() ) {
+	if( daemonCore ) {
 		name += " ";
-		name += daemonCoreSockAdapter.publicNetworkIpAddr();
+		name += daemonCore->publicNetworkIpAddr();
 	}
 	return name;
 }
@@ -122,58 +126,75 @@ CCBClient::ReverseConnect_blocking( CondorError *error )
 	counted_ptr<SharedPortEndpoint> shared_listener;
 	char const *listener_addr = NULL;
 
-	if( SharedPortEndpoint::UseSharedPort() ) {
-		shared_listener = counted_ptr<SharedPortEndpoint>(new SharedPortEndpoint());
-		shared_listener->InitAndReconfig();
-		MyString errmsg;
-		if( !shared_listener->CreateListener() ) {
-			errmsg.formatstr("Failed to create shared port endpoint for reversed connection from %s.",
-						   m_target_peer_description.Value());
-		}
-		else if( !(listener_addr = shared_listener->GetMyRemoteAddress()) ) {
-			errmsg.formatstr("Failed to get remote address for shared port endpoint for reversed connection from %s.",
-						   m_target_peer_description.Value());
-		}
-		if( !listener_addr ) {
-			if( error ) {
-				error->push("CCBClient", CEDAR_ERR_CONNECT_FAILED,errmsg.Value());
-			}
-			dprintf(D_ALWAYS,"CCBClient: %s\n",errmsg.Value());
-			return false;
-		}
-	}
-	else {
-		listen_sock = counted_ptr<ReliSock>(new ReliSock());
-		listen_sock->bind(false,0);
-		if( !listen_sock->listen() ) {
-			MyString errmsg;
-			errmsg.formatstr("Failed to listen for reversed connection from %s.",
-						   m_target_peer_description.Value());
-			if( error ) {
-				error->push("CCBClient", CEDAR_ERR_CONNECT_FAILED,errmsg.Value());
-			}
-			dprintf(D_ALWAYS,"CCBClient: %s\n",errmsg.Value());
-
-			return false;
-		}
-		listener_addr = listen_sock->get_sinful_public();
-	}
-	ASSERT( listener_addr );
-
 	m_ccb_contacts.rewind();
 	char const *ccb_contact;
 	while( (ccb_contact = m_ccb_contacts.next()) ) {
 		bool success = false;
 		MyString ccb_address, ccbid;
-		if( !SplitCCBContact( ccb_contact, ccb_address, ccbid, error ) ) {
+		if( !SplitCCBContact( ccb_contact, ccb_address, ccbid, m_target_peer_description, error ) ) {
 			continue;
 		}
 
+		//
+		// Generate a listen port for the appropriate protocol.  It will be
+		// distressing that we're passed CCB contact strings rather than
+		// sinfuls, when it's time to handle multiple addresses...
+		//
+		// FIXME: Assumes that shared port knows what it's doing.
+		//
+		if( SharedPortEndpoint::UseSharedPort() ) {
+			shared_listener = counted_ptr<SharedPortEndpoint>(new SharedPortEndpoint());
+			shared_listener->InitAndReconfig();
+			MyString errmsg;
+			if( !shared_listener->CreateListener() ) {
+				errmsg.formatstr( "Failed to create shared port endpoint for reversed connection from %s.",
+					m_target_peer_description.Value() );
+			}
+			else if( !(listener_addr = shared_listener->GetMyRemoteAddress()) ) {
+				errmsg.formatstr( "Failed to get remote address for shared port endpoint for reversed connection from %s.",
+					m_target_peer_description.Value() );
+			}
+			if( !listener_addr ) {
+				if( error ) {
+					error->push( "CCBClient", CEDAR_ERR_CONNECT_FAILED, errmsg.Value() );
+				}
+				dprintf(D_ALWAYS,"CCBClient: %s\n",errmsg.Value());
+				return false;
+			}
+		} else {
+			condor_sockaddr ccbSA;
+			MyString faked_sinful = "<" + ccb_address + ">";
+			if( ! ccbSA.from_sinful( faked_sinful ) ) {
+				dprintf( D_FULLDEBUG, "Failed to generate condor_sockaddr from faked sinful '%s', ignoring this broker.\n", faked_sinful.Value() );
+				continue;
+			}
+
+			listen_sock = counted_ptr<ReliSock>( new ReliSock() );
+			// Should bind() should accept a condor_sockaddr directly?
+			listen_sock->bind( ccbSA.get_protocol(), false, 0, false );
+			if( ! listen_sock->listen() ) {
+				MyString errmsg;
+				errmsg.formatstr( "Failed to listen for reversed connection from %s.",
+					m_target_peer_description.Value() );
+				if( error ) {
+					error->push("CCBClient", CEDAR_ERR_CONNECT_FAILED,errmsg.Value());
+				}
+				dprintf(D_ALWAYS,"CCBClient: %s\n",errmsg.Value());
+
+				return false;
+			}
+
+			listener_addr = listen_sock->get_sinful_public();
+		}
+
 		ClassAd msg;
 		msg.Assign(ATTR_CCBID,ccbid);
 		msg.Assign(ATTR_CLAIM_ID,m_connect_id);
 		// purely for debugging purposes: identify ourselves
 		msg.Assign(ATTR_NAME, myName());
+		// ATTR_MY_ADDRESS is functional and subject to address rewriting,
+		// but it's not OK to be protocol-blind, because we won't recognize
+		// the port number (it's not a command socket).
 		msg.Assign(ATTR_MY_ADDRESS, listener_addr);
 
 		dprintf(D_NETWORK|D_FULLDEBUG,
@@ -314,14 +335,14 @@ CCBClient::ReverseConnect_blocking( CondorError *error )
 	return false;
 }
 
-bool CCBClient::SplitCCBContact( char const *ccb_contact, MyString &ccb_address, MyString &ccbid, CondorError *error )
+bool CCBClient::SplitCCBContact( char const *ccb_contact, MyString &ccb_address, MyString &ccbid, const MyString & peer, CondorError *error )
 {
 	// expected format: "<address>#ccbid"
 	char const *ptr = strchr(ccb_contact,'#');
 	if( !ptr ) {
 		MyString errmsg;
 		errmsg.formatstr("Bad CCB contact '%s' when connecting to %s.",
-					   ccb_contact, m_target_peer_description.Value());
+					   ccb_contact, peer.Value());
 
 		if( error ) {
 			error->push("CCBClient",CEDAR_ERR_CONNECT_FAILED,errmsg.Value());
@@ -499,11 +520,11 @@ CCBClient::try_next_ccb()
 	}
 
 	MyString ccbid;
-	if( !SplitCCBContact( ccb_contact, m_cur_ccb_address, ccbid, NULL ) ) {
+	if( !SplitCCBContact( ccb_contact, m_cur_ccb_address, ccbid, m_target_peer_description, NULL ) ) {
 		return try_next_ccb();
 	}
 
-	char const *return_address = daemonCoreSockAdapter.publicNetworkIpAddr();
+	char const *return_address = daemonCore->publicNetworkIpAddr();
 
 	// For now, we require that this daemon has a command port.
 	// If needed, we could add support for opening a listen socket here.
@@ -542,6 +563,8 @@ CCBClient::try_next_ccb()
 	ad.Assign(ATTR_CLAIM_ID,m_connect_id);
 	// purely for debugging purposes: identify ourselves
 	ad.Assign(ATTR_NAME, myName());
+	// ATTR_MY_ADDRESS is functional and subject to address rewriting, so
+	// it's OK to use the protocol-blind publicNetworkIpAddr() above.
 	ad.Assign(ATTR_MY_ADDRESS, return_address);
 
 	classy_counted_ptr<CCBRequestMsg> msg = new CCBRequestMsg(ad);
@@ -580,7 +603,7 @@ CCBClient::try_next_ccb()
 
 			// bypass startCommand() and call the command handler directly
 			// this call will take care of deleting server_sock when done
-		daemonCoreSockAdapter.CallCommandHandler(CCB_REQUEST,server_sock);
+		daemonCore->CallCommandHandler(CCB_REQUEST,server_sock);
 	}
 	else {
 		ccb_server->sendMsg(msg.get());
@@ -593,7 +616,7 @@ CCBClient::try_next_ccb()
 void
 CCBClient::CancelReverseConnect()
 {
-	if( m_target_sock ) {
+	if( daemonCore && m_target_sock ) {
 		ReverseConnectCallback( NULL );
 	}
 }
@@ -681,7 +704,7 @@ CCBClient::RegisterReverseConnectCallback()
 		// security session management.  Again, that stuff all happens
 		// later in the reverse direction.
 
-		daemonCoreSockAdapter.Register_Command(
+		daemonCore->Register_Command(
 			CCB_REVERSE_CONNECT,
 			"CCB_REVERSE_CONNECT",
 			CCBClient::ReverseConnectCommandHandler,
@@ -706,7 +729,7 @@ CCBClient::RegisterReverseConnectCallback()
 		if( timeout < 0 ) {
 			timeout = 0;
 		}
-		m_deadline_timer = daemonCoreSockAdapter.Register_Timer (
+		m_deadline_timer = daemonCore->Register_Timer (
 			timeout,
 			(TimerHandlercpp)&CCBClient::DeadlineExpired,
 			"CCBClient::DeadlineExpired",
@@ -732,7 +755,7 @@ void
 CCBClient::UnregisterReverseConnectCallback()
 {
 	if( m_deadline_timer != -1 ) {
-		daemonCoreSockAdapter.Cancel_Timer(m_deadline_timer);
+		daemonCore->Cancel_Timer(m_deadline_timer);
 		m_deadline_timer = -1;
 	}
 
@@ -806,7 +829,7 @@ CCBClient::ReverseConnectCallback(Sock *sock)
 	// We are all done creating the connection (or failing to do so).
 	// Call any function that may have been registered with
 	// DaemonCore::Register_Socket().
-	daemonCoreSockAdapter.CallSocketHandler(m_target_sock,false);
+	daemonCore->CallSocketHandler(m_target_sock,false);
 
 	// No more use of m_target_sock, because it may have been deleted
 	// by now.
