@@ -276,6 +276,19 @@ GetComponents( vector< pair< string, ExprTree* > > &attrs ) const
 	}
 }
 
+void ClassAd::
+GetComponents( vector< pair< string, ExprTree* > > &attrs,
+	const References &whitelist ) const
+{
+	attrs.clear( );
+	for ( References::const_iterator wl_itr = whitelist.begin(); wl_itr != whitelist.end(); wl_itr++ ) {
+		AttrList::const_iterator attr_itr = attrList.find( *wl_itr );
+		if ( attr_itr != attrList.end() ) {
+			attrs.push_back( make_pair( attr_itr->first, attr_itr->second ) );
+		}
+	}
+}
+
 
 // --- begin integer attribute insertion ----
 bool ClassAd::
@@ -287,7 +300,7 @@ InsertAttr( const string &name, int value, Value::NumberFactor f )
 	val.SetIntegerValue( value );
 	plit  = Literal::MakeLiteral( val, f );
 	
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false ) );
 }
 
 
@@ -299,7 +312,7 @@ InsertAttr( const string &name, long value, Value::NumberFactor f )
 
 	val.SetIntegerValue( value );
 	plit = Literal::MakeLiteral( val, f );
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false ) );
 }
 
 
@@ -311,7 +324,7 @@ InsertAttr( const string &name, long long value, Value::NumberFactor f )
 
 	val.SetIntegerValue( value );
 	plit = Literal::MakeLiteral( val, f );
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false) );
 }
 
 
@@ -357,7 +370,7 @@ InsertAttr( const string &name, double value, Value::NumberFactor f )
 	val.SetRealValue( value );
 	plit  = Literal::MakeLiteral( val, f );
 	
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false ) );
 }
 	
 
@@ -383,7 +396,7 @@ InsertAttr( const string &name, bool value )
 	val.SetBooleanValue( value );
 	plit  = Literal::MakeLiteral( val );
 	
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false ) );
 }
 
 
@@ -408,7 +421,7 @@ InsertAttr( const string &name, const char *value )
 	val.SetStringValue( value );
 	plit  = Literal::MakeLiteral( val );
 	
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false ) );
 }
 
 bool ClassAd::
@@ -428,7 +441,7 @@ InsertAttr( const string &name, const string &value )
 	val.SetStringValue( value );
 	plit  = Literal::MakeLiteral( val );
 	
-	return( Insert( name, plit ) );
+	return( Insert( name, plit, false ) );
 }
 
 
@@ -636,6 +649,20 @@ Lookup( const string &name ) const
 	return tree;
 }
 
+ExprTree *ClassAd::
+LookupIgnoreChain( const string &name ) const
+{
+	ExprTree *tree;
+	AttrList::const_iterator itr;
+
+	itr = attrList.find( name );
+	if (itr != attrList.end()) {
+		tree = itr->second;
+	} else {
+		tree = NULL;
+	}
+	return tree;
+}
 
 ExprTree *ClassAd::
 LookupInScope( const string &name, const ClassAd *&finalScope ) const
