@@ -138,7 +138,7 @@ class ClassAd : public ExprTree
 			@param value The integer value of the attribute.
 			@param f The multiplicative factor to be attached to value.
 			@see Value::NumberFactor
-e		*/
+		*/
 		bool InsertAttr( const std::string &attrName,int value, 
 				Value::NumberFactor f=Value::NO_FACTOR );
 		bool InsertAttr( const std::string &attrName,long value, 
@@ -541,14 +541,16 @@ e		*/
 
         /** Return the number of attributes at the root level of this ClassAd.
          */
-        int size(void) const { return attrList.size(); }
+        int size(void) const { return (int)attrList.size(); }
 		//@}
 
+		void rehash(size_t s) { attrList.rehash(s);}
 		/** Deconstructor to get the components of a classad
 		 * 	@param vec A vector of (name,expression) pairs which are the
 		 * 		attributes of the classad
 		 */
 		void GetComponents( std::vector< std::pair< std::string, ExprTree *> > &vec ) const;
+		void GetComponents( std::vector< std::pair< std::string, ExprTree *> > &vec, const References &whitelist ) const;
 
         /** Make sure everything in the ad is in this ClassAd.
          *  This is different than CopyFrom() because we may have many 
