@@ -155,9 +155,16 @@ ReplaceLeftAd( ClassAd *ad )
 		// its parent scope to be the context ad.
 		lCtx->SetParentScope(this);
 		lad->SetParentScope(lCtx);
+
+		if ( _useOldClassAdSemantics ) {
+			lad->alternateScope = rad;
+		}
 	} else {
 		Delete( "LEFT" );
 	}
+	if ( rad && _useOldClassAdSemantics ) {
+		rad->alternateScope = lad;
+	}
 	return true;
 }
 
@@ -181,9 +188,16 @@ ReplaceRightAd( ClassAd *ad )
 		// its parent scope to be the context ad.
 		rCtx->SetParentScope(this);
 		rad->SetParentScope(rCtx);
+
+		if ( _useOldClassAdSemantics ) {
+			rad->alternateScope = lad;
+		}
 	} else {
 		Delete( "RIGHT" );
 	}
+	if ( lad && _useOldClassAdSemantics ) {
+		lad->alternateScope = rad;
+	}
 	return true;
 }
 
@@ -223,6 +237,10 @@ RemoveLeftAd( )
 	Remove( "LEFT" );
 	if( lad ) {
 		lad->SetParentScope( ladParent );
+		if ( _useOldClassAdSemantics && rad ) {
+			lad->alternateScope = NULL;
+			rad->alternateScope = NULL;
+		}
 	}
 	ladParent = NULL;
 	lad = NULL;
@@ -237,6 +255,10 @@ RemoveRightAd( )
 	Remove( "RIGHT" );
 	if( rad ) {
 		rad->SetParentScope( radParent );
+		if ( _useOldClassAdSemantics && lad ) {
+			lad->alternateScope = NULL;
+			rad->alternateScope = NULL;
+		}
 	}
 	radParent = NULL;
 	rad = NULL;
@@ -279,10 +301,12 @@ OptimizeAdForMatchmaking( ClassAd *ad, bool is_right, std::string *error_msg )
 
 		// insert "my" into this ad so that references that use it
 		// can be flattened
-	Value me;
-	ExprTree * pLit;
-	me.SetClassAdValue( ad );
-	ad->Insert("my",(pLit=Literal::MakeLiteral(me)));
+	if ( !_useOldClassAdSemantics ) {
+		Value me;
+		ExprTree * pLit;
+		me.SetClassAdValue( ad );
+		ad->Insert("my",(pLit=Literal::MakeLiteral(me)));
+	}
 
 		// insert "target" and "other" into this ad so references can be
 		// _partially_ flattened to the more efficient .RIGHT or .LEFT
@@ -332,7 +356,9 @@ OptimizeAdForMatchmaking( ClassAd *ad, bool is_right, std::string *error_msg )
 		// After flatenning, no references should remain to MY or TARGET.
 		// Even if there are, those can be resolved by the context ads, so
 		// we don't need to leave these attributes in the ad.
-	ad->Delete("my");
+	if ( !_useOldClassAdSemantics ) {
+		ad->Delete("my");
+	}
 	ad->Delete("other"); 
 	ad->Delete("target");
 
@@ -361,7 +387,7 @@ EvalMatchExpr(ExprTree *match_expr)
 
 	if( EvaluateExpr( match_expr, val ) ) {
 		bool result = false;
-		if( val.IsBooleanValue( result ) ) {
+		if( val.IsBooleanValueEquiv( result ) ) {
 			return result;
 		}
 		long long int_result = 0;
