@@ -191,16 +191,6 @@ bool ClassAdParser::ParseClassAd(FILE *file, ClassAd &classad, bool full)
 	return success;
 }
 
-bool ClassAdParser::ParseClassAd(istream &stream, ClassAd &classad, bool full)
-{
-	bool                    success;
-	InputStreamLexerSource lexer_source(stream);
-
-	success = ParseClassAd(&lexer_source, classad, full);
-
-	return success;
-}
-
 bool ClassAdParser::
 ParseClassAd(LexerSource *lexer_source, ClassAd &classad, bool full)
 {
@@ -288,16 +278,6 @@ ParseClassAd(FILE *file, bool full)
 	return ad;
 }
 
-ClassAd *ClassAdParser::
-ParseClassAd(istream &stream, bool full)
-{
-	ClassAd                *ad;
-	InputStreamLexerSource lexer_source(stream);
-
-	ad = ParseClassAd(&lexer_source, full);
-	return ad;
-}
-
 ClassAd *ClassAdParser::
 ParseClassAd(LexerSource *lexer_source, bool full)
 {
@@ -342,18 +322,25 @@ parseExpression( ExprTree *&tree, bool full )
 		lexer.ConsumeToken();
 		treeL = tree;
 
-		parseExpression(treeM);
-		if( ( tt = lexer.ConsumeToken() ) != Lexer::LEX_COLON ) {
-			CondorErrno = ERR_PARSE_ERROR;
-			CondorErrMsg="expected LEX_COLON, but got "+
-				string(Lexer::strLexToken(tt));
-			if( treeL ) delete treeL; 
-			if( treeM ) delete treeM;
-			tree = NULL;
-			return false;
+		if (lexer.PeekToken() == Lexer::LEX_COLON) {
+			// middle expression is empty
+			lexer.ConsumeToken(); // consume the colon
+			treeM = NULL; // mean return the lhs
+		} else {
+			// we have a middle expression
+			parseExpression(treeM);
+			if( ( tt = lexer.ConsumeToken() ) != Lexer::LEX_COLON ) {
+				CondorErrno = ERR_PARSE_ERROR;
+				CondorErrMsg="expected LEX_COLON, but got "+
+					string(Lexer::strLexToken(tt));
+				if( treeL ) delete treeL; 
+				if( treeM ) delete treeM;
+				tree = NULL;
+				return false;
+			}
 		}
 		parseExpression(treeR);
-		if( treeL && treeM && treeR && ( newTree=Operation::MakeOperation( 
+		if( treeL && treeR && ( newTree=Operation::MakeOperation( 
 				Operation::TERNARY_OP, treeL, treeM, treeR ) ) ) {
 			tree = newTree;
 			return( true );
@@ -1378,13 +1365,4 @@ Lexer::TokenType ClassAdParser::ConsumeToken(void)
     }
 }
 
-std::istream & operator>>(std::istream &stream, ClassAd &ad)
-{
-	ClassAdParser parser;
-
-	parser.ParseClassAd(stream, ad);
-	return stream;
-}
-
-
 } // classad
