@@ -76,17 +76,6 @@ ParseClassAd( FILE *file, ClassAd &ad )
 	return classad_out != NULL;
 }
 
-bool ClassAdXMLParser::
-ParseClassAd( std::istream& stream, ClassAd &ad )
-{
-	ClassAd *classad_out;
-	InputStreamLexerSource lexer_source(stream);
-
-	lexer.SetLexerSource(&lexer_source);
-	classad_out = ParseClassAd(&ad);
-	return classad_out != NULL;
-}
-
 ClassAd *ClassAdXMLParser::
 ParseClassAd( const string &buffer)
 {
@@ -119,18 +108,6 @@ ParseClassAd(FILE *file)
 	return classad;
 }
 
-ClassAd *ClassAdXMLParser::
-ParseClassAd( istream& stream)
-{
-	ClassAd *classad;
-	InputStreamLexerSource lexer_source(stream);
-	
-	lexer.SetLexerSource(&lexer_source);
-	classad = ParseClassAd();
-	return classad;
-}
-
-
 ClassAd *ClassAdXMLParser::
 ParseClassAd(ClassAd *classad_in)
 {
@@ -358,7 +335,6 @@ ParseNumberOrString(XMLLexer::TagID tag_id)
 		}
 		else {        // its a string
 			bool validStr = true;
-			//token.text.push_back('\0'); // force an explicit null terminator (because that's that the normal lexer does.)
 			convert_escapes(token.text, validStr );
 			if(!validStr) {  // invalid string because it had /0 escape sequence
 				return NULL;
