@@ -37,7 +37,7 @@ public:
 	virtual int open( const char *url, int flags, int mode );
 	virtual int close();
 	virtual int read( off_t offset, char *data, int length );
-	virtual int write( off_t offset, char *data, int length );
+	virtual int write( off_t offset, const char *data, int length );
 
 	virtual int fcntl( int cmd, int arg );
 	virtual int ioctl( int cmd, long arg );
