@@ -46,11 +46,9 @@
 
 #include "collector.h"
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
-#if defined(HAVE_DLOPEN) || defined(WIN32)
+#if defined(HAVE_DLOPEN) && !defined(DARWIN)
 #include "CollectorPlugin.h"
 #endif
-#endif
 
 #include "ccb_server.h"
 
@@ -179,6 +177,8 @@ void CollectorDaemon::Init()
 	}
 	daemonCore->Register_CommandWithPayload(QUERY_STORAGE_ADS,"QUERY_STORAGE_ADS",
 		(CommandHandler)receive_query_cedar,"receive_query_cedar",NULL,READ);
+	daemonCore->Register_CommandWithPayload(QUERY_ACCOUNTING_ADS,"QUERY_ACCOUNTING_ADS",
+		(CommandHandler)receive_query_cedar,"receive_query_cedar",NULL,READ);
 	daemonCore->Register_CommandWithPayload(QUERY_NEGOTIATOR_ADS,"QUERY_NEGOTIATOR_ADS",
 		(CommandHandler)receive_query_cedar,"receive_query_cedar",NULL,READ);
 	daemonCore->Register_CommandWithPayload(QUERY_HAD_ADS,"QUERY_HAD_ADS",
@@ -222,6 +222,9 @@ void CollectorDaemon::Init()
 	daemonCore->Register_CommandWithPayload(INVALIDATE_STORAGE_ADS,
 		"INVALIDATE_STORAGE_ADS", (CommandHandler)receive_invalidation,
 		"receive_invalidation",NULL,DAEMON);
+	daemonCore->Register_CommandWithPayload(INVALIDATE_ACCOUNTING_ADS,
+		"INVALIDATE_ACCOUNTING_ADS", (CommandHandler)receive_invalidation,
+		"receive_invalidation",NULL,NEGOTIATOR);
 	daemonCore->Register_CommandWithPayload(INVALIDATE_NEGOTIATOR_ADS,
 		"INVALIDATE_NEGOTIATOR_ADS", (CommandHandler)receive_invalidation,
 		"receive_invalidation",NULL,NEGOTIATOR);
@@ -278,6 +281,8 @@ void CollectorDaemon::Init()
 		(CommandHandler)receive_update,"receive_update", NULL, DAEMON);
     daemonCore->Register_CommandWithPayload(UPDATE_GRID_AD,"UPDATE_GRID_AD",
 		(CommandHandler)receive_update,"receive_update",NULL,DAEMON);
+	daemonCore->Register_CommandWithPayload(UPDATE_ACCOUNTING_AD,"UPDATE_ACCOUNTING_AD",
+		(CommandHandler)receive_update,"receive_update",NULL,NEGOTIATOR);
 
     // install command handlers for updates with acknowledgement
 
@@ -413,7 +418,7 @@ int CollectorDaemon::receive_query_cedar(Service* /*s*/,
 			if (stats_config != "stored") {
 				stats_ad = new ClassAd();
 				daemonCore->dc_stats.Publish(*stats_ad, stats_config.Value());
-				daemonCore->monitor_data.ExportData(stats_ad);
+				daemonCore->monitor_data.ExportData(stats_ad, true);
 				collectorStats.publishGlobal(stats_ad, stats_config.Value());
 				stats_ad->ChainToAd(curr_ad);
 				curr_ad = stats_ad; // send the stats ad instead of the self ad.
@@ -538,6 +543,11 @@ CollectorDaemon::receive_query_public( int command )
 		whichAds = STORAGE_AD;
 		break;
 
+	  case QUERY_ACCOUNTING_ADS:
+		dprintf (D_FULLDEBUG,"Got QUERY_ACCOUNTING_ADS\n");
+		whichAds = ACCOUNTING_AD;
+		break;
+
 	  case QUERY_NEGOTIATOR_ADS:
 		dprintf (D_FULLDEBUG,"Got QUERY_NEGOTIATOR_ADS\n");
 		whichAds = NEGOTIATOR_AD;
@@ -681,6 +691,11 @@ int CollectorDaemon::receive_invalidation(Service* /*s*/,
 		whichAds = STORAGE_AD;
 		break;
 
+	  case INVALIDATE_ACCOUNTING_ADS:
+		dprintf (D_ALWAYS, "Got INVALIDATE_ACCOUNTING_ADS\n");
+		whichAds = ACCOUNTING_AD;
+		break;
+
 	  case INVALIDATE_ADS_GENERIC:
 		dprintf(D_ALWAYS, "Got INVALIDATE_ADS_GENERIC\n");
 		whichAds = GENERIC_AD;
@@ -708,10 +723,8 @@ int CollectorDaemon::receive_invalidation(Service* /*s*/,
     /* let the off-line plug-in invalidate the given ad */
     offline_plugin_.invalidate ( command, cad );
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
-#if defined(HAVE_DLOPEN) || defined(WIN32)
+#if defined(HAVE_DLOPEN) && !defined(DARWIN)
 	CollectorPluginManager::Invalidate(command, cad);
-#endif
 #endif
 
 	if (viewCollectorTypes) {
@@ -774,10 +787,8 @@ int CollectorDaemon::receive_update(Service* /*s*/, int command, Stream* sock)
 	/* let the off-line plug-in have at it */
 	offline_plugin_.update ( command, *cad );
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
-#if defined(HAVE_DLOPEN) || defined(WIN32)
+#if defined(HAVE_DLOPEN) && !defined(DARWIN)
 	CollectorPluginManager::Update(command, *cad);
-#endif
 #endif
 
 	if (viewCollectorTypes) {
@@ -907,10 +918,8 @@ int CollectorDaemon::receive_update_expect_ack( Service* /*s*/,
 	if(cad)
     offline_plugin_.update ( command, *cad );
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
-#if defined(HAVE_DLOPEN) || defined(WIN32)
+#if defined(HAVE_DLOPEN) && !defined(DARWIN)
     CollectorPluginManager::Update ( command, *cad );
-#endif
 #endif
 
 	if (viewCollectorTypes) {
@@ -1421,6 +1430,8 @@ void CollectorDaemon::Config()
 							"sendCollectorAd" );
 	}
 
+	collector.m_allowOnlyOneNegotiator = param_boolean("COLLECTOR_ALLOW_ONLY_ONE_NEGOTIATOR", false);
+
 	tmp = param(COLLECTOR_REQUIREMENTS);
 	MyString collector_req_err;
 	if( !collector.setCollectorRequirements( tmp, collector_req_err ) ) {
@@ -1734,6 +1745,28 @@ void CollectorDaemon::send_classad_to_sock(int cmd, ClassAd* theAd) {
         return;
     }
 
+	ClassAd *pvtAd = NULL;
+	if (cmd == UPDATE_STARTD_AD) {
+		// Forward the startd private ad as well.  This allows the
+		// target collector to act as an aggregator for multiple collectors
+		// that balance the load of authenticating connections from
+		// the rest of the pool.
+		AdNameHashKey hk;
+		ASSERT( makeStartdAdHashKey (hk, theAd) );
+		pvtAd = collector.lookup(STARTD_PVT_AD,hk);
+	}
+
+	bool should_forward = true;
+	theAd->LookupBool( ATTR_SHOULD_FORWARD, should_forward );
+	if ( !should_forward && pvtAd ) {
+		pvtAd->LookupBool( ATTR_SHOULD_FORWARD, should_forward );
+	}
+	if ( !should_forward ) {
+		// TODO Should we remove the ShouldForward attribute?
+		dprintf( D_FULLDEBUG, "Trying to forward ad on, but %s=False\n", ATTR_SHOULD_FORWARD );
+		return;
+	}
+
     for (vector<vc_entry>::iterator e(vc_list.begin());  e != vc_list.end();  ++e) {
         DCCollector* view_coll = e->collector;
         Sock* view_sock = e->sock;
@@ -1802,22 +1835,13 @@ void CollectorDaemon::send_classad_to_sock(int cmd, ClassAd* theAd) {
             }
         }
 
-        if (cmd == UPDATE_STARTD_AD) {
-            // Forward the startd private ad as well.  This allows the
-            // target collector to act as an aggregator for multiple collectors
-            // that balance the load of authenticating connections from
-            // the rest of the pool.
-            AdNameHashKey hk;
-            ClassAd *pvt_ad;
-            ASSERT( makeStartdAdHashKey (hk, theAd) );
-            pvt_ad = collector.lookup(STARTD_PVT_AD,hk);
-            if (pvt_ad) {
-                if (!putClassAd(view_sock, *pvt_ad)) {
-                    dprintf( D_ALWAYS, "Can't forward startd private classad to View Collector %s\n", view_name);
-                    view_sock->end_of_message();
-                    view_sock->close();
-                    continue;
-                }
+        // If there's a private startd ad, send that as well.
+        if (pvtAd) {
+            if (!putClassAd(view_sock, *pvtAd)) {
+                dprintf( D_ALWAYS, "Can't forward startd private classad to View Collector %s\n", view_name);
+                view_sock->end_of_message();
+                view_sock->close();
+                continue;
             }
         }
 
