@@ -65,6 +65,7 @@ CollectorEngine::CollectorEngine (CollectorStats *stats ) :
 	MasterAds     (GREATER_TABLE_SIZE, &adNameHashFunction),
 	StorageAds    (GREATER_TABLE_SIZE, &adNameHashFunction),
 	XferServiceAds(GREATER_TABLE_SIZE, &adNameHashFunction),
+	AccountingAds (GREATER_TABLE_SIZE, &adNameHashFunction),
 	CkptServerAds (LESSER_TABLE_SIZE , &adNameHashFunction),
 	GatewayAds    (LESSER_TABLE_SIZE , &adNameHashFunction),
 	CollectorAds  (LESSER_TABLE_SIZE , &adNameHashFunction),
@@ -77,8 +78,12 @@ CollectorEngine::CollectorEngine (CollectorStats *stats ) :
 {
 	clientTimeout = 20;
 	machineUpdateInterval = 30;
+	m_forwardInterval = machineUpdateInterval / 3;
+	m_forwardFilteringEnabled = false;
 	housekeeperTimerID = -1;
 
+	m_allowOnlyOneNegotiator = param_boolean("COLLECTOR_ALLOW_ONLY_ONE_NEGOTIATOR", false);
+
 	collectorStats = stats;
 	m_collector_requirements = NULL;
 }
@@ -98,6 +103,7 @@ CollectorEngine::
 	killHashTable (LicenseAds);
 	killHashTable (MasterAds);
 	killHashTable (StorageAds);
+	killHashTable (AccountingAds);
 	killHashTable (CkptServerAds);
 	killHashTable (GatewayAds);
 	killHashTable (CollectorAds);
@@ -152,6 +158,14 @@ CollectorEngine::setCollectorRequirements( char const *str, MyString &error_desc
 int CollectorEngine::
 scheduleHousekeeper (int timeout)
 {
+	// Are we filtering updates that we forward to the view collector?
+	std::string watch_list;
+	param(watch_list,"COLLECTOR_FORWARD_WATCH_LIST", "State,Cpus,Memory,IdleJobs,ClaimId,Capability,ClaimIdList,ChildClaimIds");
+	m_forwardWatchList.clearAll();
+	m_forwardWatchList.initializeFromString(watch_list.c_str());
+
+	m_forwardFilteringEnabled = param_boolean( "COLLECTOR_FORWARD_FILTERING", false );
+
 	// cancel outstanding housekeeping requests
 	if (housekeeperTimerID != -1)
 	{
@@ -165,6 +179,8 @@ scheduleHousekeeper (int timeout)
 	// set to new timeout interval
 	machineUpdateInterval = timeout;
 
+	m_forwardInterval = param_integer("COLLECTOR_FORWARD_INTERVAL", machineUpdateInterval / 3, 0);
+
 	// if timeout interval was non-zero (i.e., housekeeping required) ...
 	if (timeout > 0)
 	{
@@ -276,6 +292,7 @@ walkHashTable (AdTypes adType, int (*scanFunction)(ClassAd *))
 		return walkGenericTables(scanFunction);
 	} else if (ANY_AD == adType) {
 		return
+			AccountingAds.walk(scanFunction) &&
 			StorageAds.walk(scanFunction) &&
 			CkptServerAds.walk(scanFunction) &&
 			LicenseAds.walk(scanFunction) &&
@@ -362,9 +379,11 @@ collect (int command, Sock *sock, const condor_sockaddr& from, int &insert)
 	const char* authn_user = sock->getFullyQualifiedUser();
 	if (authn_user) {
 		clientAd->Assign("AuthenticatedIdentity", authn_user);
+		clientAd->Assign("AuthenticationMethod", sock->getAuthenticationMethodUsed());
 	} else {
 		// remove it from the ad if it's not authenticated.
 		clientAd->Delete("AuthenticatedIdentity");
+		clientAd->Delete("AuthenticationMethod");
 	}
 
 	rval = collect(command, clientAd, from, insert, sock);
@@ -722,6 +741,19 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 							  clientAd, hk, hashString, insert, from );
 		break;
 
+	  case UPDATE_ACCOUNTING_AD:
+		if (!makeAccountingAdHashKey (hk, clientAd))
+		{
+			dprintf (D_ALWAYS, "Could not make hashkey --- ignoring ad\n");
+			insert = -3;
+			retVal = 0;
+			break;
+		}
+		hashString.Build( hk );
+		retVal=updateClassAd (AccountingAds, "AccountingAd  ", "Accouting",
+							  clientAd, hk, hashString, insert, from );
+		break;
+
 	  case UPDATE_NEGOTIATOR_AD:
 		if (!makeNegotiatorAdHashKey (hk, clientAd))
 		{
@@ -731,10 +763,12 @@ collect (int command,ClassAd *clientAd,const condor_sockaddr& from,int &insert,S
 			break;
 		}
 		hashString.Build( hk );
+		if (m_allowOnlyOneNegotiator) {
 			// first, purge all the existing negotiator ads, since we
 			// want to enforce that *ONLY* 1 negotiator is in the
 			// collector any given time.
-		purgeHashTable( NegotiatorAds );
+			purgeHashTable( NegotiatorAds );
+		}
 		retVal=updateClassAd (NegotiatorAds, "NegotiatorAd  ", "Negotiator",
 							  clientAd, hk, hashString, insert, from );
 		break;
@@ -1009,8 +1043,10 @@ updateClassAd (CollectorHashTable &hashTable,
 		// no ... new ad
 		dprintf (D_ALWAYS, "%s: Inserting ** \"%s\"\n", adType, hashString.Value() );
 
-		// Update statistics
-		collectorStats->update( label, NULL, new_ad );
+		// Update statistics, but not for private ads we can't see
+		if (strcmp(label, "StartdPvt") != 0) {
+			collectorStats->update( label, NULL, new_ad );
+		}
 
 		// Now, store it away
 		if (hashTable.insert (hk, new_ad) == -1)
@@ -1020,6 +1056,10 @@ updateClassAd (CollectorHashTable &hashTable,
 		
 		insert = 1;
 		
+		if ( m_forwardFilteringEnabled && ( strcmp( label, "Start" ) == 0 || strcmp( label, "StartdPvt" ) == 0 || strcmp( label, "Submittor" ) == 0 ) ) {
+			new_ad->Assign( ATTR_LAST_FORWARDED, (int)time(NULL) );
+		}
+
 		return new_ad;
 	}
 	else
@@ -1028,7 +1068,9 @@ updateClassAd (CollectorHashTable &hashTable,
 		dprintf (D_FULLDEBUG, "%s: Updating ... \"%s\"\n", adType, hashString.Value() );
 
 		// Update statistics
-		collectorStats->update( label, old_ad, new_ad );
+		if (strcmp(label, "StartdPvt") != 0) {
+			collectorStats->update( label, old_ad, new_ad );
+		}
 
 		// Now, finally, store the new ClassAd
 		if (hashTable.remove(hk) == -1) {
@@ -1038,6 +1080,33 @@ updateClassAd (CollectorHashTable &hashTable,
 			EXCEPT( "Error inserting ad" );
 		}
 
+		if ( m_forwardFilteringEnabled && ( strcmp( label, "Start" ) == 0 || strcmp( label, "StartdPvt" ) == 0 || strcmp( label, "Submittor" ) == 0 ) ) {
+			bool forward = false;
+			int last_forwarded = 0;
+			old_ad->LookupInteger( "LastForwarded", last_forwarded );
+			if ( last_forwarded + m_forwardInterval < time(NULL) ) {
+				forward = true;
+			} else {
+				classad::Value old_val;
+				classad::Value new_val;
+				const char *attr;
+				m_forwardWatchList.rewind();
+				while ( (attr = m_forwardWatchList.next()) ) {
+					// This treats attribute-not-present and
+					// attribute-evaluates-to-UNDEFINED as equivalent.
+					if ( old_ad->EvaluateAttr( attr, old_val ) &&
+						 new_ad->EvaluateAttr( attr, new_val ) &&
+						 !new_val.SameAs( old_val ) )
+					{
+						forward = true;
+						break;
+					}
+				}
+			}
+			new_ad->Assign( ATTR_SHOULD_FORWARD, forward );
+			new_ad->Assign( ATTR_LAST_FORWARDED, forward ? (int)time(NULL) : last_forwarded );
+		}
+
 		if (isSelfAd(old_ad)) { __self_ad__ = new_ad; }
 
 		delete old_ad;
@@ -1138,6 +1207,9 @@ housekeeper()
 	dprintf (D_ALWAYS, "\tCleaning StorageAds ...\n");
 	cleanHashTable (StorageAds, now, makeStorageAdHashKey);
 
+	dprintf (D_ALWAYS, "\tCleaning AccountingAds ...\n");
+	cleanHashTable (AccountingAds, now, makeAccountingAdHashKey);
+
 	dprintf (D_ALWAYS, "\tCleaning NegotiatorAds ...\n");
 	cleanHashTable (NegotiatorAds, now, makeNegotiatorAdHashKey);
 
@@ -1273,6 +1345,10 @@ CollectorEngine::LookupByAdType(AdTypes adType,
 			table = &StorageAds;
 			func = makeStorageAdHashKey;
 			break;
+		case ACCOUNTING_AD:
+			table = &AccountingAds;
+			func = makeAccountingAdHashKey;
+			break;
 		case NEGOTIATOR_AD:
 			table = &NegotiatorAds;
 			func = makeStorageAdHashKey; // XXX
