@@ -72,8 +72,14 @@ class CollectorEngine : public Service
 	// walk specified hash table with the given visit procedure
 	int walkHashTable (AdTypes, int (*)(ClassAd *));
 
+	// register the collector's own ad pointer, and check to see if a given ad is that ad.
+	// this is used to allow us to recognise the collector ad during iteration and automatically
+	// insert fresh stats into it when it is fetched.
+	void identifySelfAd(ClassAd * ad);
+	bool isSelfAd(void * ad) { return __self_ad__ != NULL && __self_ad__ == ad; }
+
 	// Publish stats into the collector's ClassAd
-	int publishStats( ClassAd *ad );
+	//int publishStats( ClassAd *ad );
 
 		// returns true on success; false on failure (and sets error_desc)
 	bool setCollectorRequirements( char const *str, MyString &error_desc );
@@ -104,6 +110,7 @@ class CollectorEngine : public Service
 	CollectorHashTable MasterAds;
 	CollectorHashTable StorageAds;
 	CollectorHashTable XferServiceAds;
+	CollectorHashTable AccountingAds;
 
 	// the lesser tables
 	enum {LESSER_TABLE_SIZE = 32};
@@ -148,10 +155,20 @@ class CollectorEngine : public Service
 
 	bool ValidateClassAd(int command,ClassAd *clientAd,Sock *sock);
 
+	void* __self_ad__; // contains address of last Ad for this collector added to the hashtable, do NOT free from here
+					   // this pointer is only used to recognise this collector's ad during a condor_status query
+					   // so it's harmless if this pointer is out of date.
+
 	// Statistics
 	CollectorStats	*collectorStats;
 
 	ClassAd *m_collector_requirements;
+
+	bool m_forwardFilteringEnabled;
+	StringList m_forwardWatchList;
+	int m_forwardInterval;
+public: // so that the config code can set it.
+	bool m_allowOnlyOneNegotiator; // prior to 8.5.8, this was hard-coded to be true.
 };
 
 
