@@ -55,6 +55,14 @@ public:
 	    */
 	void invalidate ( int command, const ClassAd &ad );
 
+	/** This ClassAd is about to removed because its lease has
+		expired (aka it is stale).
+		If it returns false, that means the Collector should go ahead
+		and remove the ad as planned; if returns true,
+		then the collector should NOT delete the ad.
+		*/
+	bool expire ( ClassAd &ad );
+
     /** Start iterations on all class-ads in the repository.
         */
     void rewind ();
@@ -78,8 +86,33 @@ private:
     /** Storage destination for persistent ads */
     char                *_persistent_store;
 
+	/** Storage for parsed ABSENT_REQUIREMENTS contraint */
+	ExprTree *AbsentReq;
+
 	/** overwrite attributes of ad in collection from given ad */
 	void mergeClassAd ( ClassAd &ad, char const *key );
+
+	/** Given an ad, create a key for this ad to use with the
+		persistent collection and store result in mykey.
+		On success, return a const char* pointing to
+		mykey.Value()  (this is what the persistent ad collection
+		code wants); on failure, return NULL.
+		*/
+	const char* makeOfflineKey( const ClassAd &ad, MyString & mykey);
+
+	/** Store ad into the persistent log with the given key, first
+		removing any ad w/ the same key already stored.
+		If key is passed in as NULL, then a key will be created via
+		makeOfflineKey().
+		Return true on success, false on failure. 
+		*/
+	bool persistentStoreAd(const char *key, ClassAd &ad);
+
+	/** Remove the ad in persistent log with the given key.
+		Return true on success, false on failure. 
+		*/
+	bool persistentRemoveAd(const char *key);
+
 };
 
 #endif // _GREEN_COMPUTING_PLUGIN_H_
