@@ -16,11 +16,11 @@
 
 #include "condor_common.h"
 
-#include "../condor_startd.V6/StartdPlugin.h"
+#include <StartdPlugin.h>
 
 #include "hashkey.h"
 
-#include "../condor_daemon_core.V6/condor_daemon_core.h"
+#include <condor_daemon_core.h>
 
 #include "condor_config.h"
 #include "get_daemon_name.h"
