@@ -17,53 +17,38 @@
  *
  ***************************************************************/
 
-#ifndef __CREDD_H__
-#define __CREDD_H__
-#include "condor_common.h"
-#include "sock.h"
+
+#ifndef __WIN_CREDD__
+#define __WIN_CREDD__
+
 #include "condor_daemon_core.h"
-#include "stream.h"
-#include "simplelist.h"
-#include "credential.h"
-#include "X509credentialWrapper.h"
 
+class CredDaemon : public Service {
 
-int CheckCredentials ();
+public:
+	CredDaemon();
+	~CredDaemon();
 
-int LoadCredentialList ();
-int SaveCredentialList ();
+	void reconfig();
+	void cleanup();
 
-int StoreData (const char *, const void *, const int);
-int LoadData (const char *, void *&, int &);
+private:
 
-int
-store_cred_handler(Service * service, int i, Stream *socket);
+	void nop_handler(int, Stream*);
+	void initialize_classad();
+	void update_collector();
+	void invalidate_ad();
 
-int
-get_cred_handler(Service * service, int i, Stream *socket);
+	void refresh_all_handler( int, Stream*);
+	void sweep_timer_handler( void );
 
-int
-rm_cred_handler(Service * service, int i, Stream *socket);
+	char* m_name;
 
-int
-query_cred_handler(Service * service, int i, Stream *stream);
- 
-
-int 
-RefreshProxyThruMyProxy(X509CredentialWrapper * proxy);
-
-int 
-MyProxyGetDelegationReaper(Service *, int exitPid, int exitStatus);
-
-bool
-isSuperUser( const char* user );
-
-void
-Init();
-
-
-int
-init_user_id_from_FQN (const char * _fqn);
+	int m_update_collector_tid;
+	int m_update_collector_interval;
+	int m_cred_sweep_tid;
 
+	ClassAd m_classad;
+};
 
 #endif
