@@ -41,6 +41,10 @@ DCSchedd::DCSchedd( const char* the_name, const char* the_pool )
 {
 }
 
+DCSchedd::DCSchedd( const ClassAd& ad, const char* the_pool )
+	: Daemon( &ad, DT_SCHEDD, the_pool )
+{
+}
 
 DCSchedd::~DCSchedd( void )
 {
@@ -301,7 +305,7 @@ DCSchedd::clearDirtyAttrs( StringList* ids, CondorError * errstack,
 bool
 DCSchedd::reschedule()
 {
-	return sendCommand(RESCHEDULE, Stream::safe_sock, 0);
+	return sendCommand(RESCHEDULE, hasUDPCommandPort() ? Stream::safe_sock : Stream::reli_sock, 0);
 }
 
 bool 
