@@ -65,7 +65,7 @@ public:
 			check that the ClassAd was valid and we have the info we
 			need.  
 			@return true if we have the info, false if not */
-	bool locate( void );
+	virtual bool locate( LocateType method=LOCATE_FULL );
 
 		/**
 		   @param timeout -1 to leave things alone; 0 for no timeout
