@@ -80,6 +80,7 @@ class Probe;
 
 static const int KEEP_STREAM = 100;
 static const int CLOSE_STREAM = 101;
+static const int PASS_STREAM = 102;
 static const int MAX_SOCKS_INHERITED = 4;
 
 /**
@@ -183,6 +184,7 @@ const int DCJOBOPT_NO_ENV_INHERIT   = (1<<2);        // do not pass our env or C
 const int DCJOBOPT_NEVER_USE_SHARED_PORT   = (1<<3);
 const int DCJOBOPT_NO_UDP           = (1<<4);
 const int DCJOBOPT_NO_CONDOR_ENV_INHERIT = (1<<5);   // do not pass CONDOR_INHERIT to the child
+const int DCJOBOPT_USE_SYSTEMD_INET_SOCKET = (1<<6);	     // Pass the reli sock from systemd as the command socket.
 
 #define HAS_DCJOBOPT_SUSPEND_ON_EXEC(mask)  ((mask)&DCJOBOPT_SUSPEND_ON_EXEC)
 #define HAS_DCJOBOPT_NO_ENV_INHERIT(mask)  ((mask)&DCJOBOPT_NO_ENV_INHERIT)
@@ -190,6 +192,7 @@ const int DCJOBOPT_NO_CONDOR_ENV_INHERIT = (1<<5);   // do not pass CONDOR_INHER
 #define HAS_DCJOBOPT_NEVER_USE_SHARED_PORT(mask) ((mask)&DCJOBOPT_NEVER_USE_SHARED_PORT)
 #define HAS_DCJOBOPT_NO_UDP(mask) ((mask)&DCJOBOPT_NO_UDP)
 #define HAS_DCJOBOPT_CONDOR_ENV_INHERIT(mask)  (!((mask)&DCJOBOPT_NO_CONDOR_ENV_INHERIT))
+#define HAS_DCJOBOPT_USE_SYSTEMD_INET_SOCKET(mask) ((mask)&DCJOBOPT_USE_SYSTEMD_INET_SOCKET)
 
 // structure to be used as an argument to Create_Process for tracking process
 // families
@@ -682,8 +685,10 @@ class DaemonCore : public Service
         @return Not_Yet_Documented
     */
     int Was_Not_Responding(pid_t pid);
+    int Got_Alive_Messages(pid_t pid, bool & not_responding); // returns number of DC_CHILDALIVE messages received
+
 	//@}
-        
+
 	/** @name Socket events.
 	 */
 	//@{
@@ -997,6 +1002,16 @@ class DaemonCore : public Service
                         Service *    s);
 
     /** 
+        @param timeslice       Timeslice object specifying interval parameters
+        @param event           Function to call when timer fires.
+        @param event_descrip   String describing the function.
+        @return                Timer id or -1 on error
+    */
+    int Register_Timer (const Timeslice &timeslice,
+                        TimerHandler handler,
+                        const char * event_descrip);
+
+	/** 
         @param timeslice       Timeslice object specifying interval parameters
         @param event           Function to call when timer fires.
         @param event_descrip   String describing the function.
@@ -1044,6 +1059,11 @@ class DaemonCore : public Service
     */
     bool GetTimerTimeslice ( int id, Timeslice &timeslice );
 
+    /** Get the timestamp of when the timer is expected to fire
+     *  @param id The timer's ID
+     *  @return timestamp; 0 if the timer does not exist.
+     */
+    time_t GetNextRuntime(int id) {return t.GetNextRuntime(id);}
 	//@}
 
     /** Not_Yet_Documented
@@ -1936,6 +1956,7 @@ class DaemonCore : public Service
         int reaper_id;
         int hung_tid;   // Timer to detect hung processes
         int was_not_responding;
+        int got_alive_msg; // number of child alive messages received
         int std_pipes[3];  // Pipe handles for automagic DC std pipes.
         MyString* pipe_buf[3];  // Buffers for data written to DC std pipes.
         int stdin_offset;
@@ -2133,6 +2154,12 @@ class DaemonCore : public Service
 
 	char* m_private_network_name;
 
+	// This is the argument passed to InitDCCommandSocket(), taken from
+	// the -port command line argument (default -1). We need this to
+	// correctly handle configuration of the shared port endpoint for
+	// the command socket (if there is one).
+	int m_command_port_arg;
+
 	class CCBListeners *m_ccb_listeners;
 	class SharedPortEndpoint *m_shared_port_endpoint;
 	MyString m_daemon_sock_name;
@@ -2141,6 +2168,17 @@ class DaemonCore : public Service
 	std::vector<Sinful> m_command_sock_sinfuls; // Cached copy of our command sockets' sinful strings.
 	bool m_dirty_command_sock_sinfuls; // true if m_command_sock_sinfuls needs to be reinitialized.
 
+	//
+	// For compabitility with existing configurations and code, when we
+	// enabled mixed-mode (IPv4 and IPv6) by default, we decided to change
+	// to advertising IPv4 addresses first.  Since older mixed-mode clients
+	// will always use the earliest of the addresses it can, this avoids
+	// a number of problems with host-based ALLOW settings.  Of course,
+	// this is against the RFC, and may not be what administrators want to
+	// do as they continue to migrate to IPv6, so now we have a new knob.
+	//
+	bool m_advertise_ipv4_first;
+
 	bool CommandNumToTableIndex(int cmd,int *cmd_index);
 
 	void InitSharedPort(bool in_init_dc_command_socket=false);
@@ -2178,6 +2216,26 @@ class DaemonCore : public Service
  */
 bool InitCommandSockets(int tcp_port, int udp_port, DaemonCore::SockPairVec & socks, bool want_udp, bool fatal);
 
+// helper function to extract the parent address and inherited socket information from
+// the inherit string that is normally passed via the CONDOR_INHERIT environment variable
+// This function extracts parent & socket info then tokenizes the remaining items from
+// the string into the supplied StringList.
+//
+// @return
+//    number of entries in the socks[] array that were populated.
+//    negative values for failure.
+//
+// note: the size of the socks array should be 1 more than the maximum if you want to have room
+// for a terminating null.
+//
+int extractInheritedSocks (
+	const char * inherit,  // in: inherit string, usually from CONDOR_INHERIT environment variable
+	pid_t & ppid,          // out: pid of the parent
+	std::string & psinful, // out: sinful of the parent
+	Stream* socks[],   // out: filled in with items from the inherit string
+	int     cMaxSocks, // in: number of items in the socks array
+	StringList & remaining_items); // out: unparsed items from the inherit string are appended
+
 // helper class that uses C++ constructor/destructor to automatically
 // time a function call. 
 class dc_stats_auto_runtime_probe
