@@ -146,6 +146,16 @@ class TimerManager
                   const char * event_descrip,
                   unsigned     period          =  0);
 
+    /** Create a timer using a timeslice object to control interval.
+        @param timeslice      Timeslice object specifying interval parameters
+        @param handler        Function to call when timer fires.
+        @param event_descrip  String describing the function.
+        @return The ID of the new timer, or -1 on failure
+    */
+    int NewTimer (const Timeslice &timeslice,
+                  TimerHandler handler,
+                  const char * event_descrip);
+
     /** Create a timer using a timeslice object to control interval.
         @param s              Service object of which function is a member.
         @param timeslice      Timeslice object specifying interval parameters
@@ -201,6 +211,13 @@ class TimerManager
 	 */
 	bool GetTimerTimeslice(int id, Timeslice &timeslice);
 
+	/**
+	 * Get the time_t of when the given timer ID will fire.
+	 * @param tid The ID of the timer
+	 * @return Timestamp of when this will fire; 0 if there is no such timer.
+	 */
+	time_t GetNextRuntime(int id);
+
     /// Not_Yet_Documented.
     void CancelAllTimers();
 
