@@ -108,7 +108,7 @@ static unsigned char gperf_downcase[256] =
 #ifndef GPERF_CASE_STRCMP
 #define GPERF_CASE_STRCMP 1
 static int
-gperf_case_strcmp (register const char *s1, register const char *s2)
+gperf_case_strcmp (const char *s1, const char *s2)
 {
   for (;;)
     {
@@ -129,7 +129,7 @@ inline
 #endif
 #endif
 static unsigned int
-mime_hash (register const char *str, register unsigned int len)
+mime_hash (const char *str, unsigned int len)
 {
   static unsigned short asso_values[] =
     {
@@ -160,7 +160,7 @@ mime_hash (register const char *str, register unsigned int len)
       259, 259, 259, 259, 259, 259, 259, 259, 259, 259,
       259, 259, 259, 259, 259, 259
     };
-  register int hval = len;
+  int hval = len;
 
   switch (hval)
     {
@@ -182,7 +182,7 @@ __attribute__ ((__gnu_inline__))
 #endif
 static
 struct mimetype *
-mime_lookup (register const char *str, register unsigned int len)
+mime_lookup (const char *str, unsigned int len)
 {
   static struct mimetype wordlist[] =
     {
@@ -327,11 +327,11 @@ mime_lookup (register const char *str, register unsigned int len)
 
   if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
     {
-      register int key = mime_hash (str, len);
+      int key = mime_hash (str, len);
 
       if (key <= MAX_HASH_VALUE && key >= 0)
         {
-          register const char *s = wordlist[key].ext;
+          const char *s = wordlist[key].ext;
 
           if ((((unsigned char)*str ^ (unsigned char)*s) & ~32) == 0 && !gperf_case_strcmp (str, s))
             return &wordlist[key];
