@@ -65,7 +65,19 @@ dc_soap_accept(Sock *socket, const struct soap *soap)
 		//   3. increase size of send and receive buffers
 		//   4. set SO_KEEPALIVE [done automatically by CEDAR accept()]
 	cursoap->socket = socket->get_file_desc();
-	cursoap->peer = socket->peer_addr().to_sin();
+		// If you use Fedora's IPv6 patch for gsoap, peer is sockaddr_storage.
+		// Upstream has peer of type sockaddr_in.
+		// This trickery is done to keep the code compatible with both.
+	if (sizeof(cursoap->peer) == sizeof(sockaddr_storage))
+	{
+		sockaddr_storage store = socket->peer_addr().to_storage();
+		memcpy(&cursoap->peer, &store, sizeof(cursoap->peer));
+	}
+	else
+	{
+		sockaddr_in store = socket->peer_addr().to_sin();
+		memcpy(&cursoap->peer, &store, sizeof(cursoap->peer));
+	}
 	cursoap->recvfd = soap->socket;
 	cursoap->sendfd = soap->socket;
 	if ( cursoap->recv_timeout > 0 ) {
@@ -246,7 +258,7 @@ dc_soap_init(struct soap *&soap)
 			if (!sock) {
 				EXCEPT("DaemonCore: Failed to allocate SOAP SSL socket");
 			}
-			if (!sock->assign(sock_fd)) {
+			if (!sock->assignSocket(sock_fd)) {
 				EXCEPT("DaemonCore: Failed to use bound SOAP SSL socket");
 			}
 			int index;
