@@ -97,6 +97,11 @@ int TimerManager::NewTimer(Service* s, unsigned deltawhen, TimerHandlercpp handl
 	return( NewTimer(s,deltawhen,(TimerHandler)NULL,handler,(Release)NULL,(Releasecpp)NULL,event_descrip,period,NULL) );
 }
 
+int TimerManager::NewTimer (const Timeslice &timeslice,TimerHandler handler,const char * event_descrip)
+{
+	return NewTimer(NULL,0,handler,(TimerHandlercpp)NULL,(Release)NULL,(Releasecpp)NULL,event_descrip,0,&timeslice);
+}
+
 int TimerManager::NewTimer (Service* s,const Timeslice &timeslice,TimerHandlercpp handler,const char * event_descrip)
 {
 	return NewTimer(s,0,(TimerHandler)NULL,handler,(Release)NULL,(Releasecpp)NULL,event_descrip,0,&timeslice);
@@ -187,6 +192,13 @@ bool TimerManager::GetTimerTimeslice(int id, Timeslice &timeslice)
 	return true;
 }
 
+time_t TimerManager::GetNextRuntime(int id)
+{
+	Timer *timer_ptr = GetTimer( id, NULL );
+	if (!timer_ptr) { return false; }
+
+	return timer_ptr->when;
+}
 int TimerManager::ResetTimer(int id, unsigned when, unsigned period,
 							 bool recompute_when,
 							 Timeslice const *new_timeslice)
@@ -490,7 +502,11 @@ TimerManager::Timeout(int * pNumFired /*= NULL*/, double * pruntime /*=NULL*/)
 				if ( in_timeout->timeslice ) {
 					in_timeout->when += in_timeout->timeslice->getTimeToNextRun();
 				} else {
-					in_timeout->when += in_timeout->period;
+					if( in_timeout->period == TIMER_NEVER ) {
+						in_timeout->when = TIME_T_NEVER;
+					} else {
+						in_timeout->when += in_timeout->period;
+					}
 				}
 				InsertTimer( in_timeout );
 			} else {
