@@ -57,14 +57,16 @@ int parseJobOrDagLine( const char *dagLine, StringList &tokens,
 			const char *fileType, const char *&submitOrDagFile,
 			const char *&directory );
 int setUpOptions( SubmitDagDeepOptions &deepOpts,
-			SubmitDagShallowOptions &shallowOpts );
+			SubmitDagShallowOptions &shallowOpts,
+			StringList &dagFileAttrLines );
 void ensureOutputFilesExist(const SubmitDagDeepOptions &deepOpts,
 			SubmitDagShallowOptions &shallowOpts);
 int getOldSubmitFlags( SubmitDagShallowOptions &shallowOpts );
 int parseArgumentsLine( const MyString &subLine,
 			SubmitDagShallowOptions &shallowOpts );
 void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
-			/* const */ SubmitDagShallowOptions &shallowOpts);
+			/* const */ SubmitDagShallowOptions &shallowOpts,
+			/* const */ StringList &dagFileAttrLines );
 int submitDag( SubmitDagShallowOptions &shallowOpts );
 
 //---------------------------------------------------------------------------
@@ -72,7 +74,7 @@ int main(int argc, char *argv[])
 {
 	printf("\n");
 
-		// Set up the dprintf stuff to write to stderr, so that Condor
+		// Set up the dprintf stuff to write to stderr, so that HTCondor
 		// libraries which use it will write to the right place...
 	dprintf_set_tool_debug("TOOL", 0);
 	set_debug_flags(NULL, D_ALWAYS | D_NOHEADER);
@@ -81,6 +83,9 @@ int main(int argc, char *argv[])
 		// Initialize our Distribution object -- condor vs. hawkeye, etc.
 	myDistro->Init( argc, argv );
 
+		// Save submit append lines from DAG file here (see gittrac #5107).
+	StringList dagFileAttrLines;
+
 		// Load command-line arguments into the deepOpts and shallowOpts
 		// structures.
 	SubmitDagDeepOptions deepOpts;
@@ -117,7 +122,7 @@ int main(int argc, char *argv[])
 	}
 	
 		// Further work to get the shallowOpts structure set up properly.
-	tmpResult = setUpOptions( deepOpts, shallowOpts );
+	tmpResult = setUpOptions( deepOpts, shallowOpts, dagFileAttrLines );
 	if ( tmpResult != 0 ) return tmpResult;
 
 		// Check whether the output files already exist; if so, we may
@@ -137,7 +142,7 @@ int main(int argc, char *argv[])
 	}
 
 		// Write the actual submit file for DAGMan.
-	writeSubmitFile( deepOpts, shallowOpts );
+	writeSubmitFile( deepOpts, shallowOpts, dagFileAttrLines );
 
 	return submitDag( shallowOpts );
 }
@@ -203,7 +208,8 @@ doRecursionNew( SubmitDagDeepOptions &deepOpts,
 
 						// Now run condor_submit_dag on the DAG file.
 					if ( runSubmitDag( deepOpts, submitFile.Value(),
-								directory, false ) != 0 ) {
+								directory, shallowOpts.priority,
+								false ) != 0 ) {
 						result = 1;
 					}
 				}
@@ -228,7 +234,7 @@ doRecursionNew( SubmitDagDeepOptions &deepOpts,
 
 					// Now run condor_submit_dag on the DAG file.
 				if ( runSubmitDag( deepOpts, nestedDagFile, directory,
-							false ) != 0 ) {
+							shallowOpts.priority, false ) != 0 ) {
 					result = 1;
 				}
 			}
@@ -292,7 +298,8 @@ parseJobOrDagLine( const char *dagLine, StringList &tokens,
 */
 int
 setUpOptions( SubmitDagDeepOptions &deepOpts,
-			SubmitDagShallowOptions &shallowOpts )
+			SubmitDagShallowOptions &shallowOpts,
+			StringList &dagFileAttrLines )
 {
 	shallowOpts.strLibOut = shallowOpts.primaryDagFile + ".lib.out";
 	shallowOpts.strLibErr = shallowOpts.primaryDagFile + ".lib.err";
@@ -348,8 +355,9 @@ setUpOptions( SubmitDagDeepOptions &deepOpts,
 	}
 
 	MyString	msg;
-	if ( !GetConfigFile( shallowOpts.dagFiles, deepOpts.useDagDir,
-				shallowOpts.strConfigFile, msg) ) {
+	if ( !GetConfigAndAttrs( shallowOpts.dagFiles, deepOpts.useDagDir,
+				shallowOpts.strConfigFile,
+				dagFileAttrLines, msg) ) {
 		fprintf( stderr, "ERROR: %s\n", msg.Value() );
 		return 1;
 	}
@@ -366,13 +374,13 @@ int
 submitDag( SubmitDagShallowOptions &shallowOpts )
 {
 	printf("-----------------------------------------------------------------------\n");
-	printf("File for submitting this DAG to Condor           : %s\n", 
+	printf("File for submitting this DAG to HTCondor           : %s\n", 
 			shallowOpts.strSubFile.Value());
 	printf("Log of DAGMan debugging messages                 : %s\n",
 		   	shallowOpts.strDebugLog.Value());
-	printf("Log of Condor library output                     : %s\n", 
+	printf("Log of HTCondor library output                     : %s\n", 
 			shallowOpts.strLibOut.Value());
-	printf("Log of Condor library error messages             : %s\n", 
+	printf("Log of HTCondor library error messages             : %s\n", 
 			shallowOpts.strLibErr.Value());
 	printf("Log of the life of condor_dagman itself          : %s\n",
 		   	shallowOpts.strSchedLog.Value());
@@ -416,7 +424,7 @@ submitDag( SubmitDagShallowOptions &shallowOpts )
 	}
 	else
 	{
-		printf("-no_submit given, not submitting DAG to Condor.  "
+		printf("-no_submit given, not submitting DAG to HTCondor.  "
 					"You can do this with:\n");
 		printf("\"condor_submit %s\"\n", shallowOpts.strSubFile.Value());
 	}
@@ -520,7 +528,7 @@ void ensureOutputFilesExist(const SubmitDagDeepOptions &deepOpts,
 				 shallowOpts.strRescueFile.Value() );
 	    fprintf( stderr, "  You may want to resubmit your DAG using that "
 				 "file, instead of \"%s\"\n", shallowOpts.primaryDagFile.Value());
-	    fprintf( stderr, "  Look at the Condor manual for details about DAG "
+	    fprintf( stderr, "  Look at the HTCondor manual for details about DAG "
 				 "rescue files.\n" );
 	    fprintf( stderr, "  Please investigate and either remove \"%s\",\n",
 				 shallowOpts.strRescueFile.Value() );
@@ -641,8 +649,9 @@ EnvFilter::ImportFilter( const MyString &var, const MyString &val ) const
 }
 
 //---------------------------------------------------------------------------
-void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
-			/* const */ SubmitDagShallowOptions &shallowOpts)
+void writeSubmitFile( /* const */ SubmitDagDeepOptions &deepOpts,
+			/* const */ SubmitDagShallowOptions &shallowOpts,
+			/* const */ StringList &dagFileAttrLines )
 {
 	FILE *pSubFile = safe_fopen_wrapper_follow(shallowOpts.strSubFile.Value(), "w");
 	if (!pSubFile)
@@ -683,6 +692,10 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 	fprintf(pSubFile, "output\t\t= %s\n", shallowOpts.strLibOut.Value());
     fprintf(pSubFile, "error\t\t= %s\n", shallowOpts.strLibErr.Value());
     fprintf(pSubFile, "log\t\t= %s\n", shallowOpts.strSchedLog.Value());
+	if ( ! deepOpts.batchName.empty() ) {
+		fprintf(pSubFile, "+%s\t= \"%s\"\n", ATTR_JOB_BATCH_NAME,
+					deepOpts.batchName.c_str());
+	}
 #if !defined ( WIN32 )
     fprintf(pSubFile, "remove_kill_sig\t= SIGUSR1\n" );
 #endif
@@ -770,21 +783,12 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg(shallowOpts.iMaxPost);
     }
 
-	if(shallowOpts.bNoEventChecks)
-	{
-		// strArgs += " -NoEventChecks";
-		printf( "Warning: -NoEventChecks is ignored; please use "
-					"the DAGMAN_ALLOW_EVENTS config parameter instead\n");
-	}
-
-	if(!shallowOpts.bPostRun)
-	{
-		args.AppendArg("-DontAlwaysRunPost");
-	}
-
-	if(deepOpts.bAllowLogError)
-	{
-		args.AppendArg("-AllowLogError");
+	if ( shallowOpts.bPostRunSet ) {
+		if (shallowOpts.bPostRun) {
+			args.AppendArg("-AlwaysRunPost");
+		} else {
+			args.AppendArg("-DontAlwaysRunPost");
+		}
 	}
 
 	if(deepOpts.useDagDir)
@@ -847,9 +851,9 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg("-Import_env");
 	}
 
-	if( deepOpts.priority != 0 ) {
+	if( shallowOpts.priority != 0 ) {
 		args.AppendArg("-Priority");
-		args.AppendArg(deepOpts.priority);
+		args.AppendArg(shallowOpts.priority);
 	}
 
 	MyString arg_str,args_error;
@@ -890,23 +894,25 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 
 	MyString env_str;
 	MyString env_errors;
-	if(!env.getDelimitedStringV1RawOrV2Quoted(&env_str,&env_errors)) {
-		fprintf(stderr,"Failed to insert environment: %s",env_errors.Value());
+	if ( !env.getDelimitedStringV1RawOrV2Quoted( &env_str, &env_errors ) ) {
+		fprintf( stderr,"Failed to insert environment: %s",
+					env_errors.Value() );
 		exit(1);
 	}
     fprintf(pSubFile, "environment\t= %s\n",env_str.Value());
 
-    if(deepOpts.strNotification != "") 
-	{	
-		fprintf(pSubFile, "notification\t= %s\n", deepOpts.strNotification.Value());
+    if ( deepOpts.strNotification != "" ) {	
+		fprintf( pSubFile, "notification\t= %s\n",
+					deepOpts.strNotification.Value() );
     }
 
 		// Append user-specified stuff to submit file...
+
 		// ...first, the insert file, if any...
-	if (shallowOpts.appendFile != "") {
-		FILE *aFile = safe_fopen_wrapper_follow(shallowOpts.appendFile.Value(), "r");
-		if (!aFile)
-		{
+	if ( shallowOpts.appendFile != "" ) {
+		FILE *aFile = safe_fopen_wrapper_follow(
+					shallowOpts.appendFile.Value(), "r");
+		if ( !aFile ) {
 			fprintf( stderr, "ERROR: unable to read submit append file (%s)\n",
 				 	shallowOpts.appendFile.Value() );
 			exit( 1 );
@@ -914,18 +920,27 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 
 		char *line;
 		int lineno = 0;
-		while ((line = getline_trim(aFile, lineno)) != NULL) {
+		while ( (line = getline_trim( aFile, lineno )) != NULL ) {
     		fprintf(pSubFile, "%s\n", line);
 		}
 
-		fclose(aFile);
+		fclose( aFile );
+	}
+
+		// ...now append lines specified in the DAG file...
+	dagFileAttrLines.rewind();
+	char *attrCmd;
+	while ( (attrCmd = dagFileAttrLines.next()) != NULL ) {
+			// Note:  prepending "+" here means that this only works
+			// for setting ClassAd attributes.
+    	fprintf( pSubFile, "+%s\n", attrCmd );
 	}
 
 		// ...now things specified directly on the command line.
 	shallowOpts.appendLines.rewind();
 	char *command;
-	while ((command = shallowOpts.appendLines.next()) != NULL) {
-    	fprintf(pSubFile, "%s\n", command);
+	while ( (command = shallowOpts.appendLines.next()) != NULL ) {
+    	fprintf( pSubFile, "%s\n", command );
 	}
 
     fprintf(pSubFile, "queue\n");
@@ -1034,11 +1049,12 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			}
 			else if (strArg.find("-noev") != -1) // -noeventchecks
 			{
-				shallowOpts.bNoEventChecks = true;
+				printf( "Warning: -NoEventChecks is ignored; please use "
+							"the DAGMAN_ALLOW_EVENTS config parameter instead\n");
 			}
 			else if (strArg.find("-allowlog") != -1) // -allowlogerror
 			{
-				deepOpts.bAllowLogError = true;
+				fprintf( stderr, "Warning: -AllowLogError is no longer supported\n" );
 			}
 			else if (strArg.find("-use") != -1) // -usedagdir
 			{
@@ -1076,6 +1092,15 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 				}
 				shallowOpts.appendLines.append(argv[++iArg]);
 			}
+			else if (strArg.find("-bat") != -1) // -batch-name
+			{
+				if (iArg + 1 >= argc) {
+					fprintf(stderr, "-batch-name argument needs a value\n");
+					printUsage();
+				}
+				deepOpts.batchName = argv[++iArg];
+				deepOpts.batchName.trim_quotes("\""); // trim "" if any
+			}
 			else if (strArg.find("-insert") != -1) // -insert_sub_file
 			{
 				if (iArg + 1 >= argc) {
@@ -1142,8 +1167,22 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			}
 			else if ( (strArg.find("-dontalwaysrun") != -1) ) // DontAlwaysRunPost
 			{
+				if ( shallowOpts.bPostRunSet && shallowOpts.bPostRun ) {
+					fprintf( stderr, "ERROR: -DontAlwaysRunPost and -AlwaysRunPost are both set!\n" );
+					exit(1);
+				}
+				shallowOpts.bPostRunSet = true;
 				shallowOpts.bPostRun = false;
 			}
+			else if ( (strArg.find("-alwaysrun") != -1) ) // AlwaysRunPost
+			{
+				if ( shallowOpts.bPostRunSet && !shallowOpts.bPostRun ) {
+					fprintf( stderr, "ERROR: -DontAlwaysRunPost and -AlwaysRunPost are both set!\n" );
+					exit(1);
+				}
+				shallowOpts.bPostRunSet = true;
+				shallowOpts.bPostRun = true;
+			}
 			else if ( (strArg.find("-dont_use_default_node_log") != -1) )
 			{
 				fprintf( stderr, "Error: -dont_use_default_node_log is no longer allowed\n" );
@@ -1163,7 +1202,7 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 					fprintf(stderr, "-priority argument needs a value\n");
 					printUsage();
 				}
-				deepOpts.priority = atoi(argv[++iArg]);
+				shallowOpts.priority = atoi(argv[++iArg]);
 			}
 			else if ( (strArg.find("-dorecov") != -1) )
 			{
@@ -1264,7 +1303,7 @@ int printUsage(int iExitCode)
 	printf("    -help               (print usage info and exit)\n");
 	printf("    -version            (print version and exit)\n");
 	printf("    -dagman <path>      (Full path to an alternate condor_dagman executable)\n");
-    printf("    -no_submit          (DAG is not submitted to Condor)\n");
+    printf("    -no_submit          (DAG is not submitted to HTCondor)\n");
     printf("    -verbose            (Verbose error messages from condor_submit_dag)\n");
     printf("    -force              (Overwrite files condor_submit_dag uses if they exist)\n");
     printf("    -r <schedd_name>    (Submit to the specified remote schedd)\n");
@@ -1274,12 +1313,14 @@ int printUsage(int iExitCode)
     printf("    -maxjobs <number>   (Maximum number of jobs ever submitted at once)\n");
     printf("    -MaxPre <number>    (Maximum number of PRE scripts to run at once)\n");
     printf("    -MaxPost <number>   (Maximum number of POST scripts to run at once)\n");
-    printf("    -notification <value> (Determines how much email you get from Condor.\n");
+    printf("    -notification <value> (Determines how much email you get from HTCondor.\n");
     printf("        See the condor_submit man page for values.)\n");
     printf("    -NoEventChecks      (Now ignored -- use DAGMAN_ALLOW_EVENTS)\n"); 
     printf("    -DontAlwaysRunPost  (Don't run POST script if PRE script fails)\n");
-    printf("    -AllowLogError      (Allows the DAG to attempt execution even if the log\n");
+    printf("    -AlwaysRunPost      (Run POST script if PRE script fails)\n");
+	printf("    -AllowLogError      (Allows the DAG to attempt execution even if the log\n");
     printf("        reading code finds errors when parsing the submit files)\n"); 
+	printf("        (-AllowLogError is no longer supported as of 8.5.5)\n");
 	printf("    -UseDagDir          (Run DAGs in directories specified in DAG file paths)\n");
     printf("    -debug <number>     (Determines how verbosely DAGMan logs its work\n");
     printf("         about the life of the condor_dagman job.  'value' must be\n");
@@ -1289,6 +1330,7 @@ int printUsage(int iExitCode)
     printf("    -config <filename>  (Specify a DAGMan configuration file)\n");
 	printf("    -append <command>   (Append specified command to .condor.sub file)\n");
 	printf("    -insert_sub_file <filename>   (Insert specified file into .condor.sub file)\n");
+	printf("    -batch-name <name>  (Set the batch name for the dag)\n");
 	printf("    -AutoRescue 0|1     (whether to automatically run newest rescue DAG;\n");
 	printf("         0 = false, 1 = true)\n");
 	printf("    -DoRescueFrom <number>  (run rescue DAG of given number)\n");
