@@ -92,7 +92,7 @@ class Dag {
   
     /** Create a DAG
 		@param dagFile the DAG file name
-        @param maxJobsSubmitted the maximum number of jobs to submit to Condor
+        @param maxJobsSubmitted the maximum number of jobs to submit to HTCondor
                at one time
         @param maxPreScripts the maximum number of PRE scripts to spawn at
 		       one time
@@ -108,8 +108,8 @@ class Dag {
 		       job, to put the node at the head of the ready queue
 		@param retryNodeFirst whether, when a node fails and has retries,
 			   to put the node at the head of the ready queue
-		@param condorRmExe executable to remove Condor jobs
-		@param DAGManJobId Condor ID of this DAGMan process
+		@param condorRmExe executable to remove HTCondor jobs
+		@param DAGManJobId HTCondor ID of this DAGMan process
 		@param prohibitMultiJobs whether submit files queueing multiple
 			   job procs are prohibited
 		@param submitDepthFirst whether ready nodes should be submitted
@@ -128,7 +128,6 @@ class Dag {
     Dag( /* const */ StringList &dagFiles,
 		 const int maxJobsSubmitted,
 		 const int maxPreScripts, const int maxPostScripts, 
-		 bool allowLogError,
 		 bool useDagDir, int maxIdleJobProcs, bool retrySubmitFirst,
 		 bool retryNodeFirst, const char *condorRmExe,
 		 const CondorID *DAGManJobId,
@@ -169,20 +168,21 @@ class Dag {
 
     /// Add a job to the collection of jobs managed by this Dag.
     bool Add( Job& job );
+
     /** Specify a dependency between two jobs. The child job will only
         run after the parent job has finished.
         @param parent The parent job
         @param child The child job (depends on the parent)
         @return true: successful, false: failure
     */
-    bool AddDependency (Job * parent, Job * child);
+    static bool AddDependency (Job * parent, Job * child);
 
 	/** Run waiting/deferred scripts that are ready to run.  Note: scripts
 	    are also limited by halt status and maxpre/maxpost.
 	*/
 	void RunWaitingScripts();
   
-    /** Blocks until the Condor Log file grows.
+    /** Blocks until the HTCondor Log file grows.
         @return true: log file grew, false: timeout or shrinkage
     */
     bool DetectCondorLogGrowth();
@@ -291,11 +291,30 @@ class Dag {
     Job * FindNodeByName (const char * jobName) const;
 
     /** Get pointer to job with condor ID condorID
-        @param condorID the CondorID of the job in the DAG
+        @param condorID the HTCondorID of the job in the DAG
         @return address of Job object, or NULL if not found
     */
     Job * FindNodeByEventID ( const CondorID condorID ) const;
 
+	/** Find all nodes by name -- either a specific node or all
+	 	nodes in the DAG.  To find all nodes in the DAG, pass
+		"ALL_NODES" as the node name on the first call; then NULL
+		in subsequent calls
+		@param nodeName the name of the node to find, or "ALL_NODES",
+			or NULL
+		@param finalSkipMsg the message to print when skipping final
+			nodes, in a form like this: "In parse_script(): skipping node
+			%s because final nodes must have SCRIPT set explicitly
+			(%s: %d)\n"
+		@param file the name of the DAG file that is being parsed
+		@param line the line in the DAG file that is being parsed
+	 	@return a pointer to a Job (node) object, or NULL if there are
+	 		no more node to be found
+	*/
+	Job * FindAllNodesByName( const char* nodeName,
+				const char *finalSkipMsg,
+				const char *file, int line) const;
+
     /** Ask whether a node name exists in the DAG
         @param nodeName the name of the node in the DAG
         @return true if the node exists, false otherwise
@@ -447,14 +466,14 @@ class Dag {
     /** Remove all jobs (using condor_rm) that are currently running.
         All jobs currently marked Job::STATUS_SUBMITTED will be fed
         as arguments to condor_rm via popen.  This function is called
-        when the Dagman Condor job itself is removed by the user via
+        when the Dagman HTCondor job itself is removed by the user via
         condor_rm.  This function <b>is not</b> called when the schedd
         kills Dagman.
-		@param dmJobId: the Condor ID of this DAGMan job.
-		@param removeCondorJobs: iff true we, remove our Condor node jobs.
+		@param dmJobId: the HTCondor ID of this DAGMan job.
+		@param removeCondorJobs: iff true we, remove our HTCondor node jobs.
 			This is set to false when DAGMan itself is condor_rm'ed,
 			because in that case the schedd removes the node jobs.
-		@param bForce: iff true, we run the command to remove Condor
+		@param bForce: iff true, we run the command to remove HTCondor
 			node jobs even if we don't think we have any -- I think this
 			is in case we have a failure in recovery mode before we've
 			read the logs.  Setting bForce to true automatically
@@ -465,7 +484,7 @@ class Dag {
 
     /** Remove all pre- and post-scripts that are currently running.
 	All currently running scripts will be killed via daemoncore.
-	This function is called when the Dagman Condor job itself is
+	This function is called when the Dagman HTCondor job itself is
 	removed by the user via condor_rm.  This function <b>is not</b>
 	called when the schedd kills Dagman.
     */
@@ -615,8 +634,6 @@ class Dag {
 
 	int MaxJobsSubmitted(void) { return _maxJobsSubmitted; }
 
-	bool AllowLogError(void) { return _allowLogError; }
-
 	bool UseDagDir(void) { return _useDagDir; }
 
 	int MaxIdleJobProcs(void) { return _maxIdleJobProcs; }
@@ -640,17 +657,17 @@ class Dag {
 
 	StringList& DagFiles(void) { return _dagFiles; }
 
-	/** Determine whether a job is a NOOP job based on the Condor ID.
-		@param the Condor ID of the job
+	/** Determine whether a job is a NOOP job based on the HTCondor ID.
+		@param the HTCondor ID of the job
 		@return true iff the job is a NOOP
 	*/
 	static bool JobIsNoop( const CondorID &id ) {
 		return (id._cluster == 0) && (id._proc == Job::NOOP_NODE_PROCID);
 	}
 
-	/** Get the part of the CondorID that we're indexing by (cluster ID
+	/** Get the part of the HTCondorID that we're indexing by (cluster ID
 		for "normal" jobs, subproc ID for NOOP jobs).
-		@param the Condor ID of the job
+		@param the HTCondor ID of the job
 		@return the part of the ID to index by
 	*/
 	static int GetIndexID( const CondorID &id ) {
@@ -676,9 +693,9 @@ class Dag {
 	ExtArray<Job*>* FinalRecordedNodes(void);
 
 	// called just after a parse of a dag, this will keep track of the
-	// original intial and final nodes of a dag (after all parent and
+	// original intial and terminal nodes of a dag (after all parent and
 	// child dependencies have been added or course).
-	void RecordInitialAndFinalNodes(void);
+	void RecordInitialAndTerminalNodes(void);
 
 	// Recursively lift all splices into each other and then me
 	OwnedMaterials* LiftSplices(SpliceLayer layer);
@@ -706,7 +723,26 @@ class Dag {
 	// After the nodes in the dag have been made, we take our DIR setting,
 	// and if it isn't ".", we prefix it to the directory setting for each
 	// node, unless it is an absolute path, in which case we ignore it.
-	void PropogateDirectoryToAllNodes(void);
+	void PropagateDirectoryToAllNodes(void);
+
+	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+	// Splice connections.
+
+	/** Set a pin in or pin out connection for this DAG.
+		@param isPinIn: true if this is for a pin in, false for pin out
+		@param nodeName: the name of the node we're connecting
+		@param pinNum: the number of the pin we're connecting
+		@return: true on success, false otherwise
+	*/
+	bool SetPinInOut( bool isPinIn, const char *nodeName, int pinNum );
+
+	/** Connect two splices via pin outs/pin ins
+		@param parentSplice: the splice connected via its pin outs
+		@param childSplice: the splice connected via its pin ins
+		@return: true on success, false otherwise
+	*/
+	static bool ConnectSplices( Dag *parentSplice, Dag *childSplice );
+	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 	/** Set the maximum number of job holds before a node is declared
 		a failure.
@@ -717,9 +753,16 @@ class Dag {
 	JobstateLog &GetJobstateLog() { return _jobstateLog; }
 	bool GetPostRun() const { return _alwaysRunPost; }
 	void SetPostRun(bool postRun) { _alwaysRunPost = postRun; }	
-	void SetDefaultPriorities();
-	void SetDefaultPriority(const int prio) { _defaultPriority = prio; }
-	int GetDefaultPriority() const { return _defaultPriority; }
+
+		// Set the overall priority for this DAG (set on command
+		// line (could be from higher-level DAG) or via config).
+	void SetDagPriority(const int prio) { _dagPriority = prio; }
+
+		// Get this DAG's overall priority.
+	int GetDagPriority() const { return _dagPriority; }
+		
+		// Set priorities for the individual nodes within this DAG.
+	void SetNodePriorities();
 
 	/** Determine whether the DAG is currently halted (waiting for
 		existing jobs to finish but not submitting any new ones).
@@ -745,6 +788,8 @@ class Dag {
 	const char *GetStatusName() const {
 				return _dag_status_names[_dagStatus]; }
 
+	static const char *ALL_NODES;
+
 	/** Determine whether this DAG has a final node.
 		@return true iff the DAG has a final node.
 	*/
@@ -775,7 +820,7 @@ class Dag {
 	// dag as a parent or a child, we can always reference the correct nodes
 	// even in the face of AddDependency().
 	ExtArray<Job*> _splice_initial_nodes;
-	ExtArray<Job*> _splice_final_nodes;
+	ExtArray<Job*> _splice_terminal_nodes;
 
   	// A hash table with key of a splice name and value of the dag parse 
 	// associated with the splice.
@@ -817,24 +862,24 @@ class Dag {
 		SUBMIT_RESULT_NO_SUBMIT,
 	} submit_result_t;
 
-	/** Submit the Condor job for a node, including doing
+	/** Submit the HTCondor job for a node, including doing
 		some higher-level work such as sleeping before the actual submit
 		if necessary.
 		@param the appropriate Dagman object
 		@param the node for which to submit a job
-		@param reference to hold the Condor ID the job is assigned
+		@param reference to hold the HTCondor ID the job is assigned
 		@return submit_result_t (see above)
 	*/
 	submit_result_t SubmitNodeJob( const Dagman &dm, Job *node,
 				CondorID &condorID );
 
-	/** Do the post-processing of a successful submit of a Condor job.
+	/** Do the post-processing of a successful submit of a HTCondor job.
 		@param the node for which the job was just submitted
-		@param the Condor ID of the associated job
+		@param the HTCondor ID of the associated job
 	*/	
 	void ProcessSuccessfulSubmit( Job *node, const CondorID &condorID );
 
-	/** Do the post-processing of a failed submit of a Condor job.
+	/** Do the post-processing of a failed submit of a HTCondor job.
 		@param the node for which the job was just submitted
 		@param the maximum number of submit attempts allowed for a job.
 	*/
@@ -968,7 +1013,7 @@ class Dag {
 	bool UnmonitorLogFile();
 
 protected:
-    /// List of Job objects
+    // List of Job objects
     List<Job>     _jobs;
 
 private:
@@ -980,7 +1025,7 @@ private:
 
 	HashTable<JobID_t, Job *>		_nodeIDHash;
 
-	// Hash by CondorID (really just by the cluster ID because all
+	// Hash by HTCondorID (really just by the cluster ID because all
 	// procs in the same cluster map to the same node).
 	HashTable<int, Job *>			_condorIDHash;
 
@@ -1012,10 +1057,6 @@ private:
 		// The number of DAG job procs currently held.
 	int _numHeldJobProcs;
 
-		// Whether to allow the DAG to run even if we have an error
-		// determining the job log files.
-	bool		_allowLogError;
-
 		// If this is true, nodes for which the job submit fails are retried
 		// before any other ready nodes; otherwise a submit failure puts
 		// a node at the back of the ready queue.  (Default is true.)
@@ -1027,17 +1068,17 @@ private:
 		// queue.  (Default is false.)
 	bool		m_retryNodeFirst;
 
-		// Executable to remove Condor jobs.
+		// Executable to remove HTCondor jobs.
 	const char *	_condorRmExe;
 
-		// Condor ID of this DAGMan process.
+		// HTCondor ID of this DAGMan process.
 	const CondorID *	_DAGManJobId;
 
-	// queue of jobs ready to be submitted to Condor
+	// queue of jobs ready to be submitted to HTCondor
 	PrioritySimpleList<Job*>* _readyQ;
 
 	// queue of submitted jobs not yet matched with submit events in
-	// the Condor job log
+	// the HTCondor job log
     Queue<Job*>* _submitQ;
 
 	ScriptQ* _preScriptQ;
@@ -1138,7 +1179,7 @@ private:
 		// The last time we printed a pending node report.
 	time_t		_lastPendingNodePrintTime;
 
-		// Default Condor ID to use in reseting a node's Condor ID on
+		// Default HTCondor ID to use in reseting a node's HTCondor ID on
 		// retry.
 	static const CondorID	_defaultCondorId;
 
@@ -1190,8 +1231,8 @@ private:
 	// Defaults to true
 	bool _alwaysRunPost;
 
-		// The default priority for nodes in this DAG. (defaults to 0)
-	int _defaultPriority;
+		// The priority for this DAG. (defaults to 0)
+	int _dagPriority;
 
 		// Whether the DAG is currently halted.
 	bool _dagIsHalted;
@@ -1207,6 +1248,61 @@ private:
 	
 		// Object to deal with reporting DAGMan metrics (to Pegasus).
 	DagmanMetrics *_metrics;
+
+	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+	// Splice connections.
+
+		// This is the list of nodes connected to a given pin (in or out).
+	typedef std::vector<Job *> PinNodes;
+
+		// This is a list of pin ins or pin outs.
+	typedef std::vector<PinNodes *> PinList;
+
+		// The pin ins for this DAG.
+	PinList _pinIns;
+
+		// The pin outs for this DAG.
+	PinList _pinOuts;
+
+	/** Get the list of nodes connected to a pin in or pin out
+		@param isPinIn: true if this is for a pin in, false for pin out
+		@param pinNum: the number of the pin for which we're getting
+			nodes
+		@return: a list of nodes connected to this pin
+	*/
+	const PinNodes *GetPinInOut( bool isPinIn, int pinNum ) const;
+
+	/** Get the number of pin ins or pin outs we have in this DAG
+		@param isPinIn: true if this is for pin ins, false for pin outs
+		@return: the number of pin ins or pin outs
+	*/
+	int GetPinCount( bool isPinIn );
+
+	/** Set a pin in or pin out connection for this DAG.
+		@param pinList: the pin list to update
+		@param node: the node to connect
+		@param pinNum: the number of the pin we're connecting
+		@return: true on success, false otherwise
+	*/
+	bool SetPinInOut( PinList &pinList, Job *node, int pinNum );
+
+	/** Get the list of nodes connected to a pin in or pin out
+		@param pinList: the pin list to access
+		@param inOutStr: "in" or "out" as appropriate
+		@param pinNum: the number of the pin for which we're getting
+			nodes
+		@return: a list of nodes connected to this pin
+	*/
+	const static PinNodes *GetPinInOut( const PinList &pinList,
+				const char *inOutStr, int pinNum );
+
+	/** Delete memory allocated as part of this pin list
+		@param pinList: the pin list to delete
+	*/
+	static void DeletePinList( PinList &pinList );
+	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+		// Iterator for ALL_NODES implementation.
+	mutable ListIterator<Job> *_allNodesIt;
 };
 
 #endif /* #ifndef DAG_H */
