@@ -68,9 +68,10 @@ static void Usage() {
             "\t\t[-MaxJobs <int N>]\n"
             "\t\t[-MaxPre <int N>]\n"
             "\t\t[-MaxPost <int N>]\n"
-            "\t\t[-NoEventChecks]\n"
-            "\t\t[-AllowLogError]\n"
+            "\t\t(obsolete) [-NoEventChecks]\n"
+            "\t\t(obsolete) [-AllowLogError]\n"
             "\t\t[-DontAlwaysRunPost]\n"
+            "\t\t[-AlwaysRunPost]\n"
             "\t\t[-WaitForDebug]\n"
             "\t\t[-UseDagDir]\n"
             "\t\t[-AutoRescue <0|1>]\n"
@@ -116,7 +117,6 @@ Dagman::Dagman() :
 	primaryDagFile (""),
 	multiDags (false),
 	startup_cycle_detect (false), // so Coverity is happy
-	allowLogError (false),
 	useDagDir (false),
 	allow_events (CheckEvents::ALLOW_NONE), // so Coverity is happy
 	retrySubmitFirst (true), // so Coverity is happy
@@ -137,12 +137,14 @@ Dagman::Dagman() :
 	_defaultNodeLog(""),
 	_generateSubdagSubmits(true),
 	_maxJobHolds(100),
-	_runPost(true),
-	_defaultPriority(0),
+	_runPost(false),
+	_priority(0), // from config or command line
 	_claim_hold_time(20),
 	_doRecovery(false),
 	_suppressJobLogs(false),
-	_dagmanClassad(NULL)
+	_batchName(""),
+	_dagmanClassad(NULL),
+	_removeNodeJobs(true)
 {
     debug_level = DEBUG_VERBOSE;  // Default debug level is verbose output
 }
@@ -236,10 +238,10 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_USER_LOG_SCAN_INTERVAL setting: %d\n",
 				m_user_log_scan_interval );
 
-	_defaultPriority = param_integer( "DAGMAN_DEFAULT_PRIORITY",
-				_defaultPriority, INT_MIN, INT_MAX, false );
+	_priority = param_integer( "DAGMAN_DEFAULT_PRIORITY",
+				_priority, INT_MIN, INT_MAX, false );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_DEFAULT_PRIORITY setting: %d\n",
-				_defaultPriority );
+				_priority );
 
 	if ( !param_boolean( "DAGMAN_ALWAYS_USE_NODE_LOG", true ) ) {
        	debug_printf( DEBUG_QUIET,
@@ -324,9 +326,12 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_MAX_POST_SCRIPTS setting: %d\n",
 				maxPostScripts );
 
-	allowLogError = param_boolean( "DAGMAN_ALLOW_LOG_ERROR", allowLogError );
-	debug_printf( DEBUG_NORMAL, "DAGMAN_ALLOW_LOG_ERROR setting: %s\n",
-				allowLogError ? "True" : "False" );
+	bool allowLogError = param_boolean( "DAGMAN_ALLOW_LOG_ERROR", false );
+	if ( allowLogError ) {
+		debug_printf( DEBUG_NORMAL, "Warning: DAGMAN_ALLOW_LOG_ERROR is "
+					"no longer supported\n" );
+		check_warning_strictness( DAG_STRICT_2 );
+	}
 
 	mungeNodeNames = param_boolean( "DAGMAN_MUNGE_NODE_NAMES",
 				mungeNodeNames );
@@ -343,7 +348,7 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_SUBMIT_DEPTH_FIRST setting: %s\n",
 				submitDepthFirst ? "True" : "False" );
 
-	_runPost = param_boolean( "DAGMAN_ALWAYS_RUN_POST", true );
+	_runPost = param_boolean( "DAGMAN_ALWAYS_RUN_POST", _runPost );
 	debug_printf( DEBUG_NORMAL, "DAGMAN_ALWAYS_RUN_POST setting: %s\n",
 			_runPost ? "True" : "False" );
 
@@ -451,6 +456,11 @@ Dagman::Config()
 	debug_printf( DEBUG_NORMAL, "DAGMAN_SUPPRESS_JOB_LOGS setting: %s\n",
 				_suppressJobLogs ? "True" : "False" );
 
+	_removeNodeJobs = param_boolean( "DAGMAN_REMOVE_NODE_JOBS",
+				_removeNodeJobs );
+	debug_printf( DEBUG_NORMAL, "DAGMAN_REMOVE_NODE_JOBS setting: %s\n",
+				_removeNodeJobs ? "True" : "False" );
+
 	// enable up the debug cache if needed
 	if (debug_cache_enabled) {
 		debug_cache_set_size(debug_cache_size);
@@ -569,7 +579,8 @@ int main_shutdown_remove(Service *, int) {
     debug_printf( DEBUG_QUIET, "Received SIGUSR1\n" );
 	// We don't remove Condor node jobs here because the schedd will
 	// automatically remove them itself.
-	main_shutdown_rescue( EXIT_ABORT, Dag::DAG_STATUS_RM, false );
+	main_shutdown_rescue( EXIT_ABORT, Dag::DAG_STATUS_RM,
+				dagman._removeNodeJobs );
 	return FALSE;
 }
 
@@ -668,6 +679,8 @@ void main_init (int argc, char ** const argv) {
     //
     // Process command-line arguments
     //
+	bool alwaysRunPostSet = false;
+
     for (i = 1; i < argc; i++) {
         if( !strcasecmp( "-Debug", argv[i] ) ) {
             i++;
@@ -728,6 +741,7 @@ void main_init (int argc, char ** const argv) {
                 Usage();
             }
             dagman.maxPostScripts = atoi( argv[i] );
+
         } else if( !strcasecmp( "-NoEventChecks", argv[i] ) ) {
 			debug_printf( DEBUG_QUIET, "Warning: -NoEventChecks is "
 						"ignored; please use the DAGMAN_ALLOW_EVENTS "
@@ -735,11 +749,27 @@ void main_init (int argc, char ** const argv) {
 			check_warning_strictness( DAG_STRICT_2 );
 
         } else if( !strcasecmp( "-AllowLogError", argv[i] ) ) {
-			dagman.allowLogError = true;
+			debug_printf( DEBUG_QUIET, "Warning: -AllowLogError is "
+						"no longer supported\n" );
+			check_warning_strictness( DAG_STRICT_2 );
 
-        } else if( !strcasecmp( "-DontAlwaysRunPost",argv[i] ) ) {
+        } else if( !strcasecmp( "-DontAlwaysRunPost", argv[i] ) ) {
+			if ( alwaysRunPostSet && dagman._runPost ) {
+				debug_printf( DEBUG_QUIET,
+							"ERROR: -DontAlwaysRunPost and -AlwaysRunPost are both set!\n" );
+				DC_Exit( EXIT_ERROR );
+			}
+			alwaysRunPostSet = true;
 			dagman._runPost = false;
 
+        } else if( !strcasecmp( "-AlwaysRunPost", argv[i] ) ) {
+			if ( alwaysRunPostSet && !dagman._runPost ) {
+				debug_printf( DEBUG_QUIET,
+							"ERROR: -DontAlwaysRunPost and -AlwaysRunPost are both set!\n" );
+				DC_Exit( EXIT_ERROR );
+			}
+			dagman._runPost = true;
+
         } else if( !strcasecmp( "-WaitForDebug", argv[i] ) ) {
 			wait_for_debug = 1;
 
@@ -824,7 +854,7 @@ void main_init (int argc, char ** const argv) {
 				debug_printf( DEBUG_NORMAL, "No priority value specified\n");
 				Usage();
 			}
-			dagman._submitDagDeepOpts.priority = atoi(argv[i]);
+			dagman._priority = atoi(argv[i]);
 
 		} else if( !strcasecmp( "-dont_use_default_node_log", argv[i] ) ) {
        		debug_printf( DEBUG_QUIET,
@@ -845,6 +875,13 @@ void main_init (int argc, char ** const argv) {
 	dagman.primaryDagFile = dagman.dagFiles.next();
 	dagman.multiDags = (dagman.dagFiles.number() > 1);
 
+	dagman._dagmanClassad->GetSetBatchName( dagman.primaryDagFile,
+				dagman._batchName );
+
+	dagman._dagmanClassad->GetAcctInfo(
+				dagman._submitDagDeepOpts.acctGroup,
+				dagman._submitDagDeepOpts.acctGroupUser );
+
 	dagman.ResolveDefaultLog();
 
     //
@@ -867,7 +904,7 @@ void main_init (int argc, char ** const argv) {
 
 		// Just generate this message fragment in one place.
 	MyString versionMsg;
-	versionMsg.formatstr("the version (%s) of this DAG's Condor submit "
+	versionMsg.formatstr("the version (%s) of this DAG's HTCondor submit "
 				"file (created by condor_submit_dag)", csdVersion );
 
 		// Make sure version in submit file is valid.
@@ -1068,7 +1105,6 @@ void main_init (int argc, char ** const argv) {
 		// Fill in values in the deep submit options that we haven't
 		// already set.
 		//
-	dagman._submitDagDeepOpts.bAllowLogError = dagman.allowLogError;
 	dagman._submitDagDeepOpts.useDagDir = dagman.useDagDir;
 	dagman._submitDagDeepOpts.autoRescue = dagman.autoRescue;
 	dagman._submitDagDeepOpts.doRescueFrom = dagman.doRescueFrom;
@@ -1085,7 +1121,7 @@ void main_init (int argc, char ** const argv) {
 	// wenger 2010-03-25
     dagman.dag = new Dag( dagman.dagFiles, dagman.maxJobs,
 						  dagman.maxPreScripts, dagman.maxPostScripts,
-						  dagman.allowLogError, dagman.useDagDir,
+						  dagman.useDagDir,
 						  dagman.maxIdle, dagman.retrySubmitFirst,
 						  dagman.retryNodeFirst, dagman.condorRmExe,
 						  &dagman.DAGManJobId,
@@ -1104,11 +1140,8 @@ void main_init (int argc, char ** const argv) {
 	dagman.dag->SetConfigFile( dagman._dagmanConfigFile );
 	dagman.dag->SetMaxJobHolds( dagman._maxJobHolds );
 	dagman.dag->SetPostRun(dagman._runPost);
-	if( dagman._submitDagDeepOpts.priority != 0 ) { // From command line
-		dagman.dag->SetDefaultPriority(dagman._submitDagDeepOpts.priority);
-	} else if( dagman._defaultPriority != 0 ) { // From config file
-		dagman.dag->SetDefaultPriority(dagman._defaultPriority);
-		dagman._submitDagDeepOpts.priority = dagman._defaultPriority;
+	if( dagman._priority != 0 ) {
+		dagman.dag->SetDagPriority(dagman._priority);
 	}
 
     //
@@ -1155,8 +1188,8 @@ void main_init (int argc, char ** const argv) {
 					 	dagFile );
     	}
 	}
-	if( dagman.dag->GetDefaultPriority() != 0 ) {
-		dagman.dag->SetDefaultPriorities(); // Applies to the nodes of the dag
+	if( dagman.dag->GetDagPriority() != 0 ) {
+		dagman.dag->SetNodePriorities(); // Applies to the nodes of the dag
 	}
 	dagman.dag->GetJobstateLog().WriteDagmanStarted( dagman.DAGManJobId );
 	if ( rescueDagNum > 0 ) {
@@ -1432,6 +1465,9 @@ Dagman::ResolveDefaultLog()
 				debug_printf( DEBUG_QUIET, "Ignoring value of DAGMAN_LOG_ON_NFS_IS_ERROR because ENABLE_USERLOG_LOCKING and CREATE_LOCKS_ON_LOCAL_DISK are true.\n");
 				nfsLogIsError = false;
 			}
+		} else {
+			debug_printf( DEBUG_QUIET, "Ignoring value of DAGMAN_LOG_ON_NFS_IS_ERROR because ENABLE_USERLOG_LOCKING is false.\n");
+			nfsLogIsError = false;
 		}
 	}
 
@@ -1514,7 +1550,9 @@ void condor_event_timer () {
 	dagman.dag->RunWaitingScripts();
 
 	int justSubmitted;
+	debug_printf( DEBUG_DEBUG_1, "Starting submit cycle\n" );
 	justSubmitted = dagman.dag->SubmitReadyJobs(dagman);
+	debug_printf( DEBUG_DEBUG_1, "Finished submit cycle\n" );
 	if( justSubmitted ) {
 			// Note: it would be nice to also have the proc submit
 			// count here.  wenger, 2006-02-08.
