@@ -109,11 +109,6 @@ class Dagman {
 		// (note: we perform run-time cycle-detection regardless)
 	bool startup_cycle_detect;
 
-		// Allow the job to execute even if we have an error determining
-		// the log files (e.g., the log file is missing from one of the
-		// node submit files).
-	bool allowLogError;
-
 		// Whether to treat the dirname portion of any DAG file paths
 		// as a directory that the DAG should effectively be run from.
 	bool useDagDir;
@@ -151,7 +146,7 @@ class Dagman {
 		// to breadth-first).
 	bool submitDepthFirst;
 
-		// Whether to abort on a "scary" submit event (Condor ID doesn't
+		// Whether to abort on a "scary" submit event (HTCondor ID doesn't
 		// match expected value).
 	bool abortOnScarySubmit;
 
@@ -159,7 +154,7 @@ class Dagman {
 		// are pending.
 	int pendingReportInterval;
 
-		// the Condor job id of the DAGMan job
+		// the HTCondor job id of the DAGMan job
 	CondorID DAGManJobId;
 
 		// The DAGMan configuration file (NULL if none is specified).
@@ -214,8 +209,8 @@ class Dagman {
 		// script for the node fails.
 	bool _runPost;
 
-		// Default priority that DAGman uses for nodes.
-	int _defaultPriority;
+		// Priority for this DAG.
+	int _priority;
 
 	int _claim_hold_time;
 
@@ -226,7 +221,14 @@ class Dagman {
 		// log files specified in their submit files (see gittrac #4353).
 	bool _suppressJobLogs;
 
+		// Batch-name for this workflow.
+	MyString _batchName;
+
 	DagmanClassad *_dagmanClassad;
+
+		// True iff we should remove node jobs ourself when we are
+		// condor_rm'ed.
+	bool _removeNodeJobs;
 };
 
 #endif	// ifndef DAGMAN_MAIN_H
