@@ -44,7 +44,6 @@ struct SubmitDagShallowOptions
 	int iMaxJobs;
 	int iMaxPre;
 	int iMaxPost;
-	bool bNoEventChecks;
 	MyString appendFile; // append to .condor.sub file before queue
 	StringList appendLines; // append to .condor.sub file before queue
 	MyString strConfigFile;
@@ -53,6 +52,9 @@ struct SubmitDagShallowOptions
 	MyString primaryDagFile;
 	StringList	dagFiles;
 	bool doRecovery;
+	bool bPostRun;
+	bool bPostRunSet; // whether this was actually set on the command line
+	int priority;
 
 	// non-command line options
 	MyString strLibOut;
@@ -64,12 +66,12 @@ struct SubmitDagShallowOptions
 	MyString strLockFile;
 	bool copyToSpool;
 	int iDebugLevel;
-	bool bPostRun;
 
 	SubmitDagShallowOptions() 
 	{ 
 		bSubmit = true;
-		bPostRun = true;
+		bPostRun = false;
+		bPostRunSet = false;
 		strRemoteSchedd = "";
 		strScheddDaemonAdFile = "";
 		strScheddAddressFile = "";
@@ -77,7 +79,6 @@ struct SubmitDagShallowOptions
 		iMaxJobs = 0;
 		iMaxPre = 0;
 		iMaxPost = 0;
-		bNoEventChecks = false;
 		appendFile = param( "DAGMAN_INSERT_SUB_FILE" );
 		strConfigFile = "";
 		dumpRescueDag = false;
@@ -86,6 +87,7 @@ struct SubmitDagShallowOptions
 		doRecovery = false;
 		copyToSpool = param_boolean( "DAGMAN_COPY_TO_SPOOL", false );
 		iDebugLevel = DEBUG_UNSET;
+		priority = 0;
 	}
 };
 
@@ -100,26 +102,26 @@ struct SubmitDagDeepOptions
 	bool bVerbose;
 	bool bForce;
 	MyString strNotification;
-	bool bAllowLogError;
 	MyString strDagmanPath; // path to dagman binary
 	bool useDagDir;
 	MyString strOutfileDir;
+	MyString batchName; // optional value from -batch-name argument, will be double quoted if it exists.
 	bool autoRescue;
 	int doRescueFrom;
 	bool allowVerMismatch;
 	bool recurse; // whether to recursively run condor_submit_dag on nested DAGs
 	bool updateSubmit; // allow updating submit file w/o -force
 	bool importEnv; // explicitly import environment into .condor.sub file
-	int priority; // Priority of parent of DAG node
 
 	bool suppress_notification;
+	MyString acctGroup;
+	MyString acctGroupUser;
 
 	SubmitDagDeepOptions() 
 	{ 
 		bVerbose = false;
 		bForce = false;
 		strNotification = "";
-		bAllowLogError = false;
 		useDagDir = false;
 		autoRescue = param_boolean( "DAGMAN_AUTO_RESCUE", true );
 		doRescueFrom = 0; // 0 means no rescue DAG specified
@@ -127,8 +129,9 @@ struct SubmitDagDeepOptions
 		recurse = false;
 		updateSubmit = false;
 		importEnv = false;
-		priority = 0;
 		suppress_notification = true;
+		acctGroup = "";
+		acctGroupUser = "";
 	}
 };
 
@@ -138,10 +141,13 @@ extern "C" {
 	@param dagFile: the DAG file to process
 	@param directory: the directory from which the DAG file should
 		be processed (ignored if NULL)
+	@param priority: the priority of this DAG
+	@param isRetry: whether this is a retry
 	@return 0 if successful, 1 if failed
 */
 int runSubmitDag( const SubmitDagDeepOptions &deepOpts,
-			const char *dagFile, const char *directory, bool isRetry );
+			const char *dagFile, const char *directory, int priority,
+			bool isRetry );
 }
 
 #endif	// ifndef DAGMAN_RECURSIVE_SUBMIT_H
