@@ -71,7 +71,7 @@ submit_try( ArgList &args, CondorID &condorID, bool prohibitMultiJobs )
   MyString cmd; // for debug output
   args.GetArgsStringForDisplay( &cmd );
 
-  FILE * fp = my_popen( args, "r", TRUE );
+  FILE * fp = my_popen( args, "r", MY_POPEN_OPT_WANT_STDERR );
   if (fp == NULL) {
     debug_printf( DEBUG_NORMAL, 
 		  "ERROR: my_popen(%s) in submit_try() failed!\n",
@@ -80,10 +80,10 @@ submit_try( ArgList &args, CondorID &condorID, bool prohibitMultiJobs )
   }
   
   //----------------------------------------------------------------------
-  // Parse submit command output for a Condor job ID.  This
-  // desperately needs to be replaced by Condor submit APIs.
+  // Parse submit command output for a HTCondor job ID.  This
+  // desperately needs to be replaced by HTCondor submit APIs.
   //
-  // Typical condor_submit output for Condor v6 looks like:
+  // Typical condor_submit output for HTCondor v6 looks like:
   //
   //   Submitting job(s).
   //   Logging submit event(s).
@@ -112,7 +112,7 @@ submit_try( ArgList &args, CondorID &condorID, bool prohibitMultiJobs )
   
   // Take all of the output (both stdout and stderr) from condor_submit,
   // and echo it to the dagman.out file.  Look for
-  // the line (if any) containing the word "cluster" (Condor).
+  // the line (if any) containing the word "cluster" (HTCondor).
   // If we don't find such a line, something
   // went wrong with the submit, so we return false.  The caller of this
   // function can retry the submit by repeatedly calling this function.
@@ -204,9 +204,9 @@ do_submit( ArgList &args, CondorID &condorID, bool prohibitMultiJobs )
 bool
 condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 			   const char* DAGNodeName, MyString &DAGParentNodeNames,
-			   List<Job::NodeVar> *vars, int retry,
+			   List<Job::NodeVar> *vars, int priority, int retry,
 			   const char* directory, const char *workflowLogFile,
-			   bool hold_claim )
+			   bool hold_claim, const MyString &batchName )
 {
 	TmpDir		tmpDir;
 	MyString	errMsg;
@@ -251,6 +251,12 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 				dm.DAGManJobId._cluster;
 	args.AppendArg( dagJobIdMacro.Value() );
 
+		// Pass the batch name to lower levels.
+	if ( batchName != "" ) {
+		args.AppendArg( "-batch-name" );
+		args.AppendArg( batchName.Value() );
+	}
+
 	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString submitEventNotes = MyString(
 				"submit_event_notes = DAG Node: " ) + DAGNodeName;
@@ -280,6 +286,15 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 	dmask += "\"";
 	args.AppendArg( dmask.c_str() );
 
+		// Append the priority, if we have one.
+	if ( priority != 0 ) {
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
+		MyString prioStr = "priority=";
+		prioStr += priority;
+		args.AppendArg( prioStr.Value() );
+	}
+
+
 		// Suppress the job's log file if that option is enabled.
 	if ( dm._suppressJobLogs ) {
 		debug_printf( DEBUG_VERBOSE, "Suppressing node job log file\n" );
@@ -325,7 +340,44 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 	var += dm.dag->NumNodesFailed();
 	args.AppendArg( var.Value() );
 
-		// how big is the command line so far
+	if( hold_claim ){
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
+		MyString holdit = MyString("+") + MyString(ATTR_JOB_KEEP_CLAIM_IDLE) + " = "
+			+ dm._claim_hold_time;
+		args.AppendArg( holdit.Value() );	
+	}
+	
+	if (dm._submitDagDeepOpts.suppress_notification) {
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
+		MyString notify = MyString("notification = never");
+		args.AppendArg( notify.Value() );
+	}
+
+		//
+		// Add accounting group and user if we have them.
+		//
+	if ( dm._submitDagDeepOpts.acctGroup != "" ) {
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
+		MyString arg = "accounting_group=";
+		arg += dm._submitDagDeepOpts.acctGroup;
+		args.AppendArg( arg );
+	}
+
+	if ( dm._submitDagDeepOpts.acctGroupUser != "" ) {
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
+		MyString arg = "accounting_group_user=";
+		arg += dm._submitDagDeepOpts.acctGroupUser;
+		args.AppendArg( arg );
+	}
+
+		//
+		// Add parents of this node to arguments, if we have room.
+		//
+		// This should be the last thing in the arguments, except
+		// for the submit file name!!!
+		//
+
+		// how big is the command line so far?
 	MyString display;
 	args.GetArgsStringForDisplay( &display );
 	int cmdLineSize = display.Length();
@@ -350,19 +402,6 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 		args.AppendArgsFromArgList( parentNameArgs );
 	}
 
-	if( hold_claim ){
-		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
-		MyString holdit = MyString("+") + MyString(ATTR_JOB_KEEP_CLAIM_IDLE) + " = "
-			+ dm._claim_hold_time;
-		args.AppendArg( holdit.Value() );	
-	}
-	
-	if (dm._submitDagDeepOpts.suppress_notification) {
-		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
-		MyString notify = MyString("notification = never");
-		args.AppendArg( notify.Value() );
-	}
-
 	args.AppendArg( cmdFile );
 
 	bool success = do_submit( args, condorID, dm.prohibitMultiJobs );
@@ -407,7 +446,7 @@ fake_condor_submit( CondorID& condorID, Job* job, const char* DAGNodeName,
 	}
 
 	_subprocID++;
-		// Special CondorID for NOOP jobs -- actually indexed by
+		// Special HTCondorID for NOOP jobs -- actually indexed by
 		// otherwise-unused subprocID.
 	condorID._cluster = 0;
 	condorID._proc = Job::NOOP_NODE_PROCID;
@@ -472,7 +511,7 @@ bool writePreSkipEvent( CondorID& condorID, Job* job, const char* DAGNodeName,
 		return false;
 	}
 
-		// Special CondorID for NOOP jobs -- actually indexed by
+		// Special HTCondorID for NOOP jobs -- actually indexed by
 		// otherwise-unused subprocID.
 	condorID._cluster = 0;
 	condorID._proc = Job::NOOP_NODE_PROCID;
