@@ -124,8 +124,8 @@ Job::Job( const char* jobName,
 
 	_queuedNodeJobProcs = 0;
 
-	_hasNodePriority = false;
-	_nodePriority = 0;
+	_explicitPriority = 0;
+	_effectivePriority = _explicitPriority;
 
 	_noop = false;
 
@@ -135,13 +135,14 @@ Job::Job( const char* jobName,
 
 	varsFromDag = new List<NodeVar>;
 
-	snprintf( error_text, JOB_ERROR_TEXT_MAXLEN, "unknown" );
+	error_text = "";
 
 	_timesHeld = 0;
 	_jobProcsOnHold = 0;
 
 	return;
 }
+
 //---------------------------------------------------------------------------
 void
 Job::PrefixDirectory(MyString &prefix)
@@ -188,7 +189,7 @@ void Job::Dump ( const Dag *dag ) const {
     dprintf( D_ALWAYS, "    Node Status: %s\n", GetStatusName() );
     dprintf( D_ALWAYS, "Node return val: %d\n", retval );
 	if( _Status == STATUS_ERROR ) {
-		dprintf( D_ALWAYS, "          Error: %s\n", error_text );
+		dprintf( D_ALWAYS, "          Error: %s\n", error_text.Value() );
 	}
     dprintf( D_ALWAYS, "Job Submit File: %s\n", _cmdFile );
 	if( _scriptPre ) {
@@ -515,9 +516,13 @@ Job::AddScript( bool post, const char *cmd, int defer_status, time_t defer_time,
 		return false;
 	}
 	if( post ? _scriptPost : _scriptPre ) {
-		whynot.formatstr( "%s script already assigned (%s)",
-						post ? "POST" : "PRE", GetPreScriptName() );
-		return false;
+		const char *prePost = post ? "POST" : "PRE";
+		const char *script = post ? GetPostScriptName() : GetPreScriptName();
+		debug_printf( DEBUG_NORMAL,
+					"Warning: node %s already has %s script <%s> assigned; changing to <%s>\n",
+					GetJobName(), prePost, script, cmd );
+		check_warning_strictness( DAG_STRICT_3 );
+		delete (post ? _scriptPost : _scriptPre);
 	}
 	Script* script = new Script( post, cmd, defer_status, defer_time, this );
 	if( !script ) {
@@ -540,24 +545,26 @@ Job::AddScript( bool post, const char *cmd, int defer_status, time_t defer_time,
 bool
 Job::AddPreSkip( int exitCode, MyString &whynot )
 {
-	if( exitCode < PRE_SKIP_MIN || exitCode > PRE_SKIP_MAX ) {
+	if ( exitCode < PRE_SKIP_MIN || exitCode > PRE_SKIP_MAX ) {
 		whynot.formatstr( "PRE_SKIP exit code must be between %d and %d\n",
 			PRE_SKIP_MIN, PRE_SKIP_MAX );
 		return false;
 	}
 
-	if( exitCode == 0 ) {
+	if ( exitCode == 0 ) {
 		debug_printf( DEBUG_NORMAL, "Warning: exit code 0 for a PRE_SKIP "
 			"value is weird.\n");
 	}
 
-	if( _preskip == PRE_SKIP_INVALID ) {
-		_preskip = exitCode;	
-	} else {
-		whynot = "Two definitions of PRE_SKIP for a node.\n";
-		return false;
+	if ( _preskip != PRE_SKIP_INVALID ) {
+		debug_printf( DEBUG_NORMAL,
+					"Warning: new PRE_SKIP value  %d for node %s overrides old value %d\n",
+					exitCode, GetJobName(), _preskip );
+		check_warning_strictness( DAG_STRICT_3 );
 	}
-	whynot = "n/a";
+	_preskip = exitCode;	
+
+	whynot = "";
 	return true;
 }
 
@@ -687,6 +694,8 @@ Job::NumChildren() const
 void
 Job::SetCategory( const char *categoryName, ThrottleByCategory &catThrottles )
 {
+	ASSERT( !_final );
+
 	MyString	tmpName( categoryName );
 
 	if ( (_throttleInfo != NULL) &&
@@ -801,8 +810,7 @@ Job::GetJobstateSequenceNum()
 void
 Job::SetLastEventTime( const ULogEvent *event )
 {
-	struct tm eventTm = event->eventTime;
-	_lastEventTime = mktime( &eventTm );
+	_lastEventTime = event->GetEventclock();
 }
 
 //---------------------------------------------------------------------------
@@ -816,35 +824,6 @@ Job::GetPreSkip() const
 	return _preskip;
 }
 
-//---------------------------------------------------------------------------
-// If there is a cycle, could this enter an infinite loop?
-// No: If there is a cycle, there will be equality, and recursion will stop
-// It makes no sense to insert job priorities on linear DAGs;
-
-// The scheme here is to copy the priority from parent nodes, if a parent node
-// has priority higher than the job priority currently assigned to the node, or
-// use the default priority of the DAG; otherwise, we use the priority from the
-// DAG file. Priorities calculated by DAGman will ignore and override job
-// priorities set in the submit file.
-
-// DAGman fixes the default priorities in Dag::SetDefaultPriorities
-
-void
-Job::FixPriority(Dag& dag)
-{
-	std::set<JobID_t> parents = GetQueueRef(Q_PARENTS);
-	for(std::set<JobID_t>::iterator p = parents.begin(); p != parents.end(); ++p){
-		Job* parent = dag.FindNodeByNodeID(*p);
-		if( parent->_hasNodePriority ) {
-			// Nothing to do if parent priority is small
-			if( parent->_nodePriority > _nodePriority ) {
-				_nodePriority = parent->_nodePriority;
-				_hasNodePriority = true;
-			}
-		}
-	}
-}
-
 //---------------------------------------------------------------------------
 bool
 Job::SetCondorID(const CondorID& cid)
