@@ -42,11 +42,9 @@ class DagmanMetrics;
 #include "debug.h"
 #include "script.h"
 
-#define JOB_ERROR_TEXT_MAXLEN 128
-
 typedef int JobID_t;
 
-/**  The job class represents a job in the DAG and its state in the Condor
+/**  The job class represents a job in the DAG and its state in the HTCondor
      system.  A job is given a name, a CondorID, and three queues.  The
      parents queue is a list of parent jobs that this one depends on.  That
      queue never changes once set.  The waiting queue is the same as the
@@ -55,11 +53,11 @@ typedef int JobID_t;
      Once set, this queue never changes.<p>
 
      From DAGMan's point of view, a job has six basic states in the
-     Condor system (refer to the Job::status_t enumeration).  It
+     HTCondor system (refer to the Job::status_t enumeration).  It
      starts out as READY, meaning that it has not yet been submitted.
      If the job has a PRE script defined it changes to PRERUN while
      that script executes.  The job's state changes to SUBMITTED once
-     in the Condor system.  If it completes successfully and the job
+     in the HTCondor system.  If it completes successfully and the job
      has a POST script defined it changes to POSTRUN while that script
      executes, otherwise if it completes successfully it is DONE, or
      is in the ERROR state if it completes abnormally.  Note that final
@@ -144,7 +142,7 @@ class Job {
     static const char * status_t_names[];
 
 	// explanation text for errors
-	char error_text[JOB_ERROR_TEXT_MAXLEN];
+	MyString error_text;
 
 	static int NOOP_NODE_PROCID;
   
@@ -176,7 +174,7 @@ class Job {
 	inline int GetRetries() const { return retries; }
 	const char* GetPreScriptName() const;
 	const char* GetPostScriptName() const;
-	static const char* JobTypeString() { return "Condor"; }
+	static const char* JobTypeString() { return "HTCondor"; }
 
 	bool AddScript( bool post, const char *cmd, int defer_status,
 				time_t defer_time, MyString &whynot );
@@ -390,7 +388,6 @@ class Job {
 		@return the last event time.
 	*/
 	time_t GetLastEventTime() { return _lastEventTime; }
-	void FixPriority(Dag& dag);
 
 	bool HasPreSkip() const { return _preskip != PRE_SKIP_INVALID; }
 	int GetPreSkip() const;
@@ -474,11 +471,14 @@ public:
 		// queue for this node.
 	int _queuedNodeJobProcs;
 
-		// Whether the _nodePriority value is meaningful.
-	bool _hasNodePriority;
-
 		// Node priority.  Higher number is better priority (submit first).
-	int _nodePriority;
+		// Explicit priority is the priority actually set in the DAG
+		// file (0 if not set).
+	int _explicitPriority;
+		// Effective priority is the priority at which we're going to
+		// actually submit the job (explicit priority adjusted
+		// according to the DAG priority algorithm).
+	int _effectivePriority;
 
 		// The number of times this job has been held.  (Note: the current
 		// implementation counts holds for all procs in a multi-proc cluster
@@ -559,7 +559,7 @@ private:
 	ThrottleByCategory::ThrottleInfo *_throttleInfo;
 
 		// Whether this is a noop job (shouldn't actually be submitted
-		// to Condor).
+		// to HTCondor).
 	bool _noop;
 
 		// The job tag for this node ("-" if nothing is specified;
