@@ -368,7 +368,7 @@ stdin_pipe_handler(Service*, int) {
 
 			} else if (strcasecmp (args.argv[0], GAHP_COMMAND_DOWNLOAD_SANDBOX) == 0) {
 
-				int fds[2];
+				int fds[2] = {-1,-1};
 				if ( pipe( fds ) < 0 ) {
 					EXCEPT( "Failed to create pipe!" );
 				}
@@ -405,7 +405,7 @@ stdin_pipe_handler(Service*, int) {
 
 			} else if (strcasecmp (args.argv[0], GAHP_COMMAND_UPLOAD_SANDBOX) == 0) {
 
-				int fds[2];
+				int fds[2] = {-1,-1};
 				if ( pipe( fds ) < 0 ) {
 					EXCEPT( "Failed to create pipe!" );
 				}
@@ -441,7 +441,7 @@ stdin_pipe_handler(Service*, int) {
 
 			} else if (strcasecmp (args.argv[0], GAHP_COMMAND_DESTROY_SANDBOX) == 0) {
 
-				int fds[2];
+				int fds[2] = {-1,-1};
 				if ( pipe( fds ) < 0 ) {
 					EXCEPT( "Failed to create pipe!" );
 				}
