@@ -20,7 +20,6 @@
 
 #include "condor_common.h"
 #include "condor_config.h"
-#include "job_lease.h"
 
 #include "baseresource.h"
 #include "basejob.h"
@@ -56,7 +55,6 @@ BaseResource::BaseResource( const char *resource_name )
 								"BaseResource::UpdateLeases", (Service*)this );
 	lastUpdateLeases = 0;
 	updateLeasesActive = false;
-	leaseAttrsSynched = false;
 	updateLeasesCmdActive = false;
 	m_hasSharedLeases = false;
 	m_defaultLeaseDuration = -1;
@@ -548,6 +546,27 @@ void BaseResource::DoPing( unsigned& ping_delay, bool& ping_complete,
 	ping_succeeded = true;
 }
 
+time_t BaseResource::GetLeaseExpiration( const BaseJob *job )
+{
+	// Without a job to consider, just return the shared lease
+	// expiration, if we have one.
+	if ( job == NULL ) {
+		return m_sharedLeaseExpiration;
+	}
+	// What lease expiration should be used for this job?
+	// Some jobs may have no lease (default for grid-type condor).
+	// Otherwise, if we haven't established a shared lease yet,
+	// calculate one for this job only.
+	int new_expiration = 0;
+	int job_lease_duration = m_defaultLeaseDuration;
+	job->jobAd->LookupInteger( ATTR_JOB_LEASE_DURATION, job_lease_duration );
+	if ( job_lease_duration > 0 ) {
+		new_expiration = m_sharedLeaseExpiration > 0 ?
+			m_sharedLeaseExpiration : time(NULL) + job_lease_duration;
+	}
+	return new_expiration;
+}
+
 void BaseResource::RequestUpdateLeases()
 {
 	if ( updateLeasesTimerId != TIMER_UNSET ) {
@@ -578,95 +597,54 @@ dprintf(D_FULLDEBUG,"    UpdateLeases: last update too recent, delaying %d secs\
 
     if ( updateLeasesActive == false ) {
 		BaseJob *curr_job;
-		time_t next_renew_time = INT_MAX;
-		time_t job_renew_time;
-		int min_new_expire = INT_MAX;
+		int new_lease_duration = INT_MAX;
 dprintf(D_FULLDEBUG,"    UpdateLeases: calc'ing new leases\n");
 		registeredJobs.Rewind();
-dprintf(D_FULLDEBUG,"    starting min_new_expire=%d next_renew_time=%ld\n",min_new_expire,next_renew_time);
 		while ( registeredJobs.Next( curr_job ) ) {
-			int new_expire;
-			std::string  job_id;
-			job_renew_time = next_renew_time;
-				// Don't update the lease for a job that isn't submitted
-				// anywhere. The Job object will start the lease when it
-				// submits the job.
-			if ( ( m_hasSharedLeases || curr_job->jobAd->LookupString( ATTR_GRID_JOB_ID, job_id ) ) &&
-				 CalculateJobLease( curr_job->jobAd, new_expire,
-									m_defaultLeaseDuration,
-									&job_renew_time ) ) {
-
-				if ( new_expire < min_new_expire ) {
-					min_new_expire = new_expire;
-				}
-				if ( !m_hasSharedLeases ) {
-					curr_job->UpdateJobLeaseSent( new_expire );
-					leaseUpdates.Append( curr_job );
-				}
-			} else if ( job_renew_time < next_renew_time ) {
-				next_renew_time = job_renew_time;
+			int job_lease_duration = m_defaultLeaseDuration;
+			curr_job->jobAd->LookupInteger( ATTR_JOB_LEASE_DURATION, job_lease_duration );
+			if ( job_lease_duration > 0 && job_lease_duration < new_lease_duration ) {
+				new_lease_duration = job_lease_duration;
 			}
-dprintf(D_FULLDEBUG,"    after %d.%d: min_new_expire=%d next_renew_time=%ld job_renew_time=%ld\n",curr_job->procID.cluster,curr_job->procID.proc,min_new_expire,next_renew_time,job_renew_time);
 		}
-		if ( min_new_expire == INT_MAX ||
-			 ( m_hasSharedLeases && next_renew_time < INT_MAX &&
-			   m_sharedLeaseExpiration != 0 ) ) {
-			if ( next_renew_time > time(NULL) + 3600 ) {
+dprintf(D_FULLDEBUG,"    UpdateLeases: new shared lease duration: %d\n", new_lease_duration );
+		// This is how close to the lease expiration time we want to be
+		// when we try a renewal.
+		int renew_threshold = ( new_lease_duration * 2 / 3 ) + 10;
+		if ( new_lease_duration == INT_MAX ||
+			 m_sharedLeaseExpiration > time(NULL) + renew_threshold ) {
+			// Lease doesn't need renewal, yet.
+			time_t next_renew_time = m_sharedLeaseExpiration - renew_threshold;
+			if ( new_lease_duration == INT_MAX ||
+				 next_renew_time > time(NULL) + 3600 ) {
 				next_renew_time = time(NULL) + 3600;
 			}
 dprintf(D_FULLDEBUG,"    UpdateLeases: nothing to renew, resetting timer for %ld secs\n",next_renew_time - time(NULL));
 			lastUpdateLeases = time(NULL);
 			daemonCore->Reset_Timer( updateLeasesTimerId,
 									 next_renew_time - time(NULL) );
+			return;
 		} else {
-			if ( m_hasSharedLeases ) {
+			// Time to renew the lease
+			m_sharedLeaseExpiration = time(NULL) + new_lease_duration;
+			if ( !m_hasSharedLeases ) {
 				registeredJobs.Rewind();
 				while ( registeredJobs.Next( curr_job ) ) {
 					std::string job_id;
-					if ( curr_job->jobAd->LookupString( ATTR_GRID_JOB_ID, job_id ) ) {
-						curr_job->UpdateJobLeaseSent( min_new_expire );
+					int tmp;
+					if ( curr_job->jobAd->LookupString( ATTR_GRID_JOB_ID, job_id ) &&
+						 curr_job->jobAd->LookupInteger( ATTR_JOB_LEASE_DURATION, tmp )
+					) {
+						leaseUpdates.Append( curr_job );
 					}
 				}
-				m_sharedLeaseExpiration = min_new_expire;
-dprintf(D_FULLDEBUG,"    new shared lease expiration at %ld, updating job ads...\n",m_sharedLeaseExpiration);
 			}
+dprintf(D_FULLDEBUG,"    new shared lease expiration at %ld, performing renewal...\n",m_sharedLeaseExpiration);
 			requestScheddUpdateNotification( updateLeasesTimerId );
 			updateLeasesActive = true;
-			leaseAttrsSynched = false;
 		}
-		return;
 	}
 
-	if ( leaseAttrsSynched == false ) {
-		bool still_dirty = false;
-		BaseJob *curr_job;
-		leaseUpdates.Rewind();
-		while ( leaseUpdates.Next( curr_job ) ) {
-			bool exists, dirty;
-			curr_job->jobAd->GetDirtyFlag( ATTR_JOB_LEASE_EXPIRATION,
-										   &exists, &dirty );
-			if ( !exists ) {
-					// What!? The attribute disappeared? Forget about renewing
-					// the lease then
-				dprintf( D_ALWAYS, "Lease attribute disappeared for job %d.%d, ignoring it\n",
-						 curr_job->procID.cluster, curr_job->procID.proc );
-				leaseUpdates.DeleteCurrent();
-			}
-			if ( dirty ) {
-				still_dirty = true;
-				requestScheddUpdate( curr_job, false );
-			}
-		}
-		if ( still_dirty ) {
-			requestScheddUpdateNotification( updateLeasesTimerId );
-dprintf(D_FULLDEBUG,"    UpdateLeases: waiting for schedd synch\n");
-			return;
-		}
-else dprintf(D_FULLDEBUG,"    UpdateLeases: leases synched\n");
-	}
-
-	leaseAttrsSynched = true;
-
 	unsigned update_delay = 0;
 	bool update_complete;
 	SimpleList<PROC_ID> update_succeeded;
@@ -680,7 +658,7 @@ dprintf(D_FULLDEBUG,"    UpdateLeases: calling DoUpdateLeases\n");
 
 	if ( update_delay ) {
 		daemonCore->Reset_Timer( updateLeasesTimerId, update_delay );
-dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases wants delay of %uld secs\n",update_delay);
+dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases wants delay of %u secs\n",update_delay);
 		return;
 	}
 
@@ -706,46 +684,23 @@ dprintf(D_FULLDEBUG,"    UpdateLeases: DoUpdateLeases complete, processing resul
 				// before they proceed with submission.
 				curr_job->SetEvaluateState();
 			}
-			if ( !curr_job->jobAd->LookupString( ATTR_GRID_JOB_ID, tmp ) ) {
-				continue;
-			}
-			bool curr_renewal_failed = !update_success;
-			bool last_renewal_failed = false;
-			curr_job->jobAd->LookupBool( ATTR_LAST_JOB_LEASE_RENEWAL_FAILED,
-										 last_renewal_failed );
-			if ( curr_renewal_failed != last_renewal_failed ) {
-				curr_job->jobAd->Assign( ATTR_LAST_JOB_LEASE_RENEWAL_FAILED,
-										 curr_renewal_failed );
-				requestScheddUpdate( curr_job, false );
+			if ( curr_job->jobAd->LookupString( ATTR_GRID_JOB_ID, tmp ) ) {
+				curr_job->UpdateJobLeaseSent( m_sharedLeaseExpiration );
 			}
 		}
 	} else {
-update_succeeded.Rewind();
-PROC_ID id;
 std::string msg = "    update_succeeded:";
- while(update_succeeded.Next(id)) formatstr_cat(msg, " %d.%d", id.cluster, id.proc);
-dprintf(D_FULLDEBUG,"%s\n",msg.c_str());
 		BaseJob *curr_job;
-		leaseUpdates.Rewind();
-		while ( leaseUpdates.Next( curr_job ) ) {
-			bool curr_renewal_failed;
-			bool last_renewal_failed = false;
-			if ( update_succeeded.IsMember( curr_job->procID ) ) {
-dprintf(D_FULLDEBUG,"    %d.%d is in succeeded list\n",curr_job->procID.cluster,curr_job->procID.proc);
-				curr_renewal_failed = false;
-			} else {
-dprintf(D_FULLDEBUG,"    %d.%d is not in succeeded list\n",curr_job->procID.cluster,curr_job->procID.proc);
-				curr_renewal_failed = true;
+		PROC_ID curr_id;
+		update_succeeded.Rewind();
+		while ( update_succeeded.Next( curr_id ) ) {
+formatstr_cat(msg, " %d.%d", curr_id.cluster, curr_id.proc);
+			if ( BaseJob::JobsByProcId.lookup( curr_id, curr_job ) == 0 ) {
+				curr_job->UpdateJobLeaseSent( m_sharedLeaseExpiration );
 			}
-			curr_job->jobAd->LookupBool( ATTR_LAST_JOB_LEASE_RENEWAL_FAILED,
-										 last_renewal_failed );
-			if ( curr_renewal_failed != last_renewal_failed ) {
-				curr_job->jobAd->Assign( ATTR_LAST_JOB_LEASE_RENEWAL_FAILED,
-										 curr_renewal_failed );
-				requestScheddUpdate( curr_job, false );
-			}
-			leaseUpdates.DeleteCurrent();
 		}
+dprintf(D_FULLDEBUG,"%s\n",msg.c_str());
+		leaseUpdates.Clear();
 	}
 
 	updateLeasesActive = false;
