@@ -60,6 +60,7 @@ class BaseResource : public Service
 	void AlreadySubmitted( BaseJob *job );
 
 	void RequestUpdateLeases();
+	time_t GetLeaseExpiration( const BaseJob *job = NULL );
 
     static void setProbeInterval( int new_interval )
 		{ probeInterval = new_interval; }
@@ -67,9 +68,6 @@ class BaseResource : public Service
 	static void setProbeDelay( int new_delay )
 		{ probeDelay = new_delay; }
 
-	// TODO Make this private and provide an accessor function?
-	time_t m_sharedLeaseExpiration;
-
 	int m_paramJobPollRate;
 	int m_paramJobPollInterval;
 	int m_jobPollInterval;
@@ -120,11 +118,11 @@ class BaseResource : public Service
 	bool hasLeases;
 	int updateLeasesTimerId;
 	time_t lastUpdateLeases;
-	List<BaseJob> leaseUpdates;
+	SimpleList<BaseJob*> leaseUpdates;
 	bool updateLeasesActive;
-	bool leaseAttrsSynched;
 	bool updateLeasesCmdActive;
 	bool m_hasSharedLeases;
+	time_t m_sharedLeaseExpiration;
 	time_t m_defaultLeaseDuration;
 
 	int _updateCollectorTimerId;
