@@ -27,7 +27,6 @@
 #include "spooled_job_files.h"
 #include "daemon.h"
 #include "dc_schedd.h"
-#include "job_lease.h"
 #include "nullfile.h"
 
 #include "gridmanager.h"
@@ -561,8 +560,8 @@ void CondorJob::doEvaluateState()
 				}
 				char *job_id_string = NULL;
 				if ( gahpAd == NULL ) {
-					int new_expiration;
-					if ( CalculateJobLease( jobAd, new_expiration ) ) {
+					int new_expiration = myResource->GetLeaseExpiration( this );
+					if ( new_expiration > 0 ) {
 							// This will set the job lease sent attrs,
 							// which get referenced in buildSubmitAd()
 						UpdateJobLeaseSent( new_expiration );
@@ -1457,11 +1456,11 @@ ClassAd *CondorJob::buildSubmitAd()
 	submit_ad->Delete( ATTR_JOB_STATUS_ON_RELEASE );
 	submit_ad->Delete( ATTR_LAST_JOB_LEASE_RENEWAL );
 	submit_ad->Delete( ATTR_JOB_LEASE_DURATION );
-	submit_ad->Delete( ATTR_LAST_JOB_LEASE_RENEWAL_FAILED );
 	submit_ad->Delete( ATTR_TIMER_REMOVE_CHECK );
 	submit_ad->Delete( ATTR_JOB_LEASE_EXPIRATION );
 	submit_ad->Delete( ATTR_AUTO_CLUSTER_ID );
 	submit_ad->Delete( ATTR_AUTO_CLUSTER_ATTRS );
+	submit_ad->Delete( ATTR_TOTAL_SUBMIT_PROCS );
 
 	submit_ad->Assign( ATTR_JOB_STATUS, HELD );
 	submit_ad->Assign( ATTR_HOLD_REASON, "Spooling input data files" );
@@ -1551,6 +1550,10 @@ ClassAd *CondorJob::buildSubmitAd()
 
 	submit_ad->Assign( ATTR_SUBMITTER_ID, submitterId );
 
+	expr.clear();
+	jobAd->LookupString( ATTR_GLOBAL_JOB_ID, expr );
+	submit_ad->Assign( ATTR_SUBMITTER_GLOBAL_JOB_ID, expr );
+
 		// If JOB_PROXY_OVERRIDE_FILE is set in the config file, then
 		// these attributes aren't set in the local job ad, so we need
 		// to set them explicitly.
