@@ -75,7 +75,7 @@ class CreamResource : public BaseResource
 	static HashTable <HashKey, CreamResource *> ResourcesByName;
 
  private:
-	void DoPing( time_t& ping_delay, bool& ping_complete,
+	void DoPing( unsigned& ping_delay, bool& ping_complete,
 				 bool& ping_succeeded );
 	void checkDelegation();
 
@@ -104,7 +104,7 @@ protected:
 	BatchStatusResult FinishBatchStatus();
 	GahpClient * BatchGahp();
 
-	void DoUpdateSharedLease( time_t& update_delay, bool& update_complete, 
+	void DoUpdateSharedLease( unsigned& update_delay, bool& update_complete,
 							  bool& update_succeeded );
 };
 
