@@ -76,12 +76,36 @@ public:
 	time_t lastSubmitAttempt;
 	int numSubmitAttempts;
 
+
+	//
+	// Hold reason codes were assigned as follows.  Hold reason sub codes
+	// were assigned sequentially by order of their appearance in the code,
+	// and as such are unique between hold codes.
+	//
+	//  37: (apparent) user error.  Example: an unspecified access key ID.
+	//  38: internal error.  Example: the grid resource type was not "ec2";
+	//      the GAHP should never have seen this job ad.
+	//  39: administrator error.  Example: EC2_GAHP being unset.
+	//  40: connection problem.  Example: E_CURL_IO.
+	//  41: server error.  Example: an unrecognized or unexpected job state.
+	//  42: INSTANCE MAY HAVE BEEN LOST.  Example: we tried and failed to
+	//      cancel a spot instance request.
+	//
+	// Of these, only code 40 has a decent chance of working if tried again.
+	// Code 41 may work, but probably won't (for example, the instance would
+	// have to change to a recognized state).  Codes 37, 38, and 39 would need
+	// to debugged; CODE 42 REQUIRES THE EC2 ACCOUNT OWNER TO INVESTIGATE.
+	//
+	int holdReasonCode;
+	int holdReasonSubCode;
+
+
 	std::string errorString;
 	std::string m_remoteJobId;
 	std::string remoteJobState;
 
 	EC2Resource *myResource;
-	GahpClient *gahp;
+	EC2GahpClient *gahp;
 
     void StatusUpdate( const char * instanceID,
                        const char * status,
