@@ -131,6 +131,7 @@ int GCEJob::maxRetryTimes = 3;
 
 GCEJob::GCEJob( ClassAd *classad ) :
 	BaseJob( classad ),
+	m_preemptible( false ),
 	m_retry_times( 0 ),
 	probeNow( false )
 {
@@ -174,6 +175,10 @@ GCEJob::GCEJob( ClassAd *classad ) :
 
 	jobAd->LookupString( ATTR_GCE_METADATA, m_metadata );
 
+	jobAd->LookupBool( ATTR_GCE_PREEMPTIBLE, m_preemptible );
+
+	jobAd->LookupString( ATTR_GCE_JSON_FILE, m_jsonFile );
+
 	// get VM machine type
 	jobAd->LookupString( ATTR_GCE_MACHINE_TYPE, m_machineType );
 
@@ -436,48 +441,23 @@ void GCEJob::doEvaluateState()
 					}
 				}
 
-				// If we're not doing recovery, start with GM_CLEAR_REQUEST.
-				gmState = GM_CLEAR_REQUEST;
-
-				if( ! m_instanceName.empty() ) {
-
-					// Because we have the instance name, we know that
-					// we still have the SSH keypair from the previous
-					// execution of GM_SAVE_INSTANCE_NAME.  We can therefore
-					// jump directly to GM_START_VM, where (using the
-					// instance name), we will call RunInstances()
-					// idempotently.
-					gmState = GM_START_VM;
-
-					// As an optimization, if we already have the instance
-					// ID, we can jump directly to GM_SUBMITTED.  This
-					// also allows us to avoid logging the submission and
-					// execution events twice.
-					//
-					// FIXME: If a job were removed before we begin
-					// recovery, would the execute event be logged twice?
-					if (!m_instanceId.empty()) {
-						submitLogged = true;
-						if ( condorState == RUNNING || condorState == COMPLETED ) {
-							executeLogged = true;
-						}
-						// Do NOT set probeNow; if we're recovering from
-						// a queue with 5000 jobs, we'd hit the service
-						// with 5000 status update requests, which is
-						// precisely what we're trying to avoid.
-						gmState = GM_SUBMITTED;
-					} else if( condorState == REMOVED ) {
-						// We don't know if the corresponding instance
-						// exists or not. And if we're creating the
-						// ssh keypair, we don't know if that exists.
-						// Rather than unconditionally
-						// create it (in GM_START_VM), check to see if
-						// exists first.  While this may be more efficient,
-						// the real benefit is that invalid jobs won't
-						// stay on hold when the user removes them
-						// (because we won't try to start them).
-						gmState = GM_SEEK_INSTANCE_ID;
+				if ( m_instanceName.empty() ) {
+					// This is a fresh job submission.
+					gmState = GM_CLEAR_REQUEST;
+				} else if ( m_instanceId.empty() ) {
+					// We died during submission. There may be a running
+					// instance whose ID we don't know.
+					// Wait for a bulk query of instances and see if an
+					// instance that matches our m_instanceName appears.
+					gmState = GM_SEEK_INSTANCE_ID;
+				} else {
+					// There is (or was) a running instance and we know
+					// its ID.
+					submitLogged = true;
+					if ( condorState == RUNNING || condorState == COMPLETED ) {
+						executeLogged = true;
 					}
+					gmState = GM_SUBMITTED;
 				}
 				break;
 
@@ -553,6 +533,8 @@ void GCEJob::doEvaluateState()
 													m_image,
 													m_metadata,
 													m_metadataFile,
+													m_preemptible,
+													m_jsonFile,
 													instance_id );
 
 					if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
@@ -909,7 +891,8 @@ void GCEJob::doEvaluateState()
 				// If the bulk query found this job, it has an instance ID.
 				// (If the job had an instance ID before, we would be in
 				// an another state.)  Otherwise, the service doesn't know
-				// about this job, and we can remove it from the queue.
+				// about this job, and we can submit the job or let it
+				// leave the queue.
 				if( ! m_instanceId.empty() ) {
 					WriteGridSubmitEventToUserLog( jobAd );
 					gmState = GM_SAVE_INSTANCE_ID;
