@@ -66,7 +66,7 @@ class GlobusJob : public BaseJob
 	bool GetCallbacks();
 	void ClearCallbacks();
 	BaseResource *GetResource();
-	void SetRemoteJobId( const char *job_id, bool is_gt5 = false );
+	void GlobusSetRemoteJobId( const char *job_id, bool is_gt5 );
 
 	/* If true, then ATTR_ON_EXIT_BY_SIGNAL, ATTR_ON_EXIT_SIGNAL, and
 	   ATTR_ON_EXIT_CODE are valid.  If false, no exit status is available.
@@ -84,15 +84,12 @@ class GlobusJob : public BaseJob
 
 	int ProxyCallback();
 
-	static int probeInterval;
 	static int submitInterval;
 	static int restartInterval;
 	static int gahpCallTimeout;
 	static int maxConnectFailures;
 	static int outputWaitGrowthTimeout;
 
-	static void setProbeInterval( int new_interval )
-		{ probeInterval = new_interval; }
 	static void setSubmitInterval( int new_interval )
 		{ submitInterval = new_interval; }
 	static void setRestartInterval( int new_interval )
