@@ -49,10 +49,6 @@
 #  include "creamjob.h"
 #endif
 
-#if defined( LINUX )
-#  include "dcloudjob.h"
-#endif
-
 #define QMGMT_TIMEOUT 15
 
 #define UPDATE_SCHEDD_DELAY		5
@@ -369,16 +365,6 @@ Init()
 	jobTypes.Append( new_type );
 #endif
 	
-#if defined( LINUX )
-	new_type = new JobType;
-	new_type->Name = strdup( "Deltacloud" );
-	new_type->InitFunc = DCloudJobInit;
-	new_type->ReconfigFunc = DCloudJobReconfig;
-	new_type->AdMatchFunc = DCloudJobAdMatch;
-	new_type->CreateFunc = DCloudJobCreate;
-	jobTypes.Append( new_type );
-#endif
-	
 	new_type = new JobType;
 	new_type->Name = strdup( "Unicore" );
 	new_type->InitFunc = UnicoreJobInit;
@@ -604,6 +590,7 @@ doContactSchedd()
 	char *job_id_str;
 	PROC_ID job_id;
 	CondorError errstack;
+	int tmp_int;
 
 	dprintf(D_FULLDEBUG,"in doContactSchedd()\n");
 
@@ -1065,7 +1052,10 @@ contact_schedd_next_add_job:
 					failure_line_num = __LINE__;
 					commit_transaction = false;
 					goto contact_schedd_disconnect;
-				} else {
+				} else if ( GetAttributeInt( curr_job->procID.cluster,
+											 curr_job->procID.proc,
+											 ATTR_CLUSTER_ID,
+											 &tmp_int ) < 0 ) {
 						// The job is not in the schedd's job queue. This
 						// probably means that the user did a condor_rm -f,
 						// so pretend that all updates for the job succeed.
@@ -1075,6 +1065,8 @@ contact_schedd_next_add_job:
 						//   to the job, so it can do what cleanup it can.
 					fake_job_in_queue = true;
 					break;
+				} else {
+					dprintf( D_FULLDEBUG, "     (Schedd rejected)\n" );
 				}
 			}
 		}
