@@ -148,6 +148,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	std::string error_string = "";
 	char *gahp_path = NULL;
 	ArgList gahp_args;
+	MyString args_str;
 
 	gahpAd = NULL;
 	gmState = GM_INIT;
@@ -237,6 +238,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 			formatstr( error_string, "REMOTE_GAHP not defined" );
 			goto error_exit;
 		}
+		gahp_args.GetArgsStringV2Raw( &args_str, NULL );
 	} else {
 		// CRUFT: BATCH_GAHP was added in 7.7.6.
 		//   Checking <batch-type>_GAHP should be removed at some
@@ -257,7 +259,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 
 	buff = batchType;
 	if ( gahp_args.Count() > 0 ) {
-		formatstr_cat( buff, "/%s", gahp_args.GetArg( 0 ) );
+		formatstr_cat( buff, "/%s", args_str.Value() );
 		gahp_args.AppendArg( "batch_gahp" );
 	}
 	gahp = new GahpClient( buff.c_str(), gahp_path, &gahp_args );
@@ -274,7 +276,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 			goto error_exit;
 		}
 
-		formatstr( buff, "xfer/%s/%s", batchType, gahp_args.GetArg( 0 ) );
+		formatstr( buff, "xfer/%s/%s", batchType, args_str.Value() );
 		gahp_args.RemoveArg( gahp_args.Count() - 1 );
 		gahp_args.AppendArg( "condor_ft-gahp" );
 		m_xfer_gahp = new GahpClient( buff.c_str(), gahp_path, &gahp_args );
@@ -637,7 +639,7 @@ void INFNBatchJob::doEvaluateState()
 						 gahp->getErrorString() );
 				errorString = gahp->getErrorString();
 				myResource->CancelSubmit( this );
-				gmState = GM_DELETE_SANDBOX;
+				gmState = GM_HOLD;
 				reevaluate_state = true;
 			}
 			if ( job_id_string != NULL ) {
@@ -1307,6 +1309,14 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 		ATTR_JOB_IWD,
 		ATTR_GRID_RESOURCE,
 		ATTR_REQUEST_MEMORY,
+		"CERequirements",
+		"NodeNumber",
+		"HostNumber",
+		"SMPGranularity",
+		"WholeNodes",
+		"HostSMPSize",
+		"BatchExtraSubmitArgs",
+		"StageCmd",
 		NULL };		// list must end with a NULL
 
 	submit_ad = new ClassAd;
@@ -1453,10 +1463,13 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 			}
 		}
 
+		// Older versions of the blahp will fail if the x509userproxy
+		// attribute isn't a full path.
 		old_value = "";
 		submit_ad->LookupString( ATTR_X509_USER_PROXY, old_value );
 		if ( !old_value.empty() ) {
-			submit_ad->InsertAttr( ATTR_X509_USER_PROXY, condor_basename( old_value.c_str() ) );
+			new_value = m_sandboxPath + DIR_DELIM_CHAR + condor_basename( old_value.c_str() );
+			submit_ad->InsertAttr( ATTR_X509_USER_PROXY, new_value.c_str() );
 		}
 
 		old_value = "";
