@@ -672,10 +672,11 @@ HADStateMachine::setReplicationDaemonSinfulString( void )
     replicationAddressList.rewind( );
 
     free( tmp );
-    char* host = getHostFromAddr( daemonCore->InfoCommandSinfulString( ) );
 
 	int replicationDaemonIndex = replicationAddressList.number() - 1;
 
+	const char * mySinfulString = daemonCore->InfoCommandSinfulString();
+	Sinful mySinful( mySinfulString );
     while( ( replicationAddress = replicationAddressList.next( ) ) ) {
         char* sinfulAddress     = utilToSinful( replicationAddress );
 
@@ -688,32 +689,44 @@ HADStateMachine::setReplicationDaemonSinfulString( void )
 
             //continue;
         }
-        char* sinfulAddressHost = getHostFromAddr( sinfulAddress );
 
-        if(  (replicationDaemonIndex == m_selfId)     &&
-			 (strcmp( sinfulAddressHost, host ) == 0)  ) {
-            m_replicationDaemonSinfulString = sinfulAddress;
-            free( sinfulAddressHost );
-			dprintf( D_ALWAYS,
-					"HADStateMachine::setReplicationDaemonSinfulString "
-					"corresponding replication daemon - %s\n",
-					 sinfulAddress );
-            // not freeing 'sinfulAddress', since its referent is pointed by
-            // 'replicationDaemonSinfulString' too
-            break;
-        } else if( replicationDaemonIndex == m_selfId ) {
-			sprintf( buffer,
-					 "HADStateMachine::setReplicationDaemonSinfulString"
-					 "host names of machine and replication daemon do "
-					 "not match: %s vs. %s\n", host, sinfulAddressHost);
-			utilCrucialError( buffer );
-		}
+		// We're checking if the replication daemon listed in the position
+		// corresponding to ours in the HAD list refers to the same machine
+		// or not.  We want to end up with 's' having the replication daemon's
+		// shared port ID and port number, while ignoring the shared port
+		// ID and port number in our comparison.  We temporarily set the
+		// replication daemon's shared port ID to our own (to make sure
+		// they match) and set (this sinful's copy of ) our own port number
+		// to the replication daemon's port number (to make sure they match).
+		// If the sinfuls do point to the same address, we change the
+		// replication daemon's socket name to REPLICATION_SOCKET_NAME
+		// (for backwards compatability, we can't just set the replication
+		// daemon's socket name in the replication daemon list).
+        if( replicationDaemonIndex == m_selfId ) {
+            Sinful s( sinfulAddress );
+            mySinful.setPort( s.getPort() );
+            s.setSharedPortID( mySinful.getSharedPortID() );
+            if( s.valid() && mySinful.addressPointsToMe( s ) ) {
+            	if( s.getSharedPortID() ) {
+            		s.setSharedPortID( param( "REPLICATION_SOCKET_NAME" ) );
+            	}
+                m_replicationDaemonSinfulString = strdup( s.getSinful() );
+                dprintf( D_ALWAYS,
+                    "HADStateMachine::setReplicationDaemonSinfulString "
+                    "corresponding replication daemon - %s\n",
+                    s.getSinful() );
+            } else {
+                sprintf( buffer,
+                         "HADStateMachine::setReplicationDaemonSinfulString "
+                         "host names of machine and replication daemon do "
+                         "not match: %s vs. %s\n", mySinful.getSinful(), s.getSinful() );
+                utilCrucialError( buffer );
+            }
+        }
 
 		replicationDaemonIndex --;
-        free( sinfulAddressHost );
         free( sinfulAddress );
     }
-    free( host );
 
     // if failed to found the replication daemon in REPLICATION_LIST, just
     // switch off the replication feature
@@ -833,21 +846,33 @@ HADStateMachine::getHadList( const char *str,
 
     bool iAmPresent = false;
     while( (try_address = had_list.next()) ) {
-        char *sinful_addr = utilToSinful( try_address );
+        char * sinful_addr = utilToSinful( try_address );
 		dprintf(D_ALWAYS,
-				"HADStateMachine::initializeHADList my address %s "
-				"vs. next address in the list%s\n",
+				"HADStateMachine::initializeHADList my address '%s' "
+				"vs. address in the list '%s'\n",
 				my_addr.getSinful(), sinful_addr );
         if( sinful_addr == NULL ) {
             dprintf( D_ALWAYS,
-					 "HAD CONFIGURATION ERROR: pid %d", daemonCore->getpid() );
-            dprintf( D_ALWAYS,"not valid address %s\n", try_address );
-
+					 "HAD CONFIGURATION ERROR: pid %d "
+					 "address '%s' not valid\n",
+					 daemonCore->getpid(), try_address );
             utilCrucialError( "" );
             continue;
         }
-		allIps.insert( sinful_addr );
-        if( my_addr.addressPointsToMe( Sinful(sinful_addr) ) ) {
+		// The list doesn't include shared port IDs, so if we've got one
+		// give it to each other address in the list before we check to
+		// see if they're the same.  We know that sinful_addr can't
+		// already have a shared port ID because we called utilToSinful,
+		// and not just the Sinful constructor.  (The Sinful object and/or
+		// constructor should probably resolve hostnames, but HAD does it
+		// itself.)
+		Sinful s( sinful_addr );
+		free( sinful_addr );
+		s.setSharedPortID( my_addr.getSharedPortID() );
+		allIps.insert( s.getSinful() );
+dprintf( D_ALWAYS, "Checking address with shared port ID '%s'...\n", s.getSinful() );
+        if( my_addr.addressPointsToMe( s ) ) {
+dprintf( D_ALWAYS, "... found myself in list: %s\n", s.getSinful() );
             iAmPresent = true;
             // HAD id of each HAD is just the index of its <ip:port>
             // in HAD_LIST in reverse order
@@ -860,13 +885,8 @@ HADStateMachine::getHadList( const char *str,
 				isPrimaryCopy = true;
             }
         } else {
-            otherIps.insert( sinful_addr );
+            otherIps.insert( s.getSinful() );
         }
-
-		// put attention to release memory allocated by malloc with
-		// free and by new with delete here utilToSinful returns
-		// memory allocated by malloc
-        free( sinful_addr );
         counter-- ;
     } // end while
 
