@@ -56,6 +56,8 @@
 #define ATTR_AVAIL_SINCE  "AvailSince"
 #define ATTR_AVAIL_TIME  "AvailTime"
 #define ATTR_AVAIL_TIME_ESTIMATE  "AvailTimeEstimate"
+#define ATTR_BADPUT_CAUSED_BY_DRAINING  "BadputCausedByDraining"
+#define ATTR_BADPUT_CAUSED_BY_PREEMPTION  "BadputCausedByPreemption"
 #define ATTR_BANDWIDTH_TO_SUBMIT_MACHINE  "BandwidthToSubmitMachine"
 #define ATTR_BANDWIDTH_TO_LAST_CKPT_SERVER  "BandwidthToLastCkptServer"
 #define ATTR_BANDWIDTH_TO_CKPT_SERVER  "BandwidthToCkptServer"
@@ -121,6 +123,9 @@
 #define ATTR_CPU_BUSY_TIME  "CpuBusyTime"
 #define ATTR_CPU_IS_BUSY  "CpuIsBusy"
 #define ATTR_CPUS  "Cpus"
+#define ATTR_CPU_MODEL_NUMBER  "CpuModelNumber"
+#define ATTR_CPU_FAMILY  "CpuFamily"
+#define ATTR_CPU_CACHE_SIZE  "CpuCacheSize"
 #define ATTR_CURRENT_HOSTS  "CurrentHosts"
 #define ATTR_CURRENT_JOBS_RUNNING  "CurrentJobsRunning"
 #define ATTR_CURRENT_RANK  "CurrentRank"
@@ -148,26 +153,6 @@
 #define ATTR_DEFERRAL_PREP_TIME  "DeferralPrepTime"
 #define ATTR_DEFERRAL_TIME  "DeferralTime"
 #define ATTR_DEFERRAL_WINDOW  "DeferralWindow"
-//--------------------------------------------------------------------
-// Deltacloud attributes
-#define ATTR_DELTACLOUD_AVAILABLE_ACTIONS  "DeltacloudAvailableActions"
-#define ATTR_DELTACLOUD_PRIVATE_NETWORK_ADDRESSES  "DeltacloudPrivateNetworkAddresses"
-#define ATTR_DELTACLOUD_PROVIDER_ID  "DeltacloudProviderId"
-#define ATTR_DELTACLOUD_PUBLIC_NETWORK_ADDRESSES  "DeltacloudPublicNetworkAddresses"
-#define ATTR_DELTACLOUD_RETRY_TIMEOUT  "DeltacloudRetryTimeout"
-#define ATTR_DELTACLOUD_USERNAME  "DeltacloudUsername"
-#define ATTR_DELTACLOUD_PASSWORD_FILE  "DeltacloudPasswordFile"
-#define ATTR_DELTACLOUD_IMAGE_ID  "DeltacloudImageId"
-#define ATTR_DELTACLOUD_INSTANCE_NAME  "DeltacloudInstanceName"
-#define ATTR_DELTACLOUD_REALM_ID  "DeltacloudRealmId"
-#define ATTR_DELTACLOUD_HARDWARE_PROFILE  "DeltacloudHardwareProfile"
-#define ATTR_DELTACLOUD_HARDWARE_PROFILE_CPU  "DeltacloudHardwareProfileCpu"
-#define ATTR_DELTACLOUD_HARDWARE_PROFILE_MEMORY  "DeltacloudHardwareProfileMemory"
-#define ATTR_DELTACLOUD_HARDWARE_PROFILE_STORAGE  "DeltacloudHardwareProfileStorage"
-#define ATTR_DELTACLOUD_KEYNAME  "DeltacloudKeyname"
-#define ATTR_DELTACLOUD_USER_DATA  "DeltacloudUserData"
-#define ATTR_DELTACLOUD_NEEDS_START  "DeltacloudNeedsStart"
-//--------------------------------------------------------------------
 #define ATTR_DESTINATION  "Destination"
 #define ATTR_DISK  "Disk"
 #define ATTR_DISK_USAGE  "DiskUsage"
@@ -205,11 +190,14 @@
 #define ATTR_FLOCKED_JOBS  "FlockedJobs"
 #define ATTR_FLAVOR  "Flavor"
 #define ATTR_FORCE  "Force"
+#define ATTR_GAHP_PID "GahpPid"
 #define ATTR_GCE_AUTH_FILE  "GceAuthFile"
 #define ATTR_GCE_IMAGE  "GceImage"
 #define ATTR_GCE_MACHINE_TYPE  "GceMachineType"
 #define ATTR_GCE_METADATA  "GceMetadata"
 #define ATTR_GCE_METADATA_FILE  "GceMetadataFile"
+#define ATTR_GCE_PREEMPTIBLE  "GcePreemptible"
+#define ATTR_GCE_JSON_FILE  "GceJsonFile"
 #define ATTR_GCE_PROJECT  "GceProject"
 #define ATTR_GCE_ZONE  "GceZone"
 #define ATTR_GID  "Gid"
@@ -263,6 +251,7 @@
 #define ATTR_HAS_PVM  "HasPVM"
 #define ATTR_HAS_RECONNECT  "HasReconnect"
 #define ATTR_HAS_REMOTE_SYSCALLS  "HasRemoteSyscalls"
+#define ATTR_HAS_SINGULARITY "HasSingularity"
 #define ATTR_HAS_TDP  "HasTDP"
 #define ATTR_HAS_SOAP_API  "HasSOAPInterface"
 #define ATTR_HAS_WIN_RUN_AS_OWNER  "HasWindowsRunAsOwner"
@@ -363,12 +352,17 @@
 #define ATTR_JOB_LEAVE_IN_QUEUE  "LeaveJobInQueue"
 #define ATTR_JOB_REMOTE_SYS_CPU  "RemoteSysCpu"
 #define ATTR_JOB_REMOTE_USER_CPU  "RemoteUserCpu"
+#define ATTR_JOB_CUMULATIVE_REMOTE_SYS_CPU  "CumulativeRemoteSysCpu"
+#define ATTR_JOB_CUMULATIVE_REMOTE_USER_CPU  "CumulativeRemoteUserCpu"
 #define ATTR_JOB_REMOTE_WALL_CLOCK  "RemoteWallClockTime"
 #define ATTR_JOB_ROOT_DIR  "RootDir"
 #define ATTR_JOB_RUN_COUNT  "JobRunCount"
+#define ATTR_JOB_MAX_RETRIES "JobMaxRetries"
+#define ATTR_JOB_SUCCESS_EXIT_CODE "JobSuccessExitCode"
 #define ATTR_JOB_SANDBOX_JOBAD  "DropJobAdInSandbox"
 #define ATTR_JOB_SUBMISSION  "Submission"
 #define ATTR_JOB_SUBMISSION_ID  "SubmissionId"
+#define ATTR_JOB_BATCH_NAME "JobBatchName"
 #define ATTR_JOB_START  "JobStart"
 #define ATTR_JOB_START_DATE  "JobStartDate"
 #define ATTR_JOB_STATE  "JobState"
@@ -391,6 +385,7 @@
 #define ATTR_JOB_VM_HARDWARE_VT  "JobVMHardwareVT"
 #define ATTR_JOB_VM_VNC  "JobVMVNCConsole"
 #define ATTR_JOB_VM_CPU_UTILIZATION  "JobVMCpuUtilization"
+#define ATTR_JOB_SEND_CREDENTIAL  "SendCredential"
 #define ATTR_KEYBOARD_IDLE  "KeyboardIdle"
 #define ATTR_KEYSTORE_FILE  "KeystoreFile"
 #define ATTR_KEYSTORE_ALIAS  "KeystoreAlias"
@@ -408,13 +403,13 @@
 #define ATTR_LAST_CLAIM_STATE  "LastClaimState"
 #define ATTR_LAST_FETCH_WORK_SPAWNED  "LastFetchWorkSpawned"
 #define ATTR_LAST_FETCH_WORK_COMPLETED  "LastFetchWorkCompleted"
+#define ATTR_LAST_FORWARDED	"LastForwarded"
 #define ATTR_LAST_VACATE_TIME  "LastVacateTime"
 #define ATTR_LAST_HEARD_FROM  "LastHeardFrom"
 #define ATTR_LAST_HOLD_REASON  "LastHoldReason"
 #define ATTR_LAST_HOLD_REASON_CODE  "LastHoldReasonCode"
 #define ATTR_LAST_HOLD_REASON_SUBCODE  "LastHoldReasonSubCode"
 #define ATTR_LAST_JOB_LEASE_RENEWAL  "LastJobLeaseRenewal"
-#define ATTR_LAST_JOB_LEASE_RENEWAL_FAILED  "LastJobLeaseRenewalFailed"
 #define ATTR_LAST_MATCH_TIME  "LastMatchTime"
 #define ATTR_MACHINE_LAST_MATCH_TIME  "MachineLastMatchTime"
 #define ATTR_LAST_MATCH_LIST_PREFIX  "LastMatchName"
@@ -426,9 +421,11 @@
 #define ATTR_LAST_REMOTE_HOST  "LastRemoteHost"
 #define ATTR_LAST_REMOTE_POOL  "LastRemotePool"
 #define ATTR_LAST_REMOTE_STATUS_UPDATE  "LastRemoteStatusUpdate"
+#define ATTR_LAST_SLOT_NAME  "LastSlotName"
 #define ATTR_LAST_UPDATE  "LastUpdate"
 #define ATTR_LOCAL_CREDD  "LocalCredd"
 #define ATTR_LOCAL_FILES  "LocalFiles"
+#define ATTR_LOCATION_QUERY  "LocationQuery"
 #define ATTR_LOAD_AVG  "LoadAvg"
 #define ATTR_MACHINE  "Machine"
 #define ATTR_MACHINE_COUNT  "MachineCount"
@@ -454,6 +451,8 @@
 #define ATTR_NAME  "Name"
 #define ATTR_NICE_USER  "NiceUser"
 #define ATTR_NEGOTIATOR_REQUIREMENTS  "NegotiatorRequirements"
+#define ATTR_NETWORK_IN  "NetworkInputMb"
+#define ATTR_NETWORK_OUT  "NetworkOutputMb"
 #define ATTR_NEXT_CLUSTER_NUM  "NextClusterNum"
 #define ATTR_NEXT_FETCH_WORK_DELAY  "NextFetchWorkDelay"
 #define ATTR_NEXT_JOB_START_DELAY  "NextJobStartDelay"
@@ -480,6 +479,7 @@
 #define ATTR_NUM_HOPS_TO_LAST_CKPT_SERVER  "NumHopsToLastCkptServer"
 #define ATTR_NUM_HOPS_TO_CKPT_SERVER  "NumHopsToCkptServer"
 #define ATTR_NUM_JOB_STARTS  "NumJobStarts"
+#define ATTR_NUM_JOB_COMPLETIONS "NumJobCompletions"
 #define ATTR_NUM_JOB_RECONNECTS  "NumJobReconnects"
 #define ATTR_NUM_PIDS  "NumPids"
 #define ATTR_NUM_RESTARTS  "NumRestarts"
@@ -487,6 +487,7 @@
 #define ATTR_NUM_SHADOW_STARTS  "NumShadowStarts"
 #define ATTR_NUM_SYSTEM_HOLDS  "NumSystemHolds"
 #define ATTR_NUM_USERS  "NumUsers"
+#define ATTR_NUM_OWNERS "NumOwners"
 #define ATTR_OFFLINE  "Offline"
 #define ATTR_OPSYS  "OpSys"
 #define ATTR_OPSYSVER  "OpSysVer"
@@ -517,6 +518,7 @@
 #define ATTR_PREV_RECV_ESTIMATE  "PrevRecvEstimate"
 #define ATTR_PRIO  "Prio"
 #define ATTR_PROC_ID  "ProcId"
+#define ATTR_TOTAL_SUBMIT_PROCS "TotalSubmitProcs"
 #define ATTR_PSLOT_ROLLUP_INFORMATION "PslotRollupInformation"
 #define ATTR_SUB_PROC_ID  "SubProcId"
 #define ATTR_PRIVATE_NETWORK_NAME  "PrivateNetworkName"
@@ -559,6 +561,7 @@
 #define ATTR_RUN_BENCHMARKS  "RunBenchmarks"
 #define ATTR_SHADOW_IP_ADDR  "ShadowIpAddr"
 #define ATTR_MY_ADDRESS  "MyAddress"
+#define ATTR_ADDRESS_V1 "AddressV1"
 #define ATTR_NUM_JOB_STARTS_DELAYED  "NumJobStartsDelayed"
 #define ATTR_NUM_PENDING_CLAIMS  "NumPendingClaims"
 #define ATTR_SCHEDD_SWAP_EXHAUSTED  "ScheddSwapExhausted"
@@ -567,6 +570,8 @@
 #define ATTR_SCHEDD_NAME  "ScheddName"
 #define ATTR_SCHEDULER  "Scheduler"
 #define ATTR_SHADOW_WAIT_FOR_DEBUG  "ShadowWaitForDebug"
+#define ATTR_SHOULD_FORWARD	"ShouldForward"
+#define ATTR_SINGULARITY_VERSION "SingularityVersion"
 #define ATTR_SLOT_ID  "SlotID"
 #define ATTR_SLOT_PAIR_NAME  "SlotPairName"
 #define ATTR_SLOT_PARTITIONABLE  "PartitionableSlot"
@@ -582,6 +587,7 @@
 #define ATTR_STARTD_IP_ADDR  "StartdIpAddr"
 #define ATTR_STARTD_PRINCIPAL  "StartdPrincipal"
 #define ATTR_STARTD_SENDS_ALIVES  "StartdSendsAlives"
+#define ATTR_STARTER_EXIT_STATUS "StarterExitStatus"
 #define ATTR_STARTER_HANDLES_ALIVES "_condor_StartdHandlesAlives"
 #define ATTR_STATE  "State"
 #define ATTR_STARTER_IP_ADDR  "StarterIpAddr"
@@ -596,6 +602,7 @@
 #define ATTR_STREAM_ERROR  "StreamErr"
 #define ATTR_SUBMIT_EVENT_NOTES  "SubmitEventNotes"
 #define ATTR_SUBMIT_EVENT_USER_NOTES  "SubmitEventUserNotes"
+#define ATTR_SUBMITTER_GLOBAL_JOB_ID  "SubmitterGlobalJobId"
 #define ATTR_SUBMITTER_ID  "SubmitterId"
 #define ATTR_SUBMITTER_TAG  "SubmitterTag"
 #define ATTR_SUBMITTOR_PRIO  "SubmittorPrio"   // old-style for ATTR_SUBMITTER_USER_PRIO
@@ -704,6 +711,7 @@
 #define ATTR_VISA_IP  "VisaIpAddr"
 #define ATTR_WOL_PORT  "WakePort"
 #define ATTR_WANT_CHECKPOINT  "WantCheckpoint"
+#define ATTR_WANT_FT_ON_CHECKPOINT "WantFTOnCheckpoint"
 #define ATTR_WANT_CLAIMING  "WantClaiming"
 #define ATTR_WANT_IO_PROXY  "WantIOProxy"
 #define ATTR_WANT_REMOTE_UPDATES "WantRemoteUpdates"
@@ -853,6 +861,10 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_LAST_CHECKPOINT_PLATFORM  "LastCheckpointPlatform"
 #define ATTR_IS_VALID_CHECKPOINT_PLATFORM  "IsValidCheckpointPlatform"
 
+#define ATTR_CHECKPOINT_EXIT_CODE  "CheckpointExitCode"
+#define ATTR_CHECKPOINT_EXIT_SIGNAL  "CheckpointExitSignal"
+#define ATTR_CHECKPOINT_EXIT_BY_SIGNAL  "CheckpointExitBySignal"
+
 #define ATTR_WITHIN_RESOURCE_LIMITS  "WithinResourceLimits"
 
 #define ATTR_HAD_IS_ACTIVE  "HadIsActive"
@@ -903,6 +915,10 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 
 #define ATTR_WEIGHTED_IDLE_JOBS "WeightedIdleJobs"
 #define ATTR_WEIGHTED_RUNNING_JOBS "WeightedRunningJobs"
+#define ATTR_IDLE_LOCAL_JOBS "LocalJobsIdle"
+#define ATTR_RUNNING_LOCAL_JOBS "LocalJobsRunning"
+#define ATTR_IDLE_SCHEDULER_JOBS "SchedulerJobsIdle"
+#define ATTR_RUNNING_SCHEDULER_JOBS "SchedulerJobsRunning"
 
 //************* Added for EC2 Jobs ***************************//
 #define ATTR_EC2_ACCESS_KEY_ID  "EC2AccessKeyId"
