@@ -177,6 +177,7 @@ NAMETABLE_DIRECTIVE:TABLE:DCTranslation
 #define QUERY_JOB_ADS (SCHED_VERS+116)
 #define SWAP_CLAIM_AND_ACTIVATION (SCHED_VERS+117) // swap claim & activation between two STARTD resources, for moving a job into a 'transfer' slot.
 #define SEND_RESOURCE_REQUEST_LIST	(SCHED_VERS+118)     // used in negotiation protocol
+#define QUERY_JOB_ADS_WITH_AUTH (SCHED_VERS+119) // Same as QUERY_JOB_ADS but requires authentication
 
 // values used for "HowFast" in the draining request
 #define DRAIN_GRACEFUL 0
@@ -224,8 +225,11 @@ NAMETABLE_DIRECTIVE:TABLE:DCTranslation
 // other commands that use the ClassAd-only protocol
 // CA_LOCATE_STARTER used to be (CA_AUTH_CMD_BASE+7), but no more
 // CA_RECONNECT_JOB used to be  (CA_AUTH_CMD_BASE+8), but no more
+
 // Use the ClassAd-based protocol for updating the machine ClassAd.
 #define CA_UPDATE_MACHINE_AD	(CA_AUTH_CMD_BASE+9)
+// Use the ClassAd-based protocol for communicating with the annex daemon.
+#define CA_BULK_REQUEST			(CA_AUTH_CMD_BASE+10)
 
 #define CA_CMD                  (CA_CMD_BASE+0)
 #define CA_LOCATE_STARTER       (CA_CMD_BASE+1)
@@ -327,6 +331,11 @@ const int QUERY_GENERIC_ADS = 74;
 const int SHARED_PORT_CONNECT = 75;
 const int SHARED_PORT_PASS_SOCK = 76;
 
+const int UPDATE_ACCOUNTING_AD = 77;
+const int QUERY_ACCOUNTING_ADS = 78;
+const int INVALIDATE_ACCOUNTING_ADS = 79;
+
+
 /* these comments are used to control command_table_generator.pl
 NAMETABLE_DIRECTIVE:END_SECTION:collector
 */
@@ -395,6 +404,8 @@ NAMETABLE_DIRECTIVE:END_SECTION:collector
 #define DC_SEC_QUERY        (DC_BASE+40)
 #define DC_SET_FORCE_SHUTDOWN (DC_BASE+41)
 #define DC_OFF_FORCE       (DC_BASE+42)
+#define DC_SET_READY       (DC_BASE+43)  // sent to parent to indicate a demon is ready for use
+#define DC_QUERY_READY     (DC_BASE+44)  // daemon command handler should reply only once it and children are ready
 
 
 /*
@@ -493,6 +504,7 @@ NAMETABLE_DIRECTIVE:END_SECTION:collector
 #define CREDD_GET_CRED (CREDD_BASE+1)
 #define CREDD_REMOVE_CRED (CREDD_BASE+2)
 #define CREDD_QUERY_CRED (CREDD_BASE+3)
+#define CREDD_REFRESH_ALL (CREDD_BASE+20)
 #define CREDD_GET_PASSWD (CREDD_BASE+99)	// used by the Win32 credd only
 #define CREDD_NOP (CREDD_BASE+100)			// used by the Win32 credd only
 
