@@ -32,10 +32,12 @@
 #include <string>
 
 typedef std::vector<const char *> MACRO_SOURCES;
+class CondorError;
 namespace condor_params { typedef struct key_value_pair key_value_pair; }
 typedef const struct condor_params::key_value_pair MACRO_DEF_ITEM;
 #else // ! __cplusplus
 typedef void* MACRO_SOURCES; // placeholder for use in C
+typedef void  CondorError;   // placeholder for use in C
 typedef struct key_value_pair { const char * key; const void * def; } key_value_pair;
 typedef const struct key_value_pair MACRO_DEF_ITEM;
 #endif
@@ -45,6 +47,7 @@ typedef const struct key_value_pair MACRO_DEF_ITEM;
 
 #define PARAM_USE_COUNTING
 #define CALL_VIA_MACRO_SET
+#define USE_MACRO_STREAMS 1
 #define MACRO_SET_KNOWS_DEFAULT
 #define COLON_DEFAULT_FOR_MACRO_EXPAND  // enable $(FOO:default-value) and $(FOO:$(OTHER)) for config
 #define WARN_COLON_FOR_PARAM_ASSIGN   0 // parameter assigment with : (instead of =) is disallowed, a value of 0 is warn, 1 is fail
@@ -67,6 +70,8 @@ typedef struct macro_meta {
 	    unsigned matches_default :1;
 	    unsigned inside          :1;
 	    unsigned param_table     :1;
+	    unsigned live            :1; // future
+	    unsigned checkpointed    :1; // future
 	  };
 	};
 	short int    source_id;    // index into MACRO_SOURCES table
@@ -95,16 +100,54 @@ typedef struct macro_set {
 	ALLOCATION_POOL apool;
 	MACRO_SOURCES sources;
 	MACRO_DEFAULTS * defaults; // optional reference to const defaults table
+	CondorError * errors; // optional error stack, if non NULL, use instead of fprintf to stderr
+#ifdef __cplusplus
+	// fprintf an error if the above errors field is NULL, otherwise format an error and add it to the above errorstack
+	// the preface is printed with fprintf but not with the errors stack.
+	void push_error(FILE * fh, int code, const char* preface, const char* format, ... ) CHECK_PRINTF_FORMAT(5,6);
+#endif
 } MACRO_SET;
 
+// Used as the header for a MACRO_SET checkpoint, the actual allocation is larger than this.
+typedef struct macro_set_checkpoint_hdr {
+	int cSources;
+	int cTable;
+	int cMetaTable;
+	int spare;
+} MACRO_SET_CHECKPOINT_HDR;
+
 // this holds context during macro lookup/expansion
 typedef struct macro_eval_context {
 	const char *localname;
-	const char *subsys;
-	int without_default;
-	int use_mask;
+	const char *subsys;  // default subsys prefix
+	const char *cwd;     // current working directory, used for $F macro expansion
+	char without_default;
+	char use_mask;
+	char also_in_config; // also do lookups in the config hashtable (used by submit)
+	char is_context_ex;
+#if defined(__cplusplus)
+	void init(const char * sub, char mask=2) {
+		memset(this, 0, sizeof(*this));
+		this->subsys = sub;
+		this->use_mask = mask;
+	}
+#endif
 } MACRO_EVAL_CONTEXT;
 
+#if defined(__cplusplus)
+typedef struct macro_eval_context_ex : macro_eval_context {
+	// to do lookups of last resort into the given Ad, set these. they are useually null.
+	const char *adname; // name prefix for lookups into the ad
+	const ClassAd * ad; // classad for lookups
+	void init(const char * sub, char mask=2) {
+		memset(this, 0, sizeof(*this));
+		this->subsys = sub;
+		this->use_mask = mask;
+		this->is_context_ex = true;
+	}
+} MACRO_EVAL_CONTEXT_EX;
+#endif
+
 
 #if defined(__cplusplus)
 	extern MyString global_config_source;
@@ -114,9 +157,21 @@ typedef struct macro_eval_context {
 
 	// class that can be used to hold a malloc'd pointer such as the one returned by param
 	// it will free the pointer when this object is destroyed.
+	// The intended use is:
+	//   auto_free_ptr value(param("param_name"));
+	//   if (value) { dprintf(D_ALWAYS, "param_name has value %s\n", value.ptr()); }
+	//
+	// NOTE: it is NOT SAFE to use this class as a member of a class or struct that you intend to copy.
+	//   This class has minimal support for copy construction/assigment using the swap() idiom, which necessary
+	//   for populating STL containers. but it does NOT support deep copying or reference counting
+	//   which would be needed to support its use as a member in a class that you intend to copy and keep
+	//   both copies around. 
 	class auto_free_ptr {
 	public:
 		auto_free_ptr(char* str=NULL) : p(str) {}
+		friend void swap(auto_free_ptr& first, auto_free_ptr& second) { char*t = first.p; first.p = second.p; second.p = t; }
+		auto_free_ptr(const auto_free_ptr& that) { if (that.p) set(strdup(that.p)); else set(NULL); }
+		auto_free_ptr & operator=(auto_free_ptr that) { swap(*this, that); return *this; } // swap on assigment.
 		~auto_free_ptr() { clear(); }
 		void set(char*str) { clear(); p = str; }   // set a new pointer, freeing the old pointer (if any)
 		void clear() { if (p) free(p); p = NULL; } // free the pointer if any
@@ -133,6 +188,7 @@ typedef struct macro_eval_context {
 	int param_names_matching(Regex& re, std::vector<std::string>& names);
 
     bool param_defined(const char* name);
+	const char * param_unexpanded(const char *name);
 	char* param_or_except( const char *name );
     int param_integer( const char *name, int default_value = 0,
 					   int min_value = INT_MIN, int max_value = INT_MAX, bool use_param_table = true );
@@ -165,6 +221,13 @@ typedef struct macro_eval_context {
 	bool string_is_double_param(const char * string, double& result, ClassAd *me = NULL, ClassAd *target = NULL, const char * name=NULL, int* err_reason=NULL);
 	bool string_is_long_param(const char * string, long long& result, ClassAd *me = NULL, ClassAd *target = NULL, const char * name=NULL, int* err_reason=NULL);
 
+	// A convenience function for use with trinary parameters.
+	bool param_true( const char * name );
+
+	// A convenience function for use with trinary parameters.
+	bool param_false( const char * name );
+
+	const char * param_append_location(const MACRO_META * pmet, MyString & value);
 	const char * param_get_location(const MACRO_META * pmet, MyString & value);
 
 	const char * param_get_info(const char * name,
@@ -174,6 +237,8 @@ typedef struct macro_eval_context {
 								const char ** pdef_value,
 								const MACRO_META **ppmet);
 
+	MACRO_SET * param_get_macro_set();
+
 	// lookup "name" in ALL of relevant tables as indicated by the MACRO_EVAL_CONTEXT.
 	// as soon as an item is found (even an empty item) lookup stops.  the lookup order is
 	//    localname.name in MACRO_SET
@@ -230,11 +295,21 @@ typedef struct macro_eval_context {
 	bool param(MyString &buf,char const *param_name,char const *default_value=NULL);
 
 	/* A convenience function that calls param() with a std::string buffer. */
-	bool param(std::string &buf,char const *param_name,char const *default_value=NULL);
+	bool param(std::string &buf, char const *param_name, char const *default_value=NULL);
+
+	/* A convenience function that evaluates a config parameter to a string. */
+	bool param_eval_string(std::string &buf, const char *param_name, const char *default_value=NULL,
+		classad::ClassAd *me=NULL, classad::ClassAd *target=NULL);
 
 	/* A convenience function that calls param() then inserts items from the value into the given StringList if they are not already there */
 	bool param_and_insert_unique_items(const char * param_name, StringList & items, bool case_sensitive=false);
 
+	/*  A convenience function that calls param() then inserts items from the value
+		into the given classad:References set.  Useful whenever a param knob contains
+		a string list of ClassAd attribute names, e.g. IMMUTABLE_JOB_ATTRS.
+		Return true if given param name was found, false if not. */
+	bool param_and_insert_attrs(const char * param_name, classad::References & attrs);
+
 	/* Call this after loading the config files to see if the given user has access to all of the files
 	   Used when running as root to verfiy that the config files will still be accessible after we switch
 	   to the condor user. returns true on success, false if access check fails
@@ -256,11 +331,12 @@ extern "C" {
 	#define CONFIG_OPT_DEFAULTS_ARE_PARAM_INFO 0x80 // the defaults table is the table defined in param_info.in.
 	#define CONFIG_OPT_NO_EXIT 0x100 // If a config file is missing or the config is invalid, do not abort/exit the process.
 	#define CONFIG_OPT_WANT_QUIET 0x200 // Keep printing to stdout/err to a minimum
+	#define CONFIG_OPT_DEPRECATION_WARNINGS 0x400 // warn about obsolete syntax/elements
 	#define CONFIG_OPT_SUBMIT_SYNTAX 0x1000 // allow +Attr and -Attr syntax like submit files do.
 	bool config();
 	bool config_ex(int opt);
 	bool config_host(const char* host, int config_options);
-	bool validate_config(bool abort_if_invalid);
+	bool validate_config(bool abort_if_invalid, int opt);
 	void config_dump_string_pool(FILE * fh, const char * sep);
 	void config_dump_sources(FILE * fh, const char * sep);
 	const char * config_source_by_id(int source_id);
@@ -275,8 +351,12 @@ extern "C" {
 	int  set_runtime_config(char *admin, char *config);
 	int is_valid_param_name(const char *name);
 	char * is_valid_config_assignment(const char *config);
-	// this function allows tests to pretend that a param was set to a given value.	
+	// this function allows tests to pretend that a param was set to a given value. but it leaks memory if used frequently
     void  param_insert(const char * name, const char * value);
+	// this function allows tests to set the actual backend data for a param value and returns the old value.
+	// make sure that live_value stays in scope until you put the old value back
+	const char * set_live_param_value(const char * name, const char * live_value);
+
 } // end extern "C"
 
 
@@ -333,7 +413,6 @@ int write_config_file(const char* pathname, int options);
 
 extern "C" {
 
-#if 1
 	/** Find next $$(MACRO) or $$([expression]) in value
 		search begins at pos and continues to terminating null
 
@@ -354,39 +433,6 @@ extern "C" {
 	returns non-zero if a $$() is found, zero if not found.
 	*/
 	int next_dollardollar_macro(char * value, int pos, char** left, char** name, char** right);
-#else
-
-	/** Find next $(MACRO) or $$(MACRO) in a string
-
-	The caller is expected to concatenate *leftp, the evaluated
-	*namep, then *rightp to get the expanded setting.
-
-	- value - The null-terminated string to scan. WILL BE MODIFIED!
-
-	- leftp - OUTPUT. *leftp will be set to value+search_pos.  It
-	  will be null terminated at the $ for the first $(MACRO) found.
-
-	- namep - OUTPUT. The name of the MACRO (the bit between the
-	  parenthesis).  Pointer into value.  Null terminated at the
-	  closing parenthesis.
-
-	- rightp - OUTPUT. Everything to the right of the $(MACRO).
-	  Pointer into value.
-
-	- self - Default to null. If non-null, only macros whose name is
-	  identical to self will be expanded. (Used for the special
-	  $(DOLLAR) case?)
-
-	- getdollardollar - Defaults false. If true, scans for $$(MACRO)
-	  and $$([expression]) instead of $(MACRO)
-
-	- search_pos - 0-indexed position in value to start scanning at.
-	  Defaults to 0.
-	*/
-	int find_config_macro( register char *value, register char **leftp,
-		register char **namep, register char **rightp,
-		const char *self=NULL, bool getdollardollar=false, int search_pos=0);
-#endif
 
 	void init_config (int options);
 }
@@ -397,14 +443,14 @@ BEGIN_C_DECLS
 
 	char * get_tilde(void);
 	char * param ( const char *name );
+	// do param lookup with explicit subsys and localname
+	char * param_with_context ( const char *name, const char *subsys, const char *localname, const char * cwd );
 
-	/** Insert a value into a hash table.
-
-	The value of 'value' is associated with the name 'name'.  This is
-	inserted into the table 'table' which is 'table_size' big.
-	insert keeps copies of the name and value.
-	*/
 #ifdef __cplusplus
+	// do param lookup with explicit subsys and localname
+	char * param_ctx (const char *name, MACRO_EVAL_CONTEXT & ctx);
+
+	// config source file info
 	typedef struct macro_source { bool is_inside; bool is_command; short int id; int line; short int meta_id; short int meta_off; } MACRO_SOURCE;
 	void insert_source(const char * filename, MACRO_SET& macro_set, MACRO_SOURCE & source);
 	extern const MACRO_SOURCE EnvMacro;
@@ -422,6 +468,89 @@ BEGIN_C_DECLS
 	int get_macro_ref_count (const char *name, MACRO_SET& macro_set);
 	bool config_test_if_expression(const char * expr, bool & result, const char * localname, const char * subsys, std::string & err_reason);
 
+	// simple class to split metaknob name from arguments and store the result in public member variables
+	class MetaKnobAndArgs {
+	public:
+		std::string knob;
+		std::string args;
+		std::string extra;
+		MetaKnobAndArgs(const char * p=NULL) { if (p) init_from_string(p); }
+		// set member variables by parsing p, returns pointer to the next metaknob name or NULL
+		// if there are no characters after. leading & trailing whitespace and , are skipped.
+		const char* init_from_string(const char * p);
+	};
+
+	// expand $(N) from value against arguments in argstr
+	// returns a malloc()'d ptr to expanded value, caller is responsible from freeing returned pointer.
+	char * expand_meta_args(const char *value, std::string & argstr);
+
+	// macro stream class wraps up an fp or string so that the macro parser can read from either.
+	//
+	class MacroStream {
+	public:
+		virtual ~MacroStream() {};
+		virtual char * getline(int gl_opt) = 0;
+		virtual MACRO_SOURCE& source() = 0;
+	};
+
+	// A MacroStream that uses, but does not own a FILE* and MACRO_SOURCE
+	class MacroStreamYourFile : public MacroStream {
+	public:
+		MacroStreamYourFile(FILE * _fp, MACRO_SOURCE& _src) : fp(_fp), src(&_src) {}
+		virtual ~MacroStreamYourFile() { fp = NULL; src = NULL; }
+		virtual char * getline(int gl_opt);
+		virtual MACRO_SOURCE& source() { return *src; }
+		void set(FILE* _fp, MACRO_SOURCE& _src) { fp =  _fp; src = &_src; }
+		void reset() { fp = NULL; src = NULL; }
+	protected:
+		FILE * fp;
+		MACRO_SOURCE * src;
+	};
+
+	// A MacroStream that owns the FILE* and MACRO_SOURCE
+	class MacroStreamFile : public MacroStream {
+	public:
+		MacroStreamFile() : fp(NULL) { memset(&src, 0, sizeof(src)); }
+		virtual ~MacroStreamFile() { if (fp) fclose(fp); fp = NULL; memset(&src, 0, sizeof(src)); }
+		virtual char * getline(int gl_opt);
+		virtual MACRO_SOURCE& source() { return src; }
+		bool open(const char * filename, bool is_command, MACRO_SET& set, std::string &errmsg);
+		int  close(MACRO_SET& set, int parsing_return_val);
+	protected:
+		FILE * fp;
+		MACRO_SOURCE src;
+	};
+
+	class MacroStreamCharSource : public MacroStream {
+	public:
+		MacroStreamCharSource() 
+			: input(NULL)
+			, cbBufAlloc(0)
+			, file_string(NULL)
+		{
+			memset(&src, 0, sizeof(src));
+		}
+		virtual ~MacroStreamCharSource() { if (input) delete input; input = NULL; }
+		virtual char * getline(int gl_opt);
+		virtual MACRO_SOURCE& source() { return src; }
+		bool open(const char * src_string, const MACRO_SOURCE & _src);
+		int  close(MACRO_SET& set, int parsing_return_val);
+		int  load(FILE* fp, MACRO_SOURCE & _src, bool preserve_linenumbers = false);
+		void rewind();
+	protected:
+		StringTokenIterator * input;
+		MACRO_SOURCE src;
+		size_t cbBufAlloc;
+		auto_free_ptr line_buf;
+		auto_free_ptr file_string; // holds file content when load is called.
+	private:
+		// copy construction and assignment are not permitted.
+		MacroStreamCharSource(const MacroStreamCharSource&);
+		MacroStreamCharSource & operator=(MacroStreamCharSource that); 
+
+	};
+
+
 	// populate a MACRO_SET from either a config file or a submit file.
 	#define READ_MACROS_SUBMIT_SYNTAX           0x01
 	#define READ_MACROS_EXPAND_IMMEDIATE        0x02
@@ -449,8 +578,7 @@ BEGIN_C_DECLS
 
 	int
 	Parse_macros(
-		FILE* conf_fp,
-		MACRO_SOURCE& FileMacro,
+		MacroStream& ms,
 		int depth, // a simple recursion detector
 		MACRO_SET& macro_set,
 		int options,
@@ -506,6 +634,5 @@ void condor_auth_config(int is_daemon);
 
 END_C_DECLS
 
-
 #endif /* CONFIG_H */
 
