@@ -74,7 +74,7 @@ enum {
    D_NFS,
    D_AUDIT, // messages for the audit log
    D_TEST,  // messages with this category are parsed by various tests.
-   D_UNUSED29,
+   D_STATS,
    D_UNUSED30,
    D_BUG,   // messages that indicate the daemon is going down.
 
@@ -127,7 +127,6 @@ enum {
 #ifdef __cplusplus
 #include <string>
 #include <map>
-#include "param_functions.h"
 extern "C" {
 #endif
 
@@ -164,7 +163,7 @@ void dprintf ( int flags, const char *fmt, ... ) CHECK_PRINTF_FORMAT(2,3);
 }
 void dprintf ( int flags, DPF_IDENT ident, const char *fmt, ... ) CHECK_PRINTF_FORMAT(3,4);
 extern "C" {
-// parse config files (via param_functions) and use them to fill out the array of dprintf_output_settings
+// parse config files and use them to fill out the array of dprintf_output_settings
 // one for each output log file. returns the number of entries needed in p_info, (may be larger than c_info!)
 // if p_info is NULL, then dprintf_set_outputs is called with the dprintf_output_settings array.  if != NULL, then
 // the array is returned, calling dprintf_set_outputs is left to the caller.
@@ -213,6 +212,11 @@ void dprintf_after_shared_mem_clone( void );
 
 void dprintf_dump_stack(void);
 
+/* If outputs haven't been configured yet, stop buffering dprintf()
+ * output until they are configured.
+ */
+void dprintf_pause_buffering();
+
 time_t dprintf_last_modification(void);
 void dprintf_touch_log(void);
 /* write dprintf contribution to the daemon header */
