@@ -50,6 +50,8 @@ const SetAttributeFlags_t NONDURABLE = (1<<0); // do not fsync
 const SetAttributeFlags_t SetAttribute_NoAck = (1<<1);
 const SetAttributeFlags_t SETDIRTY = (1<<2);
 const SetAttributeFlags_t SHOULDLOG = (1<<3);
+const SetAttributeFlags_t SetAttribute_OnlyMyJobs = (1<<4);
+const SetAttributeFlags_t SetAttribute_QueryOnly = (1<<5); // check if change is allowed, but don't actually change.
 
 #define SHADOW_QMGMT_TIMEOUT 300
 
@@ -172,6 +174,23 @@ int SetAttributeFloat(int cluster, int proc, const char *attr, float value, SetA
 int SetAttributeString(int cluster, int proc, const char *attr,
 					   const char *value, SetAttributeFlags_t flags = 0);
 
+/** Set attr = value for a job with the specified cluster and proc.  The value
+ *  will be converted to a valid, quoted classad string
+ *  @return -1 on failure; 0 on success
+ */
+int SetAttributeRawString(int cluster, int proc, const char *attr,
+                          const char *value, SetAttributeFlags_t flags = 0);
+
+// Internal function for only the schedd to use.
+int SetSecureAttributeInt(int cluster_id, int proc_id,
+                          const char *attr_name, int attr_value,
+                          SetAttributeFlags_t flags);
+
+// Internal function for only the schedd to use.
+int SetSecureAttributeRawString(int cluster_id, int proc_id,
+                                const char *attr_name, const char *attr_value,
+                                SetAttributeFlags_t flags);
+
 /** Set LastJobLeaseRenewalReceived = <xact start time> and
     JobLeaseDurationReceived = dur for the specified cluster/proc.
 	@return -1 on failure; 0 on success
@@ -208,8 +227,10 @@ int RemoteCommitTransaction(SetAttributeFlags_t flags=0, CondorError *errstack=N
 /** The difference between this and RemoteCommitTransaction is that
 	this function never returns if there is a failure.  This function
 	should only be called from the schedd.
+    Exception: This function can return failure if a SUBMIT_REQUIREMEMT
+      expression evaluates to False.
 */
-void CommitTransaction(SetAttributeFlags_t flags=0);
+int CommitTransaction(SetAttributeFlags_t flags=0, CondorError *errstack=NULL);
 
 int AbortTransaction();
 void AbortTransactionAndRecomputeClusters();
@@ -344,6 +365,16 @@ int float_to_rusage(double, double, struct rusage *);
 
 int QmgmtSetEffectiveOwner(char const *owner);
 
+/* Set to TRUE (1) if changes to protected job attributes should be allowed,
+   or FALSE (0) to refuse changes to protected attributes by having
+   SetAttribute() fail.  Defaults to TRUE.
+   Note that this function has no effect unless the real authenticated
+   owner is a queue super user, as changes to protected attributes
+   also always require that the real owner is a queue super user.
+   Returns previous value of the flag. */
+
+int QmgmtSetAllowProtectedAttrChanges(int val);
+
 /* Call this to begin iterating over jobs in the queue that match
    a constraint.
    Returns 0 on success, -1 on error*/
