@@ -191,9 +191,20 @@ public:
 	// returns -1 on failure, 0 for ok
 	int put_empty_file( filesize_t *size );
 
-	/// returns -1 on failure, 0 for ok
-	int get_x509_delegation( filesize_t *size, const char *destination,
-							 bool flush_buffers=false );
+	/// returns delegation_error on failure, delegation_ok on success,
+	/// and delegation_continue if the delegation is incomplete.
+	///
+	/// The continuations are allowed only if state_ptr is non-NULL.
+	/// When more data is available on the socket, the caller should call
+	/// get_x509_delegation_finish.
+	enum x509_delegation_result {delegation_ok, delegation_continue, delegation_error};
+	x509_delegation_result get_x509_delegation( const char *destination, bool flush_buffers, void **state_ptr );
+
+	/// The second half of get_x509_delegation.  If state_ptr was non-NULL to the original function call,
+	/// one must pass the same pointer here.  The function takes ownership of the value and will call delete
+	/// on it.
+	x509_delegation_result get_x509_delegation_finish( const char *destination, bool flush_buffers, void *state_ptr );
+
 	/// returns -1 on failure, 0 for ok
 	// expiration_time: 0 if none; o.w. timestamp of delegated proxy expiration
 	// result_expiration_time: if non-NULL will be set to actual expiration
@@ -216,6 +227,12 @@ public:
 	void enter_reverse_connecting_state();
 	void exit_reverse_connecting_state(ReliSock *sock);
 
+		// returns a pointer to an internally managed
+		// buffer with a human-readable string containing
+		// tcp statistics from the TCP_INFO sockopt.  Don't free.
+		// may return null
+	char *get_statistics();
+
 #ifndef WIN32
 	// interface no longer supported 
 	int attach_to_file_desc(int);
@@ -259,6 +276,11 @@ public:
 	int clear_read_block_flag() {bool state = m_read_would_block; m_read_would_block = false; return state;}
 
 	bool is_closed() {return rcv_msg.m_closed;}
+
+	// serialize and deserialize
+	const char * serialize(const char *);	// restore state from buffer
+	char * serialize() const;	// save state into buffer
+
 //	PROTECTED INTERFACE TO RELIABLE SOCKS
 //
 protected:
@@ -278,8 +300,6 @@ protected:
 	/*
 	**	Methods
 	*/
-	char * serialize(char *);	// restore state from buffer
-	char * serialize() const;	// save state into buffer
 
 	int prepare_for_nobuffering( stream_coding = stream_unknown);
 	int perform_authenticate( bool with_key, KeyInfo *& key, 
@@ -351,6 +371,8 @@ protected:
 
 	int is_client;
 	char *hostAddr;
+	char *statsBuf;
+
 	classy_counted_ptr<class CCBClient> m_ccb_client; // for reverse connects
 
 		// after connecting, request to be routed to this daemon
