@@ -127,6 +127,10 @@ public:
 	///
 	virtual int peek(char &);
 
+	// serialize and deserialize
+	const char * serialize(const char *);
+	char * serialize() const;
+
 //	PRIVATE INTERFACE TO SAFE SOCKS
 //
 
@@ -143,8 +147,6 @@ protected:
 
 	enum safesock_state { safesock_none, safesock_listen };
 
-	char * serialize(char *);
-	char * serialize() const;
 	inline bool same(const _condorMsgID msgA,
 					 const _condorMsgID msgB)
 	{
