@@ -62,6 +62,10 @@ We want to define a callback function to be invoked when certain actions happen
 
 typedef void (CedarHandler) (Stream *s);
 
+namespace classad {
+class ClassAd;
+}
+
 /*
 **	B A S E    S O C K
 */
@@ -162,6 +166,19 @@ public:
 	int assignInvalidSocket();
 	int assignInvalidSocket( condor_protocol proto );
 	int assignDomainSocket( SOCKET s );
+	//
+	// Normally, we verify that the socket (fd) you're assigning to this
+	// Sock (object) has the same address family (protocol).  However,
+	// when using CCB, this restriction must be relaxed.  Consider an
+	// IPv4-only schedd matches against a mixed-mode startd using a
+	// mixed-mode CCB.  If the mixed-mode startd prefers its IPv6 address,
+	// and the schedd is configured to respect that preference, it will
+	// attempt to connect to the IPv6 address via CCB.  However, since the
+	// schedd connected to CCB using IPv4, CCB will ask the startd to
+	// also connect over IPv4, leading to a reverse connection over IPv4
+	// for an IPv6 address.
+	//
+	int assignCCBSocket( SOCKET s );
 #if defined(WIN32) && defined(_WINSOCK2API_)
 	int assign(LPWSAPROTOCOL_INFO);		// to inherit sockets from other processes
 #endif
@@ -367,6 +384,12 @@ public:
 	void setCryptoMethodUsed(char const *crypto_method);
 	const char* getCryptoMethodUsed();
 
+	void setSessionID(const std::string &session_id) {_session = session_id;}
+	const std::string &getSessionID() const {return _session;}
+
+	void getPolicyAd(classad::ClassAd &ad) const;
+	void setPolicyAd(const classad::ClassAd &ad);
+
 		/// True if socket has tried to authenticate or socket is
 		/// using a security session that tried to authenticate.
 		/// Authentication may or may not have succeeded and
@@ -404,6 +427,13 @@ public:
 
 	unsigned int getUniqueId() { return m_uniqueId; }
 
+#ifdef WIN32
+	int set_inheritable( int flag );
+#else
+	// On unix, sockets are always inheritable
+	inline int set_inheritable( int ) { return TRUE; }
+#endif
+
 //	PRIVATE INTERFACE TO ALL SOCKS
 //
 protected:
@@ -463,12 +493,12 @@ protected:
 	void set_connect_addr(char const *addr);
 
 	inline SOCKET get_socket (void) { return _sock; }
-	char * serialize(char *);
+	const char * serialize(const char *);
 	static void close_serialized_socket(char const *buf);
 	char * serialize() const;
-    char * serializeCryptoInfo(char * buf);
+    const char * serializeCryptoInfo(const char * buf);
     char * serializeCryptoInfo() const;
-    char * serializeMdInfo(char * buf);
+    const char * serializeMdInfo(const char * buf);
     char * serializeMdInfo() const;
         
 	virtual int encrypt(bool);
@@ -478,12 +508,6 @@ protected:
 	virtual bool is_hdr_encrypt();
     ///
 	virtual bool is_encrypt();
-#ifdef WIN32
-	int set_inheritable( int flag );
-#else
-	// On unix, sockets are always inheritable
-	inline int set_inheritable( int ) { return TRUE; }
-#endif
 
     ///
 	bool test_connection();
@@ -524,6 +548,8 @@ protected:
 	char *          _auth_methods;
 	char *          _auth_name;
 	char *          _crypto_method;
+	std::string     _session;
+	classad::ClassAd *_policy_ad;
 	bool            _tried_authentication;
 
 	bool ignore_connect_timeout;	// Used by HA Daemon
@@ -555,7 +581,7 @@ private:
 
 	int _condor_read(SOCKET fd, char *buf, int sz, int timeout);
 	int _condor_write(SOCKET fd, const char *buf, int sz, int timeout);
-	int bindWithin(condor_protocol proto, const int low, const int high, bool outbound);
+	int bindWithin(condor_protocol proto, const int low, const int high);
 	///
 	// Buffer to hold the string version of our peer's IP address. 
 	char _peer_ip_buf[IP_STRING_BUF_SIZE];	
@@ -631,15 +657,6 @@ private:
 	   connection attempt.
 	 **/
 	void cancel_connect();
-
-	/**
-	   Private helper that sees if we're CCB enabled, if we're doing
-	   an outbound connection, and if so, uses CCB_local_bind() to
-	   avoid pounding the CCB broker for all outbound connections.
-	*/
-	//int _bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
-	//	SOCKET_LENGTH_TYPE len, bool outbound, bool loopback);
-	int _bind_helper(int fd, const condor_sockaddr& addr, bool outbound, bool loopback);
 };
 
 void dprintf ( int flags, Sock & sock, const char *fmt, ... ) CHECK_PRINTF_FORMAT(3,4);
