@@ -292,6 +292,16 @@ int Authentication::authenticate_continue( CondorError* errstack, bool non_block
 					(m_method_name.size()?m_method_name.c_str():"?!?") );
 		}
 
+		// We have just picked a new method, so we definitely want to
+		// make this value true.  We log a message only if we changed
+		// it.
+		if(!do_authenticate) {
+			do_authenticate = true;
+			if (IsDebugVerbose(D_SECURITY)) {
+				dprintf(D_SECURITY, "AUTHENTICATE: forcing do_authenticate to true.\n");
+			}
+		}
+
 		//------------------------------------------
 		// Now authenticate
 		//------------------------------------------
@@ -303,6 +313,10 @@ authenticate:
 			break;
 		}
 
+		if (IsDebugVerbose(D_SECURITY)) {
+			dprintf (D_SECURITY, "AUTHENTICATE: do_authenticate is %i.\n", do_authenticate);
+		}
+
 		if (do_authenticate) {
 			auth_rc = m_auth->authenticate(m_host_addr.c_str(), errstack, non_blocking);
 			if (auth_rc == 2) {
@@ -402,12 +416,7 @@ int Authentication::authenticate_finish(CondorError *errstack)
 	// check to see if CERTIFICATE_MAPFILE was defined.  if so, use it.  if
 	// not, do nothing.  the user and domain have been filled in by the
 	// authentication method itself, so just leave that alone.
-	char * cert_map_file = param("CERTIFICATE_MAPFILE");
-	bool use_mapfile = (cert_map_file != NULL);
-	if (cert_map_file) {
-		free(cert_map_file);
-		cert_map_file = 0;
-	}
+	bool use_mapfile = param_defined("CERTIFICATE_MAPFILE");
 
 	// if successful so far, invoke the security MapFile.  the output of that
 	// is the "canonical user".  if that has an '@' sign, split it up on the
@@ -492,22 +501,24 @@ void Authentication::map_authentication_name_to_canonical_name(int authenticatio
 			global_map_file = NULL;
 		}
 
-		global_map_file = new MapFile();
-
 		dprintf (D_SECURITY, "ZKM: Parsing map file.\n");
-        char * credential_mapfile;
-        if (NULL == (credential_mapfile = param("CERTIFICATE_MAPFILE"))) {
-            dprintf(D_SECURITY, "ZKM: No CERTIFICATE_MAPFILE defined\n");
-			delete global_map_file;
-			global_map_file = NULL;
-        } else {
-        	int line;
-        	if (0 != (line = global_map_file->ParseCanonicalizationFile(credential_mapfile))) {
-            	dprintf(D_SECURITY, "ZKM: Error parsing %s at line %d", credential_mapfile, line);
+		auto_free_ptr credential_mapfile(param("CERTIFICATE_MAPFILE"));
+		if ( ! credential_mapfile) {
+			dprintf(D_SECURITY, "ZKM: No CERTIFICATE_MAPFILE defined\n");
+		} else {
+			global_map_file = new MapFile();
+
+			// prior to 8.5.8 all keys in CERTIFICATE_MAPFILE were assumed to be regexes
+			// this is both slower, and less secure, so it is now possible to opt in to a syntax where
+			// keys are assumed to be literals (i.e. hashable keys) unless the start and end with /
+			bool assume_hash = param_boolean("CERTIFICATE_MAPFILE_ASSUME_HASH_KEYS", false);
+
+			int line;
+			if (0 != (line = global_map_file->ParseCanonicalizationFile(credential_mapfile.ptr(), assume_hash))) {
+				dprintf(D_SECURITY, "ZKM: Error parsing %s at line %d", credential_mapfile.ptr(), line);
 				delete global_map_file;
 				global_map_file = NULL;
 			}
-			free( credential_mapfile );
 		}
 		global_map_file_load_attempted = true;
 	} else {
@@ -728,9 +739,6 @@ int Authentication::setOwner( const char *owner )
 #if defined(SKIP_AUTHENTICATION)
 	return 0;
 #else
-	if ( !this ) {
-            return 0;
-	}
 	if ( authenticator_ ) {
             authenticator_->setRemoteUser(owner);
             return 1;
@@ -986,7 +994,12 @@ int Authentication::handshake(MyString my_methods, bool non_blocking) {
         dprintf (D_SECURITY, "HANDSHAKE: handshake() - i am the client\n");
         mySock->encode();
 		int method_bitmask = SecMan::getAuthBitmask(my_methods.Value());
-		if ( (method_bitmask & CAUTH_KERBEROS) && Condor_Auth_Kerberos::Initialize() == false ) {
+#if defined(HAVE_EXT_KRB5)
+		if ( (method_bitmask & CAUTH_KERBEROS) && Condor_Auth_Kerberos::Initialize() == false )
+#else
+		if (method_bitmask & CAUTH_KERBEROS)
+#endif
+		{
 			dprintf (D_SECURITY, "HANDSHAKE: excluding KERBEROS: %s\n", "Initialization failed");
 			method_bitmask &= ~CAUTH_KERBEROS;
 		}
@@ -1036,7 +1049,12 @@ Authentication::handshake_continue(MyString my_methods, bool non_blocking)
 	dprintf ( D_SECURITY, "HANDSHAKE: client sent (methods == %i)\n", client_methods);
 
 	shouldUseMethod = selectAuthenticationType( my_methods, client_methods );
-	if ( (shouldUseMethod & CAUTH_KERBEROS) && Condor_Auth_Kerberos::Initialize() == false ) {
+#if defined(HAVE_EXT_KRB5) 
+	if ( (shouldUseMethod & CAUTH_KERBEROS) && Condor_Auth_Kerberos::Initialize() == false )
+#else
+	if (shouldUseMethod & CAUTH_KERBEROS)
+#endif
+	{
 		dprintf (D_SECURITY, "HANDSHAKE: excluding KERBEROS: %s\n", "Initialization failed");
 		shouldUseMethod &= ~CAUTH_KERBEROS;
 	}
