@@ -86,7 +86,11 @@ const char SecMan::sec_req_rev[][10] = {
 	"REQUIRED"
 };
 
-KeyCache SecMan::session_cache;
+KeyCache SecMan::m_default_session_cache;
+std::map<std::string,KeyCache*> *SecMan::m_tagged_session_cache = NULL;
+std::string SecMan::m_tag;
+KeyCache *SecMan::session_cache = &SecMan::m_default_session_cache;
+std::string SecMan::m_pool_password;
 HashTable<MyString,MyString> SecMan::command_map(209, MyStringHash, updateDuplicateKeys);
 HashTable<MyString,classy_counted_ptr<SecManStartCommand> > SecMan::tcp_auth_in_progress(256, MyStringHash, rejectDuplicateKeys);
 int SecMan::sec_man_ref_count = 0;
@@ -95,6 +99,27 @@ char* SecMan::_my_parent_unique_id = 0;
 bool SecMan::_should_check_env_for_unique_id = true;
 IpVerify *SecMan::m_ipverify = NULL;
 
+void
+SecMan::setTag(const std::string &tag) {
+        m_tag = tag;
+	if (tag.size() == 0) {
+		session_cache = &m_default_session_cache;
+		return;
+	}
+	if (m_tagged_session_cache == NULL) {
+		m_tagged_session_cache = new std::map<std::string, KeyCache*>();
+	}
+	std::map<std::string, KeyCache*>::const_iterator iter = m_tagged_session_cache->find(tag);
+	KeyCache *tmp = NULL;
+	if (iter == m_tagged_session_cache->end()) {
+		tmp = new KeyCache();
+		m_tagged_session_cache->insert(std::make_pair(tag, tmp));
+	} else {
+		tmp = iter->second;
+	}
+	session_cache = tmp;
+}
+
 SecMan::sec_req
 SecMan::sec_alpha_to_sec_req(char *b) {
 	if (!b || !*b) {  
@@ -121,7 +146,7 @@ SecMan::sec_alpha_to_sec_req(char *b) {
 
 
 SecMan::sec_feat_act
-SecMan::sec_lookup_feat_act( ClassAd &ad, const char* pname ) {
+SecMan::sec_lookup_feat_act( const ClassAd &ad, const char* pname ) {
 
 	char* res = NULL;
 	ad.LookupString(pname, &res);
@@ -184,7 +209,7 @@ SecMan::sec_alpha_to_sec_act(char *b) {
 
 /*
 SecMan::sec_act
-SecMan::sec_lookup_act( ClassAd &ad, const char* pname ) {
+SecMan::sec_lookup_act( const ClassAd &ad, const char* pname ) {
 
 	char* res = NULL;
 	ad.LookupString(pname, &res);
@@ -204,7 +229,7 @@ SecMan::sec_lookup_act( ClassAd &ad, const char* pname ) {
 
 
 SecMan::sec_req
-SecMan::sec_lookup_req( ClassAd &ad, const char* pname ) {
+SecMan::sec_lookup_req( const ClassAd &ad, const char* pname ) {
 
 	char* res = NULL;
 	ad.LookupString(pname, &res);
@@ -442,7 +467,15 @@ SecMan::FillInSecurityPolicyAd( DCpermission auth_level, ClassAd* ad,
 	// auth methods
 	paramer = SecMan::getSecSetting ("SEC_%s_AUTHENTICATION_METHODS", auth_level);
 	if (paramer == NULL) {
-		paramer = strdup(SecMan::getDefaultAuthenticationMethods().Value());
+		MyString methods = SecMan::getDefaultAuthenticationMethods();
+		if(auth_level == READ) {
+			methods += ",CLAIMTOBE";
+			dprintf(D_SECURITY, "SECMAN: default READ methods: %s\n", methods.Value());
+		} else if (auth_level == CLIENT_PERM) {
+			methods += ",CLAIMTOBE";
+			dprintf(D_SECURITY, "SECMAN:: default CLIENT methods: %s\n", methods.Value());
+		}
+		paramer = strdup(methods.Value());
 	}
 
 	if (paramer) {
@@ -568,6 +601,41 @@ SecMan::FillInSecurityPolicyAd( DCpermission auth_level, ClassAd* ad,
 	return true;
 }
 
+// Fill'ing in the security policy ad is very expensive, sometimes requiring
+// 400 param calls.  Usually, we call it with the same parameters over and over,
+// so keep track of the last set of parameters, and cache the last one returned.
+
+bool
+SecMan::FillInSecurityPolicyAdFromCache(DCpermission auth_level, ClassAd* &ad, 
+								bool raw_protocol,
+								bool use_tmp_sec_session,
+								bool force_authentication )
+{
+	if ((m_cached_auth_level == auth_level) &&
+		(m_cached_raw_protocol == raw_protocol) &&
+		(m_cached_use_tmp_sec_session == use_tmp_sec_session) &&
+		(m_cached_force_authentication == force_authentication)) {
+
+			// A Hit!
+		if (m_cached_return_value) {
+			ad = &m_cached_policy_ad;
+
+		}
+		return m_cached_return_value;
+	}	
+
+		// Miss...
+	m_cached_auth_level = auth_level;
+	m_cached_raw_protocol = raw_protocol;
+	m_cached_use_tmp_sec_session = use_tmp_sec_session;
+	m_cached_force_authentication = force_authentication;
+	
+	m_cached_policy_ad.Clear(); 
+	m_cached_return_value = FillInSecurityPolicyAd(auth_level, &m_cached_policy_ad, raw_protocol, use_tmp_sec_session, force_authentication);
+	ad = & m_cached_policy_ad;
+	return m_cached_return_value;
+}
+
 bool
 SecMan::ReconcileSecurityDependency (sec_req &a, sec_req &b) {
 	if (a == SEC_REQ_NEVER) {
@@ -587,7 +655,7 @@ SecMan::ReconcileSecurityDependency (sec_req &a, sec_req &b) {
 
 SecMan::sec_feat_act
 SecMan::ReconcileSecurityAttribute(const char* attr,
-								   ClassAd &cli_ad, ClassAd &srv_ad,
+								   const ClassAd &cli_ad, const ClassAd &srv_ad,
 								   bool *required ) {
 
 	// extract the values from the classads
@@ -695,7 +763,7 @@ SecMan::ReconcileSecurityAttribute(const char* attr,
 
 
 ClassAd *
-SecMan::ReconcileSecurityPolicyAds(ClassAd &cli_ad, ClassAd &srv_ad) {
+SecMan::ReconcileSecurityPolicyAds(const ClassAd &cli_ad, const ClassAd &srv_ad) {
 
 	// figure out what to do
 	sec_feat_act authentication_action;
@@ -901,14 +969,23 @@ class SecManStartCommand: Service, public ClassyCountedPtr {
 	}
 
 	~SecManStartCommand() {
-		if( m_pending_socket_registered ) {
-			m_pending_socket_registered = false;
-			daemonCore->decrementPendingSockets();
-		}
 		if( m_private_key ) {
 			delete m_private_key;
 			m_private_key = NULL;
 		}
+
+			// we may be called after the daemonCore object
+			// has been destructed and nulled out in a 
+			// shutdown.  If so, just return now.
+
+		if (!daemonCore) {
+			return;
+		}
+
+		if( m_pending_socket_registered ) {
+			m_pending_socket_registered = false;
+			daemonCore->decrementPendingSockets();
+		}
 			// The callback function _must_ have been called
 			// (and set to NULL) by now.
 		ASSERT( !m_callback_fn );
@@ -1237,7 +1314,7 @@ SecManStartCommand::startCommand_inner()
 bool
 SecMan::LookupNonExpiredSession(char const *session_id, KeyCacheEntry *&session_key)
 {
-	if(!session_cache.lookup(session_id,session_key)) {
+	if(!session_cache->lookup(session_id,session_key)) {
 		return false;
 	}
 
@@ -1245,7 +1322,7 @@ SecMan::LookupNonExpiredSession(char const *session_id, KeyCacheEntry *&session_
 	time_t cutoff_time = time(0);
 	time_t expiration = session_key->expiration();
 	if (expiration && expiration <= cutoff_time) {
-		session_cache.expire(session_key);
+		session_cache->expire(session_key);
 		session_key = NULL;
 		return false;
 	}
@@ -1271,7 +1348,12 @@ SecManStartCommand::sendAuthInfo_inner()
 		}
 	}
 
-	m_session_key.formatstr ("{%s,<%i>}", m_sock->get_connect_addr(), m_cmd);
+	const std::string &tag = SecMan::getTag();
+	if (tag.size()) {
+		m_session_key.formatstr ("{%s,%s,<%i>}", tag.c_str(), m_sock->get_connect_addr(), m_cmd);
+	} else {
+		m_session_key.formatstr ("{%s,<%i>}", m_sock->get_connect_addr(), m_cmd);
+	}
 	bool found_map_ent = false;
 	if( !m_have_session && !m_raw_protocol && !m_use_tmp_sec_session ) {
 		found_map_ent = (m_sec_man.command_map.lookup(m_session_key, sid) == 0);
@@ -2011,15 +2093,21 @@ SecManStartCommand::receivePostAuthInfo_inner()
 				// gather some additional data for useful error reporting
 				MyString response_user;
 				MyString response_method = m_sock->getAuthenticationMethodUsed();
-				if (response_method == "") {
-					response_method = "(no authentication)";
-				}
 				post_auth_info.LookupString(ATTR_SEC_USER,response_user);
 
 				// push error message on the stack and print to the log
 				MyString errmsg;
-				errmsg.formatstr("Received \"%s\" from server for user %s using method %s.",
-					response_rc.Value(), response_user.Value(), response_method.Value());
+				if (response_method == "") {
+					response_method = "(no authentication)";
+					errmsg.formatstr( "Received \"%s\" from server for user %s using no authentication method, which may imply host-based security.  Our address was '%s', and server's address was '%s'.  Check your ALLOW settings and IP protocols.",
+						response_rc.Value(), response_user.Value(),
+						m_sock->my_addr().to_ip_string().Value(),
+						m_sock->peer_addr().to_ip_string().Value()
+						);
+				} else {
+					errmsg.formatstr("Received \"%s\" from server for user %s using method %s.",
+						response_rc.Value(), response_user.Value(), response_method.Value());
+				}
 				dprintf (D_ALWAYS, "SECMAN: FAILED: %s\n", errmsg.Value());
 				m_errstack->push ("SECMAN", SECMAN_ERR_AUTHORIZATION_FAILED, errmsg.Value());
 				return StartCommandFailed;
@@ -2117,7 +2205,7 @@ SecManStartCommand::receivePostAuthInfo_inner()
             }
 
 			// stick the key in the cache
-			m_sec_man.session_cache.insert(tmp_key);
+			m_sec_man.session_cache->insert(tmp_key);
 
 
 			// now add entrys which map all the {<sinful_string>,<command>} pairs
@@ -2129,7 +2217,12 @@ SecManStartCommand::receivePostAuthInfo_inner()
 			coms.rewind();
 			while ( (p = coms.next()) ) {
 				MyString keybuf;
-				keybuf.formatstr ("{%s,<%s>}", m_sock->get_connect_addr(), p);
+				const std::string &tag = SecMan::getTag();
+				if (tag.size()) {
+					keybuf.formatstr ("{%s,%s,<%s>}", tag.c_str(), m_sock->get_connect_addr(), p);
+				} else {
+					keybuf.formatstr ("{%s,<%s>}", m_sock->get_connect_addr(), p);
+				}
 
 				// NOTE: HashTable returns ZERO on SUCCESS!!!
 				if (m_sec_man.command_map.insert(keybuf, sesid) == 0) {
@@ -2142,6 +2235,7 @@ SecManStartCommand::receivePostAuthInfo_inner()
 				}
 			}
 			
+			m_sock->setSessionID(sesid);
 			free( sesid );
             free( cmd_list );
 
@@ -2432,7 +2526,7 @@ SecManStartCommand::SocketCallback( Stream *stream )
 void
 SecMan::invalidateHost(const char * sin)
 {
-	StringList *keyids = session_cache.getKeysForPeerAddress(sin);
+	StringList *keyids = session_cache->getKeysForPeerAddress(sin);
 	if( !keyids ) {
 		return;
 	}
@@ -2450,7 +2544,7 @@ SecMan::invalidateHost(const char * sin)
 
 void
 SecMan::invalidateByParentAndPid(const char * parent, int pid) {
-	StringList *keyids = session_cache.getKeysForProcess(parent,pid);
+	StringList *keyids = session_cache->getKeysForProcess(parent,pid);
 	if( !keyids ) {
 		return;
 	}
@@ -2471,7 +2565,7 @@ bool SecMan :: invalidateKey(const char * key_id)
     bool removed = true;
     KeyCacheEntry * keyEntry = NULL;
 
-	session_cache.lookup(key_id, keyEntry);
+	session_cache->lookup(key_id, keyEntry);
 
 	if ( keyEntry && keyEntry->expiration() <= time(NULL) ) {
 		dprintf( D_SECURITY,
@@ -2482,7 +2576,7 @@ bool SecMan :: invalidateKey(const char * key_id)
 	remove_commands(keyEntry);
 
 	// Now, remove session id
-	if (session_cache.remove(key_id)) {
+	if (session_cache->remove(key_id)) {
 		dprintf ( D_SECURITY, 
 				  "DC_INVALIDATE_KEY: removed key id %s.\n", 
 				  key_id);
@@ -2612,6 +2706,7 @@ SecMan::SecMan()
 	if ( NULL == m_ipverify ) {
 		m_ipverify = new IpVerify( );
 	}
+	m_cached_auth_level = (DCpermission)-1; // intentionally invalid
 	sec_man_ref_count++;
 }
 
@@ -2652,7 +2747,7 @@ SecMan::Verify(DCpermission perm, const condor_sockaddr& addr, const char * fqu,
 
 
 bool
-SecMan::sec_copy_attribute( ClassAd &dest, ClassAd &source, const char* attr ) {
+SecMan::sec_copy_attribute( classad::ClassAd &dest, const ClassAd &source, const char* attr ) {
 	ExprTree *e = source.LookupExpr(attr);
 	if (e) {
 		ExprTree *cp = e->Copy();
@@ -2664,7 +2759,7 @@ SecMan::sec_copy_attribute( ClassAd &dest, ClassAd &source, const char* attr ) {
 }
 
 bool
-SecMan::sec_copy_attribute( ClassAd &dest, const char *to_attr, ClassAd &source, const char *from_attr ) {
+SecMan::sec_copy_attribute( ClassAd &dest, const char *to_attr, const ClassAd &source, const char *from_attr ) {
 	ExprTree *e = source.LookupExpr(from_attr);
 	if (!e) {
 		return false;
@@ -2678,16 +2773,16 @@ SecMan::sec_copy_attribute( ClassAd &dest, const char *to_attr, ClassAd &source,
 
 void
 SecMan::invalidateAllCache() {
-	session_cache.clear();
+	session_cache->clear();
 
 	command_map.clear();
 }
 
 void 
-SecMan :: invalidateExpiredCache()
+SecMan::invalidateOneExpiredCache(KeyCache *cache)
 {
     // Go through all cache and invalide the ones that are expired
-    StringList * list = session_cache.getExpiredKeys();
+    StringList * list = cache->getExpiredKeys();
 
     // The current session cache, command map does not allow
     // easy random access based on host direcly. Therefore,
@@ -2703,6 +2798,22 @@ SecMan :: invalidateExpiredCache()
     delete list;
 }
 
+void
+SecMan::invalidateExpiredCache()
+{
+	invalidateOneExpiredCache(&m_default_session_cache);
+	if (!m_tagged_session_cache) {return;}
+	std::map<std::string,KeyCache*>::iterator session_cache_iter;
+	for (session_cache_iter = m_tagged_session_cache->begin();
+		session_cache_iter != m_tagged_session_cache->end();
+		session_cache_iter++)
+	{
+		if (session_cache_iter->second) {
+			invalidateOneExpiredCache(session_cache_iter->second);
+		}
+	}
+}
+
 /*
 
 			// a failure here signals that the cache may be invalid.
@@ -2971,25 +3082,25 @@ SecMan::CreateNonNegotiatedSecuritySession(DCpermission auth_level, char const *
 
 	KeyCacheEntry key(sesid,peer_sinful ? &peer_addr : NULL,keyinfo,&policy,expiration_time,0);
 
-	if( !session_cache.insert(key) ) {
+	if( !session_cache->insert(key) ) {
 		KeyCacheEntry *existing = NULL;
 		bool fixed = false;
-		if( !session_cache.lookup(sesid,existing) ) {
+		if( !session_cache->lookup(sesid,existing) ) {
 			existing = NULL;
 		}
 		if( existing ) {
 			if( !LookupNonExpiredSession(sesid,existing) ) {
 					// the existing session must have expired, so try again
 				existing = NULL;
-				if( session_cache.insert(key) ) {
+				if( session_cache->insert(key) ) {
 					fixed = true;
 				}
 			}
 			else if( existing && existing->getLingerFlag() ) {
 				dprintf(D_ALWAYS,"SECMAN: removing lingering non-negotiated security session %s because it conflicts with new request\n",sesid);
-				session_cache.expire(existing);
+				session_cache->expire(existing);
 				existing = NULL;
-				if( session_cache.insert(key) ) {
+				if( session_cache->insert(key) ) {
 					fixed = true;
 				}
 			}
@@ -3024,7 +3135,12 @@ SecMan::CreateNonNegotiatedSecuritySession(DCpermission auth_level, char const *
 	coms.rewind();
 	while ( (p = coms.next()) ) {
 		MyString keybuf;
-		keybuf.formatstr ("{%s,<%s>}", peer_sinful, p);
+		const std::string &tag = SecMan::getTag();
+		if (tag.size()) {
+			keybuf.formatstr ("{%s,%s,<%s>}", tag.c_str(), peer_sinful, p);
+		} else {
+			keybuf.formatstr ("{%s,<%s>}", peer_sinful, p);
+		}
 
 		// NOTE: HashTable returns ZERO on SUCCESS!!!
 		if (command_map.insert(keybuf, sesid) == 0) {
@@ -3098,12 +3214,29 @@ SecMan::ImportSecSessionInfo(char const *session_info,ClassAd &policy) {
 	return true;
 }
 
+bool
+SecMan::getSessionPolicy(const char *session_id, classad::ClassAd &policy_ad)
+{
+	KeyCacheEntry *session_key = NULL;
+	if (!session_cache->lookup(session_id, session_key)) {return false;}
+	ClassAd *policy = session_key->policy();
+	if (!policy) {return false;}
+
+	sec_copy_attribute(policy_ad, *policy, ATTR_X509_USER_PROXY_SUBJECT);
+	sec_copy_attribute(policy_ad, *policy, ATTR_X509_USER_PROXY_EXPIRATION);
+	sec_copy_attribute(policy_ad, *policy, ATTR_X509_USER_PROXY_EMAIL);
+	sec_copy_attribute(policy_ad, *policy, ATTR_X509_USER_PROXY_VONAME);
+	sec_copy_attribute(policy_ad, *policy, ATTR_X509_USER_PROXY_FIRST_FQAN);
+	sec_copy_attribute(policy_ad, *policy, ATTR_X509_USER_PROXY_FQAN);
+	return true;
+}
+
 bool
 SecMan::ExportSecSessionInfo(char const *session_id,MyString &session_info) {
 	ASSERT( session_id );
 
 	KeyCacheEntry *session_key = NULL;
-	if(!session_cache.lookup(session_id,session_key)) {
+	if(!session_cache->lookup(session_id,session_key)) {
 		dprintf(D_ALWAYS,"SECMAN: ExportSecSessionInfo failed to find "
 				"session %s\n",session_id);
 		return false;
@@ -3149,7 +3282,7 @@ SecMan::SetSessionExpiration(char const *session_id,time_t expiration_time) {
 	ASSERT( session_id );
 
 	KeyCacheEntry *session_key = NULL;
-	if(!session_cache.lookup(session_id,session_key)) {
+	if(!session_cache->lookup(session_id,session_key)) {
 		dprintf(D_ALWAYS,"SECMAN: SetSessionExpiration failed to find "
 				"session %s\n",session_id);
 		return false;
@@ -3166,7 +3299,7 @@ SecMan::SetSessionLingerFlag(char const *session_id) {
 	ASSERT( session_id );
 
 	KeyCacheEntry *session_key = NULL;
-	if(!session_cache.lookup(session_id,session_key)) {
+	if(!session_cache->lookup(session_id,session_key)) {
 		dprintf(D_ALWAYS,"SECMAN: SetSessionLingerFlag failed to find "
 				"session %s\n",session_id);
 		return false;
