@@ -55,6 +55,7 @@ ReliSock::init()
 	_special_state = relisock_none;
 	is_client = 0;
 	hostAddr = NULL;
+	statsBuf = NULL;
 	snd_msg.reset();
 	rcv_msg.reset();
 	rcv_msg.init_parent(this);
@@ -97,6 +98,10 @@ ReliSock::~ReliSock()
 		free( hostAddr );
 		hostAddr = NULL;
 	}
+	if (statsBuf) {
+		free(statsBuf);
+		statsBuf = NULL;
+	}
 	if( m_target_shared_port_id ) {
 		free( m_target_shared_port_id );
 		m_target_shared_port_id = NULL;
@@ -1004,6 +1009,19 @@ ReliSock::attach_to_file_desc( int fd )
 
 	_sock = fd;
 	_state = sock_connect;
+
+	int accepting = 0;
+	socklen_t l = sizeof(accepting);
+
+	if ((getsockopt(fd, SOL_SOCKET, SO_ACCEPTCONN, &accepting, &l) == 0) && (l == sizeof(accepting)))
+	{
+		if (accepting == 1)
+		{
+			_state = sock_special;
+			_special_state = relisock_listen;
+		}
+	}
+
 	timeout(0);	// make certain in blocking mode
 	return TRUE;
 }
@@ -1023,8 +1041,8 @@ ReliSock::serialize() const
 	// first, get the state from our parent class
 	char * parent_state = Sock::serialize();
     // now concatenate our state
-	char * outbuf = new char[50];
-    memset(outbuf, 0, 50);
+	char * outbuf = new char[150];
+    memset(outbuf, 0, 150);
 	sprintf(outbuf,"%d*%s*",_special_state,_who.to_sinful().Value());
 	strcat(parent_state,outbuf);
 
@@ -1044,12 +1062,12 @@ ReliSock::serialize() const
 	return( parent_state );
 }
 
-char * 
-ReliSock::serialize(char *buf)
+const char *
+ReliSock::serialize(const char *buf)
 {
 	char * sinful_string = NULL;
 	char fqu[256];
-	char *ptmp, * ptr = NULL;
+	const char *ptmp, * ptr = NULL;
 	int len = 0;
 
     ASSERT(buf);
@@ -1287,9 +1305,10 @@ ReliSock::connect_socketpair( ReliSock & sock, char const * asIfConnectingTo ) {
 bool
 ReliSock::connect_socketpair( ReliSock & sock ) {
 	condor_protocol proto = CP_IPV4;
-	bool ipV4Enabled = param_boolean( "ENABLE_IPV4", true );
-	bool ipV6Enabled = param_boolean( "ENABLE_IPV6", false );
-	if( ipV6Enabled && (! ipV4Enabled) ) {
+	bool ipV4Allowed = ! param_false( "ENABLE_IPV4" );
+	bool ipV6Allowed = ! param_false( "ENABLE_IPV6" );
+
+	if( ipV6Allowed && (! ipV4Allowed) ) {
 		proto = CP_IPV6;
 	}
 
@@ -1316,7 +1335,7 @@ ReliSock::exit_reverse_connecting_state(ReliSock *sock)
 	_state = sock_virgin;
 
 	if( sock ) {
-		int assign_rc = assignSocket(sock->get_file_desc());
+		int assign_rc = assignCCBSocket(sock->get_file_desc());
 		ASSERT( assign_rc );
 		isClient(true);
 		if( sock->_state == sock_connect ) {
@@ -1331,6 +1350,77 @@ ReliSock::exit_reverse_connecting_state(ReliSock *sock)
 	m_ccb_client = NULL;
 }
 
+/*
+ * Use the Linux-specific TCP_INFO getsockopt to return a human readable
+ * string describing low-level tcp statistics
+ *
+ */
+char *
+ReliSock::get_statistics() {
+#ifndef LINUX
+	return 0;
+#else
+			// 20 entries, each with 16 bytes of text and up to 10 bytes numeric
+	int maxSize = 20 * (16 + 10);
+	if (statsBuf == 0) {
+		statsBuf = (char *)malloc(maxSize + 1); // dtor frees this
+		statsBuf[0] = '\0';
+	}
+
+	struct tcp_info ti;
+	socklen_t tcp_info_len = sizeof(struct tcp_info);
+
+	int ret = getsockopt(this->get_file_desc(), SOL_TCP, TCP_INFO, &ti, &tcp_info_len);
+	if (ret != 0) {
+			// maybe the sock was closed?  Return the last value, if any.
+		return statsBuf;
+	}
+
+	snprintf(statsBuf, maxSize,
+		"rto: %d "
+		"ato: %d "
+		"snd_mss: %d "
+		"rcv_mss: %d "
+		"unacked: %d "
+		"sacked: %d "
+		"lost: %d "
+		"retrans: %d "
+		"fackets: %d "
+		"pmtu: %d "
+		"rcv_ssthresh: %d "
+		"rtt: %d "
+		"snd_ssthresh: %d "
+		"snd_cwnd: %d "
+		"advmss: %d "
+		"reordering: %d "
+		"rcv_rtt: %d "
+		"rcv_space: %d "
+		"total_retrans: %d ",
+		
+		ti.tcpi_rto,
+		ti.tcpi_ato,
+		ti.tcpi_snd_mss,
+		ti.tcpi_rcv_mss,
+		ti.tcpi_unacked,
+		ti.tcpi_sacked,
+		ti.tcpi_lost,
+		ti.tcpi_retrans,
+		ti.tcpi_fackets,
+		ti.tcpi_pmtu,
+		ti.tcpi_rcv_ssthresh,
+		ti.tcpi_rtt,
+		ti.tcpi_snd_ssthresh,
+		ti.tcpi_snd_cwnd,
+		ti.tcpi_advmss,
+		ti.tcpi_reordering,
+		ti.tcpi_rcv_rtt,
+		ti.tcpi_rcv_space,
+		ti.tcpi_total_retrans);
+		
+	return statsBuf;
+#endif
+}
+
 void
 ReliSock::setTargetSharedPortID( char const *id )
 {
