@@ -1098,7 +1098,11 @@ SharedPortEndpoint::ReceiveSocket( ReliSock *named_sock, ReliSock *return_remote
 	if( !remote_sock ) {
 		remote_sock = new ReliSock();
 	}
-	remote_sock->assignSocket( passed_fd );
+	// Don't EXCEPT if the connection we just accepted isn't of the same
+	// protocol as the connection we were expecting, which can happen
+	// a CCB client calls back a daemon.  See the comment for this
+	// function in condor_includes/sock.h.
+	remote_sock->assignCCBSocket( passed_fd );
 	remote_sock->enter_connected_state();
 	remote_sock->isClient(false);
 
@@ -1132,7 +1136,8 @@ SharedPortEndpoint::ReceiveSocket( ReliSock *named_sock, ReliSock *return_remote
 bool
 SharedPortEndpoint::serialize(MyString &inherit_buf,int &inherit_fd)
 {
-	inherit_buf.formatstr_cat("%s*",m_full_name.Value());
+	inherit_buf.serialize_string(m_full_name.c_str());
+	inherit_buf.serialize_sep("*");
 #ifdef WIN32
 	/*
 	Serializing requires acquiring the handles of the respective pipes and seeding them into
@@ -1149,7 +1154,8 @@ SharedPortEndpoint::serialize(MyString &inherit_buf,int &inherit_fd)
 		dprintf(D_ALWAYS, "SharedPortEndpoint: Failed to duplicate named pipe for inheritance.\n");
 		return false;
 	}
-	inherit_buf.formatstr_cat("%d", inheritable_to_child);
+	inherit_buf.serialize_int((LONG_PTR)inheritable_to_child);
+	inherit_buf.serialize_sep("*");
 #else
 	inherit_fd = m_listener_sock.get_file_desc();
 	ASSERT( inherit_fd != -1 );
@@ -1163,29 +1169,27 @@ SharedPortEndpoint::serialize(MyString &inherit_buf,int &inherit_fd)
 	return true;
 }
 
-char *
-SharedPortEndpoint::deserialize(char *inherit_buf)
+const char *
+SharedPortEndpoint::deserialize(const char *inherit_buf)
 {
-	char *ptr;
-	ptr = strchr(inherit_buf,'*');
-	ASSERT( ptr );
-	m_full_name.formatstr("%.*s",(int)(ptr-inherit_buf),inherit_buf);
-	inherit_buf = ptr+1;
+	YourStringDeserializer in(inherit_buf);
+	if ( ! in.deserialize_string(m_full_name, "*") || ! in.deserialize_sep("*") ) {
+		EXCEPT("Failed to parse serialized shared-port information at offset %d: '%s'", (int)in.offset(), inherit_buf);
+	}
 
-	m_local_id = condor_basename( m_full_name.Value() );
-	char *socket_dir = condor_dirname( m_full_name.Value() );
-	m_socket_dir = socket_dir;
-	free( socket_dir );
+	m_local_id = condor_basename(m_full_name.c_str());
+	auto_free_ptr socket_dir(condor_dirname(m_full_name.c_str()));
+	m_socket_dir = socket_dir.ptr();
 #ifdef WIN32
 	/*
 	Deserializing requires getting the handles out of the buffer and getting the pid pipe name
 	stored.  Registering the pipe is handled by StartListener().
 	*/
-	sscanf_s(inherit_buf, "%d", (int*)&pipe_end);
-
-	//m_pipe_out = daemonCore->Inherit_Pipe_Handle(out_pipe, false, true, true, 4096);
+	in.deserialize_int((LONG_PTR*)&pipe_end);
+	in.deserialize_sep("*"); // note: terminator is missing from HTCondor prior to 8.5.7 so it is optional here...
+	inherit_buf = in.next_pos();
 #else
-	inherit_buf = m_listener_sock.serialize(inherit_buf);
+	inherit_buf = m_listener_sock.serialize(in.next_pos());
 #endif
 	m_listening = true;
 
@@ -1218,13 +1222,7 @@ SharedPortEndpoint::UseSharedPort(MyString *why_not,bool already_open)
 		// to decide whether to create a shared port for our child.
 		// In the latter case, other methods are used to determine
 		// that a shared port should not be used.
-		// The HAD and REPLICATION daemons can't properly handle a
-		// shared port address for themselves or their peers.
-
-	bool never_use_shared_port =
-		get_mySubSystem()->isType(SUBSYSTEM_TYPE_SHARED_PORT) ||
-		get_mySubSystem()->nameMatch("HAD") ||
-		get_mySubSystem()->nameMatch("REPLICATION");
+	bool never_use_shared_port = get_mySubSystem()->isType(SUBSYSTEM_TYPE_SHARED_PORT);
 
 	if( never_use_shared_port ) {
 		if( why_not ) {
@@ -1233,7 +1231,14 @@ SharedPortEndpoint::UseSharedPort(MyString *why_not,bool already_open)
 		return false;
 	}
 
-	if( !param_boolean("USE_SHARED_PORT",false) ) {
+	std::string uspParameterName;
+	const char * subsystem = get_mySubSystem()->getName();
+	formatstr( uspParameterName, "%s_USE_SHARED_PORT", subsystem );
+	if( param( uspParameterName.c_str() ) == NULL ) {
+		uspParameterName = "USE_SHARED_PORT";
+	}
+
+	if( !param_boolean(uspParameterName.c_str(),false) ) {
 		if( why_not ) {
 			*why_not = "USE_SHARED_PORT=false";
 		}
