@@ -39,6 +39,7 @@
 #include "condor_sockfunc.h"
 #include "condor_config.h"
 #include "condor_sinful.h"
+#include <classad/classad.h>
 
 #if defined(WIN32)
 // <winsock2.h> already included...
@@ -67,7 +68,7 @@ void dprintf ( int flags, Sock & sock, const char *fmt, ... )
 
 unsigned int Sock::m_nextUniqueId = 1;
 
-Sock::Sock() : Stream() {
+Sock::Sock() : Stream(), _policy_ad(NULL) {
 	_sock = INVALID_SOCKET;
 	_state = sock_virgin;
 	_timeout = 0;
@@ -78,6 +79,7 @@ Sock::Sock() : Stream() {
 	_auth_methods = NULL;
 	_auth_name = NULL;
 	_crypto_method = NULL;
+	_policy_ad = NULL;
 	_tried_authentication = false;
 	ignore_connect_timeout = FALSE;		// Used by the HA Daemon
 	connect_state.connect_failed = false;
@@ -102,7 +104,7 @@ Sock::Sock() : Stream() {
     addr_changed();
 }
 
-Sock::Sock(const Sock & orig) : Stream() {
+Sock::Sock(const Sock & orig) : Stream(), _policy_ad(NULL) {
 
 	// initialize everything in the new sock
 	_sock = INVALID_SOCKET;
@@ -193,6 +195,7 @@ Sock::~Sock()
 		_auth_methods = NULL;
 	}
 	free(_auth_name);
+	delete _policy_ad;
 	if (_crypto_method) {
 		free(_crypto_method);
 		_crypto_method = NULL;
@@ -286,6 +289,24 @@ static SockInitializer _SockInitializer;
 */
 
 
+void
+Sock::setPolicyAd(const classad::ClassAd &ad)
+{
+	if (!_policy_ad) {_policy_ad = new classad::ClassAd();}
+	if (_policy_ad) {_policy_ad->CopyFrom(ad);}
+}
+
+
+void
+Sock::getPolicyAd(classad::ClassAd &ad) const
+{
+	if (_policy_ad)
+	{
+		ad.Update(*_policy_ad);
+	}
+}
+
+
 int Sock::getportbyserv(
 	char	*s
 	)
@@ -334,6 +355,15 @@ int Sock::assign(
 	int socket_fd = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, 
 					  FROM_PROTOCOL_INFO, pProtoInfo, 0, 0);
 
+	// This only works because the only call to this assign() is
+	// in SharedPortEndpoint::DoListenerAccept().  For how it works,
+	// see the comment in assignCCBSocket().  If we change the Linux code
+	// in SharedPortEndpoint::ReceiveSocket() to confirm that the
+	// connection being handed off came from CCB (before calling
+	// assignCCBSocket() instead of assignSocket()), we need to change
+	// this when we do the same for the equivalent Windows code in
+	// SharedPortEndpoint::DoListenerAccept().
+	_who.clear();
 	return assignSocket( socket_fd );
 }
 #endif
@@ -383,7 +413,7 @@ int Sock::set_inheritable( int flag )
 	{
 		int err = WSAGetLastError();
 		dprintf(D_NETWORK,
-			"Failed SIO_BASE_HANDLE in set_inheritable on sock %d to %d (err=%d, %s)",
+			"Failed SIO_BASE_HANDLE in set_inheritable on sock %d to %d (err=%d, %s)\n",
 			_sock, flag, err, GetLastErrorString(err) );
 		// Well, we failed to get the "real" handle for whatever reason.
 		// We may as well try to move forward with the original _sock handle.
@@ -401,7 +431,7 @@ int Sock::set_inheritable( int flag )
 		{
 			int err = WSAGetLastError();
 			dprintf(D_NETWORK,
-				"Failed to set_inheritable on sock %d to %d (err=%d, %s)",
+				"Failed to set_inheritable on sock %d to %d (err=%d, %s)\n",
 				_sock, flag, err, GetLastErrorString(err) );
 		} else {
 			sethandle_worked = true;
@@ -497,6 +527,27 @@ int Sock::assignInvalidSocket( condor_protocol proto ) {
 	return assignSocket( proto, INVALID_SOCKET );
 }
 
+int Sock::assignCCBSocket( SOCKET s ) {
+	ABEND( s != INVALID_SOCKET );
+
+	if( IsDebugLevel( D_NETWORK ) && _who.is_valid() ) {
+		condor_sockaddr sockAddr;
+		ABEND( condor_getsockname( s, sockAddr ) == 0 );
+		condor_protocol sockProto = sockAddr.get_protocol();
+		condor_protocol objectProto = _who.get_protocol();
+		if( objectProto != sockProto ) {
+			dprintf( D_NETWORK, "assignCCBSocket(): reverse connection made on different protocol than the request.\n"  );
+		}
+	}
+
+	// This assignSocket() is the only one that checks to see if the Sock
+	// protocol and the socket protocol match, and the call to assignSocket()
+	// clear()s _who if s is not invalid, so we can just clear _who right
+	// here and avoid both the spurious ABEND and duplicating code.
+	_who.clear();
+	return assignSocket( s );
+}
+
 int Sock::assignSocket( SOCKET sockd ) {
 	ABEND( sockd != INVALID_SOCKET );
 
@@ -574,10 +625,14 @@ int Sock::assignSocket( condor_protocol proto, SOCKET sockd ) {
 	//
 
 	int af_type = 0;
-	switch(proto) {
-		case CP_IPV4: af_type = AF_INET; break;
-		case CP_IPV6: af_type = AF_INET6; break;
-		default: ASSERT(false);
+	if (_who.is_valid()) {
+		af_type = _who.get_aftype();
+	} else {
+		switch(proto) {
+			case CP_IPV4: af_type = AF_INET; break;
+			case CP_IPV6: af_type = AF_INET6; break;
+			default: ASSERT(false);
+		}
 	}
 
 	int my_type = 0;
@@ -652,7 +707,7 @@ int Sock::assignSocket( condor_protocol proto, SOCKET sockd ) {
 
 
 int
-Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port, bool outbound)
+Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port)
 {
 	bool bind_all = (bool)_condor_bind_all_interfaces();
 
@@ -660,7 +715,7 @@ Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port,
 	// int pid = (int) getpid();
 
 	// lets use time as the starting point instead...
-	long randomish_val;
+	unsigned long randomish_val;
 #ifndef WIN32
 	struct timeval curTime;
 	(void) gettimeofday(&curTime, NULL);
@@ -668,7 +723,7 @@ Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port,
 #else
 	LARGE_INTEGER li;
 	::QueryPerformanceCounter(&li);
-	randomish_val = (int)li.LowPart;
+	randomish_val = li.LowPart % 1000000; // force into range of usec to avoid overflow when we multiply
 #endif
 
 	int range = high_port - low_port + 1;
@@ -702,7 +757,7 @@ Sock::bindWithin(condor_protocol proto, const int low_port, const int high_port,
 		}
 #endif
 
-		bind_return_val = _bind_helper(_sock, addr, outbound, false);
+		bind_return_val = condor_bind(_sock, addr);
 
         addr_changed();
 
@@ -760,6 +815,12 @@ int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
 		return FALSE;
 	}
 
+	static bool reuse = param_boolean("ALWAYS_REUSEADDR", true);
+	if (reuse) {	
+		int one = 1;
+    	this->setsockopt(SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one));
+	}
+
 	// If 'port' equals 0 and if we have 'LOWPORT' and 'HIGHPORT' defined
 	// in the config file for security, we will bind this Sock to one of
 	// the port within the range defined by these variables rather than
@@ -792,13 +853,19 @@ int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
 	int lowPort, highPort;
 	if ( port == 0 && !loopback && get_port_range((int)outbound, &lowPort, &highPort) == TRUE ) {
 			// Bind in a specific port range.
-		if ( bindWithin(proto, lowPort, highPort, outbound) != TRUE ) {
+		if ( bindWithin(proto, lowPort, highPort) != TRUE ) {
 			return FALSE;
 		}
 	} else {
 			// Bind to a dynamic port.
-
-		addr.set_protocol(proto);
+		if (_who.is_valid()) {
+			if (_who.is_ipv6()) { addr.set_ipv6(); }
+			else { addr.set_ipv4(); }
+		}
+		else
+		{
+			addr.set_protocol(proto);
+		}
 		if( loopback ) {
 			addr.set_loopback();
 		} else if( (bool)_condor_bind_all_interfaces() ) {
@@ -821,7 +888,7 @@ int Sock::bind(condor_protocol proto, bool outbound, int port, bool loopback)
 		}
 #endif
 
-		bind_return_value = _bind_helper(_sock, addr, outbound, loopback);
+		bind_return_value = condor_bind(_sock, addr);
 
         addr_changed();
 
@@ -1110,7 +1177,32 @@ bool Sock::guess_address_string(char const* host, int port, condor_sockaddr& add
 	return true;
 }
 
+bool routingParametersInitialized = false;
+bool ignoreTargetProtocolPreference = false;
+bool preferOutboundIPv4 = false;
+bool acceptIPv4 = false;
+bool acceptIPv6 = false;
+
 bool Sock::chooseAddrFromAddrs( char const * host, std::string & addr ) {
+	if(! routingParametersInitialized) {
+		ignoreTargetProtocolPreference = param_boolean( "IGNORE_TARGET_PROTOCOL_PREFERENCE", false );
+		preferOutboundIPv4 = param_boolean( "PREFER_OUTBOUND_IPV4", false );
+
+		acceptIPv4 = ! param_false( "ENABLE_IPV4" );
+		if( acceptIPv4 && ! param_defined( "IPV4_ADDRESS" ) ) {
+			acceptIPv4 = false;
+		}
+
+		acceptIPv6 = ! param_false( "ENABLE_IPV6" );
+		if( acceptIPv6 && ! param_defined( "IPV6_ADDRESS" ) ) {
+			acceptIPv6 = false;
+		}
+
+		if( (!acceptIPv4) && (!acceptIPv6) ) {
+			EXCEPT( "Unwilling or unable to try IPv4 or IPv6.  Check the settings ENABLE_IPV4, ENABLE_IPV6, and NETWORK_INTERFACE.\n" );
+		}
+	}
+
 	//
 	// If host is a Sinful string and contains an addrs parameter,
 	// choose one of the listed addresses and rewrite host to match.
@@ -1147,6 +1239,15 @@ bool Sock::chooseAddrFromAddrs( char const * host, std::string & addr ) {
 	for( unsigned i = 0; i < v->size(); ++i ) {
 		condor_sockaddr c = (*v)[i];
 		int d = -1 * c.desirability();
+		if( ignoreTargetProtocolPreference ) {
+			// This would work with d *= 2 and d -= 1.  10 and 1 may be
+			// more obvious when looking at logs, but there's no point
+			// changing it unless we need more room in the desirability
+			// spae for some reason.
+			d *= 100;
+			if( preferOutboundIPv4 && c.is_ipv4() ) { d -= 10; }
+			if( (!preferOutboundIPv4) && (!c.is_ipv4()) ) { d -= 10; }
+		}
 		sortedByDesire.insert(std::make_pair( d, c ));
 		dprintf( D_HOSTNAME, "\t%d\t%s\n", d, c.to_ip_and_port_string().c_str() );
 	}
@@ -1162,9 +1263,16 @@ bool Sock::chooseAddrFromAddrs( char const * host, std::string & addr ) {
 		// document that ENABLE_IPV6 should be false unless you have a
 		// public IPv6 address (or one which everyone in your pool can
 		// otherwise reach).
+		//
+		// That was a good idea, but in practice we want ENABLE_IPV6 to
+		// have a smarter option, in which case we don't know if we
+		// should consider IPv6 until we've examined the interfaces to
+		// determine if IPv6 exists at all.  With PREFER_IPV4, this only
+		// really matters when trying bind command sockets, but we may as
+		// well get it right here.
 		dprintf( D_HOSTNAME, "Considering address candidate %s.\n", candidate.to_ip_and_port_string().c_str() );
-		if(( candidate.is_ipv4() && param_boolean( "ENABLE_IPV4", true ) ) ||
-			( candidate.is_ipv6() && param_boolean( "ENABLE_IPV6", false ) )) {
+		if(( candidate.is_ipv4() && acceptIPv4 ) ||
+			( candidate.is_ipv6() && acceptIPv6 )) {
 			dprintf( D_HOSTNAME, "Found compatible candidate %s.\n", candidate.to_ip_and_port_string().c_str() );
 			foundAddress = true;
 			break;
@@ -2021,10 +2129,10 @@ char * Sock::serializeMdInfo() const
 	return( outbuf );
 }
 
-char * Sock::serializeCryptoInfo(char * buf)
+const char * Sock::serializeCryptoInfo(const char * buf)
 {
 	unsigned char * kserial = NULL;
-    char * ptmp = buf;
+    const char * ptmp = buf;
     int    len = 0, encoded_len = 0;
     int protocol = CONDOR_NO_PROTOCOL;
 
@@ -2084,10 +2192,10 @@ char * Sock::serializeCryptoInfo(char * buf)
 	return ptmp;
 }
 
-char * Sock::serializeMdInfo(char * buf)
+const char * Sock::serializeMdInfo(const char * buf)
 {
 	unsigned char * kmd = NULL;
-    char * ptmp = buf;
+    const char * ptmp = buf;
     int    len = 0, encoded_len = 0;
 
     // kmd may be a problem since reli_sock also has stuff after
@@ -2133,6 +2241,7 @@ char * Sock::serializeMdInfo(char * buf)
 	return ptmp;
 }
 
+
 char * Sock::serialize() const
 {
 	// here we want to save our state into a buffer
@@ -2153,14 +2262,25 @@ char * Sock::serialize() const
 		}
 	}
 
-	char * outbuf = new char[500];
-    if (outbuf) {
-        memset(outbuf, 0, 500);
-        sprintf(outbuf,"%u*%d*%d*%d*%lu*%lu*%s*%s*",_sock,_state,_timeout,triedAuthentication(),(unsigned long)fqu_len,(unsigned long)verstring_len,_fqu ? _fqu : "",verstring ? verstring : "");
-    }
-    else {
-        dprintf(D_ALWAYS, "Out of memory!\n");
-    }
+	MyString out;
+	char * outbuf = NULL;
+	// must overallocate because other people will append to this buffer!!!
+	out.reserve(1000); //(int)(verstring_len+fqu_len+8+(6*10)));
+	if (out.serialize_int(_sock)                 && out.serialize_sep("*") &&
+		out.serialize_int((int)_state)           && out.serialize_sep("*") &&
+		out.serialize_int(_timeout)              && out.serialize_sep("*") &&
+		out.serialize_int(triedAuthentication()) && out.serialize_sep("*") &&
+		out.serialize_int(fqu_len)               && out.serialize_sep("*") &&
+		out.serialize_int(verstring_len)         && out.serialize_sep("*") &&
+		out.serialize_string(_fqu)               && out.serialize_sep("*") &&
+		out.serialize_string(verstring)          && out.serialize_sep("*"))
+	{
+		outbuf = out.detach_buffer();
+	}
+	else 
+	{
+		dprintf(D_ALWAYS, "Sock::serialize failed - Out of memory?\n");
+	}
 	free( verstring );
 	return( outbuf );
 }
@@ -2176,58 +2296,48 @@ Sock::close_serialized_socket(char const *buf)
 	::close(passed_sock);
 }
 
-char * Sock::serialize(char *buf)
+
+const char * Sock::serialize(const char *buf)
 {
-	int i;
 	SOCKET passed_sock;
 	size_t fqulen = 0;
 	size_t verstring_len = 0;
-	int pos;
 	int tried_authentication = 0;
 
 	ASSERT(buf);
 
 	// here we want to restore our state from the incoming buffer
-	i = sscanf(buf,"%u*%d*%d*%d*%lu*%lu*%n",&passed_sock,(int*)&_state,&_timeout,&tried_authentication,(unsigned long *)&fqulen,(unsigned long *)&verstring_len,&pos);
-	if (i!=6) {
-		EXCEPT("Failed to parse serialized socket information (%d,%d): '%s'",i,pos,buf);
+	YourStringDeserializer in(buf);
+	if ( ! in.deserialize_int(&passed_sock)          || ! in.deserialize_sep("*") ||
+		 ! in.deserialize_int((int*)&_state)         || ! in.deserialize_sep("*") || // int* cast because std::numeric_limits<sock_state>::max() returns 0
+		 ! in.deserialize_int(&_timeout)             || ! in.deserialize_sep("*") ||
+		 ! in.deserialize_int(&tried_authentication) || ! in.deserialize_sep("*") ||
+		 ! in.deserialize_int(&fqulen)               || ! in.deserialize_sep("*") ||
+		 ! in.deserialize_int(&verstring_len)        || ! in.deserialize_sep("*")
+		 )
+	{
+		EXCEPT("Failed to parse serialized socket information at offset %d: '%s'",(int)in.offset(),buf);
 	}
-	buf += pos;
 
 	setTriedAuthentication(tried_authentication);
 
-	char *fqubuf = (char *)malloc(fqulen+1);
-	ASSERT(fqubuf);
-	memset(fqubuf,0,fqulen+1);
-	strncpy(fqubuf,buf,fqulen);
-	setFullyQualifiedUser(fqubuf);
-	free(fqubuf);
-	buf += fqulen;
-	if( *buf != '*' ) {
-		EXCEPT("Failed to parse serialized socket fqu (%lu): '%s'",(unsigned long)fqulen,buf);
+	MyString str;
+	if ( ! in.deserialize_string(str, "*") || ! in.deserialize_sep("*")) {
+		EXCEPT("Failed to parse serialized socket FullyQualifiedUser at offset %d: '%s'",(int)in.offset(),buf);
 	}
-	buf++;
+	setFullyQualifiedUser(str.c_str());
 
-	char *verstring = (char *)malloc(verstring_len+1);
-	ASSERT(verstring);
-	memset(verstring,0,verstring_len+1);
-	strncpy(verstring,buf,verstring_len);
-	verstring[verstring_len] = 0;
-	if( verstring_len ) {
-			// daemoncore does not like spaces in our serialized string
-		char *s;
-		while( (s=strchr(verstring,'_')) ) {
-			*s = ' ';
-		}
-		CondorVersionInfo peer_version(verstring);
+	str.clear();
+	if ( ! in.deserialize_string(str, "*") || ! in.deserialize_sep("*")) {
+		EXCEPT("Failed to parse serialized peer version string at offset %d: '%s'",(int)in.offset(),buf);
+	}
+	if ( ! str.empty()) {
+		// daemoncore does not like spaces in our serialized string so they were changed to _
+		// we need to put them back now.
+		str.replaceString("_"," ");
+		CondorVersionInfo peer_version(str.c_str());
 		set_peer_version( &peer_version );
 	}
-	free( verstring );
-	buf += verstring_len;
-	if( *buf != '*' ) {
-		EXCEPT("Failed to parse serialized peer version string (%lu): '%s'",(unsigned long)verstring_len,buf);
-	}
-	buf++;
 
 	// replace _sock with the one from the buffer _only_ if _sock
 	// is currently invalid.  if it is not invalid, it has already
@@ -2262,7 +2372,8 @@ char * Sock::serialize(char *buf)
 	// setsockopt() and/or ioctl() is restored.
 	timeout_no_timeout_multiplier(_timeout);
 
-	return buf;
+	// return the point at which we stopped deserializing
+	return in.next_pos();
 }
 
 void
@@ -2787,18 +2898,6 @@ bool Sock :: is_encrypt()
 }
 
 
-int
-Sock::_bind_helper(int fd, const condor_sockaddr& addr, bool outbound, bool loopback)
-{
-	int rval;
-
-	if (outbound) {} // To remove unused variable warning
-	if (loopback) {} // To remove unused variable warning
-		//rval = ::bind(fd, (SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE)addr, len);
-	rval = condor_bind(fd, addr);
-	return rval;
-}
-
 void
 Sock::set_connect_addr(char const *addr)
 {
