@@ -274,6 +274,8 @@ class CondorQClassAdFileParseHelper : public compat_classad::ClassAdFileParseHel
  public:
 	virtual int PreParse(std::string & line, ClassAd & ad, FILE* file);
 	virtual int OnParseError(std::string & line, ClassAd & ad, FILE* file);
+	// return non-zero if new parser, o if old (line oriented) parser, non-zero is returned the above functions will never be called.
+	virtual int NewParser(ClassAd & /*ad*/, FILE* /*file*/, bool & detected_long, std::string & /*errmsg*/) { detected_long = false; return 0; }
 	std::string schedd_name;
 	std::string schedd_addr;
 };
@@ -511,12 +513,12 @@ bool yield_job(classad::ClassAd const &ad,const char * pool_name,
 	return true;
 }
 
-bool submit_job( ClassAd & src, const char * schedd_name, const char * pool_name, bool is_sandboxed, int * cluster_out /*= 0*/, int * proc_out /*= 0 */)
+bool submit_job( const std::string & owner, const std::string & domain, ClassAd & src, const char * schedd_name, const char * pool_name, bool is_sandboxed, int * cluster_out /*= 0*/, int * proc_out /*= 0 */)
 {
 	return true;
 }
 
-bool submit_job( classad::ClassAd & src, const char * schedd_name, const char * pool_name, bool is_sandboxed, int * cluster_out /*= 0*/, int * proc_out /*= 0 */)
+bool submit_job( const std::string & owner, const std::string & domain, classad::ClassAd & src, const char * schedd_name, const char * pool_name, bool is_sandboxed, int * cluster_out /*= 0*/, int * proc_out /*= 0 */)
 {
 	return true;
 }
@@ -561,7 +563,7 @@ bool push_classad_diff(classad::ClassAd & src, classad::ClassAd & dest, const ch
 	return true;
 }
 
-bool finalize_job(classad::ClassAd const &ad,int cluster, int proc, const char * schedd_name, const char * pool_name, bool is_sandboxed)
+bool finalize_job(const std::string &owner, const std::string &domain, classad::ClassAd const &ad,int cluster, int proc, const char * schedd_name, const char * pool_name, bool is_sandboxed)
 {
 	return true;
 }
@@ -611,7 +613,15 @@ bool WriteHoldEventToUserLog( classad::ClassAd const &ad )
 	return true;
 }
 
+bool WriteExecuteEventToUserLog( classad::ClassAd const &ad )
+{
+	return true;
+}
 
+bool WriteEvictEventToUserLog( classad::ClassAd const &ad )
+{
+	return true;
+}
 
 // The following is copied from gridmanager/basejob.C
 // TODO: put the code into a shared file.
