@@ -32,6 +32,7 @@
 #include "condor_network.h"
 #include "condor_adtypes.h"
 #include "condor_io.h"
+#include "directory.h"
 #include "exit.h"
 #include "string_list.h"
 #include "get_daemon_name.h"
@@ -54,6 +55,9 @@ extern int load_master_mgmt(void);
 #endif
 #endif
 
+#include "systemd_manager.h"
+#include <sstream>
+
 #ifdef WIN32
 
 #include "firewall.WINDOWS.h"
@@ -91,8 +95,10 @@ void	main_config();
 int	agent_starter(ReliSock *);
 int	handle_agent_fetch_log(ReliSock *);
 int	admin_command_handler(Service *, int, Stream *);
+int	ready_command_handler(Service *, int, Stream *);
 int	handle_subsys_command(int, Stream *);
 int     handle_shutdown_program( int cmd, Stream* stream );
+int     set_shutdown_program( const char * name );
 void	time_skip_handler(void * /*data*/, int delta);
 void	restart_everyone();
 
@@ -144,6 +150,83 @@ char	default_dc_daemon_list[] =
 // create an object of class daemons.
 class Daemons daemons;
 
+class SystemdNotifier : public Service {
+
+public:
+	SystemdNotifier() : m_watchdog_timer(-1) {}
+
+	void
+	config()
+	{
+		const condor_utils::SystemdManager & sd = condor_utils::SystemdManager::GetInstance();
+		int watchdog_secs = sd.GetWatchdogUsecs() / 1e6 / 3;
+		if (watchdog_secs <= 0) { watchdog_secs = 1; }
+		Timeslice ts;
+		ts.setDefaultInterval(watchdog_secs);
+		m_watchdog_timer = daemonCore->Register_Timer(ts, static_cast<TimerHandlercpp>(&SystemdNotifier::status_handler),
+				"systemd status updater", this);
+		if (m_watchdog_timer < 0)
+		{
+			dprintf(D_ALWAYS, "Failed to register systemd update timer.\n");
+		} else {
+			dprintf(D_FULLDEBUG, "Set systemd to be notified once every %d seconds.\n", watchdog_secs);
+		}
+	}
+
+	void status_handler()
+	{
+		char *name;
+		class daemon *daemon;
+
+		daemons.ordered_daemon_names.rewind();
+		std::stringstream ss;
+		ss << "Problems: ";
+		bool had_prior = false;
+		bool missing_daemons = false;
+		while( (name = daemons.ordered_daemon_names.next()) ) {
+			daemon = daemons.FindDaemon( name );
+			if (!daemon->pid)
+			{
+				if (had_prior) { ss << ", "; }
+				else { had_prior = true; }
+				missing_daemons = true;
+				time_t starttime = daemon->GetNextRestart();
+				if (starttime)
+				{
+					time_t secs_to_start = starttime-time(NULL);
+					if (secs_to_start > 0)
+					{ ss << name << "=RESTART in " << secs_to_start << "s"; }
+					else
+					{ ss << name << "=RESTARTNG"; }
+				}
+				else
+				{ ss << name << "=STOPPED"; }
+			}
+		}
+		std::string status = ss.str();
+
+		const char * format_string = "STATUS=%s\nWATCHDOG=1";
+		if (!missing_daemons)
+		{
+			format_string = "READY=1\nSTATUS=%s\nWATCHDOG=1";
+			status = "All daemons are responding";
+		}
+
+		const condor_utils::SystemdManager &sd = condor_utils::SystemdManager::GetInstance();
+		int result = sd.Notify(format_string, status.c_str());
+		if (result == 0)
+		{
+			dprintf(D_ALWAYS, "systemd watchdog notification support not available.\n");
+			daemonCore->Cancel_Timer(m_watchdog_timer);
+			m_watchdog_timer = -1;
+		}
+	}
+
+private:
+	int m_watchdog_timer;
+};
+SystemdNotifier g_systemd_notifier;
+
 // called at exit to deallocate stuff so that memory checking tools are
 // happy and don't think we leaked any of this...
 static void
@@ -163,14 +246,87 @@ cleanup_memory( void )
 	}
 }
 
+#ifdef WIN32
+// we can't use execl on Windows (read the docs), and anyway, we don't want to
+// if we are running as a serices because we MUST call our terminate() function to exit
+static void Win32RunShutdownProgram(char * shutdown_program)
+{
+	STARTUPINFO si;
+	ZeroMemory(&si, sizeof(si)); si.cb = sizeof(si);
+
+	HANDLE hLog = INVALID_HANDLE_VALUE;
+
+	// Open a log file for the shutdown programs stdout and stderr
+	std::string shutdown_log;
+	if (param(shutdown_log, "MASTER_SHUTDOWN_LOG") &&  ! shutdown_log.empty()) {
+		SECURITY_ATTRIBUTES sa;
+		ZeroMemory(&sa, sizeof(sa));
+		sa.bInheritHandle = TRUE;
+
+		hLog = CreateFile(shutdown_log.c_str(), FILE_APPEND_DATA,
+			FILE_SHARE_READ | FILE_SHARE_WRITE, &sa, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+		if (INVALID_HANDLE_VALUE != hLog) { si.hStdError = si.hStdOutput = hLog; }
+
+		si.dwFlags = STARTF_USESTDHANDLES;
+	}
+
+	// create the shutdown program, we create it suspended so that we can log success or failure
+	// and then get a head start to exit before it actually begins running.  That still doesn't
+	// guarantee that we have exited before it starts though...
+	PROCESS_INFORMATION pi;
+	ZeroMemory(&pi, sizeof(pi));
+	priv_state p = set_root_priv( );
+	BOOL exec_status = CreateProcess( NULL, shutdown_program, NULL, NULL, TRUE, DETACHED_PROCESS | CREATE_SUSPENDED, NULL, NULL, &si, &pi );
+	set_priv( p );
+
+	if (INVALID_HANDLE_VALUE != hLog) {
+		formatstr(shutdown_log, "==== Master (pid %d) shutting down. output of shutdown script '%s' (pid %d) follows ====\n",
+			GetCurrentProcessId(), shutdown_program, pi.dwProcessId);
+		DWORD dwWrote;
+		WriteFile(hLog, shutdown_log.c_str(), shutdown_log.size(), &dwWrote, NULL);
+		CloseHandle(hLog);
+	}
+
+	if ( ! exec_status ) {
+		dprintf( D_ALWAYS, "**** CreateProcess(%s) FAILED %d %s\n",
+					shutdown_program, GetLastError(), GetLastErrorString(GetLastError()) );
+	} else {
+		dprintf( D_ALWAYS, "**** CreateProcess(%s) SUCCEEDED pid=%lu\n", shutdown_program, pi.dwProcessId );
+		ResumeThread(pi.hThread); // let the shutdown process start.
+		CloseHandle(pi.hThread);
+		CloseHandle(pi.hProcess);
+	}
+}
+#endif
+
 
 int
 master_exit(int retval)
 {
+	// If a shutdown_program has not been setup via the admin
+	// command-line tool, see if the condor_config specifies
+	// a default shutdown program to use.
+	if ( !shutdown_program ) {
+		char *defshut = param("DEFAULT_MASTER_SHUTDOWN_SCRIPT");
+		if (defshut) {
+			set_shutdown_program(defshut);
+			free(defshut);
+		}
+	}
+
 	cleanup_memory();
 
 #ifdef WIN32
 	if ( NT_ServiceFlag == TRUE ) {
+		// retval == 2 indicates that we never actually started
+		// so we don't want to invoke the shutdown program in that case.
+		if (retval != 2 && shutdown_program) {
+			dprintf( D_ALWAYS,
+						"**** %s (%s_%s) pid %lu EXECING SHUTDOWN PROGRAM %s\n",
+						"condor_master", "condor", "MASTER", GetCurrentProcessId(), shutdown_program );
+			Win32RunShutdownProgram(shutdown_program);
+			shutdown_program = NULL; // in case terminate doesn't kill us, make sure we don't run the shutdown program more than once.
+		}
 		terminate(retval);
 	}
 #endif
@@ -181,7 +337,34 @@ master_exit(int retval)
 #endif
 #endif
 
-	DC_Exit(retval, shutdown_program );
+		// If we're positive that we are going to shut down,
+		// we should clean out the shared port directory if
+		// we created it.
+	std::string dirname;
+	if ( SharedPortEndpoint::CreatedSharedPortDirectory() &&
+		 SharedPortEndpoint::GetDaemonSocketDir(dirname) ) {
+
+		TemporaryPrivSentry tps(PRIV_CONDOR);
+		Directory d(dirname.c_str());
+		d.Remove_Entire_Directory();
+		if (-1 == rmdir(dirname.c_str())) {
+			dprintf(D_ALWAYS, "ERROR: failed to remove shared port temporary directory: %s (errno=%d).\n", strerror(errno), errno);
+		}
+	}
+
+	// Exit via specified shutdown_program UNLESS retval is 2, which
+	// means the master never started and we are exiting via usage() message.
+#ifdef WIN32
+	if (retval != 2 && shutdown_program) {
+		dprintf( D_ALWAYS,
+					"**** %s (%s_%s) pid %lu RUNNING SHUTDOWN PROGRAM %s\n",
+					"condor_master", "condor", "MASTER", GetCurrentProcessId(), shutdown_program );
+		Win32RunShutdownProgram(shutdown_program);
+		shutdown_program = NULL; // don't let DC_Exit run the shutdown program on Windows.
+	}
+#endif
+	DC_Exit(retval, retval == 2 ? NULL : shutdown_program );
+
 	return 1;	// just to satisfy vc++
 }
 
@@ -189,7 +372,8 @@ void
 usage( const char* name )
 {
 	dprintf( D_ALWAYS, "Usage: %s [-f] [-t] [-n name]\n", name );
-	master_exit( 1 );
+	// Note: master_exit with value of 2 means do NOT run any shutdown_program
+	master_exit( 2 );
 }
 
 int
@@ -221,6 +405,139 @@ DoCleanup(int,int,const char*)
 	}
 }
 
+#if defined( LINUX )
+void do_linux_kernel_tuning() {
+	std::string kernelTuningScript;
+	if(! param( kernelTuningScript, "LINUX_KERNEL_TUNING_SCRIPT" )) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: LINUX_KERNEL_TUNING_SCRIPT not defined.\n" );
+		return;
+	}
+
+	std::string kernelTuningLogFile;
+	param( kernelTuningLogFile, "KERNEL_TUNING_LOG" );
+	if( kernelTuningLogFile.empty() ) {
+		kernelTuningLogFile = "/dev/null";
+	}
+
+	if(! can_switch_ids() ) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: can't switch IDs.\n" );
+		return;
+	}
+
+	priv_state prev = set_root_priv();
+	int fd = safe_open_no_create( kernelTuningScript.c_str(), O_RDONLY );
+	set_priv( prev );
+	if( -1 == fd ) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: can't open file '%s'\n", kernelTuningScript.c_str() );
+		return;
+	}
+
+	struct stat stats;
+	int rv = fstat( fd, & stats );
+	if( rv != 0 ) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: can't stat file '%s'\n", kernelTuningScript.c_str() );
+		close( fd );
+		return;
+	}
+	if( stats.st_uid != 0 ) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: file '%s' must be owned by root.\n", kernelTuningScript.c_str() );
+		close( fd );
+		return;
+	}
+	if( (stats.st_mode & S_IWGRP) || (stats.st_mode & S_IWOTH) ) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: file '%s' is group or world -writeable\n", kernelTuningScript.c_str() );
+		close( fd );
+		return;
+	}
+	if(! (stats.st_mode & S_IXUSR)) {
+		dprintf( D_FULLDEBUG, "Not tuning kernel parameters: file '%s' is not executable.\n", kernelTuningScript.c_str() );
+		close( fd );
+		return;
+	}
+	close( fd );
+
+	// Redirect the script's output into the log directory, in case anybody
+	// else cares about it.  This simplifies our code, since we don't.
+	// Do NOT block waiting for the script to exit -- shoot it in the head
+	// after twenty seconds and report an error.  Otherwise, use its exit
+	// code appropriately (may report an error).
+	pid_t childPID = fork();
+	if( childPID == 0 ) {
+		daemonCore->Forked_Child_Wants_Fast_Exit( true );
+		dprintf_init_fork_child();
+
+		priv_state prev = set_root_priv();
+
+		int fd = open( "/dev/null", O_RDONLY );
+		if( fd == -1 ) {
+			dprintf( D_FULLDEBUG, "Not tuning kernel parameters: child failed to open /dev/null: %d.\n", errno );
+			exit( 1 );
+		}
+		if( dup2( fd, 0 ) == -1 ) {
+			dprintf( D_FULLDEBUG, "Not tuning kernel parameters: child failed to dup /dev/null: %d.\n", errno );
+			exit( 1 );
+		}
+		fd = open( kernelTuningLogFile.c_str(), O_WRONLY | O_APPEND, 0644 );
+		if ((fd < 0) && (errno == ENOENT)) {
+			fd = open( kernelTuningLogFile.c_str(), O_WRONLY | O_APPEND | O_CREAT, 0644 );
+		}
+		if( fd == -1 ) {
+			dprintf( D_FULLDEBUG, "Not tuning kernel parameters: child failed to open '%s': %d.\n", kernelTuningLogFile.c_str(), errno );
+			exit( 1 );
+		}
+		if( dup2( fd, 1 ) == -1 ) {
+			dprintf( D_FULLDEBUG, "Not tuning kernel parameters: child failed to dup log file '%s' to stdout: %d.\n", kernelTuningLogFile.c_str(), errno );
+			exit( 1 );
+		}
+		if( dup2( fd, 2 ) == -1 ) {
+			dprintf( D_FULLDEBUG, "Not tuning kernel parameters: child failed to dup log file '%s' to stderr: %d.\n", kernelTuningLogFile.c_str(), errno );
+			exit( 1 );
+		}
+
+		execl( kernelTuningScript.c_str(), kernelTuningScript.c_str(), (char *) NULL );
+
+		set_priv( prev );
+		dprintf( D_ALWAYS, "Did not tune kernel paramters: execl() failed: %d.\n", errno );
+		exit( 1 );
+	} else {
+		int status = 0;
+		pid_t wait = 0;
+		unsigned timer = 20;
+
+		while( true ) {
+			wait = waitpid( childPID, & status, WNOHANG );
+			if( wait == childPID ) {
+				if( WIFEXITED(status) && WEXITSTATUS(status) == 0 ) {
+					dprintf( D_FULLDEBUG, "Kernel parameters tuned.\n" );
+					return;
+				} else {
+					dprintf( D_FULLDEBUG, "Failed to tune kernel parameters, status code %d.\n", status );
+					return;
+				}
+			} else if( wait == 0 ) {
+				if( --timer > 0 ) {
+					dprintf( D_FULLDEBUG, "Sleeping one second for kernel parameter tuning (pid %d).\n", childPID );
+					sleep( 1 );
+				} else {
+					dprintf( D_FULLDEBUG, "Waited too long for kernel parameters to be tuned, hard-killing script.\n" );
+					kill( childPID, SIGKILL );
+					// Collect the zombie.
+					wait = waitpid( childPID, &status, WNOHANG );
+					ASSERT( wait != childPID );
+					return;
+				}
+			} else {
+				dprintf( D_FULLDEBUG, "waitpid() failed while waiting for kernel tuning (%d).  Killing child %d.\n", errno, childPID );
+				kill( childPID, SIGKILL );
+				// Try again to collect the zombie?
+				waitpid( childPID, & status, WNOHANG );
+				return;
+			}
+		}
+	}
+
+}
+#endif
 
 void
 main_init( int argc, char* argv[] )
@@ -243,6 +560,9 @@ main_init( int argc, char* argv[] )
 			if( !(ptr && *ptr) ) {
 				EXCEPT( "-n requires another argument" );
 			}
+			if (MasterName) {
+				free(MasterName);
+			}
 			MasterName = build_valid_daemon_name( *ptr );
 			dprintf( D_ALWAYS, "Using name: %s\n", MasterName );
 			break;
@@ -251,6 +571,43 @@ main_init( int argc, char* argv[] )
 		}
 	}
 
+#ifdef LINUX
+	if ( can_switch_ids() &&
+		 param_boolean("DISCARD_SESSION_KEYRING_ON_STARTUP",true) )
+	{
+#ifndef KEYCTL_JOIN_SESSION_KEYRING
+  #define KEYCTL_JOIN_SESSION_KEYRING 1
+#endif
+		// Create a new empty session keyring, discarding the old one.
+		// We do this in the master so a session keyring containing
+		// root keys is not inadvertantly leaked to jobs
+		// (esp scheduler universe jobs, since
+		// KEYCTL_JOIN_SESSION_KEYRING will fail in RHEL6 if the calling
+		// process has more than one thread, which is true when spawning
+		// scheduler universe jobs as the schedd clones by default instead
+		// of forking).  We also do this in the master instead
+		// of daemonCore, since we don't want to create a brand new
+		// session keyring for every shadow (each keyring uses up kernel resources).
+		// If the syscall fails due to ENOSYS, don't worry about it,
+		// since that simply says the keyring facility is not installed.
+		if (syscall(__NR_keyctl, KEYCTL_JOIN_SESSION_KEYRING, "htcondor")==-1 &&
+			errno != ENOSYS)
+		{
+			int saved_errno = errno;
+#if defined(EDQUOT)
+			if (saved_errno == EDQUOT) {
+				dprintf(D_ALWAYS | D_FAILURE,
+				   "Error during DISCARD_SESSION_KEYRING_ON_STARTUP, suggest "
+				   "increasing /proc/sys/kernel/keys/root_maxkeys\n");
+			}
+#endif /* defined EDQUOT */
+
+			EXCEPT("Failed DISCARD_SESSION_KEYRING_ON_STARTUP=True errno=%d",
+					saved_errno);
+		}
+	}
+#endif /* defined LINUX */
+
     if (runfor != 0) {
         // We will construct an environment variable that 
         // tells the daemon what time it will be shut down. 
@@ -267,6 +624,7 @@ main_init( int argc, char* argv[] )
 	
 		// Grab all parameters needed by the master.
 	init_params();
+	validate_config(0, CONFIG_OPT_DEPRECATION_WARNINGS);
 		// param() for DAEMON_LIST and initialize our daemons object.
 	init_daemon_list();
 	if ( daemons.SetupControllers() < 0 ) {
@@ -292,6 +650,8 @@ main_init( int argc, char* argv[] )
 	MasterPluginManager::Initialize();
 #endif
 
+	g_systemd_notifier.config();
+
 		// Register admin commands
 	daemonCore->Register_Command( RESTART, "RESTART",
 								  (CommandHandler)admin_command_handler, 
@@ -346,6 +706,15 @@ main_init( int argc, char* argv[] )
 								(CommandHandler)&store_pool_cred_handler,
 								"store_pool_cred_handler", NULL, CONFIG_PERM,
 								D_FULLDEBUG );
+
+	// Command handler for handling the ready state
+	daemonCore->Register_CommandWithPayload( DC_SET_READY, "DC_SET_READY",
+								  (CommandHandler)ready_command_handler,
+								  "ready_command_handler", 0, WRITE );
+	daemonCore->Register_CommandWithPayload( DC_QUERY_READY, "DC_QUERY_READY",
+								  (CommandHandler)ready_command_handler,
+								  "ready_command_handler", 0, READ );
+
 	/*
 	daemonCore->Register_Command( START_AGENT, "START_AGENT",
 					  (CommandHandler)admin_command_handler, 
@@ -371,6 +740,51 @@ main_init( int argc, char* argv[] )
 }
 
 
+int
+ready_command_handler( Service*, int cmd, Stream* stm )
+{
+	ReliSock* stream = (ReliSock*)stm;
+	ClassAd cmdAd;
+
+	stream->decode();
+	stream->timeout(15);
+	if( !getClassAd(stream, cmdAd) || !stream->end_of_message()) {
+		dprintf( D_ALWAYS, "Failed to receive ready command (%d) on TCP: aborting\n", cmd );
+		return FALSE;
+	}
+	MyString daemon_name; // using MyString here because it will never return NULL
+	cmdAd.LookupString("DaemonName", daemon_name);
+	int daemon_pid = 0;
+	cmdAd.LookupInteger("DaemonPID", daemon_pid);
+
+	dprintf( D_FULLDEBUG, "Got ready command (%d) from %s pid=%d\n", cmd, daemon_name.c_str(), daemon_pid );
+
+	switch (cmd) {
+		case DC_SET_READY:
+		{
+			MyString state; // using MyString because its c_str() never faults or returns NULL
+			cmdAd.LookupString("DaemonState", state);
+			class daemon* daemon = daemons.FindDaemonByPID(daemon_pid);
+			if ( ! daemon) {
+				dprintf(D_ALWAYS, "Cant find daemon %s to set ready state '%s'\n", daemon_name.c_str(), state.c_str());
+			} else {
+				dprintf(D_ALWAYS, "Setting ready state '%s' for %s\n", state.c_str(), daemon_name.c_str());
+				daemon->SetReadyState(state.c_str());
+			}
+		}
+		return TRUE;
+
+		case DC_QUERY_READY:
+			return daemons.QueryReady(cmdAd, stm);
+		return FALSE;
+
+		default:
+			EXCEPT("Unknown ready command (%d) in ready_command_handler", cmd);
+	}
+
+	return FALSE;
+}
+
 int
 admin_command_handler( Service*, int cmd, Stream* stream )
 {
@@ -472,39 +886,32 @@ agent_starter( ReliSock * s )
 int
 handle_agent_fetch_log (ReliSock* stream) {
 
-	char *daemon_name = NULL;
-	char *daemon_paramname = NULL;
-	char *daemon_filename = NULL;
+	MyString daemon;
 	int  res = FALSE;
 
-	if( ! stream->code(daemon_name) ||
+	if( ! stream->code(daemon) ||
 		! stream->end_of_message()) {
 		dprintf( D_ALWAYS, "ERROR: fetch_log can't read daemon name\n" );
-		free( daemon_name );
 		return FALSE;
 	}
 
-	dprintf( D_ALWAYS, "INFO: daemon_name: %s\n", daemon_name );
+	dprintf( D_ALWAYS, "INFO: daemon_name: %s\n", daemon.c_str() );
 
-	daemon_paramname = (char*)malloc (strlen(daemon_name) + 5);
-	strcpy (daemon_paramname, daemon_name);
-	strcat (daemon_paramname, "_LOG");
+	// append _LOG to get the param name of the daemon log file
+	daemon += "_LOG";
 
-	dprintf( D_ALWAYS, "INFO: daemon_paramname: %s\n", daemon_paramname );
+	dprintf( D_ALWAYS, "INFO: daemon_paramname: %s\n", daemon.c_str() );
 
-	if( (daemon_filename = param(daemon_paramname)) ) {
+	auto_free_ptr daemon_filename(param(daemon.c_str()));
+	if ( daemon_filename ) {
 		filesize_t	size;
-		dprintf( D_ALWAYS, "INFO: daemon_filename: %s\n", daemon_filename );
+		dprintf( D_ALWAYS, "INFO: daemon_filename: %s\n", daemon_filename.ptr() );
 		stream->encode();
-		res = (stream->put_file(&size, daemon_filename) < 0);
-		free (daemon_filename);
+		res = (stream->put_file(&size, daemon_filename.ptr()) < 0);
 	} else {
-		dprintf( D_ALWAYS, "ERROR: fetch_log can't param for log name\n" );
+		dprintf( D_ALWAYS, "ERROR: fetch_log can't param for %s\n", daemon.c_str() );
 	}
 
-	free (daemon_paramname);
-	free (daemon_name);
-
 	return res;
 }
 
@@ -577,13 +984,13 @@ handle_shutdown_program( int cmd, Stream* stream )
 		EXCEPT( "Unknown command (%d) in handle_shutdown_program", cmd );
 	}
 
-	char	*name = NULL;
+	MyString name;
 	stream->decode();
 	if( ! stream->code(name) ) {
-		dprintf( D_ALWAYS, "Can't read program name\n" );
-		if ( name ) {
-			free( name );
-		}
+		dprintf( D_ALWAYS, "Can't read program name in handle_shutdown_program\n" );
+	}
+
+	if ( name.IsEmpty() ) {
 		return FALSE;
 	}
 
@@ -593,10 +1000,20 @@ handle_shutdown_program( int cmd, Stream* stream )
 	pname += name;
 	char	*path = param( pname.Value() );
 	if ( NULL == path ) {
-		dprintf( D_ALWAYS, "No shutdown program defined for '%s'\n", name );
+		dprintf( D_ALWAYS, "No shutdown program defined for '%s'\n", name.c_str() );
 		return FALSE;
 	}
 
+	int ret_val = set_shutdown_program( path );
+
+	if (path) free(path);
+
+	return ret_val;
+}
+
+int
+set_shutdown_program(const char *path)
+{
 	// Try to access() it
 # if defined(HAVE_ACCESS)
 	priv_state	priv = set_root_priv();
@@ -613,7 +1030,7 @@ handle_shutdown_program( int cmd, Stream* stream )
 	if ( shutdown_program ) {
 		free( shutdown_program );
 	}
-	shutdown_program = path;
+	shutdown_program = strdup(path);
 	dprintf( D_ALWAYS,
 			 "Shutdown program path set to %s\n", shutdown_program );
 	return TRUE;
@@ -862,6 +1279,7 @@ init_daemon_list()
 				daemon_names.insert( "SHARED_PORT" );
 			}
 		}
+
 		daemons.ordered_daemon_names.create_union( daemon_names, false );
 
 		daemon_names.rewind();
@@ -958,6 +1376,7 @@ main_config()
 
 		// Reset our config values
 	init_params();
+	validate_config(0, CONFIG_OPT_DEPRECATION_WARNINGS);
 
 		// Reset the daemon list
 	init_daemon_list();
@@ -1097,8 +1516,8 @@ invalidate_ads() {
 		default_name = default_daemon_name();
 	}
 	
-	EscapeAdStringValue( default_name, escaped_name );
-	line.formatstr( "( TARGET.%s == \"%s\" )", ATTR_NAME, escaped_name.c_str() );
+	QuoteAdStringValue( default_name, escaped_name );
+	line.formatstr( "( TARGET.%s == %s )", ATTR_NAME, escaped_name.c_str() );
 	cmd_ad.AssignExpr( ATTR_REQUIREMENTS, line.Value() );
 	cmd_ad.Assign( ATTR_NAME, default_name );
 	cmd_ad.Assign( ATTR_MY_ADDRESS, daemonCore->publicNetworkIpAddr());
@@ -1242,9 +1661,9 @@ void
 main_pre_command_sock_init()
 {
 	/* Make sure we are the only copy of condor_master running */
+	char*  p;
 #ifndef WIN32
 	MyString lock_file;
-	char*  p;
 
 	// see if a file is given explicitly
 	p = param ("MASTER_INSTANCE_LOCK");
@@ -1276,16 +1695,39 @@ main_pre_command_sock_init()
 	lock_or_except( lock_file.Value() );
 	dprintf (D_FULLDEBUG, "Obtained lock on %s.\n", lock_file.Value() );
 #endif
-	MyString daemon_list;
-	if( param(daemon_list,"DAEMON_LIST") ) {
-		StringList sl(daemon_list.Value());
-		if( sl.contains("SHARED_PORT") ) {
-				// in case a shared port address file got left behind by an
-				// unclean shutdown, clean it up now before we create our
-				// command socket to avoid confusion
-			SharedPortServer::RemoveDeadAddressFile();
-		}
+
+	// Do any kernel tuning we've been configured to do.
+	if( param_boolean( "ENABLE_KERNEL_TUNING", false ) ) {
+#ifdef LINUX
+		do_linux_kernel_tuning();
+#endif
 	}
+
+	// If using CREDENTIAL_DIRECTORY, blow away the CREDMON_COMPLETE file
+	// to force the credmon to refresh everything and to prevent the schedd
+	// from starting up until credentials are ready.
+	p = param("SEC_CREDENTIAL_DIRECTORY");
+	if(p) {
+		MyString cred_file;
+		formatstr( cred_file, "%s%cCREDMON_COMPLETE", p, DIR_DELIM_CHAR );
+		dprintf(D_SECURITY, "CREDMON: unlinking %s.", cred_file.Value());
+		unlink(cred_file.Value());
+	}
+	free(p);
+
+ 	// in case a shared port address file got left behind by an
+ 	// unclean shutdown, clean it up now before we create our
+ 	// command socket to avoid confusion
+	// Do so unconditionally, because the master will decide later (when
+	// it's ready to start daemons) if it will be starting the shared
+	// port daemon.
+ 	SharedPortServer::RemoveDeadAddressFile();
+
+	// The master and its daemons may disagree on if they're using shared
+	// port, so make sure everything's ready, just in case.
+	//
+	// FIXME: condor_preen doesn't look to know about "auto" directories.
+	SharedPortEndpoint::InitializeDaemonSocketDir();
 }
 
 #ifdef WIN32
@@ -1366,7 +1808,7 @@ void init_firewall_exceptions() {
 
 	WindowsFirewallHelper wfh;
 	
-	add_exception = param_boolean("ADD_WINDOWS_FIREWALL_EXCEPTION", true);
+	add_exception = param_boolean("ADD_WINDOWS_FIREWALL_EXCEPTION", NT_ServiceFlag);
 
 	if ( add_exception == false ) {
 		dprintf(D_FULLDEBUG, "ADD_WINDOWS_FIREWALL_EXCEPTION is false, skipping\n");
@@ -1416,6 +1858,7 @@ void init_firewall_exceptions() {
 	// We also want to add exceptions for the DAGMan we ship
 	// with Condor:
 
+	dagman_image_path = NULL; // make sure it's initialized.
 	bin_path = param ( "BIN" );
 	if ( bin_path ) {
 		dagman_image_path = (char*) malloc (
