@@ -145,6 +145,11 @@ class Matchmaker : public Service
 
 		bool getGroupInfoFromUserId(const char* user, string& groupName, float& groupQuota, float& groupUsage);
 
+		void forwardAccountingData(std::set<std::string> &names);
+		void forwardGroupAccounting(DCCollector &collector, GroupEntry *ge);
+
+		void calculateRanks(ClassAd &request, ClassAd *offer, PreemptState candidatePreemptState, double &candidateRankValue, double &candidatePreJobRankValue, double &candidatePostJobRankValue, double &candidatePreemptRankValue);
+
     protected:
 		char * NegotiatorName;
 		int update_interval;
@@ -158,10 +163,38 @@ class Matchmaker : public Service
 		void updateCollector();
 		
 		// auxillary functions
-		bool obtainAdsFromCollector (ClassAdList&, ClassAdListDoesNotDeleteAds&, ClassAdListDoesNotDeleteAds&, ClaimIdHash& );	
+		bool obtainAdsFromCollector (ClassAdList&, ClassAdListDoesNotDeleteAds&, ClassAdListDoesNotDeleteAds&, std::set<std::string> &, ClaimIdHash& );	
 		char * compute_significant_attrs(ClassAdListDoesNotDeleteAds & startdAds);
 		bool consolidate_globaljobprio_submitter_ads(ClassAdListDoesNotDeleteAds & scheddAds);
-		
+
+		/**
+		 * Start the network communication necessary for a negotiation cycle.
+		 */
+		typedef classad_shared_ptr<ResourceRequestList> RRLPtr;
+		bool startNegotiateProtocol(const std::string &submitter, const ClassAd &submitterAd, ReliSock *&sock, RRLPtr &request_list);
+
+		/**
+		 * Get a resource request list for purposes of negotiation
+		 */
+		RRLPtr startNegotiate(const std::string &submitter, const ClassAd &submitterAd, ReliSock *&sock);
+		void endNegotiate(const std::string &scheddAddr);
+
+		/**
+		 * Try starting negotiations with all schedds in parallel.
+		 */
+		void prefetchResourceRequestLists(ClassAdListDoesNotDeleteAds &scheddAds);
+		typedef std::map<std::string, classad_shared_ptr<ResourceRequestList> > RRLHash;
+		RRLHash m_cachedRRLs;
+
+		struct JobRanks {
+               double PreJobRankValue;
+               double PostJobRankValue;
+               double PreemptRankValue;
+		};
+
+		typedef std::map<ClassAd *, JobRanks> RanksMapType;
+		RanksMapType ranksMap;
+
 		/** Negotiate w/ one schedd for one user, for one 'pie spin'.
             @param groupName name of group negotiating under (or NULL)
 			@param scheddName Name attribute from the submitter ad.
@@ -315,7 +348,6 @@ class Matchmaker : public Service
 			// Recalculates ranks and re-sorts match list.
 			// ASSUMES NO_PREEMPTION for pslots.
 		bool returnPslotToMatchList(ClassAd &request, ClassAd *offer);
-		void calculateRanks(ClassAd &request, ClassAd *offer, PreemptState candidatePreemptState, double &candidateRankValue, double &candidatePreJobRankValue, double &candidatePostJobRankValue, double &candidatePreemptRankValue);
 
 
 		void RegisterAttemptedOfflineMatch( ClassAd *job_ad, ClassAd *startd_ad );
@@ -337,6 +369,7 @@ class Matchmaker : public Service
 		ExprTree *NegotiatorPostJobRank; // rank applied after job rank
 		bool want_globaljobprio;	// cached value of config knob USE_GLOBAL_JOB_PRIOS
 		bool want_matchlist_caching;	// should we cache matches per autocluster?
+		bool PublishCrossSlotPrios; // value of knob NEGOTIATOR_CROSS_SLOT_PRIOS, default of false
 		bool ConsiderPreemption; // if false, negotiation is faster (default=true)
 		bool ConsiderEarlyPreemption; // if false, do not preempt slots that still have retirement time
 		/// Should the negotiator inform startds of matches?
@@ -348,6 +381,8 @@ class Matchmaker : public Service
                                          // constraint before calculating quotas
                                          // formerly DynQuotaMachConstraint Added for CDF.
 
+		bool m_staticRanks;
+
 		StringList NegotiatorMatchExprNames;
 		StringList NegotiatorMatchExprValues;
 
@@ -538,6 +573,8 @@ class Matchmaker : public Service
 
 		int prevLHF;
 
+		char *slotWeightStr;
+		
 		#define MAX_NEGOTIATION_CYCLE_STATS 100
 		class NegotiationCycleStats *negotiation_cycle_stats[MAX_NEGOTIATION_CYCLE_STATS];
 		int num_negotiation_cycle_stats;
