@@ -52,8 +52,11 @@ class QmgmtPeer {
 		void unset();
 
 		bool setEffectiveOwner(char const *o);
+		bool setAllowProtectedAttrChanges(bool val);
+		bool getAllowProtectedAttrChanges() { return allow_protected_attr_changes_by_superuser; }
 
 		ReliSock *getReliSock() const { return sock; };
+		const CondorVersionInfo *get_peer_version() const { return sock->get_peer_version(); };
 
 		const char *endpoint_ip_str() const;
 		const condor_sockaddr endpoint() const;
@@ -65,6 +68,7 @@ class QmgmtPeer {
 	protected:
 
 		char *owner;  
+		bool allow_protected_attr_changes_by_superuser;
 		char *fquser;  // owner@domain
 		char *myendpoint; 
 		condor_sockaddr addr;
@@ -84,6 +88,9 @@ class QmgmtPeer {
 
 #define USE_JOB_QUEUE_JOB 1 // contents of the JobQueue is a class *derived* from ClassAd, (new for 8.3)
 
+#define JQJ_CHACHE_DIRTY_JOBOBJ        0x00001 // set when an attribute cached in the JobQueueJob that doesn't have it's own flag has changed
+#define JQJ_CHACHE_DIRTY_SUBMITTERDATA 0x00002 // set when an attribute that affects the submitter name is changed
+
 // used to store a ClassAd + runtime information in a condor hashtable.
 class JobQueueJob : public ClassAd {
 public:
@@ -94,7 +101,14 @@ protected:
 	char has_noop_attr; // 1 if job has ATTR_JOB_NOOP
 	char future_status; // FUTURE: keep this in sync with job status
 public:
+	int dirty_flags;	// one or more of JQJ_CHACHE_DIRTY_ flags indicating that the job ad differs from the JobQueueJob 
+	int spare;
 	int autocluster_id;
+	// cached pointer into schedulers's SubmitterDataMap and OwnerInfoMap
+	// it is set by count_jobs() or by scheduler::get_submitter_and_owner()
+	// DO NOT FREE FROM HERE!
+	struct SubmitterData * submitterdata;
+	struct OwnerInfo * ownerinfo;
 protected:
 	JobQueueJob * link; // FUTURE: jobs are linked to clusters.
 	int future_num_procs_or_hosts; // FUTURE: num_procs if cluster, num hosts if job
@@ -106,7 +120,11 @@ public:
 		, universe(0)
 		, has_noop_attr(2) // value of 2 forces PopulateFromAd() to check if it exists.
 		, future_status(0) // JOB_STATUS_MIN
+		, dirty_flags(0)
 		, autocluster_id(0)
+		, submitterdata(NULL)
+		, ownerinfo(NULL)
+		, link(NULL)
 		, future_num_procs_or_hosts(0)
 	{}
 	virtual ~JobQueueJob() {};
@@ -124,6 +142,7 @@ public:
 	int  Universe() { return universe; }
 	void SetUniverse(int uni) { universe = uni; }
 	bool IsNoopJob();
+	void DirtyNoopAttr() { has_noop_attr = 2; }
 	// FUTURE:
 	int NumProcs() { if (entry_type == entry_type_cluster) return future_num_procs_or_hosts; return 0; }
 	int IncrementNumProcs() { if (entry_type == entry_type_cluster) return ++future_num_procs_or_hosts; return 0; }
@@ -142,7 +161,6 @@ public:
 };
 
 
-void CloseJobHistoryFile();
 void SetMaxHistoricalLogs(int max_historical_logs);
 time_t GetOriginalJobQueueBirthdate();
 void DestroyJobQueue( void );
@@ -325,7 +343,6 @@ extern int grow_prio_recs(int);
 
 extern void	FindRunnableJob(PROC_ID & jobid, ClassAd* my_match_ad, char const * user);
 extern int Runnable(PROC_ID*);
-extern int Runnable(ClassAd*);
 
 extern class ForkWork schedd_forker;
 
