@@ -45,8 +45,6 @@ extern char *CondorCertDir;
 
 extern int active_cluster_num;
 
-extern int CheckTransaction( SetAttributeFlags_t, CondorError * errorStack );
-
 static bool QmgmtMayAccessAttribute( char const *attr_name ) {
 	return !ClassAdAttributeIsPrivate( attr_name );
 }
@@ -121,6 +119,30 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 		return 0;
 	}
 
+	case CONDOR_SetAllowProtectedAttrChanges:
+	{
+		int val;
+		int terrno;
+
+		assert( syscall_sock->get(val) );
+		assert( syscall_sock->end_of_message() );
+
+		rval = QmgmtSetAllowProtectedAttrChanges( val );
+		terrno = errno;
+
+		syscall_sock->encode();
+		assert( syscall_sock->code(rval) );
+		if( rval < 0 ) {
+			assert( syscall_sock->code(terrno) );
+		}
+		assert( syscall_sock->end_of_message() );
+
+		dprintf(D_SYSCALLS, "\tSetAllowProtectedAttrChanges(%d)\n", val);
+		dprintf(D_SYSCALLS, "\trval %d, errno %d\n", rval, terrno);
+
+		return 0;
+	}
+
 	case CONDOR_SetEffectiveOwner:
 	{
 		MyString owner;
@@ -508,19 +530,10 @@ do_Q_request(ReliSock *syscall_sock,bool &may_fork)
 
 		errno = 0;
 		CondorError errstack;
-		rval = CheckTransaction( flags, & errstack );
+		rval = CommitTransaction( flags, & errstack );
 		terrno = errno;
 		dprintf( D_SYSCALLS, "\tflags = %d, rval = %d, errno = %d\n", flags, rval, terrno );
 
-		if( rval >= 0 ) {
-			errno = 0;
-			CommitTransaction( flags );
-				// CommitTransaction() never returns on failure
-			rval = 0;
-			terrno = errno;
-			dprintf( D_SYSCALLS, "\tflags = %d, rval = %d, errno = %d\n", flags, rval, terrno );
-		}
-
 		syscall_sock->encode();
 		assert( syscall_sock->code(rval) );
 		if( rval < 0 ) {
