@@ -1,4 +1,3 @@
-
 /***************************************************************
  *
  * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
@@ -58,6 +57,31 @@ InitializeReadOnlyConnection( const char * /*owner*/ )
 	return( 0 );
 }
 
+int
+QmgmtSetAllowProtectedAttrChanges( int val )
+{
+	int	rval = -1;
+
+	CurrentSysCall = CONDOR_SetAllowProtectedAttrChanges;
+
+	qmgmt_sock->encode();
+	neg_on_error( qmgmt_sock->code(CurrentSysCall) );
+	neg_on_error( qmgmt_sock->code(val) );
+	neg_on_error( qmgmt_sock->end_of_message() );
+
+	qmgmt_sock->decode();
+	neg_on_error( qmgmt_sock->code(rval) );
+	if( rval < 0 ) {
+		neg_on_error( qmgmt_sock->code(terrno) );
+		neg_on_error( qmgmt_sock->end_of_message() );
+		errno = terrno;
+		return rval;
+	}
+	neg_on_error( qmgmt_sock->end_of_message() );
+
+	return rval;
+}
+
 int
 QmgmtSetEffectiveOwner(char const *o)
 {
@@ -385,7 +409,7 @@ AbortTransaction()
 }
 
 int
-RemoteCommitTransaction(SetAttributeFlags_t flags)
+RemoteCommitTransaction(SetAttributeFlags_t flags, CondorError *errstack)
 {
 	int	rval = -1;
 
@@ -408,6 +432,19 @@ RemoteCommitTransaction(SetAttributeFlags_t flags)
 	neg_on_error( qmgmt_sock->code(rval) );
 	if( rval < 0 ) {
 		neg_on_error( qmgmt_sock->code(terrno) );
+		const CondorVersionInfo *vers = qmgmt_sock->get_peer_version();
+		if (vers && vers->built_since_version(8, 3, 4))
+		{
+			ClassAd reply;
+			neg_on_error( getClassAd( qmgmt_sock, reply ) );
+
+			std::string errmsg;
+			if( errstack && reply.LookupString( "ErrorReason", errmsg ) ) {
+				int errCode = terrno;
+				reply.LookupInteger( "ErrorCode", errCode );
+				errstack->push( "SCHEDD", errCode, errmsg.c_str() );
+			}
+		}
 		neg_on_error( qmgmt_sock->end_of_message() );
 		errno = terrno;
 		return rval;
