@@ -117,6 +117,8 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	common_job_queue_attrs->insert( ATTR_DISK_USAGE );
 	common_job_queue_attrs->insert( ATTR_JOB_REMOTE_SYS_CPU );
 	common_job_queue_attrs->insert( ATTR_JOB_REMOTE_USER_CPU );
+	common_job_queue_attrs->insert( ATTR_JOB_CUMULATIVE_REMOTE_SYS_CPU );
+	common_job_queue_attrs->insert( ATTR_JOB_CUMULATIVE_REMOTE_USER_CPU );
 	common_job_queue_attrs->insert( ATTR_TOTAL_SUSPENSIONS );
 	common_job_queue_attrs->insert( ATTR_CUMULATIVE_SUSPENSION_TIME );
 	common_job_queue_attrs->insert( ATTR_COMMITTED_SUSPENSION_TIME );
@@ -136,6 +138,8 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	common_job_queue_attrs->insert( ATTR_BLOCK_READ_BYTES );
 	common_job_queue_attrs->insert( ATTR_BLOCK_WRITES );
 	common_job_queue_attrs->insert( ATTR_BLOCK_READS );
+	common_job_queue_attrs->insert( ATTR_NETWORK_IN );
+	common_job_queue_attrs->insert( ATTR_NETWORK_OUT );
     common_job_queue_attrs->insert( "Recent" ATTR_BLOCK_READ_KBYTES );
     common_job_queue_attrs->insert( "Recent" ATTR_BLOCK_WRITE_KBYTES );
     common_job_queue_attrs->insert( "Recent" ATTR_BLOCK_READ_BYTES );
@@ -153,6 +157,7 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	common_job_queue_attrs->insert( ATTR_TRANSFER_QUEUED );
 	common_job_queue_attrs->insert( ATTR_JOB_TRANSFERRING_OUTPUT );
 	common_job_queue_attrs->insert( ATTR_JOB_TRANSFERRING_OUTPUT_TIME );
+	common_job_queue_attrs->insert( ATTR_NUM_JOB_COMPLETIONS );
 
 	hold_job_queue_attrs = new StringList();
 	hold_job_queue_attrs->insert( ATTR_HOLD_REASON );
@@ -191,11 +196,14 @@ QmgrJobUpdater::initJobQueueAttrLists( void )
 	checkpoint_job_queue_attrs->insert( ATTR_VM_CKPT_IP );
 
 	x509_job_queue_attrs = new StringList();
-	x509_job_queue_attrs->insert( ATTR_X509_USER_PROXY_SUBJECT );
 	x509_job_queue_attrs->insert( ATTR_X509_USER_PROXY_EXPIRATION );
+	/* These are secure attributes, only settable by the schedd.
+	 * Assume they won't change during job execution.
+	x509_job_queue_attrs->insert( ATTR_X509_USER_PROXY_SUBJECT );
 	x509_job_queue_attrs->insert( ATTR_X509_USER_PROXY_VONAME );
 	x509_job_queue_attrs->insert( ATTR_X509_USER_PROXY_FIRST_FQAN );
 	x509_job_queue_attrs->insert( ATTR_X509_USER_PROXY_FQAN );
+	*/
 
 	m_pull_attrs = new StringList();
 	if ( job_ad->LookupExpr( ATTR_TIMER_REMOVE_CHECK ) ) {
