@@ -34,6 +34,7 @@
 #include "internet.h"
 
 #include "spooled_job_files.h"
+#include "set_user_priv_from_ad.h"
 #include "condor_config.h"
 #include "condor_open.h"
 
@@ -462,6 +463,29 @@ Job::get_file(const MyString &name,
 			  CondorError &errstack)
 {
 	memset(data, 0, length);
+
+#if !defined(WIN32)
+	TemporaryPrivSentry sentry( true );
+	if ( param_boolean( "CHOWN_JOB_SPOOL_FILES", false ) == false ) {
+		ClassAd *job_ad = GetJobAd_as_ClassAd( id.cluster, id.proc );
+		if ( job_ad == NULL ) {
+			errstack.pushf("SOAP",
+						   FAIL,
+						   "Failed to retrieve job ad for file '%s'",
+						   name.Value());
+			return 5;
+		}
+		if ( !init_user_ids_from_ad( *job_ad ) ) {
+			errstack.pushf("SOAP",
+						   FAIL,
+						   "Failed to init user ids for file '%s'",
+						   name.Value());
+			return 6;
+		}
+		set_user_priv();
+	}
+#endif
+
 	int file = safe_open_wrapper_follow((spoolDirectory + DIR_DELIM_STRING + name).Value(),
 					O_RDONLY | _O_BINARY,
 					0);
@@ -634,6 +658,7 @@ ScheddTransaction::commit()
 	Job *job;
 	ClassAd *job_ad;
 	int universe;
+	bool chown_job_spool_files = param_boolean( "CHOWN_JOB_SPOOL_FILES", false ) ;
 	jobs.startIterations();
 	while (jobs.iterate(currentKey, job)) {
 		job_ad = GetJobAd_as_ClassAd( currentKey.cluster, currentKey.proc );
@@ -641,6 +666,8 @@ ScheddTransaction::commit()
 			 universe == CONDOR_UNIVERSE_GRID ) {
 
 			aboutToSpawnJobHandler( currentKey.cluster, currentKey.proc, NULL );
+		} else if ( chown_job_spool_files == false ) {
+			SpooledJobFiles::createJobSpoolDirectory( job_ad, PRIV_USER );
 		}
 	}
 
