@@ -418,6 +418,9 @@ RemoteResource::RemoteResource( BaseShadow *shad )
 	std::string prefix;
 	param(prefix, "CHIRP_DELAYED_UPDATE_PREFIX", "CHIRP*");
 	m_delayed_update_prefix.initializeFromString(prefix.c_str());
+
+	param_and_insert_attrs("PROTECTED_JOB_ATTRS", m_protected_attrs);
+	param_and_insert_attrs("SYSTEM_PROTECTED_JOB_ATTRS", m_protected_attrs);
 }
 
 
@@ -1286,14 +1289,14 @@ RemoteResource::setJobAd( ClassAd *jA )
 	int64_t int64_value;
 	double real_value;
 
-	if( jA->LookupFloat(ATTR_JOB_REMOTE_SYS_CPU, real_value) ) {
-		remote_rusage.ru_stime.tv_sec = (time_t) real_value;
-	}
+	// REMOTE_SYS_CPU and REMOTE_USER_CPU reflect usage for this execution only.
+	// reset to 0 on start or restart.
+	real_value = 0.0;
+	remote_rusage.ru_stime.tv_sec = (time_t) real_value;
+	remote_rusage.ru_utime.tv_sec = (time_t) real_value;
+	jA->Assign(ATTR_JOB_REMOTE_USER_CPU, real_value);
+	jA->Assign(ATTR_JOB_REMOTE_SYS_CPU, real_value);
 			
-	if( jA->LookupFloat(ATTR_JOB_REMOTE_USER_CPU, real_value) ) {
-		remote_rusage.ru_utime.tv_sec = (time_t) real_value;
-	}
-
 	if( jA->LookupInteger(ATTR_IMAGE_SIZE, int64_value) ) {
 		image_size_kb = int64_value;
 	}
@@ -1362,11 +1365,35 @@ RemoteResource::updateFromStarter( ClassAd* update_ad )
 
 	double real_value;
 	if( update_ad->LookupFloat(ATTR_JOB_REMOTE_SYS_CPU, real_value) ) {
+		double prevUsage;
+		if (!jobAd->LookupFloat(ATTR_JOB_REMOTE_SYS_CPU, prevUsage)) {
+			prevUsage = 0.0;
+		}
+
+		// Remote cpu usage should be strictly increasing
+		if (real_value > prevUsage) {
+			double prevTotalUsage = 0.0;
+			jobAd->LookupFloat(ATTR_JOB_CUMULATIVE_REMOTE_SYS_CPU, prevTotalUsage);
+			jobAd->Assign(ATTR_JOB_CUMULATIVE_REMOTE_SYS_CPU, prevTotalUsage + (real_value - prevUsage));
+		}
+		
 		remote_rusage.ru_stime.tv_sec = (time_t) real_value;
 		jobAd->Assign(ATTR_JOB_REMOTE_SYS_CPU, real_value);
 	}
 
 	if( update_ad->LookupFloat(ATTR_JOB_REMOTE_USER_CPU, real_value) ) {
+		double prevUsage;
+		if (!jobAd->LookupFloat(ATTR_JOB_REMOTE_USER_CPU, prevUsage)) {
+			prevUsage = 0.0;
+		}
+
+		// Remote cpu usage should be strictly increasing
+		if (real_value > prevUsage) {
+			double prevTotalUsage = 0.0;
+			jobAd->LookupFloat(ATTR_JOB_CUMULATIVE_REMOTE_USER_CPU, prevTotalUsage);
+			jobAd->Assign(ATTR_JOB_CUMULATIVE_REMOTE_USER_CPU, prevTotalUsage + (real_value - prevUsage));
+		}
+		
 		remote_rusage.ru_utime.tv_sec = (time_t) real_value;
 		jobAd->Assign(ATTR_JOB_REMOTE_USER_CPU, real_value);
 	}
@@ -1404,6 +1431,9 @@ RemoteResource::updateFromStarter( ClassAd* update_ad )
 		}
 	}
 
+    jobAd->CopyAttribute(ATTR_NETWORK_IN, update_ad);
+    jobAd->CopyAttribute(ATTR_NETWORK_OUT, update_ad);
+
     jobAd->CopyAttribute(ATTR_BLOCK_READ_KBYTES, update_ad);
     jobAd->CopyAttribute(ATTR_BLOCK_WRITE_KBYTES, update_ad);
     jobAd->CopyAttribute("Recent" ATTR_BLOCK_READ_KBYTES, update_ad);
@@ -1903,6 +1933,13 @@ RemoteResource::resourceExit( int reason_for_exit, int exit_status )
 	}
 }
 
+void
+RemoteResource::incrementJobCompletionCount() {
+	int numJobCompletions = 0;
+	jobAd->LookupInteger( ATTR_NUM_JOB_COMPLETIONS, numJobCompletions );
+	++numJobCompletions;
+	jobAd->Assign( ATTR_NUM_JOB_COMPLETIONS, numJobCompletions );
+}
 
 void 
 RemoteResource::setResourceState( ResourceState s )
@@ -2578,6 +2615,8 @@ RemoteResource::checkX509Proxy( void )
 				   &voname, &firstfqan, &quoted_DN_and_FQAN );
 
 	jobAd->Assign(ATTR_X509_USER_PROXY_EXPIRATION, proxy_expiration_time);
+	/* These are secure attributes, only settable by the schedd.
+	 * Assume they won't change during job execution.
 	if ( proxy_subject && *proxy_subject ) {
 		jobAd->Assign(ATTR_X509_USER_PROXY_SUBJECT, proxy_subject);
 	}
@@ -2590,6 +2629,7 @@ RemoteResource::checkX509Proxy( void )
 	if ( quoted_DN_and_FQAN && *quoted_DN_and_FQAN ) {
 		jobAd->Assign(ATTR_X509_USER_PROXY_FQAN, quoted_DN_and_FQAN);
 	}
+	*/
 	free( proxy_subject );
 	free( voname );
 	free( firstfqan );
@@ -2598,7 +2638,10 @@ RemoteResource::checkX509Proxy( void )
 	// first, do the DN and expiration time, which all proxies have
 	char* proxy_subject = x509_proxy_identity_name(proxy_path.Value());
 	time_t proxy_expiration_time = x509_proxy_expiration_time(proxy_path.Value());
+	/* This is a secure attribute, only settable by the schedd.
+	 * Assume it won't change during job execution.
 	jobAd->Assign(ATTR_X509_USER_PROXY_SUBJECT, proxy_subject);
+	*/
 	jobAd->Assign(ATTR_X509_USER_PROXY_EXPIRATION, proxy_expiration_time);
 	if (proxy_subject) {
 		free(proxy_subject);
@@ -2620,9 +2663,12 @@ RemoteResource::checkX509Proxy( void )
 		if (IsDebugVerbose(D_SECURITY)) {
 			dprintf(D_SECURITY, "VOMS attributes were found\n");
 		}
+		/* These are secure attributes, only settable by the schedd.
+		 * Assume they won't change during job execution.
 		jobAd->Assign(ATTR_X509_USER_PROXY_VONAME, voname);
 		jobAd->Assign(ATTR_X509_USER_PROXY_FIRST_FQAN, firstfqan);
 		jobAd->Assign(ATTR_X509_USER_PROXY_FQAN, quoted_DN_and_FQAN);
+		*/
 		free(voname);
 		free(firstfqan);
 		free(quoted_DN_and_FQAN);
@@ -2705,6 +2751,16 @@ RemoteResource::allowRemoteWriteAttributeAccess( const std::string &name )
 	{
 		response = m_delayed_update_prefix.contains_anycase_withwildcard(name.c_str());
 	}
+
+	// Since this function is called to see if a user job is allowed to update
+	// the given attribute (via mechanisms like chirp), make certain we disallow 
+	// protected attributes. We do this here because the schedd may allow it to happen 
+	// since the shadow will likely be connected as a queue super user with access
+	// to modify protected attributes.
+	if (response && m_protected_attrs.find(name) != m_protected_attrs.end()) {
+		response = false;
+	}
+
 	// Do NOT log failures -- unfortunately, this routine doesn't know about the other
 	// whitelisted attributes (for example, ExitCode)
 	if (response) logRemoteAccessCheck(response,"write access to attribute",name.c_str());
