@@ -268,8 +268,8 @@ class RemoteResource : public Service {
 	void initFileTransfer();
 
 	virtual void resourceExit( int reason_for_exit, int exit_status );
-
 	virtual void updateFromStarter( ClassAd* update_ad );
+	virtual void incrementJobCompletionCount();
 
 	int64_t getImageSize( int64_t & memory_usage_out, int64_t & rss, int64_t & pss  ) { 
 		memory_usage_out = memory_usage_mb;
@@ -411,6 +411,7 @@ class RemoteResource : public Service {
 	bool m_want_streaming_io;
 	bool m_want_delayed;
 	StringList m_delayed_update_prefix;
+	classad::References m_protected_attrs;
 
 		// If we specially create a security session for file transfer,
 		// this records all the information we need to know about it.
