@@ -1037,7 +1037,7 @@ MachAttributes::publish( ClassAd* cp, amask_t how_much)
 #endif
 
 	// Advertise processor flags.
-	const char * pflags = sysapi_processor_flags();
+	const char * pflags = sysapi_processor_flags()->processor_flags;
 	if (pflags) {
 		char * savePointer = NULL;
 		char * processor_flags = strdup( pflags );
@@ -1050,6 +1050,18 @@ MachAttributes::publish( ClassAd* cp, amask_t how_much)
 		}
 		free( processor_flags );
 	}
+	int modelNo = -1;
+	if ((modelNo = sysapi_processor_flags()->model_no) > 0) {
+		cp->Assign(ATTR_CPU_MODEL_NUMBER, modelNo);
+	}
+	int family = -1;
+	if ((family = sysapi_processor_flags()->family) > 0) {
+		cp->Assign(ATTR_CPU_FAMILY, family);
+	}
+	int cache = -1;
+	if ((cache = sysapi_processor_flags()->cache) > 0) {
+		cp->Assign(ATTR_CPU_CACHE_SIZE, cache);
+	}
 }
 
 void
@@ -1338,7 +1350,10 @@ CpuAttributes::compute( amask_t how_much )
 			free(disk_as_str);
 			c_total_disk = temp_disk;
 		} else {
-			c_total_disk = sysapi_disk_space(rip->executeDir());
+			// only calculate total_disk once
+			if ((c_total_disk == 0) || (param_boolean("STARTD_RECOMPUTE_DISK_FREE", false))) {
+				c_total_disk = sysapi_disk_space(rip->executeDir());
+			}
 		}
 
 		if (IS_UPDATE(how_much)) {
