@@ -27,6 +27,7 @@
 #include "VMRegister.h"
 #include "overflow.h"
 #include <math.h>
+#include "credmon_interface.h"
 
 #include "slot_builder.h"
 
@@ -38,6 +39,7 @@ ResMgr::ResMgr() : extras_classad( NULL )
 	config_classad = NULL;
 	up_tid = -1;
 	poll_tid = -1;
+	m_cred_sweep_tid = -1;
 
 	draining = false;
 	draining_is_graceful = false;
@@ -1063,7 +1065,29 @@ ResMgr::send_update( int cmd, ClassAd* public_ad, ClassAd* private_ad,
 		// Increment the resmgr's count of updates.
 	num_updates++;
 		// Actually do the updates, and return the # of updates sent.
-	return daemonCore->sendUpdates(cmd, public_ad, private_ad, nonblock);
+	int res = daemonCore->sendUpdates(cmd, public_ad, private_ad, nonblock);
+
+	static bool first_time = true;
+	if (first_time) {
+		first_time = false;
+		dprintf( D_ALWAYS, "Initial update sent to collector(s)\n");
+		if ( ! param_boolean("STARTD_SEND_READY_AFTER_FIRST_UPDATE", true)) return res;
+
+		// send a DC_SET_READY message to the master to indicate the STARTD is ready to go
+		std::string master_sinful(daemonCore->InfoCommandSinfulString(-2));
+		if ( ! master_sinful.empty()) {
+			dprintf( D_ALWAYS, "Sending DC_SET_READY message to master %s\n", master_sinful.c_str());
+			ClassAd readyAd;
+			readyAd.Assign("DaemonPID", getpid());
+			readyAd.Assign("DaemonName", "STARTD"); // fix to use the environment
+			readyAd.Assign("DaemonState", "Ready");
+			classy_counted_ptr<Daemon> dmn = new Daemon(DT_ANY,master_sinful.c_str());
+			classy_counted_ptr<ClassAdMsg> msg = new ClassAdMsg(DC_SET_READY, readyAd);
+			dmn->sendMsg(msg.get());
+		}
+	}
+
+	return res;
 }
 
 
@@ -1259,11 +1283,14 @@ ResMgr::publish( ClassAd* cp, amask_t how_much )
 void
 ResMgr::updateExtrasClassAd( ClassAd * cap ) {
 	// It turns out to be colossal pain to use the ClassAd for persistence.
-	static std::set< std::string > offlineUniverses;
+	static classad::References offlineUniverses;
 
 	if( ! cap ) { return; }
 	if( ! extras_classad ) { extras_classad = new ClassAd(); }
 
+	int topping = 0;
+	int obsolete_univ = false;
+
 	//
 	// The startd maintains the set offline universes, and the offline
 	// universe timestamps.
@@ -1297,7 +1324,13 @@ ResMgr::updateExtrasClassAd( ClassAd * cap ) {
 		if( uo != attr ) { continue; }
 
 		std::string universeName( attr + 3 );
-		if( CondorUniverseNumber( universeName.c_str() ) == 0 ) { continue; }
+		int univ = CondorUniverseInfo( universeName.c_str(), &topping, &obsolete_univ );
+		if( univ == 0 || obsolete_univ) {
+			continue;
+		}
+
+		// convert universe name to canonical form
+		universeName = CondorUniverseOrToppingName(univ, topping);
 
 		std::string reasonTime = universeName + "OfflineTime";
 		std::string reasonName = universeName + "OfflineReason";
@@ -1324,18 +1357,18 @@ ResMgr::updateExtrasClassAd( ClassAd * cap ) {
 	// Construct the OfflineUniverses attribute and set it in extras_classad.
 	//
 	std::string ouListString;
-	if( offlineUniverses.empty() ) {
-		ouListString = "{}";
-	} else {
-		std::set< std::string >::const_iterator i = offlineUniverses.begin();
-		for( ; i != offlineUniverses.end(); ++i ) {
-			int offlineUniverseNumber = CondorUniverseNumber( i->c_str() );
-			formatstr( ouListString, "%s\"%s\", %d, ", ouListString.c_str(), i->c_str(), offlineUniverseNumber );
-		}
-		// Remove the trailing ", ".
-		ouListString.erase( ouListString.length() - 2 );
-		ouListString = "{ " + ouListString + " }";
+	ouListString.reserve(10 + offlineUniverses.size()*20);
+	ouListString = "{";
+	classad::References::const_iterator i = offlineUniverses.begin();
+	for( ; i != offlineUniverses.end(); ++i ) {
+		int univ = CondorUniverseInfo( i->c_str(), &topping, &obsolete_univ );
+		if ( ! univ || obsolete_univ) { continue; }
+
+		if (ouListString.size() > 1) { ouListString += ", "; }
+		formatstr_cat(ouListString, "\"%s\"", i->c_str() );
+		if ( ! topping) { formatstr_cat(ouListString, ",%d", univ ); }
 	}
+	ouListString += "}";
 	dprintf( D_ALWAYS, "OfflineUniverses = %s\n", ouListString.c_str() );
 	extras_classad->AssignExpr( "OfflineUniverses", ouListString.c_str() );
 }
@@ -1450,6 +1483,35 @@ ResMgr::check_polling( void )
 }
 
 
+void
+ResMgr::sweep_timer_handler( void )
+{
+	dprintf(D_FULLDEBUG, "STARTD: calling and resetting sweep_timer_handler()\n");
+#ifndef WIN32
+	credmon_sweep_creds();
+#endif  // WIN32
+	int sec_cred_sweep_interval = param_integer("SEC_CREDENTIAL_SWEEP_INTERVAL", 30);
+	daemonCore->Reset_Timer (m_cred_sweep_tid, sec_cred_sweep_interval, sec_cred_sweep_interval);
+}
+
+int
+ResMgr::start_sweep_timer( void )
+{
+	// only sweep if we have a cred dir
+	char* p = param("SEC_CREDENTIAL_DIRECTORY");
+	if(!p) {
+		return TRUE;
+	}
+
+	dprintf(D_FULLDEBUG, "STARTD: setting start_sweep_timer()\n");
+	int sec_cred_sweep_interval = param_integer("SEC_CREDENTIAL_SWEEP_INTERVAL", 30);
+	m_cred_sweep_tid = daemonCore->Register_Timer( sec_cred_sweep_interval, sec_cred_sweep_interval,
+							(TimerHandlercpp)&ResMgr::sweep_timer_handler,
+							"sweep_timer_handler", this );
+	return TRUE;
+}
+
+
 int
 ResMgr::start_update_timer( void )
 {
@@ -1531,6 +1593,12 @@ ResMgr::reset_timers( void )
 								 update_interval );
 	}
 
+	int sec_cred_sweep_interval = param_integer("SEC_CREDENTIAL_SWEEP_INTERVAL", 30);
+	if( m_cred_sweep_tid != -1 ) {
+		daemonCore->Reset_Timer( m_cred_sweep_tid, sec_cred_sweep_interval,
+								 sec_cred_sweep_interval );
+	}
+
 #if HAVE_HIBERNATION
 	resetHibernateTimer();
 #endif /* HAVE_HIBERNATE */
