@@ -46,6 +46,7 @@
 extern FILESQL *FILEObj;
 
 std::vector<SlotType> SlotType::types(10);
+static bool warned_startd_attrs_once = false; // used to prevent repetition of the warning about mixing STARTD_ATTRS and STARTD_EXPRS
 
 const char * SlotType::type_param(const char * name)
 {
@@ -162,6 +163,7 @@ const char * SlotType::type_param(const char * name)
 		types.resize(max_type_id);
 	}
 	for (size_t ix = 0; ix < types.size(); ++ix) { types[ix].clear(); }
+	warned_startd_attrs_once = false; // allow the warning about mixing STARTD_ATTRS and STARTD_EXPRS once again
 
 	Regex re;
 	int err = 0;
@@ -349,7 +351,7 @@ Resource::Resource( CpuAttributes* cap, int rid, bool multiple_slots, Resource*
 	r_suspended_for_cod = false;
 	r_hack_load_for_cod = false;
 	r_cod_load_hack_tid = -1;
-	r_pre_cod_total_load = 0.0;
+	r_pre_cod_total_load = -1.0;
 	r_pre_cod_condor_load = 0.0;
 	m_bUserSuspended = false;
 
@@ -432,7 +434,7 @@ Resource::~Resource()
 	delete r_load_queue; r_load_queue = NULL;
 	free( r_name ); r_name = NULL;
 	free( r_id_str ); r_id_str = NULL;
-	if (r_pair_name) free( r_pair_name ); r_pair_name = NULL;
+	free( r_pair_name ); r_pair_name = NULL;
 
 }
 
@@ -1258,24 +1260,23 @@ Resource::reconfig( void )
 void
 Resource::update( void )
 {
-	int timeout = 3;
-
 	if (r_no_collector_updates)
 		return;
 
-	if ( update_tid == -1 ) {
-			// Send no more than 16 ClassAds per second to help
-			// minimize the odds of overwhelming the collector
-			// on very large SMP machines.  So, we mod our resource num
-			// by 8 and add that to the timeout
-			// (why 8? since each update sends 2 ads).
-		if ( r_id > 0 ) {
-			timeout += r_id % 8;
+	// If we haven't already queued an update, queue one.  Wait three
+	// seconds before sending an update to allow the startd's state
+	// to quiesce; we'll implicitly coalesce the updates.
+	int delay = 3;
+	int updateSpreadTime = param_integer( "UPDATE_SPREAD_TIME", 0 );
+	if( update_tid == -1 ) {
+		if( r_id > 0 && updateSpreadTime > 0 ) {
+			// If we were doing rate limiting, this would be integer
+			// division, instead.
+			delay += (r_id - 1) % updateSpreadTime;
 		}
 
-		// set a timer for the update
 		update_tid = daemonCore->Register_Timer(
-						timeout,
+						delay,
 						(TimerHandlercpp)&Resource::do_update,
 						"do_update",
 						this );
@@ -1356,8 +1357,8 @@ Resource::final_update( void )
 	 * if you change here you will need to CollectorEngine::remove (AdTypes t_AddType, const ClassAd & c_query)
 	 * the IP was added to allow the collector to create a hash key to delete in O(1).
      */
-	 EscapeAdStringValue( r_name, escaped_name );
-     line.formatstr( "( TARGET.%s == \"%s\" )", ATTR_NAME, escaped_name.c_str() );
+	 QuoteAdStringValue( r_name, escaped_name );
+     line.formatstr( "( TARGET.%s == %s )", ATTR_NAME, escaped_name.c_str() );
      invalidate_ad.AssignExpr( ATTR_REQUIREMENTS, line.Value() );
      invalidate_ad.Assign( ATTR_NAME, r_name );
      invalidate_ad.Assign( ATTR_MY_ADDRESS, daemonCore->publicNetworkIpAddr());
@@ -2145,7 +2146,8 @@ Resource::publish( ClassAd* cap, amask_t mask )
 		tmp.set(param(param_name.c_str()));
 		if ( ! tmp.empty()) { slot_attrs.initializeFromString(tmp); }
 
-		// check for obsolete STARTD_EXPRS and generate a warning if set
+		// check for obsolete STARTD_EXPRS and generate a warning if both STARTD_ATTRS and STARTD_EXPRS is set.
+		if ( ! slot_attrs.isEmpty() && ! warned_startd_attrs_once)
 		{
 			MyString tname(slot_name); tname += "_STARTD_EXPRS";
 			auto_free_ptr tmp2(param(tname.c_str()));
@@ -2156,8 +2158,13 @@ Resource::publish( ClassAd* cap, amask_t mask )
 			if ( ! tmp2.empty()) {
 				dprintf(D_ALWAYS, "WARNING: config contains obsolete STARTD_EXPRS or SLOT_TYPE_n_STARTD_EXPRS which will be (partially) ignored! use STARTD_ATTRS instead.\n");
 			}
+			warned_startd_attrs_once = true;
 		}
 
+		// now append any attrs needed by HTCondor itself
+		tmp.set(param("SYSTEM_STARTD_ATTRS"));
+		if ( ! tmp.empty()) { slot_attrs.initializeFromString(tmp); }
+
 		slot_attrs.rewind();
 		for (char* attr = slot_attrs.first(); attr != NULL; attr = slot_attrs.next()) {
 			param_name.formatstr("%s_%s", slot_name.c_str(), attr);
@@ -2228,9 +2235,9 @@ Resource::publish( ClassAd* cap, amask_t mask )
 			// ClassAd, so be careful about that, too.
 		s = this->state();
 		if( s == claimed_state || s == preempting_state ) {
-			if( startd_job_exprs && r_cur && r_cur->ad() ) {
-				startd_job_exprs->rewind();
-				while( (ptr = startd_job_exprs->next()) ) {
+			if( startd_job_attrs && r_cur && r_cur->ad() ) {
+				startd_job_attrs->rewind();
+				while( (ptr = startd_job_attrs->next()) ) {
 					caInsert( cap, r_cur->ad(), ptr );
 				}
 			}
@@ -2407,12 +2414,15 @@ Resource::publish_private( ClassAd *ad )
 		// claimid-specific logic elsewhere, such as the private
 		// attributes in ClassAds.
 	if( r_pre_pre ) {
-		ad->Assign( ATTR_CAPABILITY, r_pre_pre->id() );
+		ad->Assign( ATTR_CLAIM_ID, r_pre_pre->id() );
+		ad->AssignExpr( ATTR_CAPABILITY, ATTR_CLAIM_ID );
 	}
 	else if( r_pre ) {
-		ad->Assign( ATTR_CAPABILITY, r_pre->id() );
+		ad->Assign( ATTR_CLAIM_ID, r_pre->id() );
+		ad->AssignExpr( ATTR_CAPABILITY, ATTR_CLAIM_ID );
 	} else if( r_cur ) {
-		ad->Assign( ATTR_CAPABILITY, r_cur->id() );
+		ad->Assign( ATTR_CLAIM_ID, r_cur->id() );
+		ad->AssignExpr( ATTR_CAPABILITY, ATTR_CLAIM_ID );
 	}		
 
     if (r_has_cp) {
@@ -2445,17 +2455,11 @@ Resource::makeChildClaimIds() {
 			}
 			Resource *child = (*i);
 			if (child->r_pre_pre) {
-				attrValue += '"';
-				attrValue += EscapeAdStringValue( child->r_pre_pre->id(), buf );
-				attrValue += '"';
+				attrValue += QuoteAdStringValue( child->r_pre_pre->id(), buf );
 			} else if (child->r_pre) {
-				attrValue += '"';
-				attrValue += EscapeAdStringValue( child->r_pre->id(), buf );
-				attrValue += '"';
+				attrValue += QuoteAdStringValue( child->r_pre->id(), buf );
 			} else if (child->r_cur) {
-				attrValue += '"';
-				attrValue += EscapeAdStringValue( child->r_cur->id(), buf );
-				attrValue += '"';
+				attrValue += QuoteAdStringValue( child->r_cur->id(), buf );
 			}
 		}
 		attrValue += "}";
@@ -2750,7 +2754,7 @@ Resource::beginCODLoadHack( void )
 		// only if we've been free of COD for over a minute (and
 		// therefore, we're completely out of COD-load hack), do we
 		// want to record the real system load as the "pre-COD" load.
-	if( r_pre_cod_total_load > 0.0 ) {
+	if( r_pre_cod_total_load < 0.0 ) {
 		r_pre_cod_total_load = r_attr->total_load();
 		r_pre_cod_condor_load = r_attr->condor_load();
 	} else {
@@ -2790,7 +2794,7 @@ Resource::endCODLoadHack( void )
 
 		// now, reset all the COD-load hack state
 	r_hack_load_for_cod = false;
-	r_pre_cod_total_load = 0.0;
+	r_pre_cod_total_load = -1.0;
 	r_pre_cod_condor_load = 0.0;
 }
 
@@ -3341,7 +3345,7 @@ Resource * initialize_resource(Resource * rip, ClassAd * req_classad, Claim* &le
                 if (j != consumption.begin()) type += " ";
                 if (MATCH == strcasecmp(j->first.c_str(),"disk")) {
                     // if it weren't for special cases, we'd have no cases at all
-                    type.formatstr_cat("disk=%d%%", max(0, (int)ceil(100 * j->second / (double)rip->r_attr->get_total_disk())));
+                    type.formatstr_cat("disk=%.1f%%", max(0, (0.1 + 100 * j->second / (double)rip->r_attr->get_total_disk())));
                 } else {
                     type.formatstr_cat("%s=%d", j->first.c_str(), int(j->second));
                 }
@@ -3383,8 +3387,8 @@ Resource * initialize_resource(Resource * rip, ClassAd * req_classad, Claim* &le
                     return NULL;
                 }
             }
-            type.formatstr_cat( "disk=%d%%",
-                                max((int) ceil((disk / (double) rip->r_attr->get_total_disk()) * 100), 1) );
+            type.formatstr_cat( "disk=%.1f%%",
+                                max((0.1 + disk / (double) rip->r_attr->get_total_disk() * 100), 0.001) );
 
 
                 // Look to see how much swap is being requested.
