@@ -31,6 +31,14 @@ class Service;
 */
 int command_handler( Service*, int, Stream* );
 
+/*
+  command_handler reads a ClaimId and classad off the socket, tries to find
+  the resource that the ClaimId belongs to, does a switch on the
+  command, and calls the proper action with the pointer to the
+  resource the ClaimId matched.
+*/
+int command_with_opts_handler( Service*, int, Stream* );
+
 /*
   ACTIVATE_CLAIM is a special case of the above case, in that more
   stuff is sent over the wire than just the command and the
@@ -110,7 +118,7 @@ int match_info( Resource*, char* );
 int request_claim( Resource*, Claim *, char*, Stream* ); 
 
 // Accept claim from schedd agent
-bool accept_request_claim( Resource* , Claim * = NULL ); 
+bool accept_request_claim( Resource* , Claim * = NULL, bool and_pair = false );
 
 // Activate a claim with a given starter
 int activate_claim( Resource*, Stream* ); 
