@@ -28,11 +28,16 @@ class StartdCronJob;
 class StartdNamedClassAd : public NamedClassAd
 {
   public:
-	StartdNamedClassAd( const char *name, StartdCronJob &job );
+	StartdNamedClassAd( const char *name, StartdCronJob &job, ClassAd * ad = NULL );
 	~StartdNamedClassAd( void ) { };
 	bool InSlotList( unsigned slot ) const;
+	StartdNamedClassAd * NewPeer( const char * name, ClassAd * ad = NULL ) { return new StartdNamedClassAd(name, m_job, ad); }
+	bool IsJob(StartdCronJob * job) const { return &m_job == job; }
+	bool ShouldMergeInto(ClassAd * merge_into, const char ** pattr_used);
+	bool MergeInto(ClassAd *merge_to);
 
   private:
+	static AttrNameSet dont_merge_attrs;
 	StartdCronJob	&m_job;
 };
 
