@@ -54,7 +54,7 @@ void WorkloadManager::query_workloads(char *script)
 	args.AppendArg(script);
 
 	priv = set_root_priv();
-	fin = my_popen(args, "r", TRUE);
+	fin = my_popen(args, "r", MY_POPEN_OPT_WANT_STDERR);
 
 	if (fin == NULL) {
 		EXCEPT("Can't execute %s", script);
