@@ -906,6 +906,33 @@ REMOTE_CONDOR_ulog( ClassAd *ad )
 	return 0;
 }
 
+int
+REMOTE_CONDOR_phase( char *phase )
+{
+	int result = 0;
+
+	dprintf ( D_SYSCALLS, "Doing CONDOR_phase\n" );
+
+	CurrentSysCall = CONDOR_phase;
+
+	if( ! phase ) {
+		EXCEPT( "CONDOR_phase called with NULL phase!" );
+		return -1;
+	}
+
+	syscall_sock->encode();
+	result = syscall_sock->code(CurrentSysCall);
+	ASSERT( result );
+	result = syscall_sock->code(phase);
+	ASSERT( result );
+	result = syscall_sock->end_of_message();
+	ASSERT( result );
+
+	//NOTE: we expect no response.
+
+	return 0;
+}
+
 int
 REMOTE_CONDOR_get_job_attr(char *  attrname , char *& expr)
 {
@@ -2153,4 +2180,45 @@ REMOTE_CONDOR_utime(char *path, int actime, int modtime)
 	return rval;
 }
 
+int
+REMOTE_CONDOR_dprintf_stats(char *message) 
+{
+	int rval = -1, result = 0;
+	condor_errno_t terrno;
+
+	const CondorVersionInfo *shadowVer = syscall_sock->get_peer_version();
+	if (!shadowVer->built_since_version(8,5,8)) {
+		dprintf ( D_SYSCALLS, "NOT Doing CONDOR_dprintf_stats, shadow is too old\n" );
+		return 0;
+	}
+
+	dprintf ( D_SYSCALLS, "Doing CONDOR_dprintf_stats\n" );
+	
+	CurrentSysCall = CONDOR_dprintf_stats;
+
+	syscall_sock->encode();
+	result = ( syscall_sock->code(CurrentSysCall) );
+	ON_ERROR_RETURN( result );
+	result = ( syscall_sock->code(message));
+	ON_ERROR_RETURN( result );
+	result = ( syscall_sock->end_of_message() );
+	ON_ERROR_RETURN( result );
+	syscall_sock->decode();
+	result = ( syscall_sock->code(rval) );
+	ON_ERROR_RETURN( result );
+	if( rval < 0 ) {
+		result = ( syscall_sock->code(terrno) );
+		ON_ERROR_RETURN( result );
+		result = ( syscall_sock->end_of_message() );
+		ON_ERROR_RETURN( result );
+		errno = terrno;
+		dprintf ( D_SYSCALLS, "Return val problem, errno = %d\n", errno );
+		return rval;
+	}
+	result = ( syscall_sock->end_of_message() );
+	ON_ERROR_RETURN( result );
+	return rval;
+}
+
+
 } // extern "C"
