@@ -20,14 +20,54 @@
 #include "condor_common.h"
 #include "condor_classad.h"
 #include "condor_config.h"
+#include "directory.h"
 #include "docker_proc.h"
 #include "starter.h"
 #include "condor_holdcodes.h"
 #include "docker-api.h"
+#include "condor_daemon_client.h"
 
 extern CStarter *Starter;
 
-static void buildExtraVolumes(std::list<std::string> &extras);
+static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, ClassAd &jobAd);
+
+static bool handleFTL(int error) {
+	if (error != 0) {
+		// dprintf( D_ALWAYS, "Failed to launch Docker universe job (%s).\n", error );
+	}
+
+	//
+	// If we failed to launch the job (as opposed to aborted the takeoff
+	// because there was something wrong with the payload), we also need
+	// to force the startd to advertise this fact so other jobs can avoid
+	// this machine.
+	//
+	DCStartd startd( (const char * const)NULL, (const char * const)NULL );
+	if( ! startd.locate() ) {
+		dprintf( D_ALWAYS | D_FAILURE, "Unable to locate startd: %s\n", startd.error() );
+		return false;
+	}
+
+	//
+	// The startd will update the list 'OfflineUniverses' for us.
+	//
+	ClassAd update;
+	if (error) {
+		update.Assign( ATTR_HAS_DOCKER, false );
+		update.Assign( "DockerOfflineReason", error );
+	} else {
+		update.Assign( ATTR_HAS_DOCKER, true );
+	}
+
+	ClassAd reply;
+	if( ! startd.updateMachineAd( &update, &reply ) ) {
+		dprintf( D_ALWAYS | D_FAILURE, "Unable to update machine ad: %s\n", startd.error() );
+		return false;
+	}
+
+	return true;
+}
+
 
 //
 // TODO: Allow the use of HTCondor file-transfer to provide the image.
@@ -39,7 +79,7 @@ static void buildExtraVolumes(std::list<std::string> &extras);
 // the full container ID as (part of) the cgroup identifier(s).
 //
 
-DockerProc::DockerProc( ClassAd * jobAd ) : VanillaProc( jobAd ) { }
+DockerProc::DockerProc( ClassAd * jobAd ) : VanillaProc( jobAd ), updateTid(-1) { }
 
 DockerProc::~DockerProc() { }
 
@@ -66,10 +106,12 @@ int DockerProc::StartJob() {
 	// introducing a version dependency -- assume the executable was
 	// transferred unless it was explicitly noted otherwise.
 	//
+	
 	bool transferExecutable = true;
 	JobAd->LookupBool( ATTR_TRANSFER_EXECUTABLE, transferExecutable );
 	if( transferExecutable ) {
-		command = sandboxPath + "/" + command;
+		// Transfered executables are still renamed (sadly)
+		command = "./condor_exec.exe";
 	}
 
 	ArgList args;
@@ -139,17 +181,26 @@ int DockerProc::StartJob() {
 	ClassAd *machineAd = Starter->jic->machClassAd();
 
 	std::list<std::string> extras;
-	buildExtraVolumes(extras);
+	buildExtraVolumes(extras, *machineAd, *JobAd);
 
-	int rv = DockerAPI::run( *machineAd, containerName, imageID, command, args, job_env, sandboxPath, extras, JobPid, childFDs, err );
+	// The following line is for condor_who to parse
+	dprintf( D_ALWAYS, "About to exec docker:%s\n", command.c_str());
+	int rv = DockerAPI::run( *machineAd, *JobAd, containerName, imageID, command, args, job_env, sandboxPath, extras, JobPid, childFDs, err );
 	if( rv < 0 ) {
 		dprintf( D_ALWAYS | D_FAILURE, "DockerAPI::run( %s, %s, ... ) failed with return value %d\n", imageID.c_str(), command.c_str(), rv );
+		handleFTL(rv);
 		return FALSE;
 	}
 	dprintf( D_FULLDEBUG, "DockerAPI::run() returned pid %d\n", JobPid );
+	// The following line is for condor_who to parse
+	dprintf( D_ALWAYS, "Create_Process succeeded, pid=%d\n", JobPid);
+	// If we manage to start the Docker job, clear the offline state for docker universe
+	handleFTL(0);
 
-
-	// TODO: Start a timer to poll for job usage updates.
+	// Start a timer to poll for job usage updates.
+	updateTid = daemonCore->Register_Timer(2, 
+			20, (TimerHandlercpp)&DockerProc::getStats, 
+				"DockerProc::getStats",this);
 
 	++num_pids; // Used by OsProc::PublishUpdateAd().
 	return TRUE;
@@ -158,7 +209,9 @@ int DockerProc::StartJob() {
 
 bool DockerProc::JobReaper( int pid, int status ) {
 	TemporaryPrivSentry sentry(PRIV_ROOT);
-	dprintf( D_ALWAYS, "DockerProc::JobReaper()\n" );
+	dprintf( D_FULLDEBUG, "DockerProc::JobReaper()\n" );
+
+	daemonCore->Cancel_Timer(updateTid);
 
 	//
 	// This should mean that the container has terminated.
@@ -239,7 +292,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 			int memory;
 			machineAd->LookupInteger(ATTR_MEMORY, memory);
 			std::string message;
-			formatstr(message, "Docker job exhaused %d Mb memory", memory);
+			formatstr(message, "Docker job has gone over memory limit of %d Mb", memory);
 			dprintf(D_ALWAYS, "%s, going on hold\n", message.c_str());
 
 			
@@ -303,7 +356,7 @@ bool DockerProc::JobReaper( int pid, int status ) {
 // JobExit() is called after file transfer.
 //
 bool DockerProc::JobExit() {
-	dprintf( D_ALWAYS, "DockerProc::JobExit()\n" );
+	dprintf( D_ALWAYS, "DockerProc::JobExit() container '%s'\n", containerName.c_str() );
 
 	{
 	TemporaryPrivSentry sentry(PRIV_ROOT);
@@ -335,7 +388,7 @@ bool DockerProc::JobExit() {
 }
 
 void DockerProc::Suspend() {
-	dprintf( D_ALWAYS, "DockerProc::Suspend()\n" );
+	dprintf( D_ALWAYS, "DockerProc::Suspend() container '%s'\n", containerName.c_str() );
 	int rv = 0;
 
 	{
@@ -353,7 +406,7 @@ void DockerProc::Suspend() {
 
 
 void DockerProc::Continue() {
-	dprintf( D_ALWAYS, "DockerProc::Continue()\n" );
+	dprintf( D_ALWAYS, "DockerProc::Continue() container '%s'\n", containerName.c_str() );
 	int rv = 0;	
 
 	if( is_suspended ) {
@@ -377,7 +430,7 @@ void DockerProc::Continue() {
 //
 
 bool DockerProc::Remove() {
-	dprintf( D_ALWAYS, "DockerProc::Remove()\n" );
+	dprintf( D_ALWAYS, "DockerProc::Remove() container '%s'\n", containerName.c_str() );
 
 	if( is_suspended ) { Continue(); }
 	requested_exit = true;
@@ -396,7 +449,7 @@ bool DockerProc::Remove() {
 
 
 bool DockerProc::Hold() {
-	dprintf( D_ALWAYS, "DockerProc::Hold()\n" );
+	dprintf( D_ALWAYS, "DockerProc::Hold() container '%s'\n", containerName.c_str() );
 
 	if( is_suspended ) { Continue(); }
 	requested_exit = true;
@@ -415,7 +468,7 @@ bool DockerProc::Hold() {
 
 
 bool DockerProc::ShutdownGraceful() {
-	dprintf( D_ALWAYS, "DockerProc::ShutdownGraceful()\n" );
+	dprintf( D_ALWAYS, "DockerProc::ShutdownGraceful() container '%s'\n", containerName.c_str() );
 
 	if( containerName.empty() ) {
 		// We haven't started a Docker yet, probably because we're still
@@ -438,7 +491,7 @@ bool DockerProc::ShutdownGraceful() {
 
 
 bool DockerProc::ShutdownFast() {
-	dprintf( D_ALWAYS, "DockerProc::ShutdownFast()\n" );
+	dprintf( D_ALWAYS, "DockerProc::ShutdownFast() container '%s'\n", containerName.c_str() );
 
 	if( containerName.empty() ) {
 		// We haven't started a Docker yet, probably because we're still
@@ -463,11 +516,14 @@ bool DockerProc::ShutdownFast() {
 }
 
 
-bool DockerProc::PublishUpdateAd( ClassAd * ad ) {
-	dprintf( D_ALWAYS, "DockerProc::PublishUpdateAd()\n" );
+int
+DockerProc::getStats(int /*tid*/) {
+	DockerAPI::stats( containerName, memUsage, netIn, netOut, userCpu, sysCpu);
+	return true;
+}
 
-	// TODO: get usage from procd somehow.  Set num_pids, if we can.  See
-	// VanillaProc::PublishUpdateAd() for the attributes to set.
+bool DockerProc::PublishUpdateAd( ClassAd * ad ) {
+	dprintf( D_FULLDEBUG, "DockerProc::PublishUpdateAd() container '%s'\n", containerName.c_str() );
 
 	//
 	// If we want to use the existing reporting code (probably a good
@@ -485,19 +541,29 @@ bool DockerProc::PublishUpdateAd( ClassAd * ad ) {
 	// or set them during our status polling.
 	//
 
+	if (memUsage > 0) {
+		// Set RSS, Memory and ImageSize to same values, best we have
+		ad->Assign(ATTR_RESIDENT_SET_SIZE, int(memUsage / 1024));
+		ad->Assign(ATTR_MEMORY_USAGE, int(memUsage / (1024 * 1024)));
+		ad->Assign(ATTR_IMAGE_SIZE, int(memUsage / (1024 * 1024)));
+		ad->Assign(ATTR_NETWORK_IN, double(netIn) / (1000 * 1000));
+		ad->Assign(ATTR_NETWORK_OUT, double(netOut) / (1000 * 1000));
+		ad->Assign(ATTR_JOB_REMOTE_USER_CPU, (int) (userCpu / (1000l * 1000l * 1000l)));
+		ad->Assign(ATTR_JOB_REMOTE_SYS_CPU, (int) (sysCpu / (1000l * 1000l * 1000l)));
+	}
 	return OsProc::PublishUpdateAd( ad );
 }
 
 
 // TODO: Implement.
 void DockerProc::PublishToEnv( Env * /* env */ ) {
-	dprintf( D_ALWAYS, "DockerProc::PublishToEnv()\n" );
+	dprintf( D_FULLDEBUG, "DockerProc::PublishToEnv()\n" );
 	return;
 }
 
 
 bool DockerProc::Detect() {
-	dprintf( D_ALWAYS, "DockerProc::Detect()\n" );
+	dprintf( D_FULLDEBUG, "DockerProc::Detect()\n" );
 
 	//
 	// To turn off Docker, unset DOCKER.  DockerAPI::detect() will fail
@@ -511,7 +577,7 @@ bool DockerProc::Detect() {
 }
 
 bool DockerProc::Version( std::string & version ) {
-	dprintf( D_ALWAYS, "DockerProc::Version()\n" );
+	dprintf( D_FULLDEBUG, "DockerProc::Version()\n" );
 
 	//
 	// To turn off Docker, unset DOCKER.  DockerAPI::version() will fail
@@ -520,13 +586,43 @@ bool DockerProc::Version( std::string & version ) {
 
 	CondorError err;
 	bool foundVersion = DockerAPI::version( version, err ) == 0;
+	if (foundVersion) {
+		dprintf( D_ALWAYS, "DockerProc::Version() found version '%s'\n", version.c_str() );
+	}
 
 	return foundVersion;
 }
 
 // Generate a list of strings that are suitable arguments to
 // docker run --volume
-static void buildExtraVolumes(std::list<std::string> &extras) {
+static void buildExtraVolumes(std::list<std::string> &extras, ClassAd &machAd, ClassAd &jobAd) {
+	// Bind mount items from MOUNT_UNDER_SCRATCH into working directory
+	std::string scratchNames;
+	if (!param_eval_string(scratchNames, "MOUNT_UNDER_SCRATCH", "", &jobAd)) {
+		// If not an expression, maybe a literal
+		param(scratchNames, "MOUNT_UNDER_SCRATCH");
+	} 
+
+	if (scratchNames.length() > 0) {
+		std::string workingDir = Starter->GetWorkingDir();
+		StringList sl(scratchNames.c_str());
+		sl.rewind();
+		char *scratchName = 0;
+			// Foreach scratch name...
+		while ( (scratchName=sl.next()) ) {
+			char * hostDir = dirscat(workingDir.c_str(), scratchName);
+			std::string volumePath;
+			volumePath.append(hostDir).append(":").append(scratchName);
+			if (mkdir_and_parents_if_needed( hostDir, S_IRWXU, PRIV_USER )) {
+				extras.push_back(volumePath);
+				dprintf(D_ALWAYS, "Adding %s as a docker volume to mount under scratch\n", volumePath.c_str());
+			} else {
+				dprintf(D_ALWAYS, "Failed to create scratch directory %s\n", hostDir);
+			}
+			delete [] hostDir; hostDir = NULL;
+		}
+	}
+
 	// These are the ones the administrator wants unconditionally mounted
 	char *volumeNames = param("DOCKER_MOUNT_VOLUMES");
 	if (!volumeNames) {
@@ -553,9 +649,30 @@ static void buildExtraVolumes(std::list<std::string> &extras) {
 				free(volumePath);
 				volumePath = volumePath2;
 			}
-			extras.push_back(volumePath);
-			dprintf(D_ALWAYS, "Adding %s as a docker volume to mount\n", volumePath);
+
+			// If there's a DOCKER_VOLUME_DIR_XXX_MOUNT_IF
+			// param, and it evals to true, add it to the list
+			// if there's no param, still add it
+			// If the param exists, but doesn't eval to true, don't
+			std::string paramNameConst = paramName + "_MOUNT_IF";
+			char *mountIfValue = param(paramNameConst.c_str());
+
+			if (mountIfValue == NULL) {
+				// there's no MOUNT_IF, assume true
+				extras.push_back(volumePath);
+				dprintf(D_ALWAYS, "Adding %s as a docker volume to mount\n", volumePath);
+			} else if (param_boolean(paramNameConst.c_str(), false /*deflt*/,
+				false /*do_log*/, &machAd, &jobAd)) {
+
+				// There is a MOUNT_IF, and it is true
+				extras.push_back(volumePath);
+				dprintf(D_ALWAYS, "Adding %s as a docker volume to mount\n", volumePath);
+			} else {
+				// There is a MOUNT_IF, and it is false/undef
+				dprintf(D_ALWAYS, "Not adding %s as a docker volume to mount -- ...MOUNT_IF evaled to false.\n", volumePath);
+			}
 			free(volumePath);
+			free(mountIfValue);
 		} else {
 			dprintf(D_ALWAYS, "WARNING: DOCKER_VOLUME_DIR_%s is missing in config file.\n", volumeName);
 		}
