@@ -244,7 +244,7 @@ public:
 	bool recordDelayedUpdate( const std::string &name, const classad::ExprTree &expr );
 
 		/* Return an attribute from the combination of the delayed ad and the starter */
-	std::auto_ptr<classad::ExprTree> getDelayedUpdate( const std::string &name );
+	std::unique_ptr<classad::ExprTree> getDelayedUpdate( const std::string &name );
 
 	virtual bool wroteChirpConfig() { return m_wrote_chirp_config; }
 	virtual const std::string chirpConfigFilename() { return m_chirp_config_filename; }
@@ -407,6 +407,9 @@ private:
 		// for file transfer and reconnect, do it.
 	void initMatchSecuritySession();
 
+		/// If the job ad says so, acquire user credentials
+	bool initUserCredentials();
+
 		/** Compare our own UIDDomain vs. where the job came from.  We
 			check in the job ClassAd for ATTR_UID_DOMAIN and compare
 			it to info we have about the shadow and the local machine.
@@ -434,6 +437,10 @@ private:
 		// The proxy is about to expire, do something!
 	void proxyExpiring();
 
+	bool refreshSandboxCredentials();
+
+	bool shadowDisconnected() { return syscall_sock_lost_time > 0; };
+
 		// // // // // // // //
 		// Private Data Members
 		// // // // // // // //
@@ -443,6 +450,10 @@ private:
 		/** The version of the shadow if known; otherwise NULL */
 	CondorVersionInfo* shadow_version;
 
+		/// timer id of the credential checking timer
+	int m_refresh_sandbox_creds_tid;
+	time_t m_sandbox_creds_last_update;
+
 		/// timer id of the proxy expiration timer
 	int m_proxy_expiration_tid;
 
