@@ -53,6 +53,7 @@ JobInfoCommunicator::JobInfoCommunicator()
 	job_remote_iwd = NULL;
 	job_output_ad_file = NULL;
 	job_output_ad_is_stdout = false;
+	job_KRB5CCNAME = NULL;
 	requested_exit = false;
 	fast_exit = false;
 	graceful_exit = false;
@@ -103,6 +104,9 @@ JobInfoCommunicator::~JobInfoCommunicator()
 	if( job_output_ad_file ) {
 		free( job_output_ad_file );
 	}
+	if( job_KRB5CCNAME ) {
+		free( job_KRB5CCNAME );
+	}
 #if HAVE_JOB_HOOKS
     if (m_hook_mgr) {
         delete m_hook_mgr;
@@ -395,6 +399,16 @@ JobInfoCommunicator::setOutputAdFile( const char* path )
 	job_output_ad_file = strdup( path );
 }
 
+void
+JobInfoCommunicator::setKRB5CCNAME( const char* path )
+{
+	if( job_KRB5CCNAME ) {
+		free( job_KRB5CCNAME );
+	}
+	job_KRB5CCNAME = strdup( path );
+}
+
+
 
 bool
 JobInfoCommunicator::writeOutputAdFile( ClassAd* ad )
@@ -421,7 +435,7 @@ JobInfoCommunicator::writeOutputAdFile( ClassAd* ad )
 		}
 	}
 		// append a delimiter?
-	ad->fPrint( fp );
+	fPrintAd( fp, *ad );
 
 	if( job_output_ad_is_stdout ) {
 		fflush( fp );
@@ -568,7 +582,7 @@ JobInfoCommunicator::checkDedicatedExecuteAccounts( char const *name )
 
 		// force the matching of the whole string
 	MyString full_pattern;
-	full_pattern.sprintf("^%s$",pattern_string);
+	full_pattern.formatstr("^%s$",pattern_string);
 
 	Regex re;
 	char const *errstr = NULL;
@@ -666,13 +680,10 @@ JobInfoCommunicator::initUserPrivWindows( void )
 	}
 
 	if ( !name ) {
-		char slot_user[255];
-		MyString slotName = "";
-		slotName = Starter->getMySlotName();
-
+		MyString slotName = Starter->getMySlotName();
 		slotName.upper_case();
-		sprintf(slot_user, "%s_USER", slotName);
-		char *run_jobs_as = param(slot_user);
+		slotName += "_USER";
+		char *run_jobs_as = param(slotName.Value());
 		if (run_jobs_as) {		
 			getDomainAndName(run_jobs_as, domain, name);
 				/* 
@@ -762,9 +773,10 @@ JobInfoCommunicator::checkForStarterDebugging( void )
 		// For debugging, see if there's a special attribute in the
 		// job ad that sends us into an infinite loop, waiting for
 		// someone to attach with a debugger
-	int starter_should_wait = 0;
-	job_ad->LookupInteger( ATTR_STARTER_WAIT_FOR_DEBUG,
-						  starter_should_wait );
+	volatile int starter_should_wait = 0;
+	int tmp = 0; // Can't pass volatile int into LookupInteger
+	job_ad->LookupInteger( ATTR_STARTER_WAIT_FOR_DEBUG, tmp );
+	starter_should_wait = tmp;
 	if( starter_should_wait ) {
 		dprintf( D_ALWAYS, "Job requested starter should wait for "
 				 "debugger with %s=%d, going into infinite loop\n",
@@ -778,9 +790,9 @@ JobInfoCommunicator::checkForStarterDebugging( void )
 
 		// Also, if the starter has D_JOB turned on, we want to dump
 		// out the job ad to the log file...
-	if( DebugFlags & D_JOB ) {
+	if( IsDebugLevel( D_JOB ) ) {
 		dprintf( D_JOB, "*** Job ClassAd ***\n" );  
-		job_ad->dPrint( D_JOB );
+		dPrintAd( D_JOB, *job_ad );
         dprintf( D_JOB, "--- End of ClassAd ---\n" );
 	}
 }
