@@ -42,6 +42,7 @@
 #include "access_desktop.WINDOWS.h"
 #endif
 #include "classad_oldnew.h"
+#include "singularity.h"
 
 extern CStarter *Starter;
 extern const char* JOB_WRAPPER_FAILURE_FILE;
@@ -60,6 +61,7 @@ OsProc::OsProc( ClassAd* ad )
 	is_checkpointed = false;
 	num_pids = 0;
 	dumped_core = false;
+	job_not_started = false;
 	m_using_priv_sep = false;
 	UserProc::initialize();
 }
@@ -80,6 +82,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 
 	if ( !JobAd ) {
 		dprintf ( D_ALWAYS, "No JobAd in OsProc::StartJob()!\n" );
+		job_not_started = true;
 		return 0;
 	}
 
@@ -87,6 +90,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 	if ( JobAd->LookupString( ATTR_JOB_CMD, JobName ) != 1 ) {
 		dprintf( D_ALWAYS, "%s not found in JobAd.  Aborting StartJob.\n", 
 				 ATTR_JOB_CMD );
+		job_not_started = true;
 		return 0;
 	}
 
@@ -177,6 +181,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 					 "Cannot find/execute USER_JOB_WRAPPER file %s\n",
 					 wrapper );
 			free( wrapper );
+			job_not_started = true;
 			return 0;
 		}
 		has_wrapper = true;
@@ -199,6 +204,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 				dprintf( D_ALWAYS, "Unable to use parrot(Cannot find/execute "
 					"at %s(%s)).\n", parrot, strerror(errno) );
 				free( parrot );
+				job_not_started = true;
 				return 0;
 			} else {
 				args.AppendArg(JobName.Value());
@@ -208,6 +214,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 		} else {
 			dprintf( D_ALWAYS, "Unable to use parrot(Undefined path in config"
 			" file)" );
+			job_not_started = true;
 			return 0;
 		}
 	}
@@ -218,6 +225,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 	if(!args.AppendArgsFromClassAd(JobAd,&args_error)) {
 		dprintf(D_ALWAYS, "Failed to read job arguments from JobAd.  "
 				"Aborting OsProc::StartJob: %s\n",args_error.Value());
+		job_not_started = true;
 		return 0;
 	}
 
@@ -233,6 +241,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 	if( !Starter->GetJobEnv(JobAd,&job_env,&env_errors) ) {
 		dprintf( D_ALWAYS, "Aborting OSProc::StartJob: %s\n",
 				 env_errors.Value());
+		job_not_started = true;
 		return 0;
 	}
 
@@ -311,6 +320,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 		dprintf(D_ALWAYS, "Failed to open some/all of the std files...\n");
 		dprintf(D_ALWAYS, "Aborting OsProc::StartJob.\n");
 		set_priv(priv); /* go back to original priv state before leaving */
+		job_not_started = true;
 		return 0;
 	}
 
@@ -334,6 +344,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 			dprintf( D_ALWAYS, "ERROR: failed to insert JOB_RENICE_INCREMENT "
 				"into job ad, Aborting OsProc::StartJob...\n" );
 			free( ptmp );
+			job_not_started = true;
 			return 0;
 		}
 			// evaluate
@@ -376,44 +387,11 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 			// it, if they need to.
 		dprintf( D_ALWAYS, "Using wrapper %s to exec %s\n", JobName.Value(), 
 				 args_string.Value() );
-
-		MyString wrapper_err;
-		wrapper_err.formatstr("%s%c%s", Starter->GetWorkingDir(),
-				 	DIR_DELIM_CHAR,
-					JOB_WRAPPER_FAILURE_FILE);
-		if( ! job_env.SetEnv("_CONDOR_WRAPPER_ERROR_FILE", wrapper_err.Value()) ) {
-			dprintf( D_ALWAYS, "Failed to set _CONDOR_WRAPPER_ERROR_FILE environment variable\n");
-		}
 	} else {
 		dprintf( D_ALWAYS, "About to exec %s %s\n", JobName.Value(),
 				 args_string.Value() );
 	}
 
-	MyString path;
-	path.formatstr("%s%c%s", Starter->GetWorkingDir(),
-			 	DIR_DELIM_CHAR,
-				MACHINE_AD_FILENAME);
-	if( ! job_env.SetEnv("_CONDOR_MACHINE_AD", path.Value()) ) {
-		dprintf( D_ALWAYS, "Failed to set _CONDOR_MACHINE_AD environment variable\n");
-	}
-
-	if( Starter->jic->wroteChirpConfig() && (! job_env.SetEnv("_CONDOR_CHIRP_CONFIG", Starter->jic->chirpConfigFilename().c_str())) ) {
-		dprintf( D_ALWAYS, "Failed to set _CONDOR_CHIRP_CONFIG environment variable.\n");
-	}
-
-	path.formatstr("%s%c%s", Starter->GetWorkingDir(),
-			 	DIR_DELIM_CHAR,
-				JOB_AD_FILENAME);
-	if( ! job_env.SetEnv("_CONDOR_JOB_AD", path.Value()) ) {
-		dprintf( D_ALWAYS, "Failed to set _CONDOR_JOB_AD environment variable\n");
-	}
-
-	std::string remoteUpdate;
-	param(remoteUpdate, "CHIRP_DELAYED_UPDATE_PREFIX", "CHIRP");
-	if( ! job_env.SetEnv("_CHIRP_DELAYED_UPDATE_PREFIX", remoteUpdate) ) {
-		dprintf( D_ALWAYS, "Failed to set _CHIRP_DELAYED_UPDATE_PREFIX environment variable\n");
-	}
-
 		// Grab the full environment back out of the Env object 
 	if(IsFulldebug(D_FULLDEBUG)) {
 		MyString env_string;
@@ -483,6 +461,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 		} else {
 			dprintf(D_ALWAYS, "Can't parse STARTER_RLIMIT_AS expression: %s\n", rlimit_expr);
 		}
+		free( rlimit_expr );
 	}
 
 	int *affinity_mask = makeCpuAffinityMask(Starter->getMySlotNumber());
@@ -516,7 +495,26 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 
 		// While we are still in user priv, print out the username
 #if defined(LINUX)
-	if( Starter->glexecPrivSepHelper() ) {
+	std::stringstream ss2;
+	ss2 << Starter->GetExecuteDir() << DIR_DELIM_CHAR << "dir_" << getpid();
+	std::string execute_dir = ss2.str();
+	htcondor::Singularity::result sing_result = htcondor::Singularity::setup(*Starter->jic->machClassAd(), *JobAd, JobName, args, job_iwd, execute_dir, job_env);
+	if (sing_result == htcondor::Singularity::SUCCESS) {
+		dprintf(D_ALWAYS, "Running job via singularity.\n");
+		if (fs_remap) {
+			dprintf(D_ALWAYS, "Disabling filesystem remapping; singularity will perform these features.\n");
+			fs_remap = NULL;
+		}
+		if (family_info && family_info->want_pid_namespace) {
+			dprintf(D_FULLDEBUG, "PID namespaces cannot be enabled for singularity jobs.\n");
+			job_not_started = true;
+			return 0;
+		}
+	} else if (sing_result == htcondor::Singularity::FAILURE) {
+		dprintf(D_ALWAYS, "Singularity enabled but setup failed; failing job.\n");
+		job_not_started = true;
+		return 0;
+	} else if( Starter->glexecPrivSepHelper() ) {
 			// TODO: if there is some way to figure out the final username,
 			// print it out here or after starting the job.
 		dprintf(D_ALWAYS,"Running job via glexec\n");
@@ -537,7 +535,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 			username = get_user_loginname();
 		}
 		if( !username ) {
-			username = "(null)";
+			username = "same uid as parent: personal condor";
 		}
 		dprintf(D_ALWAYS,"Running job %sas user %s\n",how,username);
 	}
@@ -574,6 +572,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 		                                     PRIV_USER_FINAL,
 		                                     1,
 		                                     FALSE,
+		                                     FALSE,
 		                                     &job_env,
 		                                     job_iwd,
 		                                     family_info,
@@ -602,6 +601,8 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
         create_process_err_msg += errbuf;
     }
 
+	free( affinity_mask );
+
 	// now close the descriptors in fds array.  our child has inherited
 	// them already, so we should close them so we do not leak descriptors.
 	// NOTE, we want to use a special method to close the starter's
@@ -650,6 +651,7 @@ OsProc::StartJob(FamilyInfo* family_info, FilesystemRemap* fs_remap=NULL)
 
 		dprintf(D_ALWAYS,"Create_Process(%s,%s, ...) failed: %s\n",
 			JobName.Value(), args_string.Value(), create_process_err_msg.Value());
+		job_not_started = true;
 		return 0;
 	}
 
@@ -697,6 +699,8 @@ OsProc::JobExit( void )
 		}
 	} else if( dumped_core ) {
 		reason = JOB_COREDUMPED;
+	} else if( job_not_started ) {
+		reason = JOB_NOT_STARTED;
 	} else {
 		reason = JOB_EXITED;
 	}
