@@ -38,6 +38,7 @@
 #include "vm_proc.h"
 #include "docker_proc.h"
 #include "condor_getcwd.h"
+#include "singularity.h"
 
 
 extern "C" int exception_cleanup(int,int,const char*);	/* Our function called by EXCEPT */
@@ -158,6 +159,12 @@ printClassAd( void )
 		}
 	}
 
+	// Singularity support
+	if (htcondor::Singularity::enabled()) {
+		printf("%s = True\n", ATTR_HAS_SINGULARITY);
+		printf("%s = \"%s\"\n", ATTR_SINGULARITY_VERSION, htcondor::Singularity::version());
+	}
+
 	// Detect ability to encrypt execute directory
 #ifdef LINUX
 	if ( FilesystemRemap::EncryptedMappingDetect() ) {
