@@ -37,6 +37,7 @@
 #include "subsystem_info.h"
 #include "cgroup_limits.h"
 #include "selector.h"
+#include "singularity.h"
 
 #ifdef WIN32
 #include "executable_scripts.WINDOWS.h"
@@ -168,7 +169,7 @@ VanillaProc::StartJob()
 	CHAR		interpreter[MAX_PATH+1],
 				systemshell[MAX_PATH+1];    
 	const char* jobtmp				= Starter->jic->origJobName();
-	int			joblen				= strlen(jobtmp);
+	size_t		joblen				= strlen(jobtmp);
 	const char	*extension			= joblen > 0 ? &(jobtmp[joblen-4]) : NULL;
 	bool		binary_executable	= ( extension && 
 										( MATCH == strcasecmp ( ".exe", extension ) || 
@@ -348,7 +349,7 @@ VanillaProc::StartJob()
 		 *  local universe and cgroups can be properly worked
 		 *  out. -matt 7 nov '12
 		 */
-	if (CONDOR_UNIVERSE_LOCAL != job_universe && cgroup_base.length()) {
+	if (CONDOR_UNIVERSE_LOCAL != job_universe && cgroup_base.length() && can_switch_ids()) {
 		MyString cgroup_uniq;
 		std::string starter_name, execute_str;
 		param(execute_str, "EXECUTE", "EXECUTE_UNKNOWN");
@@ -552,7 +553,7 @@ VanillaProc::StartJob()
 #if defined(LINUX)
 	// On Linux kernel 2.6.24 and later, we can give each
 	// job its own PID namespace
-	if (param_boolean("USE_PID_NAMESPACES", false)) {
+	if (param_boolean("USE_PID_NAMESPACES", false) && !htcondor::Singularity::job_enabled(*Starter->jic->machClassAd(), *JobAd)) {
 		if (!can_switch_ids()) {
 			EXCEPT("USE_PID_NAMESPACES enabled, but can't perform this "
 				"call in Linux unless running as root.");
@@ -625,7 +626,7 @@ VanillaProc::StartJob()
 	// See Note near setup of param(BASE_CGROUP)
 	if (CONDOR_UNIVERSE_LOCAL != job_universe && cgroup && retval) {
 		std::string mem_limit;
-		param(mem_limit, "CGROUP_MEMORY_LIMIT_POLICY", "soft");
+		param(mem_limit, "CGROUP_MEMORY_LIMIT_POLICY", "none");
 		bool mem_is_soft = mem_limit == "soft";
 		std::string cgroup_string = cgroup;
 		CgroupLimits climits(cgroup_string);
@@ -633,6 +634,15 @@ VanillaProc::StartJob()
 			ClassAd * MachineAd = Starter->jic->machClassAd();
 			int MemMb;
 			if (MachineAd->LookupInteger(ATTR_MEMORY, MemMb)) {
+				// cgroups prevents us from setting hard limits lower
+				// than memsw limit.  If we are reusing this cgroup,
+				// we don't know what the previous values were
+				// So, set mem to 0, memsw to +inf, so that the real
+				// values can be set without interference
+
+				climits.set_memory_limit_bytes(0);
+				climits.set_memsw_limit_bytes(LONG_MAX);
+
 				uint64_t MemMb_big = MemMb;
 				m_memory_limit = MemMb_big;
 				climits.set_memory_limit_bytes(1024*1024*MemMb_big, mem_is_soft);
@@ -699,6 +709,7 @@ bool
 VanillaProc::PublishUpdateAd( ClassAd* ad )
 {
 	dprintf( D_FULLDEBUG, "In VanillaProc::PublishUpdateAd()\n" );
+	static unsigned int max_rss = 0;
 
 	ProcFamilyUsage* usage;
 	ProcFamilyUsage cur_usage;
@@ -722,7 +733,11 @@ VanillaProc::PublishUpdateAd( ClassAd* ad )
 	ad->Assign(ATTR_JOB_REMOTE_USER_CPU, (double)usage->user_cpu_time);
 
 	ad->Assign(ATTR_IMAGE_SIZE, usage->max_image_size);
-	ad->Assign(ATTR_RESIDENT_SET_SIZE, usage->total_resident_set_size);
+
+	if (usage->total_resident_set_size > max_rss) {
+		max_rss = usage->total_resident_set_size;
+	}
+	ad->Assign(ATTR_RESIDENT_SET_SIZE, max_rss);
 
 	std::string memory_usage;
 	if (param(memory_usage, "MEMORY_USAGE_METRIC", "((ResidentSetSize+1023)/1024)")) {
@@ -757,6 +772,10 @@ VanillaProc::PublishUpdateAd( ClassAd* ad )
 
 
 int VanillaProc::pidNameSpaceReaper( int status ) {
+	if (requested_exit) {
+		return 0;
+	}
+
 	TemporaryPrivSentry sentry(PRIV_ROOT);
 	FILE *f = safe_fopen_wrapper_follow(m_pid_ns_status_filename.c_str(), "r");
 	if (f == NULL) {
@@ -776,6 +795,52 @@ int VanillaProc::pidNameSpaceReaper( int status ) {
 	return status;
 }
 
+void VanillaProc::notifySuccessfulEvictionCheckpoint() { /* FIXME (#4969) */ }
+void VanillaProc::notifySuccessfulPeriodicCheckpoint() { /* FIXME (#4969) */ }
+
+void VanillaProc::recordFinalUsage() {
+	if( daemonCore->Get_Family_Usage(JobPid, m_final_usage) == FALSE ) {
+		dprintf( D_ALWAYS, "error getting family usage for pid %d in "
+			"VanillaProc::JobReaper()\n", JobPid );
+	}
+}
+
+void VanillaProc::killFamilyIfWarranted() {
+	// Kill_Family() will (incorrectly?) kill the SSH-to-job daemon
+	// if we're using dedicated accounts, so don't unless we know
+	// we're the only job.
+	if ( ! m_dedicated_account || Starter->numberOfJobs() == 1 ) {
+		dprintf( D_PROCFAMILY, "About to call Kill_Family()\n" );
+		daemonCore->Kill_Family( JobPid );
+	} else {
+		dprintf( D_PROCFAMILY, "Postponing call to Kill_Family() "
+			"(perhaps due to ssh_to_job)\n" );
+	}
+}
+
+void VanillaProc::restartCheckpointedJob() {
+	// For the same reason that we call recordFinalUsage() from the reaper
+	// in normal exit cases, we should get the final usage of the checkpointed
+	// process now, add it to the running total of checkpointed processes,
+	// and then add the running total to the current when we publish the
+	// update ad.  FIXME (#4971)
+
+	if( Starter->jic->uploadWorkingFiles() ) {
+			notifySuccessfulPeriodicCheckpoint();
+	} else {
+			// We assume this is a transient failure and will try
+			// to transfer again after the next periodic checkpoint.
+			dprintf( D_ALWAYS, "Failed to transfer checkpoint.\n" );
+	}
+
+	// While it's arguably sensible to kill the process family
+	// before we restart the job, that would mean that checkpointing
+	// would behave differently during ssh-to-job, which seems bad.
+	// killFamilyIfWarranted();
+
+	StartJob();
+}
+
 bool
 VanillaProc::JobReaper(int pid, int status)
 {
@@ -810,72 +875,93 @@ VanillaProc::JobReaper(int pid, int status)
 	}
 #endif
 
+	//
+	// We have three cases to consider:
+	//   * if we're checkpointing; or
+	//   * if we see a special checkpoint exit code; or
+	//   * there's no special case to consider.
+	//
+
+	bool wantsFileTransferOnCheckpointExit = false;
+	JobAd->LookupBool( ATTR_WANT_FT_ON_CHECKPOINT, wantsFileTransferOnCheckpointExit );
+
+	int checkpointExitCode = 0;
+	JobAd->LookupInteger( ATTR_CHECKPOINT_EXIT_CODE, checkpointExitCode );
+	int checkpointExitSignal = 0;
+	JobAd->LookupInteger( ATTR_CHECKPOINT_EXIT_SIGNAL, checkpointExitSignal );
+	bool checkpointExitBySignal = 0;
+	JobAd->LookupBool( ATTR_CHECKPOINT_EXIT_BY_SIGNAL, checkpointExitBySignal );
+
+	int successfulCheckpointStatus = 0;
+	if( checkpointExitBySignal ) {
+		successfulCheckpointStatus = checkpointExitSignal;
+	} else if( checkpointExitCode != 0 ) {
+		successfulCheckpointStatus = checkpointExitCode << 8;
+#if defined( WINDOWS )
+		successfulCheckpointStatus = checkpointExitCode;
+#endif
+	}
+
 	if( isCheckpointing ) {
 		dprintf( D_FULLDEBUG, "Inside VanillaProc::JobReaper() during a checkpoint\n" );
-		isCheckpointing = false;
 
-		// The job signals that it's successfully written a checkpoint by
-		// exiting normally with status 0.
-		if( exit_status == 0 ) {
-			if( Starter->jic->uploadWorkingFiles() ) {
-				// FIXME: Update the job ad to reflect the succesful checkpoint.
-			} else {
-				// The logic in the base starter for failing to transfer
-				// output back is to wait for the shadow to reconnect, or,
-				// failing that, the job lease to expire.  So do nothing.
-				dprintf( D_ALWAYS, "Failed to transfer checkpoint.\n" );
+		if( exit_status == successfulCheckpointStatus ) {
+			if( isSoftKilling ) {
+				notifySuccessfulEvictionCheckpoint();
+				return true;
 			}
 
-			if( ! isSoftKilling ) {
-				StartJob();
-
-				jobExited = false;
-			}
+			restartCheckpointedJob();
+			isCheckpointing = false;
+			return false;
 		} else {
 			// The job exited without taking a checkpoint.  If we don't do
 			// anything, it will be reported as if the error code or signal
-			// had happened naturally (that job will usually exit the
+			// had happened naturally (and the job will usually exit the
 			// queue).  This could confuse the users.
 			//
-			// For now, we'll just force the job to requeue.
+			// Instead, we'll put the job on hold, figuring that if the job
+			// requested that we (periodically) send it a signal, and we
+			// did, that it's not our fault that the job failed.  This has
+			// the convenient side-effect of not overwriting the job's
+			// previous checkpoint(s), if any (since file transfer doesn't
+			// occur when the job goes on hold).
+			killFamilyIfWarranted();
+			recordFinalUsage();
 
-			// The easiest way to do that is to pretend that this was an
-			// exit requested by HTCondor.  That's not untrue, but this
-			// variable is normally only used to flag external requests
-			// which have leases.
+			std::string holdMessage;
+			formatstr( holdMessage, "Job did not exit as promised when sent its checkpoint signal.  "
+				"Promised exit was %s %u, actual exit status was %s %u.",
+				checkpointExitBySignal ? "on signal" : "with exit code",
+				checkpointExitBySignal ? checkpointExitSignal : checkpointExitCode,
+				WIFSIGNALED( exit_status ) ? "on signal" : "with exit code",
+				WIFSIGNALED( exit_status ) ? WTERMSIG( exit_status ) : WEXITSTATUS( exit_status ) );
+			Starter->jic->holdJob( holdMessage.c_str(), CONDOR_HOLD_CODE_FailedToCheckpoint, exit_status );
+			Starter->Hold();
+			return true;
+		}
+	} else if( wantsFileTransferOnCheckpointExit && exit_status == successfulCheckpointStatus ) {
+		dprintf( D_FULLDEBUG, "Inside VanillaProc::JobReaper() and the job self-checkpointed.\n" );
 
-			// This makes it show up as 'evicted', which is true but
-			// deceptive.  Fixing that may require adding a new starter
-			// exit code (JOB_FAILED_CHECKPOINT) and a new user log
-			// event ID (ULOG_JOB_FAILED_CHECKPOINT).
-			requested_exit = true;
+		if( isSoftKilling ) {
+			notifySuccessfulEvictionCheckpoint();
+			return true;
+		} else {
+			restartCheckpointedJob();
+			return false;
 		}
 	} else {
-		// If the parent job process died, clean up all of the job's processes
-		// and record its final usage stats.
-
-		// Kill_Family() will (incorrectly?) kill the SSH-to-job daemon
-		// if we're using dedicated accounts, so don't unless we know
-		// we're the only job.
-		if ( ! m_dedicated_account || Starter->numberOfJobs() == 1 ) {
-			dprintf( D_PROCFAMILY, "About to call Kill_Family()\n" );
-			daemonCore->Kill_Family( JobPid );
-		} else {
-			dprintf( D_PROCFAMILY, "Postponing call to Kill_Family() "
-				"(perhaps due to ssh_to_job)\n" );
-		}
+		// If the parent job process died, clean up all of the job's processes.
+		killFamilyIfWarranted();
 
 		// Record final usage stats for this process family, since
 		// once the reaper returns, the family is no longer
 		// registered with DaemonCore and we'll never be able to
 		// get this information again.
-		if( daemonCore->Get_Family_Usage(JobPid, m_final_usage) == FALSE ) {
-			dprintf( D_ALWAYS, "error getting family usage for pid %d in "
-				"VanillaProc::JobReaper()\n", JobPid );
-		}
-	}
+		recordFinalUsage();
 
-	return jobExited;
+		return jobExited;
+	}
 }
 
 
@@ -1022,10 +1108,12 @@ VanillaProc::outOfMemoryEvent(int /* fd */)
 	ClassAd updateAd;
 	PublishUpdateAd( &updateAd );
 	Starter->jic->periodicJobUpdate( &updateAd, true );
+	int usage;
+	updateAd.LookupInteger(ATTR_MEMORY_USAGE, usage);
 
 		//
 #ifdef LINUX
-	if (param_boolean("IGNORE_LEAF_OOM", false)) {
+	if (param_boolean("IGNORE_LEAF_OOM", true)) {
 		// if memory.use_hierarchy is 1, then hitting the limit at
 		// the parent notifies all children, even if those children
 		// are below their usage.  If we are below our usage, ignore
@@ -1033,13 +1121,10 @@ VanillaProc::outOfMemoryEvent(int /* fd */)
 		// will be killed, and when it does, this job will get unfrozen
 		// and continue running.
 
-		int usage;
-		updateAd.LookupInteger(ATTR_MEMORY_USAGE, usage);
-
 		if (usage < (0.9 * m_memory_limit)) {
 			long long oomData = 0xdeadbeef;
-			int efd;
-			daemonCore->Get_Pipe_FD(m_oom_efd, &efd);
+			int efd = -1;
+			ASSERT( daemonCore->Get_Pipe_FD(m_oom_efd, &efd) );
 				// need to drain notification fd, or it will still
 				// be hot, and we'll come right back here again
 			int r = read(efd, &oomData, 8);
@@ -1052,7 +1137,7 @@ VanillaProc::outOfMemoryEvent(int /* fd */)
 
 	std::stringstream ss;
 	if (m_memory_limit >= 0) {
-		ss << "Job has gone over memory limit of " << m_memory_limit << " megabytes.";
+		ss << "Job has gone over memory limit of " << m_memory_limit << " megabytes. Peak usage: " << usage << " megabytes.";
 	} else {
 		ss << "Job has encountered an out-of-memory event.";
 	}
@@ -1060,16 +1145,11 @@ VanillaProc::outOfMemoryEvent(int /* fd */)
 		ss << "  This occurred while the job was checkpointing.";
 	}
 
-	// this will actually clean up the job
-	Starter->Hold( );
 	dprintf( D_ALWAYS, "Job was held due to OOM event: %s\n", ss.str().c_str());
-	Starter->allJobsDone();
 
 	dprintf(D_FULLDEBUG, "Closing event FD pipe %d.\n", m_oom_efd);
 	cleanupOOM();
 
-	Starter->ShutdownFast();
-
 	// This ulogs the hold event and KILLS the shadow
 	Starter->jic->holdJob(ss.str().c_str(), CONDOR_HOLD_CODE_JobOutOfResources, 0);
 
@@ -1158,7 +1238,7 @@ VanillaProc::setupOOMEvent(const std::string &cgroup_string)
 			event_control << mount_info.path << "/";
 			break;
 		}
-		cgroup_get_controller_next(&handle, &mount_info);
+		ret = cgroup_get_controller_next(&handle, &mount_info);
 	}
 	if (!found_memcg && (ret != ECGEOF)) {
 		dprintf(D_ALWAYS,
@@ -1315,7 +1395,9 @@ bool VanillaProc::Ckpt() {
 int VanillaProc::outputOpenFlags() {
 	int wantCheckpoint = 0;
 	JobAd->LookupBool( ATTR_WANT_CHECKPOINT_SIGNAL, wantCheckpoint );
-	if( wantCheckpoint ) {
+	bool wantsFileTransferOnCheckpointExit = false;
+	JobAd->LookupBool( ATTR_WANT_FT_ON_CHECKPOINT, wantsFileTransferOnCheckpointExit );
+	if( wantCheckpoint || wantsFileTransferOnCheckpointExit ) {
 		return O_WRONLY | O_CREAT | O_APPEND | O_LARGEFILE;
 	} else {
 		return this->OsProc::outputOpenFlags();
@@ -1325,7 +1407,9 @@ int VanillaProc::outputOpenFlags() {
 int VanillaProc::streamingOpenFlags( bool isOutput ) {
 	int wantCheckpoint = 0;
 	JobAd->LookupBool( ATTR_WANT_CHECKPOINT_SIGNAL, wantCheckpoint );
-	if( wantCheckpoint ) {
+	bool wantsFileTransferOnCheckpointExit = false;
+	JobAd->LookupBool( ATTR_WANT_FT_ON_CHECKPOINT, wantsFileTransferOnCheckpointExit );
+	if( wantCheckpoint || wantsFileTransferOnCheckpointExit ) {
 		return isOutput ? O_CREAT | O_APPEND | O_WRONLY : O_RDONLY;
 	} else {
 		return this->OsProc::streamingOpenFlags( isOutput );
