@@ -20,14 +20,13 @@
 #if !defined(_SUBMIT_INTERNAL_H)
 #define _SUBMIT_INTERNAL_H
 
-#include "dc_schedd.h"
-
 #define PLUS_ATTRIBS_IN_CLUSTER_AD 1
 
 // uncomment this to get (broken) legacy behavior that attributes in
 // SUBMIT_ATTRS/SUBMIT_EXPRS behave as if they were statements in your submit file
 //#define SUBMIT_ATTRS_IS_ALSO_CONDOR_PARAM 1
 
+#ifndef EXPAND_GLOBS_WARN_EMPTY
 // functions in submit_glob.cpp
 #define EXPAND_GLOBS_WARN_EMPTY (1<<0)
 #define EXPAND_GLOBS_FAIL_EMPTY (1<<1)
@@ -37,6 +36,7 @@
 #define EXPAND_GLOBS_TO_FILES   (1<<5) // when you want files only
 
 int submit_expand_globs(StringList &items, int options, std::string & errmsg);
+#endif // EXPAND_GLOBS
 
 // functions for handling the queue statement
 int queue_connect();
@@ -64,6 +64,8 @@ const SetAttributeFlags_t NONDURABLE = (1<<0); // do not fsync
 const SetAttributeFlags_t SetAttribute_NoAck = (1<<1);
 const SetAttributeFlags_t SETDIRTY = (1<<2);
 const SetAttributeFlags_t SHOULDLOG = (1<<3);
+const SetAttributeFlags_t SetAttribute_OnlyMyJobs = (1<<4);
+const SetAttributeFlags_t SetAttribute_QueryOnly = (1<<5); // check if change is allowed, but don't actually change.
 #endif
 
 // Abstract the schedd's queue protocol so we can NOT call it when simulating
@@ -127,11 +129,12 @@ public:
 	virtual bool disconnect(bool commit_transaction, CondorError & errstack);
 	virtual int  get_type() { return AbstractQ_TYPE_SIM; }
 
-	bool Connect(FILE* fp, bool close_on_disconnect);
+	bool Connect(FILE* fp, bool close_on_disconnect, bool log_all);
 private:
 	int cluster;
 	int proc;
 	bool close_file_on_disconnect;
+	bool log_all_communication;
 	FILE * fp;
 };
 
