@@ -33,12 +33,11 @@ sub RunTests( $ );
 sub RunTestOp( $$$ );
 sub CountLiveLeases( $ );
 
-my $version = "1.0.0";
+my $version = "1.1.0";
 my $testdesc =  'lib_lease_manager - runs lease manager tests';
 my $testname = "lib_lease_manager";
-my $testbin = "../testbin_dir";
 my %programs = ( advertise	=> getcwd() . "/x_advertise.pl",
-				 tester => "$testbin/condor_lease_manager_tester",
+				 tester => "condor_lease_manager_tester",
 				 );
 
 my $now = time( );
@@ -363,7 +362,7 @@ if ( $settings{use_pid} ) {
 
 my $fulldir = getcwd() . "/$dir";
 if ( -d $dir  and  $settings{force} ) {
-    system( "/bin/rm", "-fr", $dir );
+    system( "rm", "-fr", $dir );
 }
 if ( -d $dir ) {
     die "Test directory $dir already exists! (try --force?)";
@@ -432,10 +431,8 @@ open( PARAMS, ">$param_file" ) or die "can't write to $param_file";
 print PARAMS <<ENDPARAMS;
 ports = dynamic
 condor = nightlies
-condorconfig = condor_config
-condorlocal = condor_config.local
 localpostsrc = $config_tmp
-condordaemons = MASTER,COLLECTOR,LEASEMANAGER,ADVERTISER
+daemon_list = MASTER,COLLECTOR,LEASEMANAGER,ADVERTISER
 personaldir = $fulldir
 ENDPARAMS
 close(PARAMS);
@@ -448,7 +445,7 @@ printf( "\n** Lease Manager test v%s starting test %s @ %s **\n\n",
 CondorTest::debug("About to set up Condor Personal : <<<$ltime>>>\n", 1);
 
 # Fire up my condor
-my $configrem =	CondorPersonal::StartCondor( "dummy", $param_file, $full_name );
+my $configrem =	CondorTest::StartPersonal($full_name, $param_file, $version);
 my ($config, $port) = split( /\+/, $configrem );
 $ENV{CONDOR_CONFIG} = $config;
 print "Condor config: '" . $ENV{CONDOR_CONFIG} . "'\n";
@@ -464,8 +461,7 @@ else {
 	printf "* Tests complete @ %s *\n", $ltime = localtime();
 }
 
-CondorPersonal::KillDaemonPids( $config );
-system( "rm -r ./$$" );
+CondorTest::KillPersonal( $config );
 
 my $time_end = time();
 $ltime = localtime( $time_end );
