@@ -40,7 +40,8 @@ usage( const char *cmd , const char * opt)
 	fprintf(stderr,"    -version          Display Condor version\n");
 	fprintf(stderr,"    -pool <hostname>  Use this central manager\n");
 	fprintf(stderr,"    -debug            Show extra debugging info\n");
-	fprintf(stderr,"    -tcp              Ship classad via TCP (default is UDP)\n");
+	fprintf(stderr,"    -tcp              Ship classad via TCP (default)\n");
+	fprintf(stderr,"    -udp              Ship classad via UDP\n");
 	fprintf(stderr,"    -multiple         Publish multiple ads, separated by blank lines\n");
 
 	if (opt && ( !strcmp(opt,"all") || !strcmp(opt,"cmd"))) {
@@ -67,7 +68,11 @@ class ToolClassAdFileParseHelper : public ClassAdFileParseHelper
 	// Some compilers whine when you have virtual methods but not an
 	// explicit virtual destructor
 	virtual ~ToolClassAdFileParseHelper() {}
-	ToolClassAdFileParseHelper(bool multi, FILE* errout) : multiple(multi), out(errout) {};
+	ToolClassAdFileParseHelper(bool multi) : multiple(multi) {};
+
+	// return non-zero if new parser, o if old (line oriented) parser
+	// TODO: fix this to handle new style classads also...
+	virtual int NewParser(ClassAd & /*ad*/, FILE* /*file*/, bool & detected_long, std::string & /*errmsg*/) { detected_long = false; return 0; }
 
 	// return 0 to skip (is_comment), 1 to parse line, 2 for end-of-classad, -1 for abort
 	virtual int PreParse(std::string & line, ClassAd & /*ad*/, FILE* /*file*/) {
@@ -107,7 +112,6 @@ class ToolClassAdFileParseHelper : public ClassAdFileParseHelper
 
  private:
 	bool multiple;
-	FILE * out;
 	std::string delim;
 };
 
@@ -124,7 +128,6 @@ int main( int argc, char *argv[] )
 	char *pool=0;
 	int command=-1;
 	int i;
-	bool use_tcp = false;
 	bool with_ack = false;
 	bool allow_multiple = false;
 	bool many_connections = false;
@@ -132,11 +135,13 @@ int main( int argc, char *argv[] )
 	myDistro->Init( argc, argv );
 	config();
 
+	bool use_tcp = param_boolean( "UPDATE_COLLECTOR_WITH_TCP", true );
+
 	for( i=1; i<argc; i++ ) {
 		if(!strcmp(argv[i],"-help") || !strcmp(argv[i],"--help")) {
 			usage(argv[0], argv[i+1] ? argv[i+1] : "cmd");
 			exit(0);
-		} else if(!strcmp(argv[i],"-pool")) {	
+		} else if(!strcmp(argv[i],"-pool")) {
 			i++;
 			if(!argv[i]) {
 				fprintf(stderr,"-pool requires an argument.\n\n");
@@ -146,6 +151,8 @@ int main( int argc, char *argv[] )
 			pool = argv[i];
 		} else if(!strncmp(argv[i],"-tcp",strlen(argv[i]))) {
 			use_tcp = true;
+		} else if(!strncmp(argv[i],"-udp",strlen(argv[i]))) {
+			use_tcp = false;
 		} else if(!strncmp(argv[i],"-multiple",strlen(argv[i]))) {
 				// We don't set allow_multiple=true by default, because
 				// existing users (e.g. glideinWMS) have stray blank lines
@@ -208,7 +215,7 @@ int main( int argc, char *argv[] )
 	}
 
 	// create class that we can use to influence the behavior of Classad::InsertFromFile
-	ToolClassAdFileParseHelper parse_helper(allow_multiple, stderr);
+	ToolClassAdFileParseHelper parse_helper(allow_multiple);
 
 	for (;;) {
 		ClassAd *ad = new ClassAd();
@@ -224,6 +231,7 @@ int main( int argc, char *argv[] )
 			ads.Insert(ad);
 		} else {
 			delete ad;
+			ad = 0; // so we don't double delete it
 		}
 		if (eof) {
 			break;
@@ -253,10 +261,10 @@ int main( int argc, char *argv[] )
 
 	collectors->rewind();
 	while (collectors->next(collector)) {
-		
+
 		dprintf(D_FULLDEBUG,"locating collector %s...\n", collector->name());
 
-		if(!collector->locate()) {
+		if(!collector->locate(Daemon::LOCATE_FOR_LOOKUP)) {
 			fprintf(stderr,"couldn't locate collector: %s\n",collector->error());
 			had_error = true;
 			continue;
