@@ -33,6 +33,7 @@
 #include "directory.h"
 #include "iso_dates.h"
 #include "basename.h" // for condor_dirname
+#include "historyFileFinder.h"
 
 #include "history_utils.h"
 
@@ -53,8 +54,6 @@ static char * getDBConnStr(char *&quillName, char *&databaseIp, char *&databaseN
 static bool checkDBconfig();
 
 static void readHistoryFromFiles(char *JobHistoryFileName, char* constraint, ExprTree *constraintExpr);
-static char **findHistoryFiles(int *numHistoryFiles);
-static bool isHistoryBackup(const char *fullFilename, time_t *backup_time);
 static int compareHistoryFilenames(const void *item1, const void *item2);
 static void readHistoryFromFile(char *JobHistoryFileName, char* constraint, ExprTree *constraintExpr);
 
@@ -352,7 +351,7 @@ static void readHistoryFromFiles(char *JobHistoryFileName, char* constraint, Exp
         // The user didn't specify the name of the file to read, so we read
         // the history file, and any backups (rotated versions). 
         int numHistoryFiles;
-        char **historyFiles;
+        const char **historyFiles;
 
         historyFiles = findHistoryFiles(&numHistoryFiles);
         if (historyFiles && numHistoryFiles > 0) {
@@ -360,140 +359,19 @@ static void readHistoryFromFiles(char *JobHistoryFileName, char* constraint, Exp
             if (backwards) { // Reverse reading of history files array
                 for(fileIndex = numHistoryFiles - 1; fileIndex >= 0; fileIndex--) {
                     readHistoryFromFile(historyFiles[fileIndex], constraint, constraintExpr);
-                    free(historyFiles[fileIndex]);
                 }
             }
             else {
                 for (fileIndex = 0; fileIndex < numHistoryFiles; fileIndex++) {
                     readHistoryFromFile(historyFiles[fileIndex], constraint, constraintExpr);
-                    free(historyFiles[fileIndex]);
                 }
             }
         }
-		free(historyFiles);
+        freeHistoryFilesList(historyFiles);
     }
     return;
 }
 
-// Find all of the history files that the schedd created, and put them
-// in order by time that they were created. The time comes from a
-// timestamp in the file name.
-static char **findHistoryFiles(int *numHistoryFiles)
-{
-    int  fileIndex;
-    char **historyFiles = NULL;
-    char *historyDir;
-
-    BaseJobHistoryFileName = param("HISTORY");
-	if ( BaseJobHistoryFileName == NULL ) {
-		fprintf( stderr, "Error: No history file is defined\n");
-		fprintf(stderr, "\n");
-		print_wrapped_text("Extra Info: " 
-						   "The variable HISTORY is not defined in "
-						   "your config file. If you want Condor to "
-						   "keep a history of past jobs, you must "
-						   "define HISTORY in your config file", stderr );
-
-		exit( 1 );    
-	}
-    historyDir = condor_dirname(BaseJobHistoryFileName);
-
-    *numHistoryFiles = 0;
-    if (historyDir != NULL) {
-        Directory dir(historyDir);
-        const char *current_filename;
-
-        // We walk through once and count the number of history file backups
-         for (current_filename = dir.Next(); 
-             current_filename != NULL; 
-             current_filename = dir.Next()) {
-            
-            if (isHistoryBackup(current_filename, NULL)) {
-                (*numHistoryFiles)++;
-            }
-        }
-
-        // Add one for the current history file
-        (*numHistoryFiles)++;
-
-        // Make space for the filenames
-        historyFiles = (char **) malloc(sizeof(char*) * (*numHistoryFiles));
-
-        // Walk through again to fill in the names
-        // Note that we won't get the current history file
-        dir.Rewind();
-        for (fileIndex = 0, current_filename = dir.Next(); 
-             current_filename != NULL; 
-             current_filename = dir.Next()) {
-            
-            if (isHistoryBackup(current_filename, NULL)) {
-                historyFiles[fileIndex++] = strdup(dir.GetFullPath());
-            }
-        }
-        historyFiles[fileIndex] = strdup(BaseJobHistoryFileName);
-
-        if ((*numHistoryFiles) > 2) {
-            // Sort the backup files so that they are in the proper 
-            // order. The current history file is already in the right place.
-            qsort(historyFiles, (*numHistoryFiles)-1, sizeof(char*), compareHistoryFilenames);
-        }
-        
-        free(historyDir);
-    }
-    return historyFiles;
-}
-
-// Returns true if the filename is a history file, false otherwise.
-// If backup_time is not NULL, returns the time from the timestamp in
-// the file.
-static bool isHistoryBackup(const char *fullFilename, time_t *backup_time)
-{
-    bool       is_history_filename;
-    const char *filename;
-    const char *history_base;
-    int        history_base_length;
-
-    if (backup_time != NULL) {
-        *backup_time = -1;
-    }
-    
-    is_history_filename = false;
-    history_base        = condor_basename(BaseJobHistoryFileName);
-    history_base_length = strlen(history_base);
-    filename            = condor_basename(fullFilename);
-
-    if (   !strncmp(filename, history_base, history_base_length)
-        && filename[history_base_length] == '.') {
-        // The filename begins correctly, now see if it ends in an 
-        // ISO time
-        struct tm file_time;
-        bool is_utc;
-
-        iso8601_to_time(filename + history_base_length + 1, &file_time, &is_utc);
-        if (   file_time.tm_year != -1 && file_time.tm_mon != -1 
-            && file_time.tm_mday != -1 && file_time.tm_hour != -1
-            && file_time.tm_min != -1  && file_time.tm_sec != -1
-            && !is_utc) {
-            // This appears to be a proper history file backup.
-            is_history_filename = true;
-            if (backup_time != NULL) {
-                *backup_time = mktime(&file_time);
-            }
-        }
-    }
-
-    return is_history_filename;
-}
-
-// Used by qsort in findHistoryFiles() to sort history files. 
-static int compareHistoryFilenames(const void *item1, const void *item2)
-{
-    time_t time1, time2;
-
-    isHistoryBackup((const char *) item1, &time1);
-    isHistoryBackup((const char *) item2, &time2);
-    return time1 - time2;
-}
 
 // Given a history file, returns the position offset of the last delimiter
 // The last delimiter will be found in the last line of file, 
