@@ -169,6 +169,7 @@ void SSHToJob::printUsage()
 	fprintf(stderr," -ssh <alt ssh command>    (e.g. sftp or scp)\n");
 	fprintf(stderr," -keygen-options <keygen options>\n");
 	fprintf(stderr," -X                        (enable X11 forwarding)\n");
+	fprintf(stderr," -x                        (disable X11 forwarding)\n");
 }
 
 bool SSHToJob::parseArgs(int argc,char **argv)
@@ -237,6 +238,8 @@ bool SSHToJob::parseArgs(int argc,char **argv)
 			}
 		} else if( !strcmp( argv[nextarg], "-X" ) ) {
 			m_x_forwarding = true;
+		} else if( !strcmp( argv[nextarg], "-x" ) ) {
+			m_x_forwarding = false;
 		} else if( !strcmp( argv[nextarg], "--" ) ) {
 			++nextarg;
 			break;
@@ -446,6 +449,13 @@ bool SSHToJob::execute_ssh()
 			return false;
 		}
 
+		// As long as we have the job, check to see if it is docker, and warn
+		bool isDockerJob = false;
+		if (jobAd->LookupBool(ATTR_WANT_DOCKER, isDockerJob)) {
+			fprintf(stderr, "Warning: connecting to a docker job will allow acces to input and output sandbox, but not the rest of the container\n");
+		}
+
+
 		std::string gridResource, vmName, kpName;
 		bool gotGridResource = jobAd->EvaluateAttrString( ATTR_GRID_RESOURCE, gridResource );
 		bool gotVMName = jobAd->EvaluateAttrString( ATTR_EC2_REMOTE_VM_NAME, vmName );
