@@ -25,6 +25,7 @@
 #include "condor_random_num.h"
 #include "strupr.h"
 #include "simplelist.h"
+#include <limits>
 
 /*--------------------------------------------------------------------
  *
@@ -130,15 +131,15 @@ operator=(const MyString& S)
 MyString& MyString::
 operator=(const std::string& S) 
 {
-	assign_str(S.c_str(), S.length());
+	assign_str(S.c_str(), (int)S.length());
     return *this;
 }
 
 MyString& 
 MyString::operator=( const char *s ) 
 {
-	int s_len = s ? strlen(s) : 0;
-	assign_str(s, s_len);
+	size_t s_len = s ? strlen(s) : 0;
+	assign_str(s, (int)s_len);
     return *this;
 }
 
@@ -158,7 +159,8 @@ MyString::assign_str( const char *s, int s_len )
 			capacity = s_len;
 			Data = new char[capacity+1];
 		}
-		strcpy( Data, s );
+		strncpy( Data, s, s_len );
+		Data[s_len] = 0;
 		Len = s_len;
 	}
 }
@@ -242,7 +244,7 @@ MyString&
 MyString::operator+=(const std::string& S) 
 {
 	
-    append_str( S.c_str(), S.length() );
+    append_str( S.c_str(), (int)S.length() );
     return *this;
 }
 
@@ -253,7 +255,7 @@ MyString::operator+=(const char *s)
     if( !s || *s == '\0' ) {
 		return *this;
 	}
-    append_str( s, strlen( s ) );
+    append_str( s, (int)strlen( s ) );
     return *this;
 }
 
@@ -275,13 +277,16 @@ MyString::append_str( const char *s, int s_len )
 
 	if (pCopy)
 	{
-		strcpy( Data + Len, pCopy); // b/c you invalided s w/reserve_at_least
+		strncpy( Data + Len, pCopy, s_len); // b/c you invalided s w/reserve_at_least
 		delete [] pCopy; 
 	}
 	else
-		strcpy( Data + Len, s);
+	{
+		strncpy( Data + Len, s, s_len);
+	}
 
 	Len += s_len;
+	Data[Len] = 0;
 }
 
 void
@@ -328,7 +333,7 @@ MyString::operator+=( int i )
 	const int bufLen = 64;
 	char tmp[bufLen];
 	::snprintf( tmp, bufLen, "%d", i );
-    int s_len = strlen( tmp );
+    int s_len = (int)strlen( tmp );
 	ASSERT(s_len < bufLen);
 	append_str( tmp, s_len );
     return *this;
@@ -341,7 +346,7 @@ MyString::operator+=( unsigned int ui )
 	const int bufLen = 64;
 	char tmp[bufLen];
 	::snprintf( tmp, bufLen, "%u", ui );
-	int s_len = strlen( tmp );
+	int s_len = (int)strlen( tmp );
 	ASSERT(s_len < bufLen);
 	append_str( tmp, s_len );
 	return *this;
@@ -354,7 +359,7 @@ MyString::operator+=( long l )
 	const int bufLen = 64;
 	char tmp[bufLen];
 	::snprintf( tmp, bufLen, "%ld", l );
-	int s_len = strlen( tmp );
+	int s_len = (int)strlen( tmp );
 	ASSERT(s_len < bufLen);
 	append_str( tmp, s_len );
 	return *this;
@@ -364,13 +369,13 @@ MyString::operator+=( long l )
 MyString&
 MyString::operator+=( long long l )
 {
-        const int bufLen = 64;
-        char tmp[bufLen];
-        ::snprintf( tmp, bufLen, "%lld", l );
-        int s_len = strlen( tmp );
-        ASSERT(s_len < bufLen);
-        append_str( tmp, s_len );
-        return *this;
+	const int bufLen = 64;
+	char tmp[bufLen];
+	::snprintf( tmp, bufLen, "%lld", l );
+	int s_len = (int)strlen( tmp );
+	ASSERT(s_len < bufLen);
+	append_str( tmp, s_len );
+	return *this;
 }
 
 
@@ -380,14 +385,167 @@ MyString::operator+=( double d )
 	const int bufLen = 128;
 	char tmp[bufLen];
 	::snprintf( tmp, bufLen, "%f", d );
-    int s_len = strlen( tmp );
+	int s_len = (int)strlen( tmp );
 	ASSERT(s_len < bufLen);
 	append_str( tmp, s_len );
-    return *this;
+	return *this;
 }
 
+
+// ----------------------------------------
+//           Serialization helpers
+// ----------------------------------------
+// (see YourStringDeserializer for corresponding deserialization)
+
+// append an integer for serialization
+template <class T> bool MyString::serialize_int(T val) {
+	char buf[65];
+	if (std::numeric_limits<T>::is_signed) {
+		::snprintf(buf, COUNTOF(buf), "%lld", (long long)val);
+	} else {
+		::snprintf(buf, COUNTOF(buf), "%llu", (unsigned long long)val);
+	}
+	return serialize_string(buf);
+}
+
+// bool specialization of serialize_int
+template <> bool MyString::serialize_int<bool>(bool val) { append_str(val ? "1" : "0", 1); return true; }
+
+#ifdef WIN32
+#define strtoull _strtoui64
+#pragma push_macro("min")
+#pragma push_macro("max")
+#undef min
+#undef max
+#endif
+
+// deserialize an int into the given value, and advance the deserialization pointer.
+// returns true if a valid in-range int was found at the current deserialize position
+// if true is returned, the internal buffer pointer is advanced past the parsed int
+// returns false and does NOT advance the deserialization pointer if there is no int
+// at the current position, or the int is out of range.
+template <class T> bool YourStringDeserializer::deserialize_int(T* val)
+{
+	if ( ! m_p) m_p = m_str;
+	if ( ! m_p) return false;
+
+	char * endp = const_cast<char*>(m_p);
+	if (std::numeric_limits<T>::is_signed) {
+		long long tmp;
+		tmp = strtoll(m_p, &endp, 10);
+		if (tmp < (long long)std::numeric_limits<T>::min() || tmp > (long long)std::numeric_limits<T>::max()) return false;
+		if (endp == m_p) return false;
+		*val = (T)tmp;
+	} else {
+		unsigned long long tmp;
+		tmp = strtoull(m_p, &endp, 10);
+		if (tmp > (unsigned long long)std::numeric_limits<T>::max()) return false;
+		if (endp == m_p) return false;
+		*val = (T)tmp;
+	}
+	m_p = endp;
+	return true;
+}
+// bool specialization for deserialize_int
+template <> bool YourStringDeserializer::deserialize_int<bool>(bool* val)
+{
+	if ( ! m_p) m_p = m_str;
+	if ( ! m_p) return false;
+	if (*m_p == '0') { ++m_p; *val = false; return true; }
+	if (*m_p == '1') { ++m_p; *val = true; return true; }
+	return false;
+}
+
+// check for the given separator at the current position and advance past it if found
+// returns true if the current position has the given separator
+bool YourStringDeserializer::deserialize_sep(const char * sep)
+{
+	if ( ! m_p) m_p = m_str;
+	if ( ! m_p) return false;
+	const char * p1 = m_p;
+	const char * p2 = sep;
+	while (*p2 && (*p1 == *p2)) { ++p1; ++p2; }
+	if (!*p2) { m_p = p1; return true; }
+	return false;
+}
+
+// return pointer and length of the string between the current deserialize position
+// and the next instance of the given separator character.
+// returns true if the separator was found, false if not.
+// if separator occurrs at current position, returned length will be 0.
+bool YourStringDeserializer::deserialize_string(const char *& start, size_t & len, const char * sep)
+{
+	if ( ! m_p) m_p = m_str;
+	if ( ! m_p) return false;
+	const char * p = strstr(m_p, sep);
+	if (p) { start = m_p; len = (p - m_p); m_p = p; return true; }
+	return false;
+}
+
+// Copy a string from the current deserialize position to the next separator into the given MyString
+// returns true if the separator was found, false if not.
+// if return value is true, the val will be set to the string, if false val is unchanged.
+bool YourStringDeserializer::deserialize_string(MyString & val, const char * sep)
+{
+	const char * p; size_t len;
+	if ( ! deserialize_string(p, len, sep)) return false;
+	val.set(p, (int)len);
+	return true;
+}
+
+// force instantiation of the serialize and deserialize functions that users of condor_utils will need
+template bool MyString::serialize_int<int>(int val);
+template bool MyString::serialize_int<long>(long val);
+template bool MyString::serialize_int<long long>(long long val);
+template bool MyString::serialize_int<unsigned int>(unsigned int val);
+template bool MyString::serialize_int<unsigned long>(unsigned long val);
+template bool MyString::serialize_int<unsigned long long>(unsigned long long val);
+
+template bool YourStringDeserializer::deserialize_int<int>(int* val);
+template bool YourStringDeserializer::deserialize_int<long>(long* val);
+template bool YourStringDeserializer::deserialize_int<long long>(long long* val);
+template bool YourStringDeserializer::deserialize_int<unsigned int>(unsigned int* val);
+template bool YourStringDeserializer::deserialize_int<unsigned long>(unsigned long* val);
+template bool YourStringDeserializer::deserialize_int<unsigned long long>(unsigned long long* val);
+
+#if 0
+void force_mystring_templates() {
+	MyString str;
+	str.serialize_int(false);
+	//str.serialize_int((char)0);
+	//str.serialize_int((short)0);
+	str.serialize_int((int)0);
+	str.serialize_int((long)0);
+	str.serialize_int((long long)0);
+	//str.serialize_int((unsigned char)0);
+	//str.serialize_int((unsigned short)0);
+	str.serialize_int((unsigned int)0);
+	str.serialize_int((unsigned long)0);
+	str.serialize_int((unsigned long long)0);
+
+	YourStringDeserializer buf("0*0*0*");
+	bool b;
+	int i; long l; long long ll;
+	unsigned int ui; unsigned long ul; unsigned long long ull;
+	buf.deserialize_int(&b);
+	buf.deserialize_int(&i);
+	buf.deserialize_int(&l);
+	buf.deserialize_int(&ll);
+	buf.deserialize_int(&ui);
+	buf.deserialize_int(&ul);
+	buf.deserialize_int(&ull);
+}
+#endif
+
+#ifdef WIN32
+ #pragma pop_macro("min")
+ #pragma pop_macro("max")
+#endif
+
 MSC_RESTORE_WARNING(6052) // call to snprintf might not null terminate string.
 
+
+
 /*--------------------------------------------------------------------
  *
  * Miscellaneous Functions
@@ -516,12 +674,12 @@ MyString::replaceString(
 {
 	SimpleList<int> listMatchesFound; 		
 	
-	int iToReplaceLen = strlen(pszToReplace);
+	int iToReplaceLen = (int)strlen(pszToReplace);
 	if (!iToReplaceLen) {
 		return false;
 	}
 	
-	int iWithLen = strlen(pszReplaceWith);
+	int iWithLen = (int)strlen(pszReplaceWith);
 	while (iStartFromPos <= Len){
 		iStartFromPos = find(pszToReplace, iStartFromPos);
 		if (iStartFromPos == -1)
@@ -560,19 +718,19 @@ MyString::replaceString(
 	return true;
 }
 
-bool
+const char *
 MyString::vformatstr_cat(const char *format,va_list args) 
 {
 	char *buffer = NULL;
 	int s_len;
 
     if( !format || *format == '\0' ) {
-		return true;
+		return Value();
 	}
 #ifdef HAVE_VASPRINTF
 	s_len = vasprintf(&buffer, format, args);
 	if (-1 == s_len) { // if alloc not possible or other error
-		return false;
+		return NULL;
 	}
 #else
     s_len = vprintf_length(format,args);
@@ -580,7 +738,7 @@ MyString::vformatstr_cat(const char *format,va_list args)
     if( Len + s_len > capacity || !Data ) {
 		if(!reserve_at_least( Len + s_len )) {
 			free(buffer);
-			return false;
+			return NULL;
 		}
     }
 #ifdef HAVE_VASPRINTF
@@ -593,13 +751,13 @@ MyString::vformatstr_cat(const char *format,va_list args)
 	::vsprintf(Data + Len, format, args);
 #endif
 	Len += s_len;
-    return true;
+    return Value();
 }
 
-bool 
+const char *
 MyString::formatstr_cat(const char *format,...)
 {
-	bool    succeeded;
+	const char *succeeded;
 	va_list args;
 
 	va_start(args, format);
@@ -609,7 +767,7 @@ MyString::formatstr_cat(const char *format,...)
 	return succeeded;
 }
 
-bool
+const char *
 MyString::vformatstr(const char *format,va_list args)
 {
 	Len = 0;
@@ -617,10 +775,10 @@ MyString::vformatstr(const char *format,va_list args)
 	return vformatstr_cat(format,args);
 }
 
-bool
+const char *
 MyString::formatstr(const char *format,...)
 {
-	bool    succeeded;
+	const char *succeeded;
 	va_list args;
 
 	va_start(args, format);
@@ -684,6 +842,25 @@ MyString::trim( void )
 	}
 }
 
+char
+MyString::trim_quotes(const char * quote_chars)
+{
+	if ( ! quote_chars) quote_chars = "\"";
+	if( Len < 2 ) {
+		return 0;
+	}
+	int begin = 0;
+	char ch = Data[begin];
+	if (strchr(quote_chars, ch)) {
+		int end = Length() -1;
+		if (end > begin && Data[end] == ch) {
+			*this = Substr(begin+1, end-1);
+			return ch;
+		}
+	}
+	return 0;
+}
+
 void
 MyString::compressSpaces( void )
 {
@@ -727,7 +904,7 @@ MyString::randomlyGenerate(const char *set, int len)
 	Len = len;
 	capacity = len;
 
-	set_len = strlen(set);
+	set_len = (int)strlen(set);
 
 	// now pick randomly from the set and fill stuff in
 	for (i = 0; i < len ; i++) {
@@ -891,13 +1068,77 @@ MyString::readLine( FILE* fp, bool append )
 		} else {
 			*this += buf;
 		}
-		if( Len && Data[Len-1] == '\n' ) {
+		if( Len && Data[Len-1] == '\n' )
+		{
 				// we found a newline, return success
 			return true;
 		}
 	}
 }
 
+// the MyStringFpSource can just use MyString::readLine
+bool
+MyStringFpSource::readLine(MyString & str, bool append /* = false*/)
+{
+	return str.readLine(fp, append);
+}
+
+bool
+MyStringFpSource::isEof()
+{
+	return feof(fp) != 0;
+}
+
+
+// the MyStringCharSource scans a string buffer returning
+// whenver it sees a \n
+bool
+MyStringCharSource::readLine(MyString & str, bool append /* = false*/)
+{
+	ASSERT(ptr || ! ix);
+	char * p = ptr+ix;
+
+	// if no buffer, we are at EOF
+	if ( ! p) {
+		if ( ! append) str.clear();
+		return false;
+	}
+
+	// scan for the next \n and return it plus all the chars up until it
+	int cch = 0;
+	while (p[cch] && p[cch] != '\n') ++cch;
+	if (p[cch] == '\n') ++cch;
+
+	// if we did not advance, then we are at EOF
+	if ( ! cch) {
+		if ( ! append) str.clear();
+		return false;
+	}
+
+	if (append) {
+		str.append_str(p, cch);
+	} else {
+		str.assign_str(p, cch);
+	}
+
+	// advance the current position past what we returned.
+	ix += cch;
+	return true;
+}
+
+bool
+MyStringCharSource::isEof()
+{
+	return !ptr || !ptr[ix];
+}
+
+// populate a MyString from any MyStringSource
+//
+bool MyString::readLine( MyStringSource & src, bool append /*= false*/) {
+	return src.readLine(*this, append);
+}
+
+
 /*--------------------------------------------------------------------
  *
  * Tokenize
@@ -957,3 +1198,83 @@ unsigned int MyStringHash( const MyString &str )
 }
 
 
+/*--------------------------------------------------------------------
+ *
+ * YourString
+ *
+ *--------------------------------------------------------------------*/
+
+// Note that the comparison operators here treat a NULL in YourString as valid
+// NULL is < than all other strings, equal to itself and NOT equal to ""
+//
+
+bool YourString::operator ==(const char * str) const {
+	if (m_str == str) return true;
+	if ((!m_str) || (!str)) return false;
+	return strcmp(m_str,str) == 0;
+}
+bool YourString::operator ==(const YourString &rhs) const {
+	if (m_str == rhs.m_str) return true;
+	if ((!m_str) || (!rhs.m_str)) return false;
+	return strcmp(m_str,rhs.m_str) == 0;
+}
+bool YourString::operator<(const char * str) const {
+	if ( ! m_str) { return str ? true : false; }
+	else if ( ! str) { return false; }
+	return strcmp(m_str, str) < 0;
+}
+bool YourString::operator<(const YourString &rhs) const {
+	if ( ! m_str) { return rhs.m_str ? true : false; }
+	else if ( ! rhs.m_str) { return false; }
+	return strcmp(m_str, rhs.m_str) < 0;
+}
+unsigned int YourString::hashFunction(const YourString &s) {
+	// hash function for strings
+	// Chris Torek's world famous hashing function
+	unsigned int hash = 0;
+	if (!s.m_str) return 7; // Least random number
+
+	const char *p = s.m_str;
+	while (*p) {
+		hash = (hash<<5)+hash + (unsigned char)*p;
+		p++;
+	}
+
+	return hash;
+}
+unsigned int YourString::hashFunctionNoCase(const YourString &s) {
+	// hash function for strings
+	// Chris Torek's world famous hashing function
+	unsigned int hash = 0;
+	if (!s.m_str) return 7; // Least random number
+
+	const char *p = s.m_str;
+	while (*p) {
+		hash = (hash<<5)+hash + (unsigned char)(*p & ~0x20);
+		p++;
+	}
+
+	return hash;
+}
+
+
+bool YourStringNoCase::operator ==(const char * str) const {
+	if (m_str == str) return true;
+	if ((!m_str) || (!str)) return false;
+	return strcasecmp(m_str,str) == 0;
+}
+bool YourStringNoCase::operator ==(const YourStringNoCase &rhs) const {
+	if (m_str == rhs.m_str) return true;
+	if ((!m_str) || (!rhs.m_str)) return false;
+	return strcasecmp(m_str,rhs.m_str) == 0;
+}
+bool YourStringNoCase::operator <(const char * str) const {
+	if ( ! m_str) { return str ? true : false; }
+	else if ( ! str) { return false; }
+	return strcasecmp(m_str, str) < 0;
+}
+
+
+
+
+
