@@ -134,6 +134,16 @@ Regex::isInitialized( )
 	return ( this->re != NULL );
 }
 
+size_t
+Regex::mem_used()
+{
+	if ( ! re) return 0;
+
+	size_t size = 0;
+	pcre_fullinfo(re, NULL, PCRE_INFO_SIZE, &size);
+	return size;
+}
+
 pcre *
 Regex::clone_re(pcre * re)
 {
@@ -141,7 +151,7 @@ Regex::clone_re(pcre * re)
 		return NULL;
 	}
 
-	size_t size;
+	size_t size = 0;
 	pcre_fullinfo(re, NULL, PCRE_INFO_SIZE, &size);
 
 	pcre * newre = (pcre *) pcre_malloc(size * sizeof(char));
