@@ -308,6 +308,9 @@ const struct Translation CollectorTranslation[] = {
 	{ "UPDATE_QUILL_AD", UPDATE_QUILL_AD },
 	{ "QUERY_QUILL_ADS", QUERY_QUILL_ADS },
 	{ "INVALIDATE_QUILL_ADS", INVALIDATE_QUILL_ADS },
+	{ "UPDATE_ACCOUNTING_AD", UPDATE_ACCOUNTING_AD },
+	{ "QUERY_ACCOUNTING_ADS", QUERY_ACCOUNTING_ADS },
+	{ "INVALIDATE_ACCOUNTING_ADS", INVALIDATE_ACCOUNTING_ADS },
 	{ "", 0 }
 };
 
