@@ -28,6 +28,11 @@
 #include "classad/classadCache.h"
 #include "env.h"
 #include "condor_arglist.h"
+#define CLASSAD_USER_MAP_RETURNS_STRINGLIST 1
+
+class MapFile;
+extern int reconfig_user_maps();
+extern bool user_map_do_mapping(const char * mapname, const char * input, MyString & output);
 
 #if defined(HAVE_DLOPEN)
 #include <dlfcn.h>
@@ -86,6 +91,8 @@ Reconfig()
 		}
 	}
 
+	reconfig_user_maps();
+
 	char *user_python_char = param("CLASSAD_USER_PYTHON_MODULES");
 	if (user_python_char)
 	{
@@ -159,6 +166,7 @@ void releaseTheMatchAd()
 	the_match_ad_in_use = false;
 }
 
+
 static
 bool stringListSize_func( const char * /*name*/,
 						  const classad::ArgumentList &arg_list,
@@ -450,6 +458,102 @@ bool stringListRegexpMember_func( const char * /*name*/,
 	return true;
 }
 
+// map an input string using the given mapping set, this works using the
+// same mapping function that the security layer uses to map security principals
+// usage: 
+//           // the 2 argument form returns a list of groups
+//        userMap("user_to_groups", "user") -> {"group1","group2"}
+// 
+//           // the 3 argument form returns a single string. it will return the preferred group
+//           // if the user is in that group. otherwise it will return the first group
+//        userMap("user_to_groups", "user", "group2") -> "group2"
+//        userMap("user_to_groups", "user", "group3") -> "group1"
+
+//           // the 4 argument form return a single group. it will return the preferred group
+//           // if the user is in that group. otherwise it will return the first group.
+//           // If the user is not in any groups, it will return the 4th argument
+//        userMap("user_to_groups", "user", "group2", "defgroup") -> "group2"
+//        userMap("user_to_groups", "user", "group3", "defgroup") -> "group1"
+
+static
+bool userMap_func( const char * /*name*/,
+	const classad::ArgumentList & arg_list,
+	classad::EvalState & state,
+	classad::Value & result)
+{
+	classad::Value mapVal, userVal, prefVal;
+
+	size_t cargs = arg_list.size();
+	if (cargs < 2 || cargs > 4) {
+		result.SetErrorValue();
+		return true;
+	}
+	if ( ! arg_list[0]->Evaluate(state, mapVal) ||
+		 ! arg_list[1]->Evaluate(state, userVal) ||
+		 (cargs >= 3 && !arg_list[2]->Evaluate(state, prefVal)) ||
+		 (cargs >= 4 && !arg_list[3]->Evaluate(state, result))
+		) {
+		result.SetErrorValue();
+		return false; // propagate evaluation failure.
+	}
+
+	std::string mapName, userName;
+	if ( ! mapVal.IsStringValue(mapName) || ! userVal.IsStringValue(userName)) {
+		if (mapVal.IsErrorValue() || userVal.IsErrorValue()) {
+			result.SetErrorValue();
+		} else if (cargs < 4) {
+			result.SetUndefinedValue();
+		}
+		return true;
+	}
+
+	MyString output;
+	if (user_map_do_mapping(mapName.c_str(), userName.c_str(), output)) {
+		StringList items(output.Value(), ",");
+
+		if (cargs == 2) {
+			// 2 arg form, return a list.
+		#ifdef CLASSAD_USER_MAP_RETURNS_STRINGLIST
+			result.SetStringValue(output.c_str());
+		#else
+			classad_shared_ptr<classad::ExprList> lst( new classad::ExprList() );
+			ASSERT(lst);
+			for (const char * str = items.first(); str != NULL; str = items.next()) {
+				classad::Value val; val.SetStringValue(str);
+				lst->push_back(classad::Literal::MakeLiteral(val));
+			}
+			result.SetListValue(lst);
+		#endif
+		} else {
+			// 3 arg form, return a string either the preferred item, or the first item
+			// preferred item match is case-insensitive
+			std::string pref;
+			const char * selected_item = NULL;
+			const bool any_case = true;
+			if (prefVal.IsStringValue(pref)) { selected_item = items.find(pref.c_str(), any_case); }
+		#if 1
+			else if (prefVal.IsUndefinedValue()) { selected_item = items.first(); }
+		#else
+			// if preferred item is not in the list, use the first item
+			if ( ! selected_item) { selected_item = items.first(); }
+		#endif
+			if (selected_item) {
+				result.SetStringValue(selected_item);
+			} else if (cargs < 4) {
+				// if result has not been primed with the default value (i.e. this is the 3 arg form)
+				// set the result to undefined now.
+				result.SetUndefinedValue();
+			}
+		}
+	} else if (cargs < 4) {
+		// 2 or 3 arg form, and no mapping, return undefined.
+		result.SetUndefinedValue();
+	}
+
+	return true;
+}
+
+
 // split user@domain or slot@machine, result is always a list of 2 strings
 // if there is no @ in the input, then for user@domain the domain is empty
 // for slot@machine the slot is empty
@@ -484,7 +588,7 @@ bool splitAt_func( const char * name,
 	classad::Value first;
 	classad::Value second;
 
-	unsigned int ix = str.find_first_of('@');
+	size_t ix = str.find_first_of('@');
 	if (ix >= str.size()) {
 		if (0 == strcasecmp(name, "splitslotname")) {
 			first.SetStringValue("");
@@ -561,10 +665,10 @@ bool splitArb_func( const char * /*name*/,
 	// but runs of a single separator are handled individually, thus
 	// "foo, bar" is the same as "foo ,bar" and "foo,bar".  But not the same as
 	// "foo,,bar", which produces a list of 3 items rather than 2.
-	unsigned int ixLast = 0;
+	size_t ixLast = 0;
 	classad::Value val;
 	if (seps.length() > 0) {
-		unsigned int ix = str.find_first_of(seps, ixLast);
+		size_t ix = str.find_first_of(seps, ixLast);
 		int      ch = -1;
 		while (ix < str.length()) {
 			if (ix - ixLast > 0) {
@@ -1063,6 +1167,9 @@ void registerClassadFunctions()
 	name = "userHome";
 	classad::FunctionCall::RegisterFunction(name, userHome_func);
 
+	name = "userMap";
+	classad::FunctionCall::RegisterFunction(name, userMap_func);
+
 	// user@domain, slot@machine & sinful string crackers.
 	name = "splitusername";
 	classad::FunctionCall::RegisterFunction( name, splitAt_func );
@@ -1140,14 +1247,15 @@ ClassAd( FILE *file, const char *delimitor, int &isEOF, int&error, int &empty )
 
 	int index;
 	MyString buffer;
-	int			delimLen = strlen( delimitor );
+	MyStringFpSource myfs(file, false);
+	size_t delimLen = strlen( delimitor );
 
 	empty = TRUE;
 
 	while( 1 ) {
 
 			// get a line from the file
-		if ( buffer.readLine( file, false ) == false ) {
+		if ( buffer.readLine( myfs, false ) == false ) {
 			error = ( isEOF = feof( file ) ) ? 0 : errno;
 			return;
 		}
@@ -1183,7 +1291,7 @@ ClassAd( FILE *file, const char *delimitor, int &isEOF, int&error, int &empty )
 			buffer = "";
 			while ( strncmp( buffer.Value(), delimitor, delimLen ) &&
 					!feof( file ) ) {
-				buffer.readLine( file, false );
+				buffer.readLine( myfs, false );
 			}
 			isEOF = feof( file );
 			error = -1;
@@ -1194,12 +1302,46 @@ ClassAd( FILE *file, const char *delimitor, int &isEOF, int&error, int &empty )
 	}
 }
 
-// this method is called before each line is parsed. 
+CondorClassAdFileParseHelper::~CondorClassAdFileParseHelper()
+{
+	switch (parse_type) {
+		case Parse_xml: {
+			classad::ClassAdXMLParser * parser = (classad::ClassAdXMLParser *)new_parser;
+			delete parser;
+			new_parser = NULL;
+		} break;
+		case Parse_json: {
+			classad::ClassAdJsonParser * parser = (classad::ClassAdJsonParser *)new_parser;
+			delete parser;
+			new_parser = NULL;
+		} break;
+		case Parse_new: {
+			classad::ClassAdParser * parser = (classad::ClassAdParser *)new_parser;
+			delete parser;
+			new_parser = NULL;
+		} break;
+		default: break;
+	}
+	ASSERT( ! new_parser);
+}
+
+
+bool CondorClassAdFileParseHelper::line_is_ad_delimitor(const std::string & line)
+{
+	if (blank_line_is_ad_delimitor) {
+		const char * p = line.c_str();
+		while (*p && isspace(*p)) ++p;
+		return ( ! *p || *p == '\n');
+	}
+	return starts_with(line, ad_delimitor);
+}
+
+// this method is called before each line is parsed.
 // return 0 to skip (is_comment), 1 to parse line, 2 for end-of-classad, -1 for abort
 int CondorClassAdFileParseHelper::PreParse(std::string & line, ClassAd & /*ad*/, FILE* /*file*/)
 {
 	// if this line matches the ad delimitor, tell the parser to stop parsing
-	if (starts_with(line, ad_delimitor))
+	if (line_is_ad_delimitor(line))
 		return 2; //end-of-classad
 
 	// check for blank lines or lines whose first character is #
@@ -1209,7 +1351,7 @@ int CondorClassAdFileParseHelper::PreParse(std::string & line, ClassAd & /*ad*/,
 		if (line[ix] == '#' || line[ix] == '\n')
 			return 0; // skip this line, but don't stop parsing.
 		if (line[ix] != ' ' && line[ix] != '\t')
-			return 1; // parse this line
+			break;
 	}
 	return 1; // parse this line.
 }
@@ -1218,13 +1360,19 @@ int CondorClassAdFileParseHelper::PreParse(std::string & line, ClassAd & /*ad*/,
 // return 0 to skip and continue, 1 to re-parse line, 2 to quit parsing with success, -1 to abort parsing.
 int CondorClassAdFileParseHelper::OnParseError(std::string & line, ClassAd & /*ad*/, FILE* file)
 {
+	if (parse_type >= Parse_xml && parse_type < Parse_auto) {
+		// here line is actually errmsg.
+		//PRAGMA_REMIND("report parse errors for new parsers?")
+		return -1;
+	}
+
 		// print out where we barfed to the log file
 	dprintf(D_ALWAYS,"failed to create classad; bad expr = '%s'\n",
 			line.c_str());
 
 	// read until delimitor or EOF; whichever comes first
-	line = "";
-	while ( ! starts_with(line, ad_delimitor)) {
+	line = "NotADelim=1";
+	while ( ! line_is_ad_delimitor(line)) {
 		if (feof(file))
 			break;
 		if ( ! readLine(line, file, false))
@@ -1233,13 +1381,181 @@ int CondorClassAdFileParseHelper::OnParseError(std::string & line, ClassAd & /*a
 	return -1; // abort
 }
 
+
+int CondorClassAdFileParseHelper::NewParser(ClassAd & ad, FILE* file, bool & detected_long, std::string & errmsg)
+{
+	detected_long = false;
+	if (parse_type < Parse_xml || parse_type > Parse_auto) {
+		// return 0 to indicate this is a -long form (line oriented) parse type
+		return 0;
+	}
+
+	int rval = 1;
+	switch(parse_type) {
+		case Parse_xml: {
+			classad::ClassAdXMLParser * parser = (classad::ClassAdXMLParser *)new_parser;
+			if ( ! parser) {
+				parser = new classad::ClassAdXMLParser();
+				new_parser = (void*)parser;
+			}
+			ASSERT(parser);
+			bool fok = parser->ParseClassAd(file, ad);
+			if (fok) {
+				rval = ad.size();
+			} else if (feof(file)) {
+				rval = -99;
+			} else {
+				rval = -1;
+			}
+		} break;
+
+		case Parse_json: {
+			classad::ClassAdJsonParser * parser = (classad::ClassAdJsonParser *)new_parser;
+			if ( ! parser) {
+				parser = new classad::ClassAdJsonParser();
+				new_parser = (void*)parser;
+			}
+			ASSERT(parser);
+			bool fok = parser->ParseClassAd(file, ad, false);
+			if ( ! fok) {
+				bool keep_going = false;
+				classad::Lexer::TokenType tt = parser->getLastTokenType();
+				if (inside_list) {
+					if (tt == classad::Lexer::LEX_COMMA) { keep_going = true; }
+					else if (tt == classad::Lexer::LEX_CLOSE_BOX) { keep_going = true; inside_list = false; }
+				} else {
+					if (tt == classad::Lexer::LEX_OPEN_BOX) { keep_going = true; inside_list = true; }
+				}
+				if (keep_going) {
+					fok = parser->ParseClassAd(file, ad, false); 
+				}
+			}
+			if (fok) {
+				rval = ad.size();
+			} else if (feof(file)) {
+				rval = -99;
+			} else {
+				rval = -1;
+			}
+		} break;
+
+		case Parse_new: {
+			classad::ClassAdParser * parser = (classad::ClassAdParser *)new_parser;
+			if ( ! parser) {
+				parser = new classad::ClassAdParser();
+				new_parser = (void*)parser;
+			}
+			ASSERT(parser);
+			bool fok = parser->ParseClassAd(file, ad);
+			if ( ! fok) {
+				bool keep_going = false;
+				classad::Lexer::TokenType tt = parser->getLastTokenType();
+				if (inside_list) {
+					if (tt == classad::Lexer::LEX_COMMA) { keep_going = true; }
+					else if (tt == classad::Lexer::LEX_CLOSE_BRACE) { keep_going = true; inside_list = false; }
+				} else {
+					if (tt == classad::Lexer::LEX_OPEN_BRACE) { keep_going = true; inside_list = true; }
+				}
+				if (keep_going) {
+					fok = parser->ParseClassAd(file, ad, false); 
+				}
+			}
+			if (fok) {
+				rval = ad.size();
+			} else if (feof(file)) {
+				rval = -99;
+			} else {
+				rval = -1;
+			}
+		} break;
+
+		case Parse_auto: { // parse line oriented until we figure out the parse type
+			// get a line from the file
+			std::string buffer;
+			for (;;) {
+				if ( ! readLine(buffer, file, false)) {
+					return feof(file) ? -99 : -1;
+				}
+
+				int ee = PreParse(buffer, ad, file);
+				if (ee == 1) {
+					// pre-parser says parse it. can we use it to figure out what type we are?
+					// if we are still scanning to decide what the parse type is, we should be able to figure it out now...
+					if (buffer == "<?xml version=\"1.0\"?>\n") {
+						parse_type = Parse_xml;
+						return NewParser(ad, file, detected_long, errmsg); // skip this line, but don't stop parsing, from now on
+					} else if (buffer == "[\n" || buffer == "{\n") {
+						char ch = buffer[0];
+						// could be json or new classads, read character to figure out which.
+						int ch2 = fgetc(file);
+						if (ch == '{' && ch2 == '[') {
+							inside_list = true;
+							ungetc('[', file);
+							parse_type = Parse_new;
+							return NewParser(ad, file, detected_long, errmsg);
+						} else if (ch == '[' && ch2 == '{') {
+							inside_list = true;
+							ungetc('{', file);
+							parse_type = Parse_json;
+							return NewParser(ad, file, detected_long, errmsg);
+						} else {
+							buffer = ""; buffer[0] = ch;
+							readLine(buffer, file, true);
+						}
+					}
+					// this doesn't look like a new classad prolog, so just parse it
+					// using the line oriented parser.
+					parse_type = Parse_long;
+					errmsg = buffer;
+					detected_long = true;
+					return 0;
+				}
+			}
+			
+		} break;
+
+		default: rval = -1; break;
+	}
+
+	return rval;
+}
+
+
 // returns number of attributes added to the ad
 int ClassAd::
 InsertFromFile(FILE* file, /*out*/ bool& is_eof, /*out*/ int& error, ClassAdFileParseHelper* phelp /*=NULL*/)
 {
+	int ee = 1;
 	int cAttrs = 0;
 	std::string buffer;
 
+	if (phelp) {
+		// new classad style parsers do all of the work in the NewParser callback
+		// they will return non-zero to indicate that they are new classad style parsers.
+		bool detected_long = false;
+		cAttrs = phelp->NewParser(*this, file, detected_long, buffer);
+		if (cAttrs > 0) {
+			error = 0;
+			is_eof = false;
+			return cAttrs;
+		} else if (cAttrs < 0) {
+			if (cAttrs == -99) {
+				error = 0;
+				is_eof = true;
+				return 0;
+			}
+			is_eof = feof(file);
+			error = cAttrs;
+			return phelp->OnParseError(buffer, *this, file);
+		}
+		// got a 0 from NewParser, fall down into the old (-long) style parser
+		if (detected_long && ! buffer.empty()) {
+			// buffer has the first line that we want to parse, jump into the
+			// middle of the line-oriented parse loop.
+			goto parse_line;
+		}
+	}
+
 	while( 1 ) {
 
 			// get a line from the file
@@ -1251,7 +1567,7 @@ InsertFromFile(FILE* file, /*out*/ bool& is_eof, /*out*/ int& error, ClassAdFile
 
 		// if there is a helper, give the helper first crack at the line
 		// otherwise set ee to decide what to do with this line.
-		int ee = 1;
+		ee = 1;
 		if (phelp) {
 			ee = phelp->PreParse(buffer, *this, file);
 		} else {
@@ -1277,6 +1593,7 @@ InsertFromFile(FILE* file, /*out*/ bool& is_eof, /*out*/ int& error, ClassAdFile
 			return cAttrs;
 		}
 
+parse_line:
 		// Insert the string into the classad
 		if (Insert(buffer.c_str()) !=  0) {
 			++cAttrs;
@@ -1311,6 +1628,261 @@ InsertFromFile(FILE* file, /*out*/ bool& is_eof, /*out*/ int& error, ClassAdFile
 }
 
 
+bool CondorClassAdFileIterator::begin(
+	FILE* fh,
+	bool close_when_done,
+	CondorClassAdFileParseHelper::ParseType type)
+{
+	parse_help = new CondorClassAdFileParseHelper("\n", type);
+	free_parse_help = true;
+	file = fh;
+	close_file_at_eof = close_when_done;
+	error = 0;
+	at_eof = false;
+	return true;
+}
+
+bool CondorClassAdFileIterator::begin(
+	FILE* fh,
+	bool close_when_done,
+	CondorClassAdFileParseHelper & helper)
+{
+	parse_help = &helper;
+	free_parse_help = false;
+	file = fh;
+	close_file_at_eof = close_when_done;
+	error = 0;
+	at_eof = false;
+	return true;
+}
+
+int CondorClassAdFileIterator::next(ClassAd & classad, bool merge /*=false*/)
+{
+	if ( ! merge) classad.Clear();
+	if (at_eof) return 0;
+	if ( ! file) { error = -1; return -1; }
+
+	int cAttrs = classad.InsertFromFile(file, at_eof, error, parse_help);
+	if (cAttrs > 0) return cAttrs;
+	if (at_eof) {
+		if (file && close_file_at_eof) { fclose(file); file = NULL; }
+		return 0;
+	}
+	if (error < 0)
+		return error;
+	return 0;
+}
+
+ClassAd * CondorClassAdFileIterator::next(classad::ExprTree * constraint)
+{
+	if (at_eof) return NULL;
+
+	for (;;) {
+		ClassAd * ad = new ClassAd();
+		int cAttrs = this->next(*ad, true);
+		bool include_classad = cAttrs > 0 && error >= 0;
+		if (include_classad && constraint) {
+			classad::Value val;
+			if (ad->EvaluateExpr(constraint,val)) {
+				if ( ! val.IsBooleanValueEquiv(include_classad)) {
+					include_classad = false;
+				}
+			}
+		}
+		if (include_classad) {
+			return ad;
+		}
+		delete ad;
+		ad = NULL;
+
+		if (at_eof || error < 0) break;
+	}
+	return NULL;
+}
+
+CondorClassAdFileParseHelper::ParseType CondorClassAdFileIterator::getParseType()
+{
+	if (parse_help) return parse_help->getParseType();
+	return (CondorClassAdFileParseHelper::ParseType)-1;
+}
+
+CondorClassAdFileParseHelper::ParseType CondorClassAdListWriter::setFormat(CondorClassAdFileParseHelper::ParseType typ)
+{
+	if ( ! wrote_header && ! cNonEmptyOutputAds) out_format = typ;
+	return out_format;
+}
+
+CondorClassAdFileParseHelper::ParseType CondorClassAdListWriter::autoSetFormat(CondorClassAdFileParseHelper & parse_help)
+{
+	if (out_format != CondorClassAdFileParseHelper::Parse_auto) return out_format;
+	return setFormat(parse_help.getParseType());
+}
+
+// append a classad into the given output buffer, writing list header or separator as needed.
+// return:
+//    < 0 failure,
+//    0   nothing written
+//    1   non-empty ad appended
+int CondorClassAdListWriter::appendAd(const ClassAd & ad, std::string & output, StringList * whitelist, bool hash_order)
+{
+	if (ad.size() == 0) return 0;
+	size_t cchBegin = output.size();
+
+	classad::References attrs;
+	classad::References *print_order = NULL;
+	if ( ! hash_order || whitelist) {
+		sGetAdAttrs(attrs, ad, false, whitelist);
+		print_order = &attrs;
+	}
+
+	// if we havn't picked a format yet, pick long.
+	if (out_format < ClassAdFileParseType::Parse_long || out_format > ClassAdFileParseType::Parse_new) {
+		out_format = ClassAdFileParseType::Parse_long;
+	}
+
+	switch (out_format) {
+	default:
+	case ClassAdFileParseType::Parse_long: {
+			if (print_order) {
+				sPrintAdAttrs(output, ad, *print_order);
+			} else {
+				sPrintAd(output, ad);
+			}
+			if (output.size() > cchBegin) { output += "\n"; }
+		} break;
+
+	case ClassAdFileParseType::Parse_json: {
+			classad::ClassAdJsonUnParser  unparser;
+			output += cNonEmptyOutputAds ? ",\n" : "[\n";
+			if (print_order) {
+				unparser.Unparse(output, &ad, *print_order);
+			} else {
+				unparser.Unparse(output, &ad);
+			}
+			if (output.size() > cchBegin+2) {
+				needs_footer = wrote_header = true;
+				output += "\n";
+			} else {
+				output.erase(cchBegin);
+			}
+		} break;
+
+	case ClassAdFileParseType::Parse_new: {
+			classad::ClassAdUnParser  unparser;
+			output += cNonEmptyOutputAds ? ",\n" : "{\n";
+			if (print_order) {
+				unparser.Unparse(output, &ad, *print_order);
+			} else {
+				unparser.Unparse(output, &ad);
+			}
+			if (output.size() > cchBegin+2) {
+				needs_footer = wrote_header = true;
+				output += "\n";
+			} else {
+				output.erase(cchBegin);
+			}
+		} break;
+
+	case ClassAdFileParseType::Parse_xml: {
+			classad::ClassAdXMLUnParser  unparser;
+			unparser.SetCompactSpacing(false);
+			size_t cchTmp = cchBegin;
+			if (0 == cNonEmptyOutputAds) {
+				AddClassAdXMLFileHeader(output);
+				cchTmp = output.size();
+			}
+			if (print_order) {
+				unparser.Unparse(output, &ad, *print_order);
+			} else {
+				unparser.Unparse(output, &ad);
+			}
+			if (output.size() > cchTmp) {
+				needs_footer = wrote_header = true;
+				//no extra newlines for xml
+				// output += "\n";
+			} else {
+				output.erase(cchBegin);
+			}
+		} break;
+	}
+
+	if (output.size() > cchBegin) {
+		++cNonEmptyOutputAds;
+		return 1;
+	}
+	return 0;
+}
+
+// append a classad list footer into the given output buffer if needed
+// return:
+//    < 0 failure,
+//    0   nothing written
+//    1   non-empty ad appended
+int CondorClassAdListWriter::appendFooter(std::string & buf, bool xml_always_write_header_footer)
+{
+	int rval = 0;
+	switch (out_format) {
+	case ClassAdFileParseType::Parse_xml:
+		if ( ! wrote_header) {
+			if (xml_always_write_header_footer) {
+				// for XML output, we ALWAYS write the header and footer, even if there were no ads.
+				AddClassAdXMLFileHeader(buf);
+			} else {
+				break;
+			}
+		}
+		AddClassAdXMLFileFooter(buf);
+		rval = 1;
+		break;
+	case ClassAdFileParseType::Parse_new:
+		if (cNonEmptyOutputAds) { buf += "}\n"; rval = 1; }
+		break;
+	case ClassAdFileParseType::Parse_json:
+		if (cNonEmptyOutputAds) { buf += "]\n"; rval = 1; }
+	default:
+		// nothing to do.
+		break;
+	}
+	needs_footer = false;
+	return rval;
+}
+
+// write a classad into the given output stream attributes are sorted unless hash_order is true
+// return:
+//    < 0 failure,
+//    0   nothing written
+//    1   non-empty ad written
+static const int cchReserveForPrintingAds = 16384;
+int CondorClassAdListWriter::writeAd(const ClassAd & ad, FILE * out, StringList * whitelist, bool hash_order)
+{
+	buffer.clear();
+	if ( ! cNonEmptyOutputAds) buffer.reserve(cchReserveForPrintingAds);
+
+	int rval = appendAd(ad, buffer, whitelist, hash_order);
+	if (rval < 0) return rval;
+
+	if ( ! buffer.empty()) {
+		fputs(buffer.c_str(), out);
+	}
+	return rval;
+}
+
+// write a classad list footer into the given output stream if needed
+// return:
+//    < 0 failure,
+//    0   nothing written
+//    1   non-empty ad written
+int CondorClassAdListWriter::writeFooter(FILE* out, bool xml_always_write_header_footer)
+{
+	buffer.clear();
+	appendFooter(buffer, xml_always_write_header_footer);
+	if ( ! buffer.empty()) {
+		int rval = fputs(buffer.c_str(), out);
+		return (rval < 0) ? rval : 1;
+	}
+	return 0;
+}
+
 bool
 ClassAdAttributeIsPrivate( char const *name )
 {
@@ -1334,10 +1906,15 @@ ClassAdAttributeIsPrivate( char const *name )
 			// This attribute contains the secret file transfer cookie
 		return true;
 	}
-	if (strcasecmp(name,"ChildClaimIds") == 0) {
+	if (strcasecmp(name,ATTR_CHILD_CLAIM_IDS) == 0) {
 			// In a partitionable slot, contains all the claim ids
 		return true;
 	}
+	if (strcasecmp(name,ATTR_CLAIM_ID_LIST) == 0) {
+			// In a partitionable slot with consumption policies,
+			// contains extra claim ids
+		return true;
+	}
 	return false;
 }
 
@@ -1352,6 +1929,23 @@ int ClassAd::Insert( const char *name, classad::ExprTree *& expr, bool bCache )
 	return Insert( str, expr, bCache ) ? TRUE : FALSE;
 }
 
+int
+ClassAd::Insert(const std::string &str)
+{
+	
+	size_t backslash = str.find('\\');
+	if (backslash == std::string::npos) {
+		// No backslashes, no need to escape 'em (and mutate the string)
+		if (!classad::ClassAd::Insert(str)) {
+	  		return FALSE;
+		}
+		return TRUE;
+	}
+
+	// At least one backslash, take the slow path, allocate a new string
+	return this->Insert(str.c_str());
+}
+
 int
 ClassAd::Insert( const char *str )
 {
@@ -1981,9 +2575,11 @@ fPrintAd( FILE *file, const classad::ClassAd &ad, bool exclude_private, StringLi
 	MyString buffer;
 
 	sPrintAd( buffer, ad, exclude_private, attr_white_list );
-	fprintf( file, "%s", buffer.Value() );
-
-	return TRUE;
+	if ( fprintf(file, "%s", buffer.Value()) < 0 ) {
+		return FALSE;
+	} else {
+		return TRUE;
+	}
 }
 
 void
@@ -2014,6 +2610,9 @@ sPrintAd( MyString &output, const classad::ClassAd &ad, bool exclude_private, St
 			if ( attr_white_list && !attr_white_list->contains_anycase(itr->first.c_str()) ) {
 				continue; // not in white-list
 			}
+			if ( ad.LookupIgnoreChain(itr->first) ) {
+				continue; // attribute exists in child ad; we will print it below
+			}
 			if ( !exclude_private ||
 				 !ClassAdAttributeIsPrivate( itr->first.c_str() ) ) {
 				value = "";
@@ -2049,6 +2648,97 @@ sPrintAd( std::string &output, const classad::ClassAd &ad, bool exclude_private,
 	return rc;
 }
 
+/** Get a sorted list of attributes that are in the given ad, and also match the given whitelist (if any)
+	and privacy criteria.
+	@param attrs the set of attrs to insert into. This is set is NOT cleared first.
+	@return TRUE
+*/
+int
+sGetAdAttrs( classad::References &attrs, const classad::ClassAd &ad, bool exclude_private, StringList *attr_white_list, bool ignore_parent )
+{
+	classad::ClassAd::const_iterator itr;
+
+	for ( itr = ad.begin(); itr != ad.end(); itr++ ) {
+		if ( attr_white_list && !attr_white_list->contains_anycase(itr->first.c_str()) ) {
+			continue; // not in white-list
+		}
+		if ( !exclude_private ||
+			 !ClassAdAttributeIsPrivate( itr->first.c_str() ) ) {
+			attrs.insert(itr->first);
+		}
+	}
+
+	const classad::ClassAd *parent = ad.GetChainedParentAd();
+	if ( parent && ! ignore_parent) {
+		for ( itr = parent->begin(); itr != parent->end(); itr++ ) {
+			if ( attrs.find(itr->first) != attrs.end() ) {
+				continue; // we already inserted this into the attrs list.
+			}
+			if ( attr_white_list && !attr_white_list->contains_anycase(itr->first.c_str()) ) {
+				continue; // not in white-list
+			}
+			if ( !exclude_private ||
+				 !ClassAdAttributeIsPrivate( itr->first.c_str() ) ) {
+				attrs.insert(itr->first);
+			}
+		}
+	}
+
+	return TRUE;
+}
+
+/** Format the given attributes from the ClassAd as an old ClassAd into the given string
+	@param output The std::string to write into
+	@return TRUE
+	Note: this function and its sister that outputs to std::string do not call each other for reasons of efficiency...
+*/
+int
+sPrintAdAttrs( MyString &output, const classad::ClassAd &ad, const classad::References &attrs )
+{
+	classad::ClassAdUnParser unp;
+	unp.SetOldClassAd( true, true );
+	std::string line;
+
+	classad::References::const_iterator it;
+	for (it = attrs.begin(); it != attrs.end(); ++it) {
+		const ExprTree * tree = ad.Lookup(*it); // use Lookup rather than find in case we have a parent ad.
+		if (tree) {
+			line = *it;
+			line += " = ";
+			unp.Unparse( line, tree );
+			line += "\n";
+			output += line;
+		}
+	}
+
+	return TRUE;
+}
+
+/** Format the given attributes from the ClassAd as an old ClassAd into the given string
+	@param output The std::string to write into
+	@return TRUE
+	Note: this function and its sister the outputs to MyString do not call each other for reasons of efficiency...
+*/
+int
+sPrintAdAttrs( std::string &output, const classad::ClassAd &ad, const classad::References &attrs )
+{
+	classad::ClassAdUnParser unp;
+	unp.SetOldClassAd( true, true );
+
+	classad::References::const_iterator it;
+	for (it = attrs.begin(); it != attrs.end(); ++it) {
+		const ExprTree * tree = ad.Lookup(*it); // use Lookup rather than find in case we have a parent ad.
+		if (tree) {
+			output += *it;
+			output += " = ";
+			unp.Unparse( output, tree );
+			output += "\n";
+		}
+	}
+
+	return TRUE;
+}
+
 // Taken from the old classad's function. Got rid of incorrect documentation. 
 ////////////////////////////////////////////////////////////////////////////////// Print an expression with a certain name into a buffer. 
 // The returned buffer will be allocated with malloc(), and it needs
@@ -2058,7 +2748,7 @@ char*
 sPrintExpr(const classad::ClassAd &ad, const char* name)
 {
 	char *buffer = NULL;
-	int buffersize = 0;
+	size_t buffersize = 0;
 	classad::ClassAdUnParser unp;
     string parsedString;
 	classad::ExprTree* expr;
@@ -2380,16 +3070,20 @@ CopyAttribute( char const *target_attr, char const *source_attr,
 {
 	ASSERT( target_attr );
 	ASSERT( source_attr );
-	if( !source_ad ) {
-		source_ad = this;
-	}
+        if (!source_ad) {source_ad = this;}
+        
+	CopyAttribute(target_attr, *this, source_attr, *source_ad);
+}
 
-	classad::ExprTree *e = source_ad->Lookup( source_attr );
+void ClassAd::
+CopyAttribute(const char *target_attr, classad::ClassAd &target_ad, const char *source_attr, const classad::ClassAd &source_ad)
+{
+	classad::ExprTree *e = source_ad.Lookup( source_attr );
 	if ( e ) {
 		e = e->Copy();
-		Insert( target_attr, e, false );
+		target_ad.Insert( target_attr, e, false );
 	} else {
-		Delete( target_attr );
+		target_ad.Delete( target_attr );
 	}
 }
 
@@ -2432,14 +3126,11 @@ sPrintAdAsXML(std::string &output, const classad::ClassAd &ad, StringList *attr_
 		attr_white_list->rewind();
 		while( (attr = attr_white_list->next()) ) {
 			if ( (expr = ad.Lookup( attr )) ) {
-				tmp_ad.Insert( attr, expr, false );
+				classad::ExprTree *new_expr = expr->Copy();
+				tmp_ad.Insert( attr, new_expr, false );
 			}
 		}
 		unparser.Unparse( xml, &tmp_ad );
-		attr_white_list->rewind();
-		while( (attr = attr_white_list->next()) ) {
-			tmp_ad.Remove( attr );
-		}
 	} else {
 		unparser.Unparse( xml, &ad );
 	}
@@ -2448,11 +3139,60 @@ sPrintAdAsXML(std::string &output, const classad::ClassAd &ad, StringList *attr_
 }
 ///////////// end XML functions /////////
 
-char const *
-EscapeAdStringValue(char const *val, std::string &buf)
+int
+fPrintAdAsJson(FILE *fp, const classad::ClassAd &ad, StringList *attr_white_list)
 {
-    if(val == NULL)
+    if(!fp)
+    {
+        return FALSE;
+    }
+
+    std::string out;
+    sPrintAdAsJson(out,ad,attr_white_list);
+    fprintf(fp, "%s", out.c_str());
+    return TRUE;
+}
+
+int
+sPrintAdAsJson(MyString &output, const classad::ClassAd &ad, StringList *attr_white_list)
+{
+	std::string std_output;
+	int rc = sPrintAdAsJson(std_output, ad, attr_white_list);
+	output += std_output;
+	return rc;
+}
+
+int
+sPrintAdAsJson(std::string &output, const classad::ClassAd &ad, StringList *attr_white_list)
+{
+	classad::ClassAdJsonUnParser unparser;
+
+	if ( attr_white_list ) {
+		classad::ClassAd tmp_ad;
+		classad::ExprTree *expr;
+		const char *attr;
+		attr_white_list->rewind();
+		while( (attr = attr_white_list->next()) ) {
+			if ( (expr = ad.Lookup( attr )) ) {
+				classad::ExprTree *new_expr = expr->Copy();
+				tmp_ad.Insert( attr, new_expr, false );
+			}
+		}
+		unparser.Unparse( output, &tmp_ad );
+	} else {
+		unparser.Unparse( output, &ad );
+	}
+	return TRUE;
+}
+
+char const *
+QuoteAdStringValue(char const *val, std::string &buf)
+{
+    if(val == NULL) {
         return NULL;
+    }
+
+    buf.clear();
 
     classad::Value tmpValue;
     classad::ClassAdUnParser unparse;
@@ -2462,7 +3202,6 @@ EscapeAdStringValue(char const *val, std::string &buf)
     tmpValue.SetStringValue(val);
     unparse.Unparse(buf, tmpValue);
 
-	buf = buf.substr( 1, buf.length() - 2 );
     return buf.c_str();
 }
 
@@ -2536,6 +3275,15 @@ GetExprReferences(const char* expr,
 	return true;
 }
 
+bool ClassAd::
+GetExprReferences(classad::ExprTree *tree,
+				  StringList *internal_refs,
+				  StringList *external_refs) const
+{
+	_GetReferences( tree, internal_refs, external_refs );
+	return true;
+}
+
 static void AppendReference( StringList &reflist, char const *name )
 {
 	char const *end = strchr(name,'.');
