@@ -38,6 +38,8 @@ class Stream;
 #define	ATTRLIST_MAX_EXPRESSION 10240
 #endif
 
+class MapFile; // forward ref
+
 namespace compat_classad {
 
 typedef enum {
@@ -75,6 +77,19 @@ int sPrintAd( MyString &output, const classad::ClassAd &ad, bool exclude_private
 	*/
 int sPrintAd( std::string &output, const classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL );
 
+	/** Get a sorted list of attributes that are in the given ad, and also match the given whitelist (if any)
+		@param attrs the set of attrs to insert into. This is set is NOT cleared first.
+		@return TRUE
+	*/
+int sGetAdAttrs( classad::References &attrs, const classad::ClassAd &ad, bool exclude_private = false, StringList *attr_white_list = NULL, bool ignore_parent = false );
+
+	/** Format the given attributes from the ClassAd as an old ClassAd into the given string
+		@param output The std::string to write into
+		@return TRUE
+	*/
+int sPrintAdAttrs( std::string &output, const classad::ClassAd &ad, const classad::References & attrs );
+int sPrintAdAttrs( MyString &output, const classad::ClassAd &ad, const classad::References & attrs );
+
 class ClassAd : public classad::ClassAd
 {
  public:
@@ -109,6 +124,7 @@ class ClassAd : public classad::ClassAd
 		 *  @param str A string of the form "Attribute = Value"
 		 */
 	int Insert(const char *str);
+	int Insert(const std::string &str); // somewhat faster than above
 
 		/* Insert an attribute/value into the Classad
 		 */
@@ -362,6 +378,11 @@ class ClassAd : public classad::ClassAd
 	// CopyAttribute(target_attr,target_attr,source_ad).
 	void CopyAttribute(char const *target_attr, classad::ClassAd *source_ad );
 
+        static void CopyAttribute(const char *target_attr, classad::ClassAd &target_ad, const char *source_attr, const classad::ClassAd &source_ad);
+        static void CopyAttribute(const std::string &target_attr, classad::ClassAd &target_ad, const std::string &source_attr, const classad::ClassAd &source_ad)
+        {CopyAttribute(target_attr.c_str(), target_ad, source_attr.c_str(), source_ad);}
+
+
     /** Takes the ad this is chained to, copies over all the 
      *  attributes from the parent ad that aren't in this classad
      *  (so attributes in both the parent ad and this ad retain the 
@@ -378,6 +399,10 @@ class ClassAd : public classad::ClassAd
                 StringList *internal_refs,
                 StringList *external_refs) const;
 
+    bool GetExprReferences(ExprTree * expr,
+                StringList *internal_refs,
+                StringList *external_refs) const;
+
 	static void Reconfig();
 	static bool m_initConfig;
 	static bool m_strictEvaluation;
@@ -422,6 +447,10 @@ class ClassAdFileParseHelper
 	virtual int PreParse(std::string & line, ClassAd & ad, FILE* file)=0;
 	// return 0 to skip and continue, 1 to re-parse line, 2 to quit parsing with success, -1 to abort parsing.
 	virtual int OnParseError(std::string & line, ClassAd & ad, FILE* FILE)=0;
+	// return non-zero if new parser, 0 if old (line oriented) parser, if parse type is auto
+	// it may return 0 and also set detected_long to indicate that errmsg should be parsed
+	// as a line from the file. we do this to avoid having to backtrack the FILE*
+	virtual int NewParser(ClassAd & ad, FILE* file, bool & detected_long, std::string & errmsg)=0;
 };
 
 // this implements a classad file parse helper that
@@ -432,16 +461,113 @@ class CondorClassAdFileParseHelper : public ClassAdFileParseHelper
  public:
 	// Some compilers whine when you have virtual methods but not an
 	// explicit virtual destructor
-	virtual ~CondorClassAdFileParseHelper() {}
+	virtual ~CondorClassAdFileParseHelper();
 	// return 0 to skip (is_comment), 1 to parse line, 2 for end-of-classad, -1 for abort
 	virtual int PreParse(std::string & line, ClassAd & ad, FILE* file);
 	// return 0 to skip and continue, 1 to re-parse line, 2 to quit parsing with success, -1 to abort parsing.
 	virtual int OnParseError(std::string & line, ClassAd & ad, FILE* FILE);
-	CondorClassAdFileParseHelper(std::string delim) : ad_delimitor(delim) {};
+	// return non-zero if new parser, 0 if old (line oriented) parser, if parse type is auto
+	// it may return 0 and also set detected_long to indicate that errmsg should be parsed
+	// as a line from the file. we do this to avoid having to backtrack the FILE*
+	virtual int NewParser(ClassAd & ad, FILE* file, bool & detected_long, std::string & errmsg);
+
+	enum ParseType {
+		Parse_long=0, // file is in the traditional -long form, possibly with a delimiter line between ads
+		Parse_xml,    // file is in -xml form
+		Parse_json,   // file is in -json form, usually begins with a line of "[" and has a with a line of "," between ads
+		Parse_new,    // file is in new classads form, may begin with a line of "{" and have a line of "," between ads, or may just begin with a line of [
+		Parse_auto,   // parse helper should figure out what the form is
+	};
+
+	CondorClassAdFileParseHelper(std::string delim, ParseType typ=Parse_long) 
+		: ad_delimitor(delim), parse_type(typ), new_parser(NULL), inside_list(false)
+		, blank_line_is_ad_delimitor(delim=="\n") {};
+	ParseType getParseType() { return parse_type; }
+	bool configure(const char * delim, ParseType typ) {
+		if (new_parser) return false;
+		if (delim) { ad_delimitor = delim; }
+		parse_type = typ;
+		inside_list = false;
+		blank_line_is_ad_delimitor = ad_delimitor=="\n";
+		return true;
+	}
  private:
+	CondorClassAdFileParseHelper(const CondorClassAdFileParseHelper & that); // no copy construction
+	CondorClassAdFileParseHelper & operator=(const CondorClassAdFileParseHelper & that); // no assignment
+	bool line_is_ad_delimitor(const std::string & line);
+
 	std::string ad_delimitor;
+	ParseType parse_type;
+	void*     new_parser; // a class whose type depends on the value of parse_type.
+	bool      inside_list;
+	bool      blank_line_is_ad_delimitor;
 };
 
+// This implements a generic classad FILE* reader that can be used as an iterator
+//
+class CondorClassAdFileIterator
+{
+public:
+	CondorClassAdFileIterator() 
+		: parse_help(NULL)
+		, file(NULL)
+		, error(0), at_eof(false)
+		, close_file_at_eof(false)
+		, free_parse_help(false)
+	{}
+	~CondorClassAdFileIterator() {
+		if (file && close_file_at_eof) { fclose(file); file = NULL; }
+		if (parse_help && free_parse_help) { delete parse_help; parse_help = NULL; }
+	}
+
+	bool begin(FILE* fh, bool close_when_done, CondorClassAdFileParseHelper & helper);
+	bool begin(FILE* fh, bool close_when_done, CondorClassAdFileParseHelper::ParseType type);
+	int  next(ClassAd & out, bool merge=false);
+	ClassAd * next(classad::ExprTree * constraint);
+	CondorClassAdFileParseHelper::ParseType getParseType();
+
+protected:
+	CondorClassAdFileParseHelper * parse_help;
+	FILE* file;
+	int  error;
+	bool at_eof;
+	bool close_file_at_eof;
+	bool free_parse_help;
+};
+
+
+// This implements a generic classad list writer. This class keeps track of
+// whether it has written any non-empty ads, and writes the appropriate xml/list header
+// before the first ad. When writeFooter or appendFooter is called, the appropriate footer
+// is written only if a header was previously written.
+class CondorClassAdListWriter
+{
+public:
+	CondorClassAdListWriter(CondorClassAdFileParseHelper::ParseType typ=CondorClassAdFileParseHelper::Parse_long)
+		: out_format(typ), cNonEmptyOutputAds(0), wrote_header(false), needs_footer(false) {}
+
+	CondorClassAdFileParseHelper::ParseType setFormat(CondorClassAdFileParseHelper::ParseType typ);
+	CondorClassAdFileParseHelper::ParseType autoSetFormat(CondorClassAdFileParseHelper & parse_help);
+
+	// these return < 0 on failure, 0 if nothing written, 1 if non-empty ad is written.
+	int writeAd(const ClassAd & ad, FILE * out, StringList * whitelist=NULL, bool hash_order=false);
+	int appendAd(const ClassAd & ad, std::string & buf, StringList * whitelist=NULL, bool hash_order=false);
+	// write the footer if one is needed. 
+	int writeFooter(FILE* out, bool xml_always_write_header_footer=true);
+	int appendFooter(std::string & buf, bool xml_always_write_header_footer=true);
+
+	int getNumAds() { return cNonEmptyOutputAds; } // returns number of ads in output list.
+	bool needsFooter() { return needs_footer; } // returns true if a header was previously written and footer has not yet been.
+
+protected:
+	std::string buffer; // internal buffer used by writeAd & writeFooter
+	CondorClassAdFileParseHelper::ParseType out_format;
+	int cNonEmptyOutputAds; // count of number of non-empty ads written, used to trigger header/footer
+	bool wrote_header;      // keep track of whether header/footer have been written
+	bool needs_footer;
+};
+
+
 /** Is this value valid for being written to the log?
  *  The value is a RHS of an expression. Only '\n' or '\r' are invalid.
  *
@@ -482,6 +608,27 @@ int sPrintAdAsXML(MyString &output, const classad::ClassAd &ad,
 int sPrintAdAsXML(std::string &output, const classad::ClassAd &ad,
 				  StringList *attr_white_list = NULL);
 
+/* Prints out the classad as json to a file.
+ * @param fp The file to be printed to.
+ * @param ad The classad containing the attribute
+ * @param An optional white-list of attributes to be printed.
+ * @return TRUE as long as the file existed.
+ */
+int fPrintAdAsJson(FILE *fp, const classad::ClassAd &ad,
+				   StringList *attr_white_list = NULL);
+
+/* Prints the classad as JSON to a string. fPrintAdAsJson calls this.
+ * @param output The string to have filled with the JSON-ified classad.
+ *   The string is appended to, not overwritten.
+ * @param ad The ad to be printed.
+ * @param An optional white-list of attributes to be printed.
+ * @return TRUE
+ */
+int sPrintAdAsJson(MyString &output, const classad::ClassAd &ad,
+				   StringList *attr_white_list = NULL);
+int sPrintAdAsJson(std::string &output, const classad::ClassAd &ad,
+				   StringList *attr_white_list = NULL);
+
 /** Given an attribute name, return a buffer containing the name
  *  and it's unevaluated value, like so:
  *    <name> = <expression>
@@ -493,11 +640,13 @@ int sPrintAdAsXML(std::string &output, const classad::ClassAd &ad,
  */
 char* sPrintExpr(const classad::ClassAd &ad, const char* name);
 
-/** Basically just calls an Unparser so we can escape strings
- *  @param val The string we're escaping stuff in. 
- *  @return The escaped string.
+/** Basically just calls an Unparser so we can escape strings.
+ *  The enclosing quote characters are included.
+ *  @param val The string to be escaped.
+ *  @param buf Buffer where the escaped form is written.
+ *  @return The escaped string (buf.c_str()).
  */
-char const *EscapeAdStringValue(char const *val, std::string &buf);
+char const *QuoteAdStringValue(char const *val, std::string &buf);
 
 	/** Set the MyType attribute */
 void SetMyTypeName(classad::ClassAd &ad, const char *);
