@@ -373,7 +373,7 @@ CronTab::initRegexObject() {
 		if ( ! CronTab::regex.compile( pattern, &errptr, &erroffset )) {
 			MyString error = "CronTab: Failed to compile Regex - ";
 			error += pattern;
-			EXCEPT( "%s", const_cast<char*>(error.Value()) );
+			EXCEPT( "%s", error.Value() );
 		}
 	}
 }
