@@ -203,18 +203,18 @@ class ULogEvent {
     /// The event last read, or to be written.
     ULogEventNumber    eventNumber;
 
-    /// The time this event occurred
-    struct tm          eventTime;
+	/** Get the time at which this event occurred, in the form
+	    of a struct tm.
+		@return The time at which this event occurred.
+	*/
+	const struct tm& GetEventTime() const { return eventTime; }
 
-/*
-define ULOG_MICROSECONDS on linux to get microsecond resolution in the
-user log.  This is write only, and probably breaks compatability with
-log readers.
-*/
+	/** Get the time at which this event occurred, in the form
+	    of a time_t.
+		@return The time at which this event occurred.
+	*/
+	time_t GetEventclock() const { return eventclock; }
 
-#ifdef ULOG_MICROSECONDS
-	struct timeval     eventTimeval;
-#endif
     /// The cluster field of the Condor ID for this event
     int                cluster;
     /// The proc    field of the Condor ID for this event
@@ -226,8 +226,6 @@ log readers.
     char *scheddname;
     //char globaljobid[100];
 
-	time_t eventclock;
-
   protected:
 
     /** Read the resource usage from the log file.
@@ -287,6 +285,23 @@ log readers.
 	/// the global job id for the job associated with this event
 	void insertCommonIdentifiers(ClassAd &adToFill);
 	const char *m_gjid;
+
+  private:
+    /// The time this event occurred (eventclock is Unix timestamp;
+	/// eventTime is local time); these MUST correspond to the same
+	/// time!!
+	/// We should get rid of one of these (see gittrac #5468).
+	time_t				eventclock;
+    struct tm			eventTime;
+
+/*
+define ULOG_MICROSECONDS on linux to get microsecond resolution in the
+user log.  This is write only, and probably breaks compatability with
+log readers.
+*/
+#ifdef ULOG_MICROSECONDS
+	struct timeval     eventTimeval;
+#endif
 };
 
 //----------------------------------------------------------------------------
