@@ -29,6 +29,7 @@
 #include "condor_classad.h"
 #include "quill_enums.h"
 #include "dc_schedd.h"
+#include "my_username.h"
 
 #ifdef HAVE_EXT_POSTGRESQL
 #include "pgsqldatabase.h"
@@ -377,8 +378,8 @@ CondorQ::fetchQueueFromHostAndProcess ( const char *host,
 	constraint = strdup( ExprTreeToString( tree ) );
 	delete tree;
 
-	if (useFastPath == 2) {
-		int result = fetchQueueFromHostAndProcessV2(host, constraint, attrs, fetch_opts, match_limit, process_func, process_func_data, connect_timeout, errstack);
+	if (useFastPath > 1) {
+		int result = fetchQueueFromHostAndProcessV2(host, constraint, attrs, fetch_opts, match_limit, process_func, process_func_data, connect_timeout, useFastPath, errstack);
 		free( constraint);
 		return result;
 	}
@@ -418,6 +419,7 @@ CondorQ::fetchQueueFromHostAndProcessV2(const char *host,
 					condor_q_process_func process_func,
 					void * process_func_data,
 					int connect_timeout,
+					int useFastPath,
 					CondorError *errstack)
 {
 	classad::ClassAdParser parser;
@@ -430,34 +432,130 @@ CondorQ::fetchQueueFromHostAndProcessV2(const char *host,
 
 	request_ad.Insert(ATTR_REQUIREMENTS, expr);
 
-	char *projection = attrs.print_to_delimed_string(",");
+	char *projection = attrs.print_to_delimed_string("\n");
 	if (projection) {
 		request_ad.InsertAttr(ATTR_PROJECTION, projection);
 		free(projection);
 	}
 
+	bool want_authentication = false;
 	if (fetch_opts == fetch_DefaultAutoCluster) {
 		request_ad.InsertAttr("QueryDefaultAutocluster", true);
 		request_ad.InsertAttr("MaxReturnedJobIds", 2); // TODO: make this settable by caller of this function.
 	} else if (fetch_opts == fetch_GroupBy) {
 		request_ad.InsertAttr("ProjectionIsGroupBy", true);
 		request_ad.InsertAttr("MaxReturnedJobIds", 2); // TODO: make this settable by caller of this function.
+	} else if (fetch_opts == fetch_MyJobs) {
+		const char * owner = my_username();
+		if (owner) { request_ad.InsertAttr("Me", owner); }
+		request_ad.InsertAttr("MyJobs", owner ? "(Owner == Me)" : "true");
+		want_authentication = true;
 	}
 
-	if (match_limit >= 0)
-	{
+	if (match_limit >= 0) {
 		request_ad.InsertAttr(ATTR_LIMIT_RESULTS, match_limit);
 	}
 
+	// determine if authentication can/will happen.  three reasons why it might not:
+	// 1) security negotiation is disabled (NEVER or OPTIONAL for outgoing connections)
+	// 2) Authentication is disabled (NEVER) by the client
+	// 3) Authentication is disabled (NEVER) by the server.  this is actually impossible to
+	//    get correct without actually querying the server but we make an educated guess by
+	//    paraming the READ auth level.
+	bool can_auth = true;
+	char *paramer = NULL;
+
+	paramer = SecMan::getSecSetting ("SEC_%s_NEGOTIATION", CLIENT_PERM);
+	if (paramer != NULL) {
+		char p = toupper(paramer[0]);
+		free(paramer);
+		if (p == 'N' || p == 'O') {
+			// authentication will not happen since no security negotiation will occur
+			can_auth = false;
+		}
+	}
+
+	paramer = SecMan::getSecSetting ("SEC_%s_AUTHENTICATION", CLIENT_PERM);
+	if (paramer != NULL) {
+		char p = toupper(paramer[0]);
+		free(paramer);
+		if (p == 'N') {
+			// authentication will not happen since client doesn't allow it.
+			can_auth = false;
+		}
+	}
+
+	// authentication will not happen since server probably doesn't allow it.
+	// on the off chance that someone's config manages to trick us, leave an
+	// undocumented knob as a last resort to disable our inference.
+	if (param_boolean("CONDOR_Q_INFER_SCHEDD_AUTHENTICATION", true)) {
+		paramer = SecMan::getSecSetting ("SEC_%s_AUTHENTICATION", READ);
+		if (paramer != NULL) {
+			char p = toupper(paramer[0]);
+			free(paramer);
+			if (p == 'N') {
+				can_auth = false;
+			}
+		}
+
+		paramer = SecMan::getSecSetting ("SCHEDD.SEC_%s_AUTHENTICATION", READ);
+		if (paramer != NULL) {
+			char p = toupper(paramer[0]);
+			free(paramer);
+			if (p == 'N') {
+				can_auth = false;
+			}
+		}
+	}
+
+	if (!can_auth) {
+		dprintf (D_ALWAYS, "detected that authentication will not happen.  falling back to QUERY_JOB_ADS without authentication.\n");
+	}
+
 	DCSchedd schedd(host);
+	int cmd = QUERY_JOB_ADS;
+	if (want_authentication && can_auth && (useFastPath > 2)) {
+		cmd = QUERY_JOB_ADS_WITH_AUTH;
+	}
 	Sock* sock;
-	if (!(sock = schedd.startCommand(QUERY_JOB_ADS, Stream::reli_sock, connect_timeout, errstack))) return Q_SCHEDD_COMMUNICATION_ERROR;
+	if (!(sock = schedd.startCommand(cmd, Stream::reli_sock, connect_timeout, errstack))) return Q_SCHEDD_COMMUNICATION_ERROR;
 
 	classad_shared_ptr<Sock> sock_sentry(sock);
 
 	if (!putClassAd(sock, request_ad) || !sock->end_of_message()) return Q_SCHEDD_COMMUNICATION_ERROR;
 	dprintf(D_FULLDEBUG, "Sent classad to schedd\n");
 
+#if 0
+	// if doing MyJobs queryies, we can expect to get a late authentication request
+	// unless we have already authenticated, or the schedd is older and doesn't know to try.
+	if (expect_late_authentication) {
+		ReliSock* rsock = (ReliSock*)sock;
+		if (rsock->triedAuthentication()) {
+			expect_late_authentication = false;
+			dprintf(D_FULLDEBUG, "MyJobs option requires authentication, and it already happened.\n");
+		} else {
+			CondorVersionInfo const *peer_ver = rsock->get_peer_version();
+			auto_free_ptr verstr(peer_ver ? peer_ver->get_version_string() : NULL);
+			dprintf(D_FULLDEBUG, "Schedd version is %s\n", verstr ? verstr.ptr() : "NULL");
+			expect_late_authentication = peer_ver && peer_ver->built_since_version(8,5,5);
+			dprintf(D_FULLDEBUG, "MyJobs option requires authentication, and schedd %s late authentication\n", 
+				expect_late_authentication ? "supports" : "does not support");
+		}
+	}
+
+	// we interrupt this condor_q query to do authentication now.
+	if (expect_late_authentication) {
+		ReliSock* rsock = (ReliSock*)sock;
+		dprintf(D_FULLDEBUG, "Attempting late authentication\n");
+		CondorError errstack;
+		bool authenticated = SecMan::authenticate_sock(rsock, CLIENT_PERM, &errstack);
+		if ( ! authenticated && IsDebugCatAndVerbosity(D_FULLDEBUG)) {
+			dprintf(D_FULLDEBUG, "Authentication failed: %s\n", errstack.message());
+		}
+		dprintf(D_FULLDEBUG, "Continuing command %s Authentication\n", authenticated ? "with" : "without");
+	}
+#endif
+
 	int rval = 0;
 	do {
 		ad = new ClassAd();
@@ -656,7 +754,7 @@ CondorQ::getFilterAndProcessAds( const char *constraint,
 
 	if (useAll) {
 			// The fast case with the new protocol
-		char *attrs_str = attrs.print_to_delimed_string();
+		char *attrs_str = attrs.print_to_delimed_string("\n");
 		GetAllJobsByConstraint_Start(constraint, attrs_str);
 		free(attrs_str);
 
@@ -729,7 +827,7 @@ CondorQ::getAndFilterAds (const char *constraint,
 						  int useAllJobs)
 {
 	if (useAllJobs == 1) {
-		char *attrs_str = attrs.print_to_delimed_string();
+		char *attrs_str = attrs.print_to_delimed_string("\n");
 		GetAllJobsByConstraint(constraint, attrs_str, list);
 		free(attrs_str);
 
