@@ -154,8 +154,14 @@ class CondorQuery
 	QueryResult addORConstraint (const char *);			 // custom constraints
 	QueryResult addANDConstraint (const char *);		 // custom constraints
 
+	// This query should perform only a location lookup.
+	bool setLocationLookup(const std::string &location);
+
 	// fetch from collector
 	QueryResult fetchAds (ClassAdList &adList, const char * pool, CondorError* errstack = NULL);
+	// fetch ads from the collector, handing each to 'callback'
+	// callback will return 'false' if it took ownership of the ad.
+	QueryResult processAds (bool (*callback)(void*, ClassAd *), void* pv, const char * pool, CondorError* errstack = NULL);
 
 
 	// filter list of ads; arg1 is 'in', arg2 is 'out'
@@ -176,6 +182,10 @@ class CondorQuery
 		// to be returned in the queried ads.
 		// If not set, all attributes are returned.
 	void setDesiredAttrs(char const * const *attrs);
+	void setDesiredAttrs(const std::vector<std::string> &attrs);
+	void setDesiredAttrs(const classad::References & attrs);
+		// set list of desired attributes as a space-separated string
+	void setDesiredAttrs(const char * attrs) { extraAttrs.Assign(ATTR_PROJECTION, attrs); }
 	void setDesiredAttrsExpr(const char *expr);
 
   private:
