@@ -240,8 +240,12 @@ private:
 	bool rmdirAttempt( const char* path, priv_state priv );
 
 #ifdef WIN32
-	long dirp;
+	LONG_PTR dirp;
+#ifdef _M_X64
+	struct _finddatai64_t filedata;
+#else
 	struct _finddata_t filedata;
+#endif
 #else
 	condor_DIR *dirp;
 	priv_state setOwnerPriv( const char* path, si_error_t &err );
@@ -332,6 +336,7 @@ bool recursive_chown( const char *path,
 	will be set to EEXIST in that case.
  */
 bool mkdir_and_parents_if_needed( const char *path, mode_t mode, priv_state priv = PRIV_UNKNOWN );
+bool mkdir_and_parents_if_needed( const char *path, mode_t mode, mode_t parent_mode, priv_state priv = PRIV_UNKNOWN );
 
 /** Create parent directories of a path if they do not exist.
     If the parent directory already exists, it is left as is
