@@ -105,7 +105,6 @@ extern unsigned int DebugHeaderOptions;	// for D_FID, D_PID, D_NOHEADER & D_
 extern DebugOutputChoice AnyDebugBasicListener; /* Bits to look for in dprintf */
 extern DebugOutputChoice AnyDebugBasicListener; /* verbose bits for dprintf */
 //extern  int		DebugFlags;
-//extern param_functions *dprintf_param_funcs;
 
 /*
    This is a global flag that tells us if we've successfully ran
@@ -947,6 +946,7 @@ _condor_dprintf_va( int cat_and_flags, DPF_IDENT ident, const char* fmt, va_list
 			switch ((*it).outputTarget) {
 				case STD_ERR: it->debugFP = stderr; break;
 				case STD_OUT: it->debugFP = stdout; break;
+				case SYSLOG: break;
 				default:
 				case FILE_OUT:
 					debug_lock_it(&(*it), NULL, 0, it->dont_panic);
@@ -1770,7 +1770,7 @@ _condor_dprintf_exit( int error_code, const char* msg )
 		/* If _EXCEPT_Cleanup is set for cleaning up during EXCEPT(),
 		   we call that here, as well. */
 	if( _EXCEPT_Cleanup ) {
-		(*_EXCEPT_Cleanup)( __LINE__, errno, "dprintf hit fatal errors\n" );
+		(*_EXCEPT_Cleanup)( __LINE__, errno, "dprintf hit fatal errors" );
 	}
 
 		/* Actually exit now */
@@ -2444,6 +2444,16 @@ dprintf_dump_stack(void) {
 
 #endif
 
+// If outputs haven't been configured yet, stop buffering dprintf()
+// output until they are configured.
+void dprintf_pause_buffering()
+{
+	_condor_dprintf_works = 1;
+	if ( DebugLogs == NULL ) {
+		DebugLogs = new std::vector<DebugFileInfo>;
+	}
+}
+
 bool debug_open_fds(std::map<int,bool> &open_fds)
 {
 	bool found = false;
