@@ -30,6 +30,7 @@ class ExtArray
 	// ctors/dtor
 	ExtArray (int = 64); // initial size of array
 	ExtArray (const ExtArray &);
+	ExtArray &operator =(const ExtArray &);
 	~ExtArray ();
 
 	// accessor functions
@@ -79,6 +80,23 @@ ExtArray (int sz) : filler()
 	}
 }
 
+template <class Element>
+ExtArray<Element> &
+ExtArray<Element>::
+operator =(const ExtArray &old)
+{
+	truncate(0);
+	int i;
+	resize(old.size);
+	for(i=0; i<size; i++) {
+		array[i] = old.array[i];
+	}
+	last = old.last;
+	// copy filler element as well
+	filler = old.filler;
+	return *this;
+}
+
 template <class Element>
 ExtArray<Element>::
 ExtArray (const ExtArray &old)
