@@ -171,6 +171,21 @@ class FileTransfer: public Service {
 		    success(true), in_progress(false), xfer_status(XFER_STATUS_UNKNOWN),
 			try_again(true), hold_code(0), hold_subcode(0) {}
 
+		FileTransferInfo(const FileTransferInfo &rhs) {
+			bytes = rhs.bytes;
+			duration = rhs.duration;
+			type = rhs.type;
+			success = rhs.success;
+			in_progress = rhs.in_progress;
+			xfer_status = rhs.xfer_status;
+			try_again = rhs.try_again;
+			hold_code = rhs.hold_code;
+			hold_subcode = rhs.hold_subcode;
+			error_desc = rhs.error_desc;
+			spooled_files = rhs.spooled_files;
+			tcp_stats = rhs.tcp_stats;
+			
+		}
 		void addSpooledFile(char const *name_in_spool);
 
 		filesize_t bytes;
@@ -186,6 +201,7 @@ class FileTransfer: public Service {
 			// List of files we created in remote spool.
 			// This is intended to become SpooledOutputFiles.
 		MyString spooled_files;
+		MyString tcp_stats;
 	};
 
 	FileTransferInfo GetInfo() { return Info; }
@@ -240,7 +256,7 @@ class FileTransfer: public Service {
 			@param filename Name of file to add to our list
 			@return always true
 			*/
-	bool addFileToExeptionList( const char* filename );
+	bool addFileToExceptionList( const char* filename );
 
 		/** Allows the client side of the filetransfer object to 
 			point to a different server.
@@ -295,16 +311,16 @@ class FileTransfer: public Service {
 	int GetUploadTimestamps(time_t * pStart, time_t * pEnd = NULL) {
 		if (uploadStartTime < 0)
 			return false;
-		if (pEnd) *pEnd = uploadEndTime;
-		if (pStart) *pStart = uploadStartTime;
+		if (pEnd) *pEnd = (time_t)uploadEndTime;
+		if (pStart) *pStart = (time_t)uploadStartTime;
 		return true;
 	}
 
 	bool GetDownloadTimestamps(time_t * pStart, time_t * pEnd = NULL) {
 		if (downloadStartTime < 0)
 			return false;
-		if (pEnd) *pEnd = downloadEndTime;
-		if (pStart) *pStart = downloadStartTime;
+		if (pEnd) *pEnd = (time_t)downloadEndTime;
+		if (pStart) *pStart = (time_t)downloadStartTime;
 		return true;
 	}
 
@@ -328,8 +344,8 @@ class FileTransfer: public Service {
 	int DoDownload( filesize_t *total_bytes, ReliSock *s);
 	int DoUpload( filesize_t *total_bytes, ReliSock *s);
 
-	time_t uploadStartTime, uploadEndTime;
-	time_t downloadStartTime, downloadEndTime;
+	double uploadStartTime, uploadEndTime;
+	double downloadStartTime, downloadEndTime;
 
 	void CommitFiles();
 	void ComputeFilesToSend();
@@ -416,7 +432,7 @@ class FileTransfer: public Service {
 	bool LookupInFileCatalog(const char *fname, time_t *mod_time, filesize_t *filesize);
 
 	// Called internally by DoUpload() in order to handle common wrapup tasks.
-	int ExitDoUpload(filesize_t *total_bytes, ReliSock *s, priv_state saved_priv, bool socket_default_crypto, bool upload_success, bool do_upload_ack, bool do_download_ack, bool try_again, int hold_code, int hold_subcode, char const *upload_error_desc,int DoUpload_exit_line);
+	int ExitDoUpload(filesize_t *total_bytes, int numFiles, ReliSock *s, priv_state saved_priv, bool socket_default_crypto, bool upload_success, bool do_upload_ack, bool do_download_ack, bool try_again, int hold_code, int hold_subcode, char const *upload_error_desc,int DoUpload_exit_line);
 
 	// Send acknowledgment of success/failure after downloading files.
 	void SendTransferAck(Stream *s,bool success,bool try_again,int hold_code,int hold_subcode,char const *hold_reason);
