@@ -320,6 +320,13 @@ makeStorageAdHashKey (AdNameHashKey &hk, ClassAd *ad)
 	return adLookup( "Storage", ad, ATTR_NAME, NULL, hk.name );
 }
 
+bool
+makeAccountingAdHashKey (AdNameHashKey &hk, ClassAd *ad)
+{
+	hk.ip_addr = "";
+	return adLookup( "Accounting", ad, ATTR_NAME, NULL, hk.name );
+}
+
 
 bool
 makeNegotiatorAdHashKey (AdNameHashKey &hk, ClassAd *ad)
@@ -364,6 +371,10 @@ makeGridAdHashKey (AdNameHashKey &hk, ClassAd *ad)
         }
     }
 
+	if ( adLookup( "Grid", ad, ATTR_GRIDMANAGER_SELECTION_VALUE, NULL, tmp, false ) ) {
+		hk.name += tmp;
+	}
+
     return true;
 }
 
