@@ -26,7 +26,7 @@
 #include "condor_uid.h"
 #include "hibernator.tools.h"
 #include "path_utils.h"
-#include "../condor_daemon_core.V6/condor_daemon_core.h"
+#include <condor_daemon_core.h>
 
 /***************************************************************
  * UserDefinedToolsHibernator class
@@ -232,6 +232,7 @@ UserDefinedToolsHibernator::enterState ( HibernatorBase::SLEEP_STATE state ) con
 		PRIV_CONDOR_FINAL,
 		m_reaper_id, 
 		FALSE, 
+		FALSE, 
 		NULL, 
 		NULL, 
 		&fi );	
