@@ -397,7 +397,7 @@ MyString get_hostname(const condor_sockaddr& addr) {
 	if (targ_addr.is_ipv6())
 		targ_addr.set_scope_id(0);
 
-	e = condor_getnameinfo(targ_addr, hostname, sizeof(hostname), NULL, 0, 0);
+	e = condor_getnameinfo(targ_addr, hostname, sizeof(hostname), NULL, 0, NI_NAMEREQD );
 	if (e)
 		return ret;
 
@@ -411,23 +411,27 @@ MyString get_hostname(const condor_sockaddr& addr) {
 
 bool verify_name_has_ip(MyString name, condor_sockaddr addr){
 	std::vector<condor_sockaddr> addrs;
-	bool found = false;
 
 	addrs = resolve_hostname(name);
-	dprintf(D_FULLDEBUG, "IPVERIFY: checking %s against %s\n", name.Value(), addr.to_ip_string().Value());
+	if (IsDebugVerbose(D_SECURITY)) {
+		// print the list of addrs that we are comparing against
+		MyString ips_str; ips_str.reserve_at_least(addrs.size()*40);
+		for(unsigned int i = 0; i < addrs.size(); i++) {
+			ips_str += "\n\t";
+			ips_str += addrs[i].to_ip_string().Value();
+		}
+		dprintf(D_SECURITY|D_VERBOSE, "IPVERIFY: checking %s against %s addrs are:%s\n",
+				name.Value(), addr.to_ip_string().Value(), ips_str.Value());
+	}
 	for(unsigned int i = 0; i < addrs.size(); i++) {
 		// compare MyStrings
 		// addr.to_ip_string
 		if(addrs[i].to_ip_string() == addr.to_ip_string()) {
-			dprintf(D_FULLDEBUG, "IPVERIFY: matched %s to %s\n", addrs[i].to_ip_string().Value(), addr.to_ip_string().Value());
-			found = true;
-		} else {
-			dprintf(D_FULLDEBUG, "IPVERIFY: comparing %s to %s\n", addrs[i].to_ip_string().Value(), addr.to_ip_string().Value());
+			dprintf(D_SECURITY, "IPVERIFY: for %s matched %s to %s\n", name.Value(), addrs[i].to_ip_string().Value(), addr.to_ip_string().Value());
+			return true;
 		}
 	}
-	dprintf(D_FULLDEBUG, "IPVERIFY: ip found is %i\n", found);
-
-	return found;
+	return false;
 }
 
 std::vector<MyString> get_hostname_with_alias(const condor_sockaddr& addr)
@@ -536,7 +540,23 @@ std::vector<condor_sockaddr> resolve_hostname(const MyString& hostname)
 }
 
 std::vector<condor_sockaddr> resolve_hostname_raw(const MyString& hostname) {
+	//
+	// For now, we can just document that you need the Punycoded DSN name.
+	// If somebody complains about that, we can revisit this.  (If we
+	// assume/require UTF-8 (int the config file), we can still readily
+	// distinguish Sinfuls, since ':' and '?' won't be used by the UTF-8
+	// IDNs.)
+	//
 	std::vector<condor_sockaddr> ret;
+	for( int i = 0; i < hostname.length(); ++i ) {
+		if( isalnum( hostname[i] ) || hostname[i] == '-' ) { continue; }
+		if( hostname[i] == '.' && i + 1 < hostname.length()
+			&& hostname[i+1] != '.' ) { continue; }
+
+		dprintf( D_HOSTNAME, "resolve_hostname_raw(): argument '%s' is not a valid DNS name, returning no addresses.\n", hostname.c_str() );
+		return ret;
+	}
+
 	addrinfo_iterator ai;
 	int res  = ipv6_getaddrinfo(hostname.Value(), NULL, ai);
 	if (res) {
@@ -544,9 +564,9 @@ std::vector<condor_sockaddr> resolve_hostname_raw(const MyString& hostname) {
 			hostname.Value(), gai_strerror(res), res);
 		return ret;
 	}
-	
-		// To eliminate duplicate address, here we use std::set.
-		// we also want to maintain the order given by getaddrinfo.
+
+	// To eliminate duplicate address, here we use std::set.
+	// we also want to maintain the order given by getaddrinfo.
 	std::set<condor_sockaddr> s;
 	while (addrinfo* info = ai.next()) {
 		condor_sockaddr addr(info->ai_addr);
