@@ -133,8 +133,8 @@ ReadMultipleUserLogs::readEvent (ULogEvent * & event)
 
 		if ( outcome != ULOG_NO_EVENT ) {
 			if ( oldestEventMon == NULL ||
-						(oldestEventMon->lastLogEvent->eventTime >
-						monitor->lastLogEvent->eventTime) ) {
+						(oldestEventMon->lastLogEvent->GetEventclock() >
+						monitor->lastLogEvent->GetEventclock()) ) {
 				oldestEventMon = monitor;
 			}
 		}
@@ -501,118 +501,6 @@ MultiLogFiles::makePathAbsolute(MyString &filename, CondorError &errstack)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-// Skip whitespace in a std::string buffer.
-void
-MultiLogFiles::skip_whitespace(std::string const &s,int &offset) {
-	while((int)s.size() > offset && isspace(s[offset])) offset++;
-};
-
-// Read a file into a std::string.  Return false on error.
-MyString
-MultiLogFiles::readFile(char const *filename,std::string& buf)
-{
-    char chunk[4000];
-	MyString rtnVal;
-
-	int fd = safe_open_wrapper_follow(filename, O_RDONLY);
-	if (fd < 0) {
-		rtnVal.formatstr("error opening submit file %s: %s",
-				filename, strerror(errno) );
-		dprintf(D_ALWAYS, "%s\n", rtnVal.Value() );
-		return rtnVal;
-	}
-
-    while(1) {
-        ssize_t n = read(fd,chunk,sizeof(chunk)-1);
-        if(n>0) {
-            chunk[n] = '\0';
-            buf += chunk;
-        }
-        else if(n==0) {
-            break;
-        }
-        else {
-            rtnVal.formatstr("failed to read submit file %s: %s",
-					filename, strerror(errno) );
-			dprintf(D_ALWAYS, "%s\n", rtnVal.Value() );
-			close(fd);
-			return rtnVal;
-        }
-    }
-
-	close(fd);
-    return rtnVal;
-}
-
-///////////////////////////////////////////////////////////////////////////////
-
-MyString
-MultiLogFiles::getValuesFromFileNew(const MyString &fileName, 
-			const MyString &keyword, StringList &values, int skipTokens)
-{
-	MyString	errorMsg;
-
-	FileReader reader;
-	errorMsg = reader.Open( fileName );
-	if ( errorMsg != "" ) {
-		return errorMsg;
-	}
-
-	MyString logicalLine;
-	while ( reader.NextLogicalLine( logicalLine ) ) {
-		if ( logicalLine != "" ) {
-				// Note: StringList constructor removes leading
-				// whitespace from lines.
-			StringList tokens( logicalLine.Value(), " \t" );
-			tokens.rewind();
-
-			if ( !strcasecmp(tokens.next(), keyword.Value()) ) {
-					// Skip over unwanted tokens.
-				for ( int skipped = 0; skipped < skipTokens; skipped++ ) {
-					if ( !tokens.next() ) {
-						MyString result = MyString( "Improperly-formatted "
-									"file: value missing after keyword <" ) +
-									keyword + ">";
-			    		return result;
-					}
-				}
-
-					// Get the value.
-				const char *newValue = tokens.next();
-				if ( !newValue || !strcmp( newValue, "" ) ) {
-					MyString result = MyString( "Improperly-formatted "
-								"file: value missing after keyword <" ) +
-								keyword + ">";
-			    	return result;
-				}
-
-					// Add the value we just found to the values list
-					// (if it's not already in the list -- we don't want
-					// duplicates).
-				values.rewind();
-				char *existingValue;
-				bool alreadyInList = false;
-				while ( (existingValue = values.next()) ) {
-					if (!strcmp( existingValue, newValue ) ) {
-						alreadyInList = true;
-					}
-				}
-
-				if ( !alreadyInList ) {
-						// Note: append copies the string here.
-					values.append(newValue);
-				}
-			}
-		}
-	}
-
-	reader.Close();
-
-	return "";
-}
-
-///////////////////////////////////////////////////////////////////////////////
-
 MyString
 MultiLogFiles::getParamFromSubmitLine(MyString &submitLine,
 		const char *paramName)
