@@ -24,7 +24,6 @@
 #include <time.h>
 #include "MyString.h"
 #include "condor_uid.h"
-#include "condor_xml_classads.h"
 #include "condor_config.h"
 #include "stat_wrapper.h"
 #include "read_user_log_state.h"
@@ -397,7 +396,7 @@ ReadUserLogState::GeneratePath( int rotation,
 	path = m_base_path;
 	if ( rotation ) {
 		if ( m_max_rotations > 1 ) {
-			path.sprintf_cat( ".%d", rotation );
+			path.formatstr_cat( ".%d", rotation );
 		}
 		else {
 			path += ".old";
@@ -520,34 +519,34 @@ ReadUserLogState::ScoreFile( const StatStructType &statbuf, int rot ) const
 	// Check inode match
 	if ( m_stat_buf.st_ino == statbuf.st_ino ) {
 		score += m_score_fact_inode;
-		if ( DebugFlags & D_FULLDEBUG ) MatchList += "inode ";
+		if ( IsFulldebug(D_FULLDEBUG) ) MatchList += "inode ";
 	}
 
 	// Check ctime match
 	if ( m_stat_buf.st_ctime == statbuf.st_ctime ) {
 		score += m_score_fact_ctime;
-		if ( DebugFlags & D_FULLDEBUG ) MatchList += "ctime ";
+		if ( IsFulldebug(D_FULLDEBUG) ) MatchList += "ctime ";
 	}
 
 	// If it's the same size, it's a good sign..
 	if ( same_size ) {
 		score += m_score_fact_same_size;
-		if ( DebugFlags & D_FULLDEBUG ) MatchList += "same-size ";
+		if ( IsFulldebug(D_FULLDEBUG) ) MatchList += "same-size ";
 	}
 	// If it's the current file and recently stat()ed, if it's grown
 	// we're OK with that, too
 	else if ( is_recent && is_current && has_grown ) {
 		score += m_score_fact_grown;
-		if ( DebugFlags & D_FULLDEBUG ) MatchList += "grown ";
+		if ( IsFulldebug(D_FULLDEBUG) ) MatchList += "grown ";
 	}
 
 	// If the file has shrunk, that doesn't bode well, though, in *any* case
 	if ( m_stat_buf.st_size > statbuf.st_size ) {
 		score += m_score_fact_shrunk;
-		if ( DebugFlags & D_FULLDEBUG ) MatchList += "shrunk ";
+		if ( IsFulldebug(D_FULLDEBUG) ) MatchList += "shrunk ";
 	}
 
-	if ( DebugFlags & D_FULLDEBUG ) {
+	if ( IsFulldebug(D_FULLDEBUG) ) {
 		dprintf( D_FULLDEBUG, "ScoreFile: match list: %s\n",
 				 MatchList.Value() );
 	}
@@ -762,9 +761,9 @@ ReadUserLogState::GetStateString( MyString &str, const char *label ) const
 {
 	str = "";
 	if ( NULL != label ) {
-		str.sprintf( "%s:\n", label );
+		str.formatstr( "%s:\n", label );
 	}
-	str.sprintf_cat (
+	str.formatstr_cat (
 		"  BasePath = %s\n"
 		"  CurPath = %s\n"
 		"  UniqId = %s, seq = %d\n"
@@ -788,7 +787,7 @@ ReadUserLogState::GetStateString(
 	const ReadUserLogFileState::FileState *istate;
 	if ( ( !convertState(state, istate) ) || ( !istate->m_version ) ) {
 		if ( label ) {
-			str.sprintf( "%s: no state", label );
+			str.formatstr( "%s: no state", label );
 		}
 		else {
 			str = "no state\n";
@@ -798,16 +797,16 @@ ReadUserLogState::GetStateString(
 
 	str = "";
 	if ( NULL != label ) {
-		str.sprintf( "%s:\n", label );
+		str.formatstr( "%s:\n", label );
 	}
-	str.sprintf_cat (
+	str.formatstr_cat (
 		"  signature = '%s'; version = %d; update = %ld\n"
 		"  base path = '%s'\n"
 		"  cur path = '%s'\n"
 		"  UniqId = %s, seq = %d\n"
-		"  rotation = %d; max = %d; offset = "FILESIZE_T_FORMAT";"
-		" event num = "FILESIZE_T_FORMAT"; type = %d\n"
-		"  inode = %u; ctime = %ld; size = "FILESIZE_T_FORMAT"\n",
+		"  rotation = %d; max = %d; offset = " FILESIZE_T_FORMAT";"
+		" event num = " FILESIZE_T_FORMAT"; type = %d\n"
+		"  inode = %u; ctime = %ld; size = " FILESIZE_T_FORMAT"\n",
 		istate->m_signature, istate->m_version, istate->m_update_time,
 		istate->m_base_path,
 		CurPath(state),
