@@ -23,8 +23,7 @@
 // pretty-printing options
 enum ppOption {
     PP_NOTSET,
-	
-	PP_DEFRAG_NORMAL,
+
     PP_STARTD_NORMAL,
     PP_STARTD_SERVER,
     PP_STARTD_STATE,
@@ -34,47 +33,57 @@ enum ppOption {
     PP_QUILL_NORMAL,
 
     PP_SCHEDD_NORMAL,
-    PP_SCHEDD_SUBMITTORS,	
+    PP_SCHEDD_DATA,
+    PP_SCHEDD_RUN,
+    PP_SUBMITTER_NORMAL,
 
     PP_MASTER_NORMAL,
     PP_COLLECTOR_NORMAL,
     PP_CKPT_SRVR_NORMAL,
     PP_GRID_NORMAL,
-	PP_STORAGE_NORMAL,
+    PP_STORAGE_NORMAL,
     PP_NEGOTIATOR_NORMAL,
+    PP_DEFRAG_NORMAL,
+    PP_ACCOUNTING_NORMAL,
+
     PP_ANY_NORMAL,
     PP_GENERIC_NORMAL,
     PP_GENERIC,
-    PP_XML,
     PP_CUSTOM,
-
-    PP_VERBOSE
+    PP_LONG,
+    PP_XML,
+    PP_JSON,
+    PP_NEWCLASSAD, // output new classad format, ie [] around the ad and ; between attributes
 };
 
+#define PP_IS_LONGish(pp) ((pp) >= PP_LONG && (pp) <= PP_NEWCLASSAD)
 
-// modes for condor_status
-enum Mode {
-    MODE_NOTSET,
-	MODE_DEFRAG_NORMAL,
-    MODE_STARTD_NORMAL,
-    MODE_STARTD_AVAIL,
-    MODE_STARTD_RUN,
-    MODE_STARTD_COD,
-    MODE_QUILL_NORMAL,
-    MODE_SCHEDD_NORMAL,
-    MODE_SCHEDD_SUBMITTORS,
-    MODE_MASTER_NORMAL,
-    MODE_COLLECTOR_NORMAL,
-    MODE_CKPT_SRVR_NORMAL,
-    MODE_GRID_NORMAL,
-    MODE_LICENSE_NORMAL,
-    MODE_STORAGE_NORMAL,
-    MODE_NEGOTIATOR_NORMAL,
-    MODE_GENERIC_NORMAL,
-    MODE_ANY_NORMAL,
-    MODE_OTHER,
-    MODE_GENERIC,
-    MODE_HAD_NORMAL
+
+// display modes for condor_status
+enum {
+	SDO_NotSet,			//  MODE_NOTSET,
+	SDO_Startd,			//  MODE_STARTD_NORMAL,
+	SDO_Startd_Avail,	//  MODE_STARTD_AVAIL,
+	SDO_Startd_Claimed,	//  MODE_STARTD_RUN,
+	SDO_Startd_Cod,		//  MODE_STARTD_COD,
+	SDO_Quill,			//  MODE_QUILL_NORMAL,
+	SDO_Schedd,			//  MODE_SCHEDD_NORMAL,
+	SDO_Schedd_Data,	//  MODE_SCHEDD_DATA,
+	SDO_Schedd_Run,		//  MODE_SCHEDD_RUN,
+	SDO_Submitters,		//  MODE_SCHEDD_SUBMITTORS,
+	SDO_Master,			//  MODE_MASTER_NORMAL,
+	SDO_Collector,		//  MODE_COLLECTOR_NORMAL,
+	SDO_CkptSvr,		//  MODE_CKPT_SRVR_NORMAL,
+	SDO_Grid,			//  MODE_GRID_NORMAL,
+	SDO_License,		//  MODE_LICENSE_NORMAL,
+	SDO_Storage,		//  MODE_STORAGE_NORMAL,
+	SDO_Negotiator,		//  MODE_NEGOTIATOR_NORMAL,
+	SDO_Defrag,			//  MODE_DEFRAG_NORMAL,
+	SDO_Accounting,		//  
+	SDO_Generic,		//  MODE_GENERIC_NORMAL,
+	SDO_Any,			//  MODE_ANY_NORMAL,
+	SDO_Other,			//  MODE_OTHER,
+	SDO_HAD				//  MODE_HAD_NORMAL
 };
    
 #endif //__STATUS_TYPES_H__
