@@ -56,6 +56,19 @@ StringList::StringList(const char *s, const char *delim )
 	}
 }
 
+StringList::StringList(const char *s, char delim_char, bool keep_empty_fields )
+{
+	char delims[2] = { delim_char, 0 };
+	m_delimiters = strnewp( delims );
+	if ( s ) {
+		if (keep_empty_fields) {
+			initializeFromString(s, delim_char);
+		} else {
+			initializeFromString(s);
+		}
+	}
+}
+
 StringList::StringList( const StringList &other )
 		: m_delimiters( NULL )
 {
@@ -105,13 +118,18 @@ StringList::initializeFromString (const char *s)
 
 		// mark the beginning of this String in the list.
 		const char *begin_ptr = walk_ptr;
+		const char *end_ptr = begin_ptr;
 
 		// walk to the end of this string
-		while (!isSeparator (*walk_ptr) && *walk_ptr != '\0')
+		while (!isSeparator (*walk_ptr) && *walk_ptr != '\0') {
+			if ( !isspace(*walk_ptr) ) {
+				end_ptr = walk_ptr;
+			}
 			walk_ptr++;
+		}
 
 		// malloc new space for just this item
-		int len = (walk_ptr - begin_ptr);
+		int len = (end_ptr - begin_ptr) + 1;
 		char *tmp_string = (char*)malloc( 1 + len );
 		ASSERT( tmp_string );
 		strncpy (tmp_string, begin_ptr, len);
@@ -122,6 +140,47 @@ StringList::initializeFromString (const char *s)
 	}
 }
 
+// This version allows for a single delimiter character,
+// it will trim leading and trailing whitespace from items, but
+// it will not skip empty items or extra delimiters.
+void
+StringList::initializeFromString (const char *s, char delim_char)
+{
+	if(!s)
+	{
+		EXCEPT("StringList::initializeFromString passed a null pointer");
+	}
+
+	const char * p = s;
+	while (*p) {
+
+		// skip leading whitespace but not leading separators.
+		while (isspace(*p)) ++p;
+
+		// scan for end of string or for a delimiter char
+		const char * e = p;
+		while (*e && *e != delim_char) ++e;
+
+		size_t len = e-p;
+
+		// rewind back over trailing whitespace
+		while (len && isspace(p[len-1])) --len;
+
+		char *tmp_string = (char*)malloc(1 + len);
+		ASSERT(tmp_string);
+		strncpy(tmp_string, p, len);
+		tmp_string[len] = 0;
+
+		// put the string into the StringList
+		m_strings.Append (tmp_string);
+
+		p = e;
+
+		// if we ended at a delimiter, skip over it.
+		if (*p == delim_char) ++p;
+	}
+}
+
 void
 StringList::print (void)
 {
@@ -154,7 +213,7 @@ bool
 StringList::create_union(StringList & subset, bool anycase)
 {
 	char *x;
-	BOOLEAN ret_val = TRUE;
+	bool ret_val = true;
 	bool result = false;	// true if list modified
 
 	subset.rewind ();
@@ -165,7 +224,7 @@ StringList::create_union(StringList & subset, bool anycase)
 			ret_val = contains(x);
 		}
 			// not there, add it.
-		if( ret_val == FALSE ) {
+		if( ret_val == false ) {
 			result = true;
 			append(x);
 		}
@@ -178,7 +237,7 @@ bool
 StringList::contains_list(StringList & subset, bool anycase)
 {
 	char *x;
-	BOOLEAN ret_val;
+	bool ret_val;
 
 	subset.rewind ();
 	while ((x = subset.next ())) {
@@ -187,7 +246,7 @@ StringList::contains_list(StringList & subset, bool anycase)
 		} else {
 			ret_val = contains(x);
 		}
-		if( ret_val == FALSE ) {
+		if( ret_val == false ) {
 			return false;
 		}
 	}
@@ -195,7 +254,7 @@ StringList::contains_list(StringList & subset, bool anycase)
 }
 
 
-BOOLEAN
+bool
 StringList::contains( const char *st )
 {
 	char	*x;
@@ -203,14 +262,14 @@ StringList::contains( const char *st )
 	m_strings.Rewind ();
 	while ((x = m_strings.Next ())) {
 		if( strcmp(st, x) == MATCH ) {
-			return TRUE;
+			return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 
-BOOLEAN
+bool
 StringList::contains_anycase( const char *st )
 {
 	char	*x;
@@ -218,10 +277,10 @@ StringList::contains_anycase( const char *st )
 	m_strings.Rewind ();
 	while ((x = m_strings.Next ())) {
 		if( strcasecmp(st, x) == MATCH ) {
-			return TRUE;
+			return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 
@@ -251,29 +310,28 @@ StringList::remove_anycase(const char *str)
 	}
 }
 
-BOOLEAN
+bool
 StringList::substring( const char *st )
 {
 	char    *x;
-	int len;
 	
 	m_strings.Rewind ();
 	while( (x = m_strings.Next()) ) {
-		len = strlen(x);
+		size_t len = strlen(x);
 		if( strncmp(st, x, len) == MATCH ) {
-			return TRUE;
+			return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
-BOOLEAN
+bool
 StringList::contains_withwildcard(const char *string)
 {
 	return (contains_withwildcard(string, false) != NULL);
 }
 
-BOOLEAN
+bool
 StringList::contains_anycase_withwildcard(const char *string)
 {
 	return (contains_withwildcard(string, true) != NULL);
@@ -298,8 +356,7 @@ StringList::contains_withwildcard(const char *string, bool anycase, StringList *
 	char *matchstart;
 	char *matchend;
 	char *asterisk;
-	int matchendlen, len;
-    BOOLEAN result; 
+    bool result;
 	int temp;
 	
 	if ( !string )
@@ -382,7 +439,7 @@ StringList::contains_withwildcard(const char *string, bool anycase, StringList *
 		// or in the middle somewhere, and both matchstart and matchend are set 
 		// appropiately with what we want
 
-		result = TRUE;
+		result = true;
 		*asterisk = '\0';	// replace asterisk with a NULL
 		if ( matchstart ) {
 			if ( anycase ) {
@@ -391,25 +448,25 @@ StringList::contains_withwildcard(const char *string, bool anycase, StringList *
 				temp = strncmp(matchstart,string,strlen(matchstart));
 			}
 			if ( temp != MATCH ) 
-				result = FALSE;
+				result = false;
 		}
-		if ( matchend && result == TRUE) {
-			len = strlen(string);
-			matchendlen = strlen(matchend);
+		if ( matchend && result == true) {
+			size_t len = strlen(string);
+			size_t matchendlen = strlen(matchend);
 			if ( matchendlen > len )	// make certain we do not SEGV below
-				result = FALSE;
-			if ( result == TRUE ) {
+				result = false;
+			if ( result == true ) {
 				if (anycase) {
 					temp = strcasecmp(&(string[len-matchendlen]),matchend);
 				} else {
 					temp = strcmp(&(string[len-matchendlen]),matchend);
 				}
 				if ( temp != MATCH )
-					result = FALSE;
+					result = false;
 			}
 		}
 		*asterisk = '*';	// set asterisk back no matter what the result
-		if ( result == TRUE ) {
+		if ( result == true ) {
 			if( matches ) {
 				matches->append( x );
 			}
@@ -427,7 +484,7 @@ StringList::contains_withwildcard(const char *string, bool anycase, StringList *
 	return NULL;
 }
 
-bool
+const char *
 StringList::find( const char *str, bool anycase ) const
 {
 	char	*x;
@@ -436,13 +493,13 @@ StringList::find( const char *str, bool anycase ) const
     iter.ToBeforeFirst ();
 	while ( iter.Next(x) ) {
 		if( (anycase) && (strcasecmp( str, x ) == MATCH) ) {
-			return true;
+			return x;
 		}
 		else if( (!anycase) && (strcmp(str, x) == MATCH) ) {
-			return true;
+			return x;
 		}
 	}
-	return false;
+	return NULL;
 }
 
 bool
@@ -504,7 +561,7 @@ StringList::print_to_delimed_string(const char *delim) const
 
     iter.Initialize( m_strings );
     iter.ToBeforeFirst ();
-	int		len = 1;
+	size_t len = 1;
 	while ( iter.Next(tmp) ) {
 		len += ( strlen(tmp) + strlen(delim) );
 	}
