@@ -515,6 +515,7 @@ BOOL SystemHandleInformation::Refresh()
 				pSysHandleInformation->Handles[i].HandleType = (WORD)(pSysHandleInformation->Handles[i].HandleType % 256);
 				
 				SYSTEM_HANDLE* ptmp = (SYSTEM_HANDLE*)VirtualAlloc ( NULL, sizeof ( SYSTEM_HANDLE ), MEM_COMMIT, PAGE_READWRITE );
+				ASSERT(ptmp);
 				memcpy ( ptmp, &pSysHandleInformation->Handles[i], sizeof ( SYSTEM_HANDLE ) );
 				m_HandleInfos.Append ( ptmp );
 
@@ -882,7 +883,7 @@ BOOL SystemHandleInformation::GetFileName( HANDLE h, MyString& str, DWORD proces
 	tp.pName = &str;
 	tp.rc = 0;
 
-	#pragma message(__FILE__ "(885) : tj:2012 is it really necessary to create a thread to call NtQueryInformationFile??")
+	// tj:2012 is it really necessary to create a thread to call NtQueryInformationFile??"
 	// Let's start the thread to get the file name
 	hThread = (HANDLE)_beginthread( GetFileNameThread, 0, &tp );
 
