@@ -56,15 +56,14 @@ const char *BoolString( bool tf ) { return tf ? "True" : "False"; }
 int
 main(int argc, const char **argv)
 {
-	DebugFlags = D_ALWAYS;
+	set_debug_flags(NULL, D_ALWAYS);
 
 		// initialize to read from config file
 	myDistro->Init( argc, argv );
 	config();
 
 		// Set up the dprintf stuff...
-	Termlog = true;
-	dprintf_config("TEST_NETWORK_ADAPTER", get_param_functions());
+	dprintf_set_tool_debug("TEST_NETWORK_ADAPTER", 0);
 
 	const char	*tmp;
 	int			 status = 0;
@@ -83,7 +82,7 @@ main(int argc, const char **argv)
 	}
 	else {
 		MyString	sinful;
-		sinful.sprintf( "<%s:1234>", opts.m_address );
+		sinful.formatstr( "<%s:1234>", opts.m_address );
 		printf( "Creating network adapter object for %s\n", sinful.Value() );
 		net = NetworkAdapterBase::createNetworkAdapter( sinful.Value() );
 	}
@@ -140,7 +139,7 @@ main(int argc, const char **argv)
 
 	ClassAd	ad;
 	hman.publish( ad );
-	ad.fPrint( stdout );
+	fPrintAd( stdout, ad );
 
 	const char	*method = hman.getHibernationMethod();
 	printf( "Hibernation method used: %s\n", method );
@@ -195,7 +194,7 @@ CheckArgs(int argc, const char **argv, Options &opts)
 
 		if ( arg.Match( 'd', "debug") ) {
 			if ( arg.hasOpt() ) {
-				set_debug_flags( arg.getOpt() );
+				set_debug_flags( arg.getOpt(), 0 );
 			} else {
 				fprintf(stderr, "Value needed for %s\n", arg.Arg() );
 				printf("%s", usage);
