@@ -73,7 +73,7 @@ ReadEvents( Options &opts, int &numEvents );
 void
 ReportError( const ReadUserLog &reader );
 
-const char *timestr( struct tm &tm );
+const char *timestr( const struct tm &tm );
 
 // Simple term signal handler
 static bool	global_done = false;
@@ -514,7 +514,7 @@ ReadEvents(Options &opts, int &totalEvents)
 			if ( opts.verbosity >= VERB_ALL ) {
 				printf( "Got an event from %d.%d.%d @ %s",
 						event->cluster, event->proc, event->subproc,
-						timestr(event->eventTime) );
+						timestr(event->GetEventTime()) );
 			}
 
 			if (opts.missedCheck ) {
@@ -739,7 +739,7 @@ ReportError( const ReadUserLog &reader )
 			 error_str, error, line_num );
 }
 
-const char *timestr( struct tm &t )
+const char *timestr( const struct tm &t )
 {
 	static char	tbuf[64];
 	strncpy( tbuf, asctime( &t ), sizeof(tbuf) );
