@@ -30,13 +30,15 @@
 #include <vector>
 #include <string>
 
+// for testing the userMap classad function
+extern int add_user_mapping(const char * mapname, char * mapdata);
 
 //uncomment if   SUBSYS.LOCAL.KNOB is allowed.
 //#define ALLOW_SUBSYS_LOCAL_HEIRARCHY 1
 
 extern const MACRO_SOURCE DefaultMacro;
 
-class YourSensitiveString ystr; // most recent lookup
+class YourString ystr; // most recent lookup
 auto_free_ptr gstr;    // holds a pointer from the most recent lookup that should be freed
 MyString gmstr;   // holds a formatted string that results from dumping a macro set
 
@@ -44,15 +46,15 @@ bool dash_verbose = false;
 int fail_count;
 
 template <class c>
-bool within(YourSensitiveString ys, c lb, c hb) { return false; }
+bool within(YourString ys, c lb, c hb) { return false; }
 
-template <> bool within(YourSensitiveString ys, int lb, int hb) {
+template <> bool within(YourString ys, int lb, int hb) {
 	int v = atoi(ys.Value());
 	//fprintf(stderr, "within '%s' -> %d\n", ys.Value(), v);
 	return v >= lb && v <= hb;
 }
 
-template <> bool within(YourSensitiveString ys, const char* lb, const char* hb) {
+template <> bool within(YourString ys, const char* lb, const char* hb) {
 	return strcmp(ys.Value(),lb) >= 0 && strcmp(ys.Value(),hb) <= 0;
 }
 
@@ -69,38 +71,38 @@ static MACRO_DEFAULTS TestingMacroDefaults = { 0, NULL, NULL };
 static MACRO_SET TestingMacroSet = {
 	0, 0,
 	CONFIG_OPT_WANT_META,
-	0, NULL, NULL, ALLOCATION_POOL(), std::vector<const char*>(), &TestingMacroDefaults };
+	0, NULL, NULL, ALLOCATION_POOL(), std::vector<const char*>(), &TestingMacroDefaults, NULL };
 
-MACRO_EVAL_CONTEXT def_ctx = { NULL, "TOOL", false, 2 };
+MACRO_EVAL_CONTEXT def_ctx = { NULL, "TOOL", "/home/testing", false, 2, 0, 0 };
 MACRO_SOURCE TestMacroSource = { false, false, 0, 0, -1, -2 };
 MACRO_SOURCE FileMacroSource = { false, false, 0, 0, -1, -2 };
 
 // helper functions for use within the REQUIRE macro
 // these call various MACRO_SET functions that we wish to test
-// and return a YourSensitiveString so that the result can be easily
+// and return a YourString so that the result can be easily
 // compared to a string literal.
 
 //
-YourSensitiveString & lookup(const char * name) {
+YourString & lookup(const char * name) {
 	ystr = lookup_macro(name, TestingMacroSet, def_ctx);
 	return ystr;
 }
 
-YourSensitiveString & lookup_as(const char * prefix, const char * name) {
+YourString & lookup_as(const char * prefix, const char * name) {
 	MACRO_EVAL_CONTEXT ctx = def_ctx;
 	ctx.subsys = prefix;
 	ystr = lookup_macro(name, TestingMacroSet, ctx);
 	return ystr;
 }
 
-YourSensitiveString & lookup_lcl(const char * local, const char * name) {
+YourString & lookup_lcl(const char * local, const char * name) {
 	MACRO_EVAL_CONTEXT ctx = def_ctx;
 	ctx.localname = local;
 	ystr = lookup_macro(name, TestingMacroSet, ctx);
 	return ystr;
 }
 
-YourSensitiveString & lookup_lcl_as(const char * local, const char * prefix, const char * name) {
+YourString & lookup_lcl_as(const char * local, const char * prefix, const char * name) {
 	MACRO_EVAL_CONTEXT ctx = def_ctx;
 	ctx.subsys = prefix;
 	ctx.localname = local;
@@ -111,13 +113,13 @@ YourSensitiveString & lookup_lcl_as(const char * local, const char * prefix, con
 // The expand helpers return a ystr, and also store the malloc'ed return value
 // in an auto_free_ptr so that the will be automatically freed by the next REQUIRE
 //
-YourSensitiveString expand(const char * value) {
+YourString expand(const char * value) {
 	gstr.set(expand_macro(value, TestingMacroSet, def_ctx));
 	ystr = gstr.ptr();
 	return ystr;
 }
 
-YourSensitiveString expand_as(const char * prefix, const char * value) {
+YourString expand_as(const char * prefix, const char * value) {
 	MACRO_EVAL_CONTEXT ctx = def_ctx;
 	ctx.subsys = prefix;
 	gstr.set(expand_macro(value, TestingMacroSet, ctx));
@@ -125,7 +127,7 @@ YourSensitiveString expand_as(const char * prefix, const char * value) {
 	return ystr;
 }
 
-YourSensitiveString expand_lcl(const char * local, const char * value) {
+YourString expand_lcl(const char * local, const char * value) {
 	MACRO_EVAL_CONTEXT ctx = def_ctx;
 	ctx.localname = local;
 	gstr.set(expand_macro(value, TestingMacroSet, ctx));
@@ -133,7 +135,7 @@ YourSensitiveString expand_lcl(const char * local, const char * value) {
 	return ystr;
 }
 
-YourSensitiveString expand_lcl_as(const char * local, const char * prefix, const char * value) {
+YourString expand_lcl_as(const char * local, const char * prefix, const char * value) {
 	MACRO_EVAL_CONTEXT ctx = def_ctx;
 	ctx.subsys = prefix;
 	ctx.localname = local;
@@ -142,7 +144,7 @@ YourSensitiveString expand_lcl_as(const char * local, const char * prefix, const
 	return ystr;
 }
 
-YourSensitiveString next$$(const char * value, int pos) {
+YourString next$$(const char * value, int pos) {
 	gstr.set(strdup(value));
 	char* left, *name, *right;
 	if (next_dollardollar_macro(gstr.ptr(), pos, &left, &name, &right)) {
@@ -153,7 +155,7 @@ YourSensitiveString next$$(const char * value, int pos) {
 	return ystr;
 }
 
-YourSensitiveString next$$_left(const char * value, int pos) {
+YourString next$$_left(const char * value, int pos) {
 	gstr.set(strdup(value));
 	char* left, *name, *right;
 	if (next_dollardollar_macro(gstr.ptr(), pos, &left, &name, &right)) {
@@ -164,7 +166,7 @@ YourSensitiveString next$$_left(const char * value, int pos) {
 	return ystr;
 }
 
-YourSensitiveString next$$_right(const char * value, int pos) {
+YourString next$$_right(const char * value, int pos) {
 	gstr.set(strdup(value));
 	char* left, *name, *right;
 	if (next_dollardollar_macro(gstr.ptr(), pos, &left, &name, &right)) {
@@ -210,35 +212,46 @@ static void insert_testing_macros(const char * local, const char * subsys)
 		{"fileLong", "Now is the time for all good men."},
 		{"fileCompound", "$(fileDirs)/$(fileBase).$(FileExt)"},
 		{"fileAbs", "/one/two/three.for"},
+		{"fileAbsDeep", "/six/five/four/three/two/one/file.ext"},
+		{"dirDeep", "six/five/four/three/two/one/"},
 		{"urlAbs", "file:/one/two/three.for"},
 		{"fileAbsQuoted", "\"/one/two/three.for\""},
+		{"fileAbsSQuoted", "'/one/two/three.for'"},
 		{"fileRel", "ein/zwei/drei.fir"},
+		{"fileRelDeep", "a/b/c/d/e/f.x"},
 		{"fileRelSpaces", "\"ein/zw ei/dr ei.fir\""},
 		{"fileCurRel", "./here"},
+		{"fileUp1Rel", "../file"},
+		{"fileOver1Rel", "../peer/file.dat"},
 		{"fileCurRel2", "./uno/dos.tres"},
-#ifdef WIN32
 		{"wfileAbs", "c:\\one\\two\\three.for"},
+		{"wfileAbsDeep", "c:\\six\\five\\four\\three\\two\\one\\file.ext"},
+		{"wUNCfileAbs", "\\\\server\\share\\one\\two\\file.ext"},
 		{"wfileAbsQuoted", "\"c:\\one\\two\\three.for\""},
 		{"wfileRel", "ein\\zwei\\drei.fir"},
 		{"wfileRelSpaces", "\"ein\\zw ei\\dr ei.fir\""},
 		{"wfileCurRel", ".\\here"},
 		{"wfileCurRel2", ".\\uno\\dos.tres"},
-#endif
+		{"wfileUp1Rel", "..\\file"},
+		{"wfileOver1Rel", "..\\peer\\file.dat"},
 
 		// for $F regressions
 		{"Items5", "aa bb cc dd ee"},
 		{"Items5Quoted", "\"aa bb cc dd ee\""},
 		{"List6c", "aa,bb, cc,dd,ee,ff"},
 		{"MASTER.List6c", "JMK,Vvv,XX,YY,ZKM,ZA"},
-		{"List6cfq", "$Fq(list6c)"},
+		{"List6cq", "$Fq(list6c)"},
 
 		// for $INT and $REAL tests
 		{"DoubleVanilla", "$(VANILLA)*2"},
 		{"HalfVanilla", "$(VANILLA)/2.0"},
 		{"StandardMinusVM", "$(STANDARD)-$(VM)"},
+
+		// for $STRING and $EVAL tests
+		{"Version","\"$Version: 8.5.6 May 20 2016 998822 $\""},
 	};
 
-	MACRO_EVAL_CONTEXT ctx = { local, subsys, false, 2 };
+	MACRO_EVAL_CONTEXT ctx = { local, subsys, "/home/testing", false, 2, 0, 0 };
 
 	insert_source("Insert", TestingMacroSet, TestMacroSource);
 
@@ -348,7 +361,7 @@ void testing_parser(bool verbose)
 	MACRO_SET set = {
 		0, 0,
 		CONFIG_OPT_WANT_META | CONFIG_OPT_DEFAULTS_ARE_PARAM_INFO,
-		0, NULL, NULL, ALLOCATION_POOL(), std::vector<const char*>(), &TestingMacroDefaults
+		0, NULL, NULL, ALLOCATION_POOL(), std::vector<const char*>(), &TestingMacroDefaults, NULL
 	};
 	MACRO_SOURCE source = { false, false, 0, 0, -1, -2 };
 	insert_source("parse", set, source);
@@ -513,8 +526,24 @@ void testing_$ENV_expand(bool verbose)
 	if (verbose) {
 		fprintf( stdout, "\n----- testing_$ENV_expand ----\n\n");
 	}
-	// regressions
-	// REQUIRE( expand("$ENV(username)") == "johnkn" );
+
+	// linux want's these to be char's
+	static char env1[] = "FOO=BAR";
+	static char env2[] = "Declaration=We the people, in order to form a more perfect union...";
+	// on Windows, putenv of this removes the environment var, but on *nix, it sets an empty var.
+	//static char env3[] = "EMPTY=";
+	putenv(env1);
+	putenv(env2);
+
+	// tests
+	REQUIRE( expand("$ENV(FOO)") == "BAR" );
+	REQUIRE( expand("$ENV(FOO:)") == "BAR" );
+	REQUIRE( expand("$ENV(FOO:BAZ)") == "BAR" );
+	REQUIRE( expand("$ENV(Declaration:)") == "We the people, in order to form a more perfect union..." );
+	REQUIRE( expand("$ENV(EMPTY)") == "UNDEFINED" );
+	REQUIRE( expand("$ENV(EMPTY:)") == "" );
+	REQUIRE( expand("$ENV(EMPTY:BAZ)") == "BAZ" );
+	REQUIRE( expand("$ENV(EMPTY:$(STUFF))") == "" );
 }
 
 void testing_$RAND_expand(bool verbose)
@@ -523,7 +552,7 @@ void testing_$RAND_expand(bool verbose)
 		fprintf( stdout, "\n----- testing_$RAND_expand ----\n\n");
 	}
 
-	// regressions
+	// tests
 	REQUIRE( within(expand("$RANDOM_INTEGER(-30,30,1)"), -30, 30) );
 	REQUIRE( within(expand("$RANDOM_INTEGER(2,2,1)"), 2, 2) );
 	REQUIRE( within(expand("$RANDOM_INTEGER(-10,0,2)"), -10, 0) );
@@ -550,11 +579,17 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fx(fileBase)") == "" );
 	REQUIRE( expand("$Fd(fileBase)") == "" );
 
+	REQUIRE( expand("$Fdf(fileBase)") == "testing/" );
+	REQUIRE( expand("$Fpf(fileBase)") == "/home/testing/" );
+
 	REQUIRE( expand("$Fd(fileDirs)") == "dur/" );
 	REQUIRE( expand("$Fdx(fileDirs)") == "dur/der" );
 	REQUIRE( expand("$Fn(fileDirs)") == "der" );
 	REQUIRE( expand("$Fpnx(fileDirs)") == "/dur/der" );
 
+	REQUIRE( expand("$Fpnf(fileDirs)") == "/dur/der" );
+	REQUIRE( expand("$Fdf(fileDirs)") == "dur/" );
+
 	REQUIRE( expand("$Fdnx(fileSimple)") == "simple.dat" );
 	REQUIRE( expand("$Fd(fileSimple)") == "" );
 	REQUIRE( expand("$Fn(fileSimple)") == "simple" );
@@ -568,22 +603,46 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fx(fileLong)") == "." );
 	REQUIRE( expand("$Fnx(fileLong)") == "Now is the time for all good men." );
 
-	REQUIRE( expand("$Fdnx(fileCompound)") == "/dur/der/base.ex" );
-	REQUIRE( expand("$Fd(fileCompound)") == "/dur/der/" );
+	REQUIRE( expand("$F(fileCompound)") == "/dur/der/base.ex" );
+	REQUIRE( expand("$Fdnx(fileCompound)") == "der/base.ex" );
+	REQUIRE( expand("$Fd(fileCompound)") == "der/" );
+	REQUIRE( expand("$Fdb(fileCompound)") == "der" );
+	REQUIRE( expand("$Fddb(fileCompound)") == "dur/der" );
 	REQUIRE( expand("$Fp(fileCompound)") == "/dur/der/" );
+	REQUIRE( expand("$Fpw(fileCompound)") == "\\dur\\der\\" );
+	REQUIRE( expand("$Ffqaw(fileCompound)") == "'\\dur\\der\\base.ex'" );
 	REQUIRE( expand("$Fn(fileCompound)") == "base" );
 	REQUIRE( expand("$Fx(fileCompound)") == ".ex" );
 	REQUIRE( expand("$Fnx(fileCompound)") == "base.ex" );
+	REQUIRE( expand("$Fpf(fileCompound)") == "/dur/der/" );
+	REQUIRE( expand("$Fnf(fileCompound)") == "base" );
+	REQUIRE( expand("$Fxf(fileCompound)") == ".ex" );
+	REQUIRE( expand("$Fnxf(fileCompound)") == "base.ex" );
 
 	REQUIRE( expand("$Fdnx(fileAbs)") == "two/three.for" );
 	REQUIRE( expand("$Fpnx(fileAbs)") == "/one/two/three.for" );
 	REQUIRE( expand("$Fqpnx(fileAbs)") == "\"/one/two/three.for\"" );
 	REQUIRE( expand("$Fq(fileAbs)") == "\"/one/two/three.for\"" );
+	REQUIRE( expand("$Fqapnx(fileAbs)") == "'/one/two/three.for'" );
+	REQUIRE( expand("$Fqa(fileAbs)") == "'/one/two/three.for'" );
 	REQUIRE( expand("$Fd(fileAbs)") == "two/" );
 	REQUIRE( expand("$Fp(fileAbs)") == "/one/two/" );
 	REQUIRE( expand("$Fn(fileAbs)") == "three" );
 	REQUIRE( expand("$Fx(fileAbs)") == ".for" );
 	REQUIRE( expand("$Fnx(fileAbs)") == "three.for" );
+	REQUIRE( expand("$Fpnxf(fileAbs)") == "/one/two/three.for" );
+
+	REQUIRE( expand("$Fd(fileAbsDeep)") == "one/" );
+	REQUIRE( expand("$Fdd(fileAbsDeep)") == "two/one/" );
+	REQUIRE( expand("$Fddd(fileAbsDeep)") == "three/two/one/" );
+	REQUIRE( expand("$Fddddddd(fileAbsDeep)") == "/six/five/four/three/two/one/" );
+	REQUIRE( expand("$Fddb(fileAbsDeep)") == "two/one" );
+
+	REQUIRE( expand("$Fw(fileAbsDeep)") == "\\six\\five\\four\\three\\two\\one\\file.ext" );
+	REQUIRE( expand("$Ffw(fileAbsDeep)") == "\\six\\five\\four\\three\\two\\one\\file.ext" );
+	REQUIRE( expand("$Fpw(fileAbsDeep)") == "\\six\\five\\four\\three\\two\\one\\" );
+	REQUIRE( expand("$Fddxw(fileAbsDeep)") == "two\\one\\file.ext" );
+	REQUIRE( expand("$Fddnw(fileAbsDeep)") == "two\\one\\file" );
 
 	REQUIRE( expand("$Fdnx(urlAbs)") == "two/three.for" );
 	REQUIRE( expand("$Fpnx(urlAbs)") == "file:/one/two/three.for" );
@@ -600,7 +659,23 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fp(fileAbsQuoted)") == "/one/two/" );
 	REQUIRE( expand("$Fn(fileAbsQuoted)") == "three" );
 	REQUIRE( expand("$Fx(fileAbsQuoted)") == ".for" );
+	REQUIRE( expand("$Fdb(fileAbsQuoted)") == "two" );
+	REQUIRE( expand("$Fpb(fileAbsQuoted)") == "/one/two" );
+	REQUIRE( expand("$Fnb(fileAbsQuoted)") == "three" );
+	REQUIRE( expand("$Fxb(fileAbsQuoted)") == "for" );
 	REQUIRE( expand("$Fnx(fileAbsQuoted)") == "three.for" );
+	REQUIRE( expand("$Fpf(fileAbsQuoted)") == "/one/two/" );
+	REQUIRE( expand("$Fqn(fileAbsQuoted)") == "\"three\"" );
+	REQUIRE( expand("$Fqan(fileAbsQuoted)") == "'three'" );
+
+	REQUIRE( expand("$Fdnx(fileAbsSQuoted)") == "two/three.for" );
+	REQUIRE( expand("$Fd(fileAbsSQuoted)") == "two/" );
+	REQUIRE( expand("$Fp(fileAbsSQuoted)") == "/one/two/" );
+	REQUIRE( expand("$Fn(fileAbsSQuoted)") == "three" );
+	REQUIRE( expand("$Fx(fileAbsSQuoted)") == ".for" );
+	REQUIRE( expand("$Fnx(fileAbsSQuoted)") == "three.for" );
+	REQUIRE( expand("$Fqn(fileAbsSQuoted)") == "\"three\"" );
+	REQUIRE( expand("$Fqan(fileAbsSQuoted)") == "'three'" );
 
 	REQUIRE( expand("$Fpnx(fileRel)") == "ein/zwei/drei.fir" );
 	REQUIRE( expand("$Fd(fileRel)") == "zwei/" );
@@ -609,6 +684,17 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fx(fileRel)") == ".fir" );
 	REQUIRE( expand("$Fnx(fileRel)") == "drei.fir" );
 
+	REQUIRE( expand("$Fnx(fileRelDeep)") == "f.x" );
+	REQUIRE( expand("$Fdb(fileRelDeep)") == "e" );
+	REQUIRE( expand("$Fddb(fileRelDeep)") == "d/e" );
+	REQUIRE( expand("$Ff(fileRelDeep)") == "/home/testing/a/b/c/d/e/f.x" );
+	REQUIRE( expand("$Ffq(fileRelDeep)") == "\"/home/testing/a/b/c/d/e/f.x\"" );
+	REQUIRE( expand("$Ffddddd(fileRelDeep)") == "a/b/c/d/e/" );
+	REQUIRE( expand("$Ffdddddd(fileRelDeep)") == "testing/a/b/c/d/e/" );
+	REQUIRE( expand("$Ffw(fileRelDeep)") == "\\home\\testing\\a\\b\\c\\d\\e\\f.x" );
+	REQUIRE( expand("$Ffwq(fileRelDeep)") == "\"\\home\\testing\\a\\b\\c\\d\\e\\f.x\"" );
+	REQUIRE( expand("$Ffwqa(fileRelDeep)") == "'\\home\\testing\\a\\b\\c\\d\\e\\f.x'" );
+
 	REQUIRE( expand("$Fdnx(fileRelSpaces)") == "zw ei/dr ei.fir" );
 	REQUIRE( expand("$Fd(fileRelSpaces)") == "zw ei/" );
 	REQUIRE( expand("$Fp(fileRelSpaces)") == "ein/zw ei/" );
@@ -622,6 +708,34 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fn(fileCurRel)") == "here" );
 	REQUIRE( expand("$Fx(fileCurRel)") == "" );
 	REQUIRE( expand("$Fnx(fileCurRel)") == "here" );
+	REQUIRE( expand("$Ff(fileCurRel)") == "/home/testing/here" );
+
+	REQUIRE( expand("$Fdnx(fileUp1Rel)") == "../file" );
+	REQUIRE( expand("$Fd(fileUp1Rel)") == "../" );
+	REQUIRE( expand("$Fp(fileUp1Rel)") == "../" );
+	REQUIRE( expand("$Fn(fileUp1Rel)") == "file" );
+	REQUIRE( expand("$Fx(fileUp1Rel)") == "" );
+	REQUIRE( expand("$Fnx(fileUp1Rel)") == "file" );
+	REQUIRE( expand("$Ff(fileUp1Rel)") == "/home/testing/../file" );
+
+	REQUIRE( expand("$Fdnx(fileOver1Rel)") == "peer/file.dat" );
+	REQUIRE( expand("$Fd(fileOver1Rel)") == "peer/" );
+	REQUIRE( expand("$Fp(fileOver1Rel)") == "../peer/" );
+	REQUIRE( expand("$Fn(fileOver1Rel)") == "file" );
+	REQUIRE( expand("$Fx(fileOver1Rel)") == ".dat" );
+	REQUIRE( expand("$Fnx(fileOver1Rel)") == "file.dat" );
+	REQUIRE( expand("$Ff(fileOver1Rel)") == "/home/testing/../peer/file.dat" );
+
+#ifdef WIN32
+	REQUIRE( expand("$Ff(wfileCurRel)") == "/home/testing/here" );
+	REQUIRE( expand("$Ffu(wfileCurRel)") == "/home/testing/here" );
+#endif
+	REQUIRE( expand("$Ffw(wfileCurRel)") == "\\home\\testing\\here" );
+	REQUIRE( expand("$Ff(wfileUp1Rel)") == "/home/testing/..\\file" );
+	REQUIRE( expand("$Ffu(wfileUp1Rel)") == "/home/testing/../file" );
+	REQUIRE( expand("$Ffw(wfileUp1Rel)") == "\\home\\testing\\..\\file" );
+	REQUIRE( expand("$Ffu(wfileOver1Rel)") == "/home/testing/../peer/file.dat" );
+	REQUIRE( expand("$Ffw(wfileOver1Rel)") == "\\home\\testing\\..\\peer\\file.dat" );
 
 	REQUIRE( expand("$Fpnx(fileCurRel2)") == "./uno/dos.tres" );
 	REQUIRE( expand("$Fdnx(fileCurRel2)") == "uno/dos.tres" );
@@ -630,15 +744,53 @@ void testing_$F_expand(bool verbose)
 	REQUIRE( expand("$Fn(fileCurRel2)") == "dos" );
 	REQUIRE( expand("$Fx(fileCurRel2)") == ".tres" );
 	REQUIRE( expand("$Fnx(fileCurRel2)") == "dos.tres" );
+	REQUIRE( expand("$Fpnxf(fileCurRel2)") == "/home/testing/uno/dos.tres" );
+	REQUIRE( expand("$Fdnxf(fileCurRel2)") == "uno/dos.tres" );
+	REQUIRE( expand("$Fpf(fileCurRel2)") == "/home/testing/uno/" );
+
+	REQUIRE( expand("$Ff(wfileRel)") == "/home/testing/ein\\zwei\\drei.fir" );
+	REQUIRE( expand("$Ffw(wfileRel)") == "\\home\\testing\\ein\\zwei\\drei.fir" );
+	REQUIRE( expand("$Ffu(wfileRel)") == "/home/testing/ein/zwei/drei.fir" );
 
 #ifdef WIN32
 	REQUIRE( expand("$Fdnx(wfileAbs)") == "two\\three.for" );
 	REQUIRE( expand("$Fd(wfileAbs)") == "two\\" );
 	REQUIRE( expand("$Fp(wfileAbs)") == "c:\\one\\two\\" );
+	REQUIRE( expand("$Ff(wfileAbs)") == "c:\\one\\two\\three.for" );
 	REQUIRE( expand("$Fn(wfileAbs)") == "three" );
 	REQUIRE( expand("$Fx(wfileAbs)") == ".for" );
 	REQUIRE( expand("$Fnx(wfileAbs)") == "three.for" );
 
+	REQUIRE( expand("$Fdnx(wfileAbsDeep)") == "one\\file.ext" );
+	REQUIRE( expand("$Fddnx(wfileAbsDeep)") == "two\\one\\file.ext" );
+	REQUIRE( expand("$Fd(wfileAbsDeep)") == "one\\" );
+	REQUIRE( expand("$Fdd(wfileAbsDeep)") == "two\\one\\" );
+	REQUIRE( expand("$Fddd(wfileAbsDeep)") == "three\\two\\one\\" );
+	REQUIRE( expand("$Fdddp(wfileAbsDeep)") == "three\\two\\one\\" );
+	REQUIRE( expand("$Fp(wfileAbsDeep)") == "c:\\six\\five\\four\\three\\two\\one\\" );
+	REQUIRE( expand("$Fpb(wfileAbsDeep)") == "c:\\six\\five\\four\\three\\two\\one" );
+	REQUIRE( expand("$Fn(wfileAbsDeep)") == "file" );
+	REQUIRE( expand("$Fx(wfileAbsDeep)") == ".ext" );
+	REQUIRE( expand("$Fxb(wfileAbsDeep)") == "ext" );
+	REQUIRE( expand("$Fnx(wfileAbsDeep)") == "file.ext" );
+
+	REQUIRE( expand("$Fdnx(wUNCfileAbs)") == "two\\file.ext" );
+	REQUIRE( expand("$Fddnx(wUNCfileAbs)") == "one\\two\\file.ext" );
+	REQUIRE( expand("$Fd(wUNCfileAbs)") == "two\\" );
+	REQUIRE( expand("$Fdd(wUNCfileAbs)") == "one\\two\\" );
+	REQUIRE( expand("$Fddd(wUNCfileAbs)") == "share\\one\\two\\" );
+	REQUIRE( expand("$Fdddd(wUNCfileAbs)") == "server\\share\\one\\two\\" );
+	REQUIRE( expand("$Fddddd(wUNCfileAbs)") == "\\\\server\\share\\one\\two\\" );
+	REQUIRE( expand("$Fpn(wUNCfileAbs)") == "\\\\server\\share\\one\\two\\file" );
+
+	REQUIRE( expand("$Fdnxu(wUNCfileAbs)") == "two/file.ext" );
+	REQUIRE( expand("$Fddnxu(wUNCfileAbs)") == "one/two/file.ext" );
+	REQUIRE( expand("$Fdu(wUNCfileAbs)") == "two/" );
+	REQUIRE( expand("$Fdub(wUNCfileAbs)") == "two" );
+	REQUIRE( expand("$Fdb(wUNCfileAbs)") == "two" );
+	REQUIRE( expand("$Fddu(wUNCfileAbs)") == "one/two/" );
+	REQUIRE( expand("$Fu(wUNCfileAbs)") == "//server/share/one/two/file.ext" );
+
 	REQUIRE( expand("$Fdnx(wfileAbsQuoted)") == "two\\three.for" );
 	REQUIRE( expand("$Fd(wfileAbsQuoted)") == "two\\" );
 	REQUIRE( expand("$Fp(wfileAbsQuoted)") == "c:\\one\\two\\" );
@@ -753,6 +905,35 @@ void testing_$SUBSTR_expand(bool verbose)
 	REQUIRE( expand_as("MASTER", "$SUBSTR(fileCompound,standard)") == "ur/der/base.ex" );
 }
 
+void testing_$STRING_expand(bool verbose)
+{
+	if (verbose) {
+		fprintf( stdout, "\n----- testing_$STRING_expand ----\n\n");
+	}
+
+	insert_macro("items6", "strcat($(Items5Quoted),\" ff\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("VerNum", "split($(Version))[1]", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("FullVerNum", "strcat(split($(Version))[1],\".\",split($(Version))[5])", TestingMacroSet,  TestMacroSource, def_ctx);
+
+	REQUIRE( expand("$STRING(ITEMS5quoted)") == "aa bb cc dd ee" );
+	REQUIRE( expand("$STRING(Items6)") == "aa bb cc dd ee ff" );
+	REQUIRE( expand("$STRING(VerNum)") == "8.5.6" );
+	REQUIRE( expand("$STRING(FullVerNum)") == "8.5.6.998822" );
+
+	REQUIRE( expand("$STRING(list6cq)") == "aa,bb, cc,dd,ee,ff" );
+	REQUIRE( expand_as("MASTER", "$STRING(list6cq)") == "JMK,Vvv,XX,YY,ZKM,ZA" );
+
+	// if param lookup fails, we parse as an expression, and a simple variable name
+	// will parse as an attribute reference and then round trip unchanged. it's 
+	// unclear that this is the correct thing to do.  but it's what happens right now.
+	REQUIRE( expand("$STRING(BAR)") == "BAR" );
+
+	// now do some formatting tests...
+	REQUIRE( expand("$STRING(Items6,%20s)") == "   aa bb cc dd ee ff" );
+	REQUIRE( expand("$STRING(Items6,%-20s)") == "aa bb cc dd ee ff   " );
+	REQUIRE( expand("$STRING(Items6,%10.10s)") == "aa bb cc d" );
+}
+
 void testing_$INT_expand(bool verbose)
 {
 	if (verbose) {
@@ -793,6 +974,52 @@ void testing_$REAL_expand(bool verbose)
 	REQUIRE( within(expand("$REAL(BackgroundLoad,%e)"), "3.000000e-0001", "3.000000e-01") );
 }
 
+void testing_$EVAL_expand(bool verbose)
+{
+	if (verbose) {
+		fprintf( stdout, "\n----- testing_$EVAL_expand ----\n\n");
+	}
+	REQUIRE( expand("$EVAL(4)") == "4" );
+	REQUIRE( expand("$EVAL(4+4)") == "8" );
+	REQUIRE( expand("$EVAL(bar)") == "undefined" );
+
+	insert_macro("SplitVer", "split($(Version))", TestingMacroSet,  TestMacroSource, def_ctx);
+	REQUIRE( expand("$EVAL(SplitVer)") == "{ \"$Version:\",\"8.5.6\",\"May\",\"20\",\"2016\",\"998822\",\"$\" }" );
+
+	auto_free_ptr mapping(strdup("* alice Security,MetalShop\n* bob Security,WoodShop\n"));
+	add_user_mapping("grouptest", mapping.ptr());
+	insert_macro("BobsGroups", "userMap(\"grouptest\",\"bob\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("BobsFirst", "userMap(\"grouptest\",\"bob\",undefined)", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("BobsShop", "userMap(\"grouptest\",\"bob\",\"woodshop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("BobsUnshop", "userMap(\"grouptest\",\"bob\",\"shop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	REQUIRE( expand("$EVAL(BobsGroups)") == "Security,WoodShop" );
+	REQUIRE( expand("$EVAL(BobsFirst)") == "Security" );
+	REQUIRE( expand("$EVAL(BobsShop)") == "WoodShop" );
+	REQUIRE( expand("$EVAL(BobsUnShop)") == "undefined" );
+
+	insert_macro("AlicesGroups", "userMap(\"grouptest\",\"alice\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("AlicesFirst", "userMap(\"grouptest\",\"alice\",undefined)", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("AlicesShop", "userMap(\"grouptest\",\"alice\",\"metalshop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("AlicesUnshop", "userMap(\"grouptest\",\"alice\",\"shop\",\"AutoShop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("AlicesDefGroup", "userMap(\"grouptest\",\"alice\",undefined,\"AutoShop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	REQUIRE( expand("$EVAL(AlicesGroups)") == "Security,MetalShop" );
+	REQUIRE( expand("$EVAL(AlicesFirst)") == "Security" );
+	REQUIRE( expand("$EVAL(AlicesShop)") == "MetalShop" );
+	REQUIRE( expand("$EVAL(AlicesUnShop)") == "AutoShop" );
+	REQUIRE( expand("$EVAL(AlicesDefGroup)") == "Security" );
+
+	insert_macro("JohnsGroups", "userMap(\"grouptest\",\"john\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("JohnsFirst", "userMap(\"grouptest\",\"john\",undefined)", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("JohnsShop", "userMap(\"grouptest\",\"john\",\"metalshop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("JohnsUnShop", "userMap(\"grouptest\",\"john\",\"shop\",\"AutoShop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	insert_macro("JohnsDefGroup", "userMap(\"grouptest\",\"john\",undefined,\"AutoShop\")", TestingMacroSet,  TestMacroSource, def_ctx);
+	REQUIRE( expand("$EVAL(JohnsGroups)") == "undefined" );
+	REQUIRE( expand("$EVAL(JohnsFirst)") == "undefined" );
+	REQUIRE( expand("$EVAL(JohnsShop)") == "undefined" );
+	REQUIRE( expand("$EVAL(JohnsUnShop)") == "AutoShop" );
+	REQUIRE( expand("$EVAL(JohnsDefGroup)") == "AutoShop" );
+}
+
 // runs all of the tests in non-verbose mode by default (i.e. printing only failures)
 // individual groups of tests can be run by using the -t:<tests> option where <tests>
 // is one or more of 
@@ -828,11 +1055,12 @@ int main( int /*argc*/, const char ** argv) {
 					case 'e': test_flags |= 0x0008; break; // $ENV
 					case 'F': test_flags |= 0x0010 | 0x0020; break;	 // $F
 					case 'c': test_flags |= 0x0040; break; // $CHOICE
-					case 's': test_flags |= 0x0080; break; // $SUBSTR
+					case 's': test_flags |= 0x0080; break; // $SUBSTR, $STRING
 					case 'i': test_flags |= 0x0100; break; // $INT
 					case 'f': test_flags |= 0x0200; break; // $REAL
 					case 'n': test_flags |= 0x0100 | 0x0200; break; // $INT, $REAL
-					case 'r': test_flags |= 0x0400; break; // $RANDOM_INTEGER and $RANDOM_CHOICE
+					case 'v': test_flags |= 0x0400; break; // $EVAL
+					case 'r': test_flags |= 0x0800; break; // $RANDOM_INTEGER and $RANDOM_CHOICE
 					case 'p': test_flags |= 0x1000; break; // parse
 					}
 				}
@@ -841,6 +1069,26 @@ int main( int /*argc*/, const char ** argv) {
 			}
 		} else {
 			fprintf(stderr, "unknown argument %s\n", arg);
+			fprintf(stderr, "usage: %s [ -verbose ] [ -test[:<tests>] ]\n", argv[0]);
+			fprintf(stderr,
+				"   one or more single letters select individual tests:\n"
+				"      l param lookups\n"
+				"      $ $ expansion\n"
+				"      [ $$ expansion\n"
+				"      e $ENV expansion\n"
+				"      F $F expansion\n"
+				"      c $CHOICE expansion/evaluation\n"
+				"      s $SUBSTR, $STRING expansion/evaluation\n"
+				"      i $INT expansion/evaluation\n"
+				"      f $REAL expansion/evaluation\n"
+				"      n $INT, $REAL expansion/evaluation\n"
+				"      v $EVAL expansion/evaluation\n"
+				"      r $RANDOM_INTEGER and $RANDOM_CHOICE expansion/evaluation\n"
+				"      p config parsing\n"
+				"   default is to run all tests\n"
+				"   If -verbose is specified, all tests show output, otherwise\n"
+				"   only failed tests produce output\n"
+				);
 			return 1;
 		}
 	}
@@ -863,11 +1111,13 @@ int main( int /*argc*/, const char ** argv) {
 
 	if (test_flags & 0x0040) testing_$CHOICE_expand(dash_verbose);
 	if (test_flags & 0x0080) testing_$SUBSTR_expand(dash_verbose);
+	if (test_flags & 0x0080) testing_$STRING_expand(dash_verbose);
 
 	if (test_flags & 0x0100) testing_$INT_expand(dash_verbose);
 	if (test_flags & 0x0200) testing_$REAL_expand(dash_verbose);
+	if (test_flags & 0x0400) testing_$EVAL_expand(dash_verbose);
 
-	if (test_flags & 0x0400) testing_$RAND_expand(dash_verbose);
+	if (test_flags & 0x0800) testing_$RAND_expand(dash_verbose);
 
 	if (test_flags & 0x1000) testing_parser(dash_verbose);
 
