@@ -51,13 +51,13 @@ TrackTotals::
 }
 
 int TrackTotals::
-update (ClassAd *ad)
+update (ClassAd *ad, int options, const char * _key /*=NULL*/)
 {
 	ClassTotal *ct;
-	MyString	key;
+	MyString	key(_key);
 	int		   	rval;
 
-	if (!ClassTotal::makeKey(key, ad, ppo)) 
+	if ( key.empty() && ! ClassTotal::makeKey(key, ad, ppo))
 	{
 		malformed++;
 		return 0;
@@ -74,23 +74,16 @@ update (ClassAd *ad)
 		}
 	}
 
-	rval = ct->update(ad);
-	topLevelTotal->update(ad);
+	rval = ct->update(ad, options);
+	topLevelTotal->update(ad, options);
 
 	if (rval == 0) malformed++;
 
 	return rval;
 }
 
-
-void TrackTotals::
-displayTotals (FILE *file, int keyLength)
+bool TrackTotals::haveTotals()
 {
-	ClassTotal *ct=0;
-	MyString	key;
-	int k;
-	bool auto_key_length = keyLength < 0;
-
 	// display totals only for meaningful modes
 	switch (ppo)
 	{
@@ -105,17 +98,33 @@ displayTotals (FILE *file, int keyLength)
 #endif /* HAVE_EXT_POSTGRESQL */
 
     	case PP_SCHEDD_NORMAL:
-    	case PP_SCHEDD_SUBMITTORS:   
+    	case PP_SUBMITTER_NORMAL:
     	case PP_CKPT_SRVR_NORMAL:
 			break;
 
 		default:
-			return;
+			return false;
 	}
+	return true;
+}
+
+
+void TrackTotals::
+displayTotals (FILE *file, int keyLength)
+{
+	ClassTotal *ct=0;
+	MyString	key;
+	int k;
+	bool auto_key_length = keyLength < 0;
+	if (auto_key_length) { keyLength = 5; } // must be at least 5 for "Total"
+
+	// display totals only for meaningful modes
+	if ( ! haveTotals()) return;
 
 		
 	// sort the keys (insertion sort) so we display totals in sorted order
 	const char **keys = new const char* [allTotals.getNumElements()];
+	ASSERT(keys);
 	allTotals.startIterations();
 	for (k = 0; k < allTotals.getNumElements(); k++) // for each key
 	{
@@ -143,15 +152,18 @@ displayTotals (FILE *file, int keyLength)
 	fprintf (file, "\n");
 
 	// now that our keys are sorted, display the totals in sort order
+	bool had_tot_keys = false;
 	for (k = 0; k < allTotals.getNumElements(); k++)
 	{
 		fprintf (file, "%*.*s", keyLength, keyLength, keys[k]);
 		allTotals.lookup(MyString(keys[k]), ct);
 		free((void *)const_cast<char*>(keys[k]));
 		ct->displayInfo(file);
+		had_tot_keys = true;
 	}
 	delete [] keys;
-	fprintf (file, "\n%*.*s", keyLength, keyLength, "Total");
+	if (had_tot_keys) fprintf(file, "\n");
+	fprintf (file, "%*.*s", keyLength, keyLength, "Total");
 	topLevelTotal->displayInfo(file,1);
 
 	if (malformed > 0)
@@ -181,11 +193,41 @@ StartdNormalTotal()
 
 
 int StartdNormalTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int options)
 {
 	char state[32];
 
-	if (!ad->LookupString (ATTR_STATE, state, sizeof(state))) return 0;
+	bool partitionable_slot = false;
+	bool dynamic_slot = false;
+	if (options) {
+		ad->LookupBool(ATTR_SLOT_PARTITIONABLE, partitionable_slot);
+		if ( ! partitionable_slot) { ad->LookupBool(ATTR_SLOT_DYNAMIC, dynamic_slot); }
+	}
+
+	if ((options & TOTALS_OPTION_IGNORE_PARTITIONABLE) && partitionable_slot) return 1;
+	if ((options & TOTALS_OPTION_IGNORE_DYNAMIC) && dynamic_slot) return 1;
+	if ((options & TOTALS_OPTION_ROLLUP_PARTITIONABLE) && partitionable_slot) {
+		classad::Value lval;
+		const classad::ExprList* plist = NULL;
+		if ( ! ad->EvaluateAttr("Child" ATTR_STATE, lval) || ! lval.IsListValue(plist)) return 1; // ChildState can be validly empty
+		classad::ExprList::const_iterator it;
+		for (it = plist->begin(); it != plist->end(); ++it) {
+			const classad::ExprTree * pexpr = *it;
+			classad::Value val;
+			if (pexpr->Evaluate(val) && val.IsStringValue(state,sizeof(state))) {
+				update(state);
+			}
+		}
+		return 1;
+	} else {
+		if (!ad->LookupString (ATTR_STATE, state, sizeof(state))) return 0;
+		return update(state);
+	}
+}
+
+int StartdNormalTotal::
+update (const char * state)
+{
 	switch (string_to_state (state))
 	{
 		case owner_state: 		owner++; 		break;
@@ -208,12 +250,12 @@ void StartdNormalTotal::
 displayHeader(FILE *file)
 {
 #if HAVE_BACKFILL
-	fprintf (file, "%6.6s %5.5s %7.7s %9.9s %7.7s %10.10s %8.8s\n",
+	fprintf (file, "%6.6s %5.5s %7.7s %9.9s %7.7s %10.10s %8.8s %6.6s\n",
 					"Total", "Owner", "Claimed", "Unclaimed", "Matched",
-					"Preempting", "Backfill");
+					"Preempting", "Backfill", "Drain");
 #else
-	fprintf (file, "%9.9s %5.5s %7.7s %9.9s %7.7s %10.10s\n", "Machines", 
-					"Owner", "Claimed", "Unclaimed", "Matched", "Preempting");
+	fprintf (file, "%9.9s %5.5s %7.7s %9.9s %7.7s %10.10s %6.6s\n", "Machines",
+					"Owner", "Claimed", "Unclaimed", "Matched", "Preempting", "Drain");
 #endif /* HAVE_BACKFILL */
 }
 
@@ -222,12 +264,12 @@ void StartdNormalTotal::
 displayInfo (FILE *file, int)
 {
 #if HAVE_BACKFILL
-	fprintf ( file, "%6d %5d %7d %9d %7d %10d %8d\n", machines, owner,
-			  claimed, unclaimed, matched, preempting, backfill );
+	fprintf ( file, "%6d %5d %7d %9d %7d %10d %8d %6d\n", machines, owner,
+			  claimed, unclaimed, matched, preempting, backfill, drained );
 
 #else 
-	fprintf (file, "%9d %5d %7d %9d %7d %10d\n", machines, owner, claimed,
-					unclaimed, matched, preempting);
+	fprintf (file, "%9d %5d %7d %9d %7d %10d %6d\n", machines, owner, claimed,
+					unclaimed, matched, preempting, drained);
 #endif /* HAVE_BACKFILL */
 }
 
@@ -246,12 +288,18 @@ StartdServerTotal()
 
 
 int StartdServerTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int options)
 {
 	char state[32];
-	int	 attrMem, attrDisk, attrMips, attrKflops;
+	int	 attrMem, attrDisk, attrMips, attrKflops = 0;
 	bool badAd = false;
-	State s;
+
+	bool partitionable_slot = false;
+	bool dynamic_slot = false;
+	if (options) {
+		ad->LookupBool(ATTR_SLOT_PARTITIONABLE, partitionable_slot);
+		if ( ! partitionable_slot) { ad->LookupBool(ATTR_SLOT_DYNAMIC, dynamic_slot); }
+	}
 
 	// if ATTR_STATE is not found, abort this ad
 	if (!ad->LookupString (ATTR_STATE, state, sizeof(state))) return 0;
@@ -262,7 +310,7 @@ update (ClassAd *ad)
 	if (!ad->LookupInteger(ATTR_MIPS,  attrMips)){ badAd = true; attrMips = 0;}
 	if (!ad->LookupInteger(ATTR_KFLOPS,attrKflops)){badAd= true;attrKflops = 0;}
 
-	s = string_to_state(state);
+	State s = string_to_state(state);
 	if (s == claimed_state || s == unclaimed_state)
 		avail++;
 
@@ -307,12 +355,19 @@ StartdRunTotal()
 
 
 int StartdRunTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int options)
 {
 	int attrMips, attrKflops;
 	float attrLoadAvg;
 	bool badAd = false;
 
+	bool partitionable_slot = false;
+	bool dynamic_slot = false;
+	if (options) {
+		ad->LookupBool(ATTR_SLOT_PARTITIONABLE, partitionable_slot);
+		if ( ! partitionable_slot) { ad->LookupBool(ATTR_SLOT_DYNAMIC, dynamic_slot); }
+	}
+
 	if (!ad->LookupInteger(ATTR_MIPS, attrMips)) { badAd = true; attrMips = 0;}
 	if (!ad->LookupInteger(ATTR_KFLOPS, attrKflops)){badAd=true; attrKflops=0;}
 	if (!ad->LookupFloat(ATTR_LOAD_AVG,attrLoadAvg)){badAd=true;attrLoadAvg=0;}
@@ -362,16 +417,46 @@ StartdStateTotal()
 }
 
 int StartdStateTotal::
-update( ClassAd *ad )
+update (ClassAd *ad, int options)
 {
-	char	stateStr[32];
-	State	state;
+	char state[32];
 
-	machines ++;
+	bool partitionable_slot = false;
+	bool dynamic_slot = false;
+	if (options) {
+		ad->LookupBool(ATTR_SLOT_PARTITIONABLE, partitionable_slot);
+		if ( ! partitionable_slot) { ad->LookupBool(ATTR_SLOT_DYNAMIC, dynamic_slot); }
+	}
 
-	if( !ad->LookupString( ATTR_STATE , stateStr, sizeof(stateStr) ) ) return false;
-	state = string_to_state( stateStr );
-	switch( state ) {
+	if ((options & TOTALS_OPTION_IGNORE_PARTITIONABLE) && partitionable_slot) return 1;
+	if ((options & TOTALS_OPTION_IGNORE_DYNAMIC) && dynamic_slot) return 1;
+	if ((options & TOTALS_OPTION_ROLLUP_PARTITIONABLE) && partitionable_slot) {
+		classad::Value lval;
+		const classad::ExprList* plist = NULL;
+		if ( ! ad->EvaluateAttr("Child" ATTR_STATE, lval) || ! lval.IsListValue(plist)) return 1;
+		classad::ExprList::const_iterator it;
+		for (it = plist->begin(); it != plist->end(); ++it) {
+			const classad::ExprTree * pexpr = *it;
+			classad::Value val;
+			if (pexpr->Evaluate(val) && val.IsStringValue(state,sizeof(state))) {
+				update(state);
+			}
+		}
+		return 1;
+	} else {
+		if (!ad->LookupString (ATTR_STATE, state, sizeof(state))) return 0;
+		return update(state);
+	}
+
+	++machines;
+	return 1;
+}
+
+
+int StartdStateTotal::
+update (const char * state)
+{
+	switch (string_to_state(state)) {
 		case owner_state	:	owner++;		break;
 		case unclaimed_state:	unclaimed++;	break;
 		case claimed_state	:	claimed++;		break;
@@ -383,21 +468,21 @@ update( ClassAd *ad )
 		case drained_state:		drained++;	break;
 		default				:	return false;
 	}
-
 	return 1;
 }
-		
+
+
 
 void StartdStateTotal::
 displayHeader(FILE *file)
 {
 #if HAVE_BACKFILL
-	fprintf (file, "%6.6s %5.5s %9.9s %7.7s %10.10s %7.7s %8.8s\n",
-					"Total", "Owner", "Unclaimed", "Claimed", 
-					"Preempting", "Matched", "Backfill");
+	fprintf (file, "%6.6s %5.5s %9.9s %7.7s %10.10s %7.7s %8.8s %6.6s\n",
+					"Total", "Owner", "Unclaimed", "Claimed",
+					"Preempting", "Matched", "Backfill", "Drain");
 #else
-	fprintf( file, "%10.10s %5.5s %9.9s %7.7s %10.10s %7.7s\n", "Machines", 
-				"Owner", "Unclaimed", "Claimed", "Preempting", "Matched" );
+	fprintf( file, "%10.10s %5.5s %9.9s %7.7s %10.10s %7.7s %6.6s\n", "Machines",
+				"Owner", "Unclaimed", "Claimed", "Preempting", "Matched", "Drain" );
 #endif /* HAVE_BACKFILL */
 }
 
@@ -406,11 +491,11 @@ void StartdStateTotal::
 displayInfo( FILE *file, int )
 {
 #if HAVE_BACKFILL
-	fprintf( file, "%6d %5d %9d %7d %10d %7d %8d\n", machines, owner, 
-			 unclaimed, claimed, preempt, matched, backfill );
+	fprintf( file, "%6d %5d %9d %7d %10d %7d %8d %6d\n", machines, owner, 
+			 unclaimed, claimed, preempt, matched, backfill, drained );
 #else
-	fprintf( file, "%10d %5d %9d %7d %10d %7d\n", machines, owner, 
-			 unclaimed, claimed, preempt, matched );
+	fprintf( file, "%10d %5d %9d %7d %10d %7d %6d\n", machines, owner, 
+			 unclaimed, claimed, preempt, matched, drained );
 #endif /* HAVE_BACKFILL */
 }
 
@@ -444,7 +529,7 @@ StartdCODTotal::updateTotals( ClassAd* ad, const char* id )
 }
 
 int StartdCODTotal::
-update( ClassAd *ad )
+update (ClassAd *ad, int /*options*/)
 {
 	StringList cod_claim_list;
 	char* cod_claims = NULL;
@@ -486,7 +571,7 @@ QuillNormalTotal()
 }
 
 int QuillNormalTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int /*options*/)
 {
 	int attrSqlTotal, attrSqlLastBatch;
 	bool badAd = false;
@@ -532,7 +617,7 @@ ScheddNormalTotal()
 
 
 int ScheddNormalTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int /*options*/)
 {
 	int attrRunning, attrIdle, attrHeld;;
 	bool badAd = false;
@@ -583,7 +668,7 @@ ScheddSubmittorTotal()
 
 
 int ScheddSubmittorTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int /*options*/)
 {
 	int attrRunning=0, attrIdle=0, attrHeld=0;
 	bool badAd = false;
@@ -611,14 +696,14 @@ update (ClassAd *ad)
 void ScheddSubmittorTotal::
 displayHeader(FILE *file)
 {
-	fprintf (file, "%18s %18s %18s\n", "RunningJobs", "IdleJobs", "HeldJobs");
+	fprintf (file, "%11s %10s %10s\n", "RunningJobs", "IdleJobs", "HeldJobs");
 }
 
 
 void ScheddSubmittorTotal::
 displayInfo (FILE *file, int)
 {
-	fprintf (file, "%18d %18d %18d\n", runningJobs, idleJobs, heldJobs);
+	fprintf (file, "%11d %10d %10d\n", runningJobs, idleJobs, heldJobs);
 }
 
 
@@ -630,7 +715,7 @@ CkptSrvrNormalTotal()
 }
 
 int CkptSrvrNormalTotal::
-update (ClassAd *ad)
+update (ClassAd *ad, int /*options*/)
 {
 	int attrDisk = 0;
 
@@ -688,7 +773,7 @@ makeTotalObject (ppOption ppo)
 		case PP_QUILL_NORMAL:		ct = new QuillNormalTotal; break;
 #endif /* HAVE_EXT_POSTGRESQL */
 
-		case PP_SCHEDD_SUBMITTORS:	ct = new ScheddSubmittorTotal; break;
+		case PP_SUBMITTER_NORMAL:	ct = new ScheddSubmittorTotal; break;
 		case PP_CKPT_SRVR_NORMAL:	ct = new CkptSrvrNormalTotal; break;
 
 		default:
@@ -724,7 +809,7 @@ makeKey (MyString &key, ClassAd *ad, ppOption ppo)
 			key = buf;
 			return 1;
 
-		case PP_SCHEDD_SUBMITTORS:
+		case PP_SUBMITTER_NORMAL:
 			if (!ad->LookupString(ATTR_NAME, p1, sizeof(p1))) return 0;
 			key = p1;
 			return 1;
