@@ -271,6 +271,20 @@ can_switch_ids( void )
 }
 
 
+#ifdef LINUX
+static int should_use_keyring_sessions() {
+	static int UseKeyringSessions = FALSE;
+	static int DidParamForKeyringSessions = FALSE;
+
+	if(!DidParamForKeyringSessions) {
+		UseKeyringSessions = param_boolean("USE_KEYRING_SESSIONS", false);
+		DidParamForKeyringSessions = true;
+	}
+	return UseKeyringSessions;
+}
+#endif
+
+
 /* End Common Bits */
 
 
@@ -507,6 +521,7 @@ init_user_ids(const char username[], const char domain[])
 			Daemon credd(DT_CREDD);
 			Sock * credd_sock = credd.startCommand(CREDD_GET_PASSWD,Stream::reli_sock,10);
 			if ( credd_sock ) {
+				credd_sock->set_crypto_mode(true);
 				credd_sock->put((char*)username);	// send user
 				credd_sock->put((char*)domain);		// send domain
 				credd_sock->end_of_message();
@@ -636,6 +651,7 @@ init_user_ids(const char username[], const char domain[])
 	}
 }
 
+// WINDOWS
 priv_state
 _set_priv(priv_state s, const char *file, int line, int dologging)
 {
@@ -1079,6 +1095,18 @@ static int
 set_user_ids_implementation( uid_t uid, gid_t gid, const char *username, 
 							 int is_quiet ) 
 {
+		// Don't allow changing of user ids when we're in user priv state.
+	if ( CurrentPrivState == PRIV_USER || CurrentPrivState == PRIV_USER_FINAL ) {
+		if ( uid != UserUid || gid != UserGid ) {
+			if ( !is_quiet ) {
+				dprintf( D_ALWAYS, "ERROR: Attempt to change user ids while in user privilege state\n" );
+			}
+			return FALSE;
+		} else {
+			return TRUE;
+		}
+	}
+
 	if( uid == 0 || gid == 0 ) {
 			// NOTE: we want this dprintf() even if we're in quiet
 			// mode, since we should *never* be allowing this.
@@ -1233,6 +1261,18 @@ init_user_ids_implementation( const char username[], int is_quiet )
 	uid_t 				usr_uid;
 	gid_t				usr_gid;
 
+		// Don't allow changing of user ids when we're in user priv state.
+	if ( CurrentPrivState == PRIV_USER || CurrentPrivState == PRIV_USER_FINAL ) {
+		if ( strcmp( username, UserName ) ) {
+			if ( !is_quiet ) {
+				dprintf( D_ALWAYS, "ERROR: Attempt to change user ids while in user privilege state\n" );
+			}
+			return FALSE;
+		} else {
+			return TRUE;
+		}
+	}
+
 		// So if we are not root, trying to use any user id is bogus
 		// since the OS will disallow it.  So if we are not running as
 		// root, may as well just set the user id to be the real id.
@@ -1375,9 +1415,59 @@ set_file_owner_ids( uid_t uid, gid_t gid )
 }
 
 
+// since this makes syscalls directly into linux kernel, don't compile
+// this code on non-linux unix. (e.g. OS X)
+#ifdef LINUX
+
+// Define some syscall keyring constants.  Normally these are in
+// <keyutils.h>, but we only need a few values that are not changing,
+// and by placing them here we do not require keyutils development
+// package from being installed at compile time.
+#ifndef KEYCTL_JOIN_SESSION_KEYRING   // don't redefine if keyutils.h included
+  typedef int32_t key_serial_t;
+  #define KEYCTL_JOIN_SESSION_KEYRING     1       /* join or start named session keyring */
+  #define KEYCTL_DESCRIBE                 6       /* describe a key */
+  #define KEYCTL_LINK                     8       /* link a key into a keyring */
+  #define KEYCTL_UNLINK                   9       /* unlink a key from a keyring */
+  #define KEYCTL_SEARCH                   10      /* search for a key in a keyring */
+  #define KEY_SPEC_SESSION_KEYRING        -3      /* - key ID for session-specific keyring */
+  #define KEY_SPEC_USER_KEYRING           -4      /* - key ID for UID-specific keyring */
+  #define KEY_SPEC_INVALID_KEYRING        -99     /* - key ID for "invalid keyring" */
+  #define KEY_SPEC_INVALID_UID            -1      /* - user ID for "invalid user" */
+#endif  // of ifndef KEYCTL_JOIN_SESSION_KEYRING
+
+// helper functions to avoid importing libkeyutils
+static key_serial_t condor_keyctl_session(const char* n) {
+  return syscall(__NR_keyctl, KEYCTL_JOIN_SESSION_KEYRING, n);
+}
+
+static key_serial_t condor_keyctl_search(key_serial_t k, const char* t, const char* d, key_serial_t r) {
+  return syscall(__NR_keyctl, KEYCTL_SEARCH, k, t, d, r);
+}
+
+static long condor_keyctl_link(key_serial_t k, key_serial_t r) {
+  return syscall(__NR_keyctl, KEYCTL_LINK, k, r);
+}
+
+static int   CurrentSessionKeyring = KEY_SPEC_INVALID_KEYRING;
+static uid_t CurrentSessionKeyringUID = KEY_SPEC_INVALID_UID;
+static int   PreviousSessionKeyring = KEY_SPEC_INVALID_KEYRING;
+static uid_t PreviousSessionKeyringUID = KEY_SPEC_INVALID_UID;
+
+#endif
+
+
+// UNIX
 priv_state
 _set_priv(priv_state s, const char *file, int line, int dologging)
 {
+	/* NOTE  NOTE   NOTE  NOTE
+	 * This function is called from deep inside dprintf.  To
+	 * avoid potentially nasty recursive situations, ONLY call
+	 * dprintf() from inside of this function if the
+	 * dologging parameter is non-zero.
+	 */
+
 	priv_state PrevPrivState = CurrentPrivState;
 	if (s == CurrentPrivState) return s;
 	if (CurrentPrivState == PRIV_USER_FINAL) {
@@ -1398,6 +1488,69 @@ _set_priv(priv_state s, const char *file, int line, int dologging)
 	CurrentPrivState = s;
 
 	if (can_switch_ids()) {
+		if ((s == PRIV_USER || s == PRIV_USER_FINAL) && !UserIdsInited ) {
+			EXCEPT("Programmer Error: attempted switch to user privilege, "
+				   "but user ids are not initialized");
+		}
+
+		// ultimately, to be extra-paranoid all switches need a new
+		// session (to shed any old creds).  however, that means we can
+		// no longer dprintf in any of the below code since doing so
+		// switches to condor priv, creates a new session, and messes
+		// up the whole process below.  so for now, we only do it when
+		// switching to user priv.
+
+		// We only get here if we are actually switching state. (Not if we
+		// requested to to switch to the current state, or if we requested
+		// to switch away from a _FINAL state).
+
+#ifdef LINUX
+		if(should_use_keyring_sessions()) {
+
+			// capture current priv state.  we will need to become root to link/unlink keys,
+			// create new sessions, etc.  but if we need to switch back to PRIV_UNKNOWN, or
+			// hit the 'default' case in the switch statement below, the result should be
+			// that we didn't change euid or egid.
+			uid_t saveeuid = geteuid();
+			uid_t saveegid = getegid();
+
+
+			// no matter what state we're switching to, we always create a new
+			// session.  that way we drop the user's creds if switching to condor
+			// or root priv.  we'll have our own keyring if we just recently forked
+			// as well.  and if we're switching to user priv we'll attach the AFS
+			// keyring to this new session.
+			//
+			// creating sessions is cheap and they are garbage
+			// collected very agressively so this is not a problem.
+
+
+			// create a new session
+			set_root_euid();
+			condor_keyctl_session(NULL);
+			key_serial_t sess_keyring = KEY_SPEC_SESSION_KEYRING;
+			if (dologging) dprintf(D_SECURITY, "KEYCTL: New session keyring %i\n", sess_keyring);
+
+
+			// if we were in priv user and are switching out, we record the keyring
+			// id holding the user's AFS token, since it's likely we'll switch back
+			// to it and can avoid looking it up again.
+
+			if(PrevPrivState == PRIV_USER) {
+				// update state
+				PreviousSessionKeyring = CurrentSessionKeyring;
+				PreviousSessionKeyringUID = CurrentSessionKeyringUID;
+			}
+
+			// restore us to the same euid and egid as when we were called.
+			// the if statements are there just to avoid a compiler warning
+			// about not checking the return value.
+			set_root_euid();
+			if(setegid(saveegid)) {}
+			if(seteuid(saveeuid)) {}
+		}
+#endif
+
 		switch (s) {
 		case PRIV_ROOT:
 			set_root_euid();
@@ -1409,20 +1562,83 @@ _set_priv(priv_state s, const char *file, int line, int dologging)
 			set_condor_euid();
 			break;
 		case PRIV_USER:
-			set_root_euid();	/* must be root to switch */
-			set_user_egid();
-			set_user_euid();
+		case PRIV_USER_FINAL:
+// we can only use linux-specific kernel keyrings
+#ifdef LINUX
+			if(should_use_keyring_sessions()) {
+
+				// First, see if we can simply attach the previously-used keyring
+				if(UserUid != PreviousSessionKeyringUID) {
+					set_root_euid();
+					// no. lookup the new one
+
+					// locate the master keyring.
+					// KEY_SPEC_USER_KEYRING is the uid keyring for root.
+					key_serial_t htcondor_keyring = KEY_SPEC_USER_KEYRING;
+
+					// create the keyring name for user keyring
+					MyString ring_name = "htcondor_uid";
+					ring_name = ring_name + UserUid;
+
+					// locate the user keyring
+					key_serial_t user_keyring = condor_keyctl_search(htcondor_keyring, "keyring", ring_name.Value(), 0);
+					if(user_keyring == -1) {
+						CurrentSessionKeyring = KEY_SPEC_INVALID_KEYRING;
+						CurrentSessionKeyringUID = KEY_SPEC_INVALID_UID;
+						if (dologging) dprintf(D_ALWAYS,
+							"KEYCTL: unable to find keyring '%s', error: %s\n",
+							ring_name.Value(), strerror(errno));
+					} else {
+						CurrentSessionKeyring = user_keyring;
+						CurrentSessionKeyringUID = UserUid;
+						if (dologging) dprintf(D_SECURITY,
+							 "KEYCTL: found user keyring %s (%li) for uid %i.\n",
+							 ring_name.Value(), (long)user_keyring, UserUid);
+					}
+				} else {
+					CurrentSessionKeyring = PreviousSessionKeyring;
+					CurrentSessionKeyringUID = PreviousSessionKeyringUID;
+					if (dologging) dprintf(D_SECURITY, "KEYCTL: resuming stored keyring %i and uid %i.\n",
+						CurrentSessionKeyring, CurrentSessionKeyringUID);
+				}
+
+
+				// only link if there's something to link to
+				if(CurrentSessionKeyringUID != (uid_t)KEY_SPEC_INVALID_UID) {
+					set_root_euid();
+					// locate the session keyring and uid 0 keyring.
+					key_serial_t user_keyring = CurrentSessionKeyring;
+					key_serial_t sess_keyring = KEY_SPEC_SESSION_KEYRING;
+
+					// link the user keyring to our session keyring
+					long link_success = condor_keyctl_link(user_keyring, sess_keyring);
+					if(link_success == -1) {
+						if (dologging) dprintf(D_ALWAYS, "KEYCTL: link(%li,%li) error: %s\n",
+							(long)user_keyring, (long)sess_keyring, strerror(errno));
+					} else {
+						if (dologging) dprintf(D_SECURITY, "KEYCTL: linked key %li to %li\n",
+							(long)user_keyring, (long)sess_keyring);
+					}
+				}
+			}
+#endif // LINUX
+
+			set_root_euid();
+			if(s == PRIV_USER) {
+				// PRIV_USER: change effective
+				set_user_egid();
+				set_user_euid();
+			} else {
+				// PRIV_USER_FINAL: change real
+				set_user_rgid();
+				set_user_ruid();
+			}
 			break;
 		case PRIV_FILE_OWNER:
 			set_root_euid();	/* must be root to switch */
 			set_owner_egid();
 			set_owner_euid();
 			break;
-		case PRIV_USER_FINAL:
-			set_root_euid();	/* must be root to switch */
-			set_user_rgid();
-			set_user_ruid();
-			break;
 		case PRIV_CONDOR_FINAL:
 			set_root_euid();	/* must be root to switch */
 			set_condor_rgid();
