@@ -399,9 +399,9 @@ void UserPolicy::Init()
 
 void UserPolicy::ClearConfig()
 {
-	if (m_sys_periodic_hold) delete m_sys_periodic_hold; m_sys_periodic_hold = NULL;
-	if (m_sys_periodic_release) delete m_sys_periodic_release; m_sys_periodic_release = NULL;
-	if (m_sys_periodic_remove) delete m_sys_periodic_remove; m_sys_periodic_remove = NULL;
+	delete m_sys_periodic_hold; m_sys_periodic_hold = NULL;
+	delete m_sys_periodic_release; m_sys_periodic_release = NULL;
+	delete m_sys_periodic_remove; m_sys_periodic_remove = NULL;
 }
 
 
@@ -527,6 +527,7 @@ UserPolicy::AnalyzePolicy( int mode )
 		// Clear out our stateful variables
 	m_fire_expr = NULL;
 	m_fire_expr_val = -1;
+	m_fire_unparsed_expr.clear();
 
 	/*	The user_policy is checked in this
 			order. The first one to succeed is the winner:
