@@ -2,13 +2,13 @@
  *
  * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
  * University of Wisconsin-Madison, WI.
- * 
+ *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
  * may not use this file except in compliance with the License.  You may
  * obtain a copy of the License at
- * 
+ *
  *    http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -23,48 +23,32 @@
 #include "condor_common.h"
 #include <string>
 
-/**
- *
- * This class encapsulates a DaemonCore pipe and provides buffering.
- * E.g. it allows you to hold a result read out of the pipe until
- * a full line is read. The getNextLine() and Write() methods are
- * designed to be used in read pipe and write pipe handlers,
- * respectively. Note that an object of this type is to be used
- * either strictly for reading (using getNextLine()) or strictly
- * for writing (using Write()).
- *
- **/
-
-const int PIPE_BUFFER_READAHEAD_SIZE = 1024;
+const int PIPE_BUFFER_READAHEAD_SIZE = 16384;
 
 class PipeBuffer {
 public:
-	PipeBuffer ();
-	PipeBuffer (int pipe_end);
+	PipeBuffer();
+	PipeBuffer( int pipe_end );
 
-	std::string * GetNextLine();
-	int Write (const char * toWrite = NULL);
+	bool GetNextLine( char * buffer, int length );
 
 	int getPipeEnd() { return pipe_end; }
-	void setPipeEnd(const int _pipe_end) { pipe_end = _pipe_end; }
-	const char * getBuffer() { return buffer.c_str(); }
+	void setPipeEnd( const int _pipe_end ) { pipe_end = _pipe_end; }
 
-	int IsEmpty();
 	bool IsError() { return error; }
 	bool IsEOF() { return eof; }
 
 protected:
 	int pipe_end;
-	std::string buffer;
 
 	bool error;
 	bool eof;
 
 	bool last_char_was_escape;
 
-	char readahead_buffer[PIPE_BUFFER_READAHEAD_SIZE];
-	int readahead_length;
-	int readahead_index;
+	int readBufferSize;
+	int readBufferIndex;
+	char readBuffer[ PIPE_BUFFER_READAHEAD_SIZE ];
 };
 
 #endif
