@@ -46,15 +46,19 @@
 #define AMAZON_COMMAND_VM_REGISTER_IMAGE    "EC2_VM_REGISTER_IMAGE"
 #define AMAZON_COMMAND_VM_DEREGISTER_IMAGE  "EC2_VM_DEREGISTER_IMAGE"
 #define AMAZON_COMMAND_VM_ASSOCIATE_ADDRESS "EC2_VM_ASSOCIATE_ADDRESS"
-//#define AMAZON_COMMAND_VM_DISASSOCIATE_ADDRESS   "EC2_VM_DISASSOCIATE_ADDRESS"
-#define AMAZON_COMMAND_VM_ATTACH_VOLUME		"EC_VM_ATTACH_VOLUME"
-#define AMAZON_COMMAND_VM_CREATE_TAGS		"EC2_VM_CREATE_TAGS"
+#define AMAZON_COMMAND_VM_ATTACH_VOLUME     "EC2_VM_ATTACH_VOLUME"
+#define AMAZON_COMMAND_VM_CREATE_TAGS       "EC2_VM_CREATE_TAGS"
+#define AMAZON_COMMAND_VM_SERVER_TYPE       "EC2_VM_SERVER_TYPE"
 
 #define AMAZON_COMMAND_VM_START_SPOT        "EC2_VM_START_SPOT"
 #define AMAZON_COMMAND_VM_STOP_SPOT         "EC2_VM_STOP_SPOT"
 #define AMAZON_COMMAND_VM_STATUS_SPOT       "EC2_VM_STATUS_SPOT"
 #define AMAZON_COMMAND_VM_STATUS_ALL_SPOT   "EC2_VM_STATUS_ALL_SPOT"
 
+#define AMAZON_COMMAND_BULK_START           "EC2_BULK_START"
+#define AMAZON_COMMAND_PUT_RULE             "EC2_PUT_RULE"
+#define AMAZON_COMMAND_PUT_TARGETS          "EC2_PUT_TARGETS"
+
 // S3 Commands
 #define AMAZON_COMMAND_S3_ALL_BUCKETS       "AMAZON_S3_ALL_BUCKETS"
 #define AMAZON_COMMAND_S3_CREATE_BUCKET     "AMAZON_S3_CREATE_BUCKET"
@@ -71,30 +75,68 @@
 
 class AmazonRequest {
     public:
-        AmazonRequest();
+        AmazonRequest( int i, const char * c, int sv = 2 ) :
+            includeResponseHeader(false), requestID(i), requestCommand(c),
+            signatureVersion(sv) { }
         virtual ~AmazonRequest();
 
         virtual bool SendRequest();
+        virtual bool SendJSONRequest( const std::string & payload );
 
-    protected:    
+    protected:
         typedef std::map< std::string, std::string > AttributeValueMap;
         AttributeValueMap query_parameters;
-        
+        AttributeValueMap headers;
+
         std::string serviceURL;
         std::string accessKeyFile;
         std::string secretKeyFile;
-        
+
         std::string errorMessage;
         std::string errorCode;
-        
+
         std::string resultString;
+
+		bool includeResponseHeader;
+
+		// For tracing.
+		int requestID;
+		std::string requestCommand;
+		struct timespec mutexReleased;
+		struct timespec lockGained;
+		struct timespec requestBegan;
+		struct timespec requestEnded;
+		struct timespec mutexGained;
+		struct timespec sleepBegan;
+		struct timespec liveLine;
+		struct timespec sleepEnded;
+
+		// So that we don't bother to send expired signatures.
+		struct timespec signatureTime;
+
+		int signatureVersion;
+
+		// For signature v4, in case we guess wrong (based on the URL).
+		std::string region;
+		std::string service;
+
+	private:
+		bool sendV2Request();
+		bool sendV4Request();
+
+		void canonicalizeQueryString( std::string & canonicalQueryString );
+		bool createV4Signature( const std::string & payload, std::string & authorizationHeader );
+
+		bool sendPreparedRequest(	const std::string & protocol,
+									const std::string & uri,
+									const std::string & payload );
 };
 
 // EC2 Commands
 
 class AmazonVMStart : public AmazonRequest {
 	public:
-		AmazonVMStart();
+		AmazonVMStart( int i, const char * c ) : AmazonRequest( i, c ) { }
 		virtual ~AmazonVMStart();
 
         virtual bool SendRequest();
@@ -108,11 +150,11 @@ class AmazonVMStart : public AmazonRequest {
 
 class AmazonVMStartSpot : public AmazonVMStart {
     public:
-        AmazonVMStartSpot();
+		AmazonVMStartSpot( int i, const char * c ) : AmazonVMStart( i, c ) { }
         virtual ~AmazonVMStartSpot();
-        
+
         virtual bool SendRequest();
-        
+
         static bool ioCheck( char ** argv, int argc );
         static bool workerFunction( char ** argv, int argc, std::string & result_string );
 
@@ -122,7 +164,7 @@ class AmazonVMStartSpot : public AmazonVMStart {
 
 class AmazonVMStop : public AmazonRequest {
 	public:
-		AmazonVMStop();
+		AmazonVMStop( int i, const char * c ) : AmazonRequest( i, c ) { }
 		virtual ~AmazonVMStop();
 
 		static bool ioCheck(char **argv, int argc);
@@ -131,9 +173,9 @@ class AmazonVMStop : public AmazonRequest {
 
 class AmazonVMStopSpot : public AmazonVMStop {
     public:
-        AmazonVMStopSpot();
+		AmazonVMStopSpot( int i, const char * c ) : AmazonVMStop( i, c ) { }
         virtual ~AmazonVMStopSpot();
-        
+
         // EC2_VM_STOP_SPOT uses the same argument structure as EC2_VM_STOP.
 		// static bool ioCheck( char ** argv, int argc );
 		static bool workerFunction( char ** argv, int argc, std::string & result_string );
@@ -161,7 +203,7 @@ class AmazonStatusResult {
 
 class AmazonVMStatusAll : public AmazonRequest {
 	public:
-		AmazonVMStatusAll();
+		AmazonVMStatusAll( int i, const char * c ) : AmazonRequest( i, c ) { }
 		virtual ~AmazonVMStatusAll();
 
         virtual bool SendRequest();
@@ -175,7 +217,7 @@ class AmazonVMStatusAll : public AmazonRequest {
 
 class AmazonVMStatus : public AmazonVMStatusAll {
 	public:
-		AmazonVMStatus();
+		AmazonVMStatus( int i, const char * c ) : AmazonVMStatusAll( i, c ) { }
 		virtual ~AmazonVMStatus();
 
 		static bool ioCheck(char **argv, int argc);
@@ -193,11 +235,11 @@ class AmazonStatusSpotResult {
 
 class AmazonVMStatusSpot : public AmazonVMStatus {
     public:
-        AmazonVMStatusSpot();
+		AmazonVMStatusSpot( int i, const char * c ) : AmazonVMStatus( i, c ) { }
         virtual ~AmazonVMStatusSpot();
 
         virtual bool SendRequest();
-        
+
         // EC2_VM_STATUS_SPOT uses the same argument structure as EC2_VM_STATUS_SPOT.
 		// static bool ioCheck( char ** argv, int argc );
 		static bool workerFunction( char ** argv, int argc, std::string & result_string );
@@ -208,25 +250,16 @@ class AmazonVMStatusSpot : public AmazonVMStatus {
 
 class AmazonVMStatusAllSpot : public AmazonVMStatusSpot {
     public:
-        AmazonVMStatusAllSpot();
+		AmazonVMStatusAllSpot( int i, const char * c ) : AmazonVMStatusSpot( i, c ) { }
         virtual ~AmazonVMStatusAllSpot();
 
 		static bool ioCheck( char ** argv, int argc );
 		static bool workerFunction( char ** argv, int argc, std::string & result_string );
 };
 
-class AmazonVMRunningKeypair : public AmazonVMStatusAll {
-	public:
-		AmazonVMRunningKeypair();
-		virtual ~AmazonVMRunningKeypair();
-
-		static bool ioCheck(char **argv, int argc);
-		static bool workerFunction(char **argv, int argc, std::string &result_string);
-};
-
 class AmazonVMCreateKeypair : public AmazonRequest {
 	public:
-		AmazonVMCreateKeypair();
+		AmazonVMCreateKeypair( int i, const char * c ) : AmazonRequest( i, c ) { }
 		virtual ~AmazonVMCreateKeypair();
 
         virtual bool SendRequest();
@@ -240,30 +273,16 @@ class AmazonVMCreateKeypair : public AmazonRequest {
 
 class AmazonVMDestroyKeypair : public AmazonRequest {
 	public:
-		AmazonVMDestroyKeypair();
+		AmazonVMDestroyKeypair( int i, const char * c ) : AmazonRequest( i, c ) { }
 		virtual ~AmazonVMDestroyKeypair();
 
 		static bool ioCheck(char **argv, int argc);
 		static bool workerFunction(char **argv, int argc, std::string &result_string);
 };
 
-class AmazonVMKeypairNames : public AmazonRequest {
-	public:
-		AmazonVMKeypairNames();
-		virtual ~AmazonVMKeypairNames();
-
-        virtual bool SendRequest();
-
-		static bool ioCheck(char **argv, int argc);
-		static bool workerFunction(char **argv, int argc, std::string &result_string);
-
-    protected:
-        StringList keyNames;
-};
-
 class AmazonAssociateAddress : public AmazonRequest {
     public:
-        AmazonAssociateAddress();
+		AmazonAssociateAddress( int i, const char * c ) : AmazonRequest( i, c ) { }
         virtual ~AmazonAssociateAddress();
 
         static bool ioCheck(char **argv, int argc);
@@ -272,7 +291,7 @@ class AmazonAssociateAddress : public AmazonRequest {
 
 class AmazonCreateTags : public AmazonRequest {
     public:
-        AmazonCreateTags();
+		AmazonCreateTags( int i, const char * c ) : AmazonRequest( i, c ) { }
         virtual ~AmazonCreateTags();
 
         static bool ioCheck(char **argv, int argc);
@@ -285,7 +304,7 @@ class AmazonCreateTags : public AmazonRequest {
  */
 class AmazonAttachVolume : public AmazonRequest {
     public:
-        AmazonAttachVolume();
+        AmazonAttachVolume( int i, const char * c ) : AmazonRequest( i, c ) { }
         virtual ~AmazonAttachVolume();
 
         static bool ioCheck(char **argv, int argc);
@@ -293,4 +312,60 @@ class AmazonAttachVolume : public AmazonRequest {
 };
 
 
+class AmazonVMServerType : public AmazonRequest {
+	public:
+        AmazonVMServerType( int i, const char * c ) : AmazonRequest( i, c ) { }
+		virtual ~AmazonVMServerType();
+
+		virtual bool SendRequest();
+
+		static bool ioCheck(char **argv, int argc);
+		static bool workerFunction(char **argv, int argc, std::string &result_string);
+
+	protected:
+		std::string serverType;
+};
+
+// Spot Fleet commands
+class AmazonBulkStart : public AmazonRequest {
+	public:
+		AmazonBulkStart( int i, const char * c ) : AmazonRequest( i, c ) { }
+		virtual ~AmazonBulkStart();
+
+        virtual bool SendRequest();
+
+		static bool ioCheck(char **argv, int argc);
+		static bool workerFunction(char **argv, int argc, std::string &result_string);
+
+    protected:
+    	void setLaunchSpecificationAttribute( int, std::map< std::string, std::string > &, const char *, const char * = NULL );
+
+		std::string bulkRequestID;
+};
+
+class AmazonPutRule : public AmazonRequest {
+	public:
+		AmazonPutRule( int i, const char * c ) : AmazonRequest( i, c ) { }
+		virtual ~AmazonPutRule();
+
+		virtual bool SendJSONRequest( const std::string & payload );
+
+		static bool ioCheck(char **argv, int argc);
+		static bool workerFunction(char **argv, int argc, std::string &result_string);
+
+    protected:
+		std::string ruleARN;
+};
+
+class AmazonPutTargets : public AmazonRequest {
+	public:
+		AmazonPutTargets( int i, const char * c ) : AmazonRequest( i, c ) { }
+		virtual ~AmazonPutTargets();
+
+		virtual bool SendJSONRequest( const std::string & payload );
+
+		static bool ioCheck(char **argv, int argc);
+		static bool workerFunction(char **argv, int argc, std::string &result_string);
+};
+
 #endif
