@@ -33,6 +33,12 @@
 #include "amazonCommands.h"
 #include "subsystem_info.h"
 
+// Globals from amazonCommand.cpp's statistitcs.
+extern int NumRequests;
+extern int NumDistinctRequests;
+extern int NumRequestsExceedingLimit;
+extern int NumExpiredSignatures;
+
 #define MIN_WORKER_NUM 1
 #define AMAZON_GAHP_VERSION	"1.0"
 
@@ -113,15 +119,23 @@ registerAllAmazonCommands(void)
     registerAmazonGahpCommand(AMAZON_COMMAND_VM_ASSOCIATE_ADDRESS,
             AmazonAssociateAddress::ioCheck, AmazonAssociateAddress::workerFunction);
 
-	 registerAmazonGahpCommand(AMAZON_COMMAND_VM_ATTACH_VOLUME,
+	registerAmazonGahpCommand(AMAZON_COMMAND_VM_ATTACH_VOLUME,
             AmazonAttachVolume::ioCheck, AmazonAttachVolume::workerFunction);
 
-	 registerAmazonGahpCommand(AMAZON_COMMAND_VM_CREATE_TAGS,
+	registerAmazonGahpCommand(AMAZON_COMMAND_VM_CREATE_TAGS,
             AmazonCreateTags::ioCheck, AmazonCreateTags::workerFunction);
 
-	 registerAmazonGahpCommand(AMAZON_COMMAND_VM_SERVER_TYPE,
+	registerAmazonGahpCommand(AMAZON_COMMAND_VM_SERVER_TYPE,
             AmazonVMServerType::ioCheck, AmazonVMServerType::workerFunction);
 
+	// Spot Fleet commands
+	registerAmazonGahpCommand( AMAZON_COMMAND_BULK_START,
+			AmazonBulkStart::ioCheck, AmazonBulkStart::workerFunction );
+	registerAmazonGahpCommand( AMAZON_COMMAND_PUT_RULE,
+			AmazonPutRule::ioCheck, AmazonPutRule::workerFunction );
+	registerAmazonGahpCommand( AMAZON_COMMAND_PUT_TARGETS,
+			AmazonPutTargets::ioCheck, AmazonPutTargets::workerFunction );
+
 	return true;
 }
 
@@ -161,19 +175,18 @@ main( int argc, char ** const argv )
 
 	set_mySubSystem("AMAZON_GAHP", SUBSYSTEM_TYPE_GAHP);
 
-    config();
-    dprintf_config( "EC2_GAHP" );
-    const char * debug_string = getenv( "DebugLevel" );
-    if( debug_string && * debug_string ) {
-        set_debug_flags( debug_string, 0 );
-    }
-
 	int min_workers = MIN_NUMBER_WORKERS;
 	int max_workers = -1;
+	const char * dprintfName = "EC2_GAHP";
 
 	int c = 0;
-	while ( (c = my_getopt(argc, argv, "f:d:w:m:" )) != -1 ) {
+	while ( (c = my_getopt(argc, argv, "l:f:d:w:m:" )) != -1 ) {
 		switch(c) {
+			case 'l':
+				if( my_optarg && *my_optarg ) {
+					dprintfName = my_optarg;
+				}
+				break;
 			case 'f':
 				break;
 			case 'd':
@@ -201,6 +214,13 @@ main( int argc, char ** const argv )
 		}
 	}
 
+    config();
+    dprintf_config( dprintfName );
+    const char * debug_string = getenv( "DebugLevel" );
+    if( debug_string && * debug_string ) {
+        set_debug_flags( debug_string, 0 );
+    }
+
 	dprintf(D_FULLDEBUG, "Welcome to the EC2 GAHP\n");
 
 	const char *buff;
@@ -233,7 +253,7 @@ main( int argc, char ** const argv )
 	printf ("%s\n", version);
 	fflush(stdout);
 
-	dprintf (D_FULLDEBUG, "EC2 GAHP initialized\n");
+	dprintf( D_ALWAYS, "EC2 GAHP initialized\n" );
 
 		/* Our main thread should grab the mutex first.  We will then
 		 * release it and let other threads run when we would otherwise
@@ -254,6 +274,7 @@ static int
 verify_gahp_command(char ** argv, int argc) {
 	// Special Commands First
 	if (strcasecmp (argv[0], GAHP_COMMAND_RESULTS) == 0 ||
+			strcasecmp (argv[0], GAHP_COMMAND_STATISTICS) == 0 ||
 			strcasecmp (argv[0], GAHP_COMMAND_VERSION) == 0 ||
 			strcasecmp (argv[0], GAHP_COMMAND_COMMANDS) == 0 ||
 			strcasecmp (argv[0], GAHP_COMMAND_QUIT) == 0 ||
@@ -418,6 +439,12 @@ IOProcess::stdinPipeHandler()
 			} else if (strcasecmp (args.argv[0], GAHP_COMMAND_ASYNC_MODE_OFF) == 0) {
 				m_async_mode = false;
 				gahp_output_return_success();
+			} else if (strcasecmp (args.argv[0], GAHP_COMMAND_STATISTICS) == 0) {
+				printf( "%s %d %d %d %d\n", GAHP_RESULT_SUCCESS,
+					NumRequests, NumDistinctRequests,
+					NumRequestsExceedingLimit, NumExpiredSignatures
+				);
+				fflush( stdout );
 			} else if (strcasecmp (args.argv[0], GAHP_COMMAND_COMMANDS) == 0) {
 				StringList amazon_commands;
 				int num_commands = 0;
@@ -574,6 +601,8 @@ IOProcess::addNewRequest(const char *cmd)
 
 	Request* new_req = new Request(cmd);
 	ASSERT(new_req);
+	dprintf( D_PERF_TRACE, "request #%d (%s): dispatch\n",
+		new_req->m_reqid, new_req->m_args.argv[0] );
 
 	// check if there is available worker process
 	Worker *worker = findFreeWorker();
