@@ -349,10 +349,12 @@ bool GetAccessToken( const string &auth_file, string &access_token,
 		string client_id;
 		string client_secret;
 		string refresh_token;
+		string access_token;
 		string key;
 		string value;
 		int nesting = 0;
 		int expires_in = 0;
+		long int expiration_timestamp = 0;
 		const char *pos;
 		if ( !readShortFile( auth_file, auth_file_contents ) ) {
 			auth_entry.m_err_msg = "Failed to read auth file";
@@ -367,10 +369,33 @@ bool GetAccessToken( const string &auth_file, string &access_token,
 				client_id = value;
 			} else if ( key == "client_secret" ) {
 				client_secret = value;
+			} else if ( key == "access_token" ) {
+				access_token = value;
+			} else if ( key == "expires_in" ) {
+				expires_in = atoi(value.c_str());
+			} else if ( key == "token_expiry") {
+				// Parse token expiry time in (UTC) format
+				// ""2016-07-27T18:44:12Z"
+				struct tm expiration_tm;
+				if ( strptime(value.c_str(), "%Y-%m-%dT%TZ", &expiration_tm ) != NULL) {
+					expiration_timestamp = timegm(&expiration_tm);
+				}
 			}
 		}
 		if ( refresh_token.empty() ) {
-			auth_entry.m_err_msg = "Failed to find refresh_token in auth file";
+			if ( expiration_timestamp > 0 ) {
+				expires_in = expiration_timestamp - time(NULL);
+			}
+			if ( access_token.empty() ) {
+				auth_entry.m_err_msg = "Failed to find refresh_token or access_token in auth file";
+				goto done;
+			}
+			if ( expires_in <= 0 ) {
+				auth_entry.m_err_msg = "Access_token is expired, please re-login.";
+				goto done;
+			}
+			auth_entry.m_access_token = access_token;
+			auth_entry.m_expiration = time(NULL) + expires_in;
 			goto done;
 		}
 		if ( client_id.empty() ) {
@@ -786,18 +811,19 @@ GceInstanceInsert::GceInstanceInsert() { }
 
 GceInstanceInsert::~GceInstanceInsert() { }
 
-// Expecting:GCE_INSTACE_INSERT <req_id> <serviceurl> <authfile> <project> <zone>
+// Expecting:GCE_INSTANCE_INSERT <req_id> <serviceurl> <authfile> <project> <zone>
 //     <instance_name> <machine_type> <image> <metadata> <metadata_file>
+//     <preemptible> <json_file>
 bool GceInstanceInsert::workerFunction(char **argv, int argc, string &result_string) {
 	assert( strcasecmp( argv[0], "GCE_INSTANCE_INSERT" ) == 0 );
 
 	int requestID;
 	get_int( argv[1], & requestID );
 
-	if( ! verify_number_args( argc, 11 ) ) {
+	if( ! verify_number_args( argc, 13 ) ) {
 		result_string = create_failure_result( requestID, "Wrong_Argument_Number" );
 		dprintf( D_ALWAYS, "Wrong number of arguments (%d should be >= %d) to %s\n",
-				 argc, 6, argv[0] );
+				 argc, 13, argv[0] );
 		return false;
 	}
 
@@ -833,6 +859,13 @@ bool GceInstanceInsert::workerFunction(char **argv, int argc, string &result_str
 			metadata[key] = value;
 		}
 	}
+	string json_file_contents;
+	if ( strcasecmp( argv[12], NULLSTRING ) ) {
+		if ( !readShortFile( argv[12], json_file_contents ) ) {
+			result_string = create_failure_result( requestID, "Failed to open additional JSON file" );
+			return true;
+		}
+	}
 
 	insert_request.requestBody = "{\n";
 	insert_request.requestBody += " \"machineType\": \"";
@@ -841,6 +874,11 @@ bool GceInstanceInsert::workerFunction(char **argv, int argc, string &result_str
 	insert_request.requestBody += " \"name\": \"";
 	insert_request.requestBody += argv[6];
 	insert_request.requestBody += "\",\n";
+	insert_request.requestBody += "  \"scheduling\":\n";
+	insert_request.requestBody += "  {\n";
+	insert_request.requestBody += "    \"preemptible\": ";
+	insert_request.requestBody += argv[11];
+	insert_request.requestBody += "\n  },\n";
 	insert_request.requestBody += " \"disks\": [\n  {\n";
 	insert_request.requestBody += "   \"boot\": true,\n";
 	insert_request.requestBody += "   \"autoDelete\": true,\n";
@@ -878,8 +916,33 @@ bool GceInstanceInsert::workerFunction(char **argv, int argc, string &result_str
 	insert_request.requestBody += "     \"name\": \"External NAT\",\n";
 	insert_request.requestBody += "     \"type\": \"ONE_TO_ONE_NAT\"\n";
 	insert_request.requestBody += "    }\n   ]\n";
-	insert_request.requestBody += "  }\n ]\n";
-	insert_request.requestBody += "}\n";
+	insert_request.requestBody += "  }\n ]";
+	insert_request.requestBody += "\n}\n";
+
+	if ( !json_file_contents.empty() ) {
+		classad::ClassAd instance_ad;
+		classad::ClassAd custom_ad;
+		classad::ClassAdJsonParser parser;
+		classad::ClassAdJsonUnParser unparser;
+		string wrap_custom_attrs = "{" + json_file_contents + "}";
+
+		if ( !parser.ParseClassAd( insert_request.requestBody, instance_ad, true ) ) {
+			result_string = create_failure_result( requestID,
+									"Failed to parse instance description" );
+			return true;
+		}
+
+		if ( !parser.ParseClassAd( wrap_custom_attrs, custom_ad, true ) ) {
+			result_string = create_failure_result( requestID,
+									"Failed to parse custom attributes" );
+			return true;
+		}
+
+		instance_ad.Update( custom_ad );
+
+		insert_request.requestBody.clear();
+		unparser.Unparse( insert_request.requestBody, &instance_ad );
+	}
 
 	string auth_file = argv[3];
 	if ( !GetAccessToken( auth_file, insert_request.accessToken,
@@ -1115,6 +1178,8 @@ bool GceInstanceList::workerFunction(char **argv, int argc, string &result_strin
 	assert( strcasecmp( argv[0], "GCE_INSTANCE_LIST" ) == 0 );
 
 	int requestID;
+	string next_page_token;
+	vector<string> results;
 	get_int( argv[1], & requestID );
 
 	if( ! verify_number_args( argc, 6 ) ) {
@@ -1124,87 +1189,98 @@ bool GceInstanceList::workerFunction(char **argv, int argc, string &result_strin
 		return false;
 	}
 
-	// Fill in required attributes & parameters.
-	GceInstanceList list_request;
-	list_request.serviceURL = argv[2];
-	list_request.serviceURL += "/projects/";
-	list_request.serviceURL += argv[4];
-	list_request.serviceURL += "/zones/";
-	list_request.serviceURL += argv[5];
-	list_request.serviceURL += "/instances";
-	list_request.requestMethod = "GET";
+	while( true ) {
+		// Fill in required attributes & parameters.
+		GceInstanceList list_request;
+		list_request.serviceURL = argv[2];
+		list_request.serviceURL += "/projects/";
+		list_request.serviceURL += argv[4];
+		list_request.serviceURL += "/zones/";
+		list_request.serviceURL += argv[5];
+		list_request.serviceURL += "/instances";
+		if (!next_page_token.empty() ) {
+			dprintf( D_ALWAYS, "Requesting page token %s\n", next_page_token.c_str() );
+			list_request.serviceURL += "?pageToken=";
+			list_request.serviceURL += next_page_token;
+			next_page_token="";
+		}
 
-	string auth_file = argv[3];
-	if ( !GetAccessToken( auth_file, list_request.accessToken,
-						  list_request.errorMessage ) ) {
-		result_string = create_failure_result( requestID,
-											   list_request.errorMessage.c_str() );
-		return true;
-	}
+		list_request.requestMethod = "GET";
 
-	// Send the request.
-	if( ! list_request.SendRequest() ) {
-		// TODO Fix construction of error message
-		result_string = create_failure_result( requestID,
-							list_request.errorMessage.c_str(),
-							list_request.errorCode.c_str() );
-	} else {
-		string next_id;
-		string next_name;
-		string next_status;
-		string next_status_msg;
-		vector<string> results;
-		string key;
-		string value;
-		int nesting = 0;
+		string auth_file = argv[3];
+		if ( !GetAccessToken( auth_file, list_request.accessToken,
+							  list_request.errorMessage ) ) {
+			result_string = create_failure_result( requestID,
+												   list_request.errorMessage.c_str() );
+			return true;
+		}
 
-		const char *pos = list_request.resultString.c_str();
-		while( ParseJSONLine( pos, key, value, nesting ) ) {
-			if ( nesting != 3 ) {
-				continue;
+		// Send the request.
+		if( ! list_request.SendRequest() ) {
+			// TODO Fix construction of error message
+			result_string = create_failure_result( requestID,
+								list_request.errorMessage.c_str(),
+								list_request.errorCode.c_str() );
+		} else {
+			string next_id;
+			string next_name;
+			string next_status;
+			string next_status_msg;
+			string key;
+			string value;
+			int nesting = 0;
+
+			const char *pos = list_request.resultString.c_str();
+			while( ParseJSONLine( pos, key, value, nesting ) ) {
+				if ( key == "nextPageToken") {
+					next_page_token = value;
+					dprintf( D_ALWAYS, "Found page token %s\n", next_page_token.c_str() );
+				}
+				if ( nesting != 3 ) {
+					continue;
+				}
+				if ( key == "id" ) {
+					if ( !next_id.empty() ) {
+						AddInstanceToResult( results, next_id, next_name,
+											 next_status, next_status_msg );
+					}
+					next_id = value;
+				} else if ( key == "name" ) {
+					if ( !next_name.empty() ) {
+						AddInstanceToResult( results, next_id, next_name,
+											 next_status, next_status_msg );
+					}
+					next_name = value;
+				} else if ( key == "status" ) {
+					if ( !next_status.empty() ) {
+						AddInstanceToResult( results, next_id, next_name,
+											 next_status, next_status_msg );
+					}
+					next_status = value;
+				} else if ( key == "statusMessage" ) {
+					if ( !next_status_msg.empty() ) {
+						AddInstanceToResult( results, next_id, next_name,
+											 next_status, next_status_msg );
+					}
+					next_status_msg = value;
+				}
 			}
-			if ( key == "id" ) {
-				if ( !next_id.empty() ) {
-					AddInstanceToResult( results, next_id, next_name,
-										 next_status, next_status_msg );
-				}
-				next_id = value;
-			} else if ( key == "name" ) {
-				if ( !next_name.empty() ) {
-					AddInstanceToResult( results, next_id, next_name,
-										 next_status, next_status_msg );
-				}
-				next_name = value;
-			} else if ( key == "status" ) {
-				if ( !next_status.empty() ) {
-					AddInstanceToResult( results, next_id, next_name,
-										 next_status, next_status_msg );
-				}
-				next_status = value;
-			} else if ( key == "statusMessage" ) {
-				if ( !next_status_msg.empty() ) {
-					AddInstanceToResult( results, next_id, next_name,
-										 next_status, next_status_msg );
-				}
-				next_status_msg = value;
+			if ( !next_name.empty() ) {
+				AddInstanceToResult( results, next_id, next_name,
+									 next_status, next_status_msg );
 			}
 		}
-		if ( !next_name.empty() ) {
-			AddInstanceToResult( results, next_id, next_name,
-								 next_status, next_status_msg );
+		if (next_page_token.empty()) {
+			char buff[16];
+			sprintf( buff, "%d", (int)(results.size() / 4) );
+
+			StringList response;
+			response.append( buff );
+			for ( vector<string>::iterator idx = results.begin(); idx != results.end(); idx++ ) {
+				response.append( idx->c_str() );
+			}
+			result_string = create_success_result( requestID, &response );
+			return true;
 		}
-
-		char buff[16];
-		sprintf( buff, "%d", (int)(results.size() / 4) );
-
-		StringList response;
-		response.append( buff );
-		for ( vector<string>::iterator idx = results.begin(); idx != results.end(); idx++ ) {
-			response.append( idx->c_str() );
-		}
-
-		result_string = create_success_result( requestID, &response );
 	}
-
-	return true;
 }
