@@ -70,6 +70,79 @@ bool ExprTreeHolder::ShouldEvaluate() const
            m_expr->GetKind() == classad::ExprTree::EXPR_LIST_NODE;
 }
 
+long long ExprTreeHolder::toLong() const
+{
+    classad::Value val;
+    const classad::ClassAd *origParent = m_expr->GetParentScope();
+    bool evalresult;
+    if (origParent) {
+        evalresult = m_expr->Evaluate(val);
+    } else {
+        classad::EvalState state;
+        evalresult = m_expr->Evaluate(state, val);
+    }
+    if (PyErr_Occurred()) {boost::python::throw_error_already_set();}
+    if (!evalresult)
+    {
+        THROW_EX(TypeError, "Unable to evaluate expression");
+    }
+    long long retInt;
+    std::string retStr;
+    if (val.IsNumber(retInt)) {return retInt;}
+    else if (val.IsStringValue(retStr)) {
+        errno = 0;
+        char *endptr;
+        long long val = strtoll(retStr.c_str(), &endptr, 10);
+        if (errno == ERANGE) {
+            if (val == LLONG_MIN) {THROW_EX(ValueError, "Underflow when converting to integer.");}
+            else {THROW_EX(ValueError, "Overflow when converting to integer.");}
+        }
+        if (endptr != (retStr.c_str() + retStr.size())) {
+            THROW_EX(ValueError, "Unable to convert string to integer.");
+        }
+        return val;
+    }
+    THROW_EX(ValueError, "Unable to convert expression to numeric type.");
+    return 0;  // Should never get here
+}
+
+double ExprTreeHolder::toDouble() const
+{
+    classad::Value val;
+    const classad::ClassAd *origParent = m_expr->GetParentScope();
+    bool evalresult;
+    if (origParent) {
+        evalresult = m_expr->Evaluate(val);
+    } else {
+        classad::EvalState state;
+        evalresult = m_expr->Evaluate(state, val);
+    }
+    if (PyErr_Occurred()) {boost::python::throw_error_already_set();}
+    if (!evalresult)
+    {   
+        THROW_EX(TypeError, "Unable to evaluate expression");
+    }   
+    double retDouble;
+    std::string retStr;
+    if (val.IsNumber(retDouble)) {return retDouble;}
+    else if (val.IsStringValue(retStr)) {
+        errno = 0;
+        char *endptr;
+        double val = strtod(retStr.c_str(), &endptr);
+        if (errno == ERANGE) {
+            // Any small value will indicate underflow.
+            if (fabs(val) < 1.0) {THROW_EX(ValueError, "Underflow when converting to integer.");}
+            else {THROW_EX(ValueError, "Overflow when converting to integer.");}
+        }
+        if (endptr != (retStr.c_str() + retStr.size())) {
+            THROW_EX(ValueError, "Unable to convert string to integer.");
+        }
+        return val;
+    }
+    THROW_EX(ValueError, "Unable to convert expression to numeric type.");
+    return 0;  // Should never get here
+}
+
 class ScopeGuard
 {
 public:
@@ -641,17 +714,42 @@ attribute(std::string name)
 }
 
 
+/**
+ * See if a given python function accepts an argument named `state`.
+ *
+ * We do this to determine whether a python-based classad function is able to
+ * accept the evaluation context (state).
+ *
+ * This function is likely a historical mistake.  I can think of no other case in
+ * python where the call is changed based on reflection of the target function.
+ * It has a few holes -- for example, it can only make this determination for
+ * functions implemented in Python (as it looks at the associated bytecode
+ * object).  It was done because so many users forgot to accept the state
+ * argument - and getting error messages out of ClassAds is so awkward.  We
+ * actually populate the `error` global - which is useless in multithreaded
+ * programs, but no one seems to look at this.  Python programmers expect
+ * exceptions -- but these can only be thrown when the ClassAd library is used
+ * inside a python interpreter.
+ */
 bool
 checkAcceptsState(boost::python::object pyFunc)
 {
+    // First, check all the named (positional) arguments.  Covers cases like:
+    //   def f1(arg1, arg2, state)
+    //   def f2(arg1, arg2=None, state={})
     boost::python::object varnames = pyFunc.attr("__code__").attr("co_varnames");
-    ssize_t len = py_len(varnames);
+    ssize_t len = boost::python::extract<ssize_t>(pyFunc.attr("__code__").attr("co_argcount"));
     for (int idx=0; idx<len; idx++)
     {
         std::string varname = boost::python::extract<std::string>(varnames[idx]);
         if (varname == "state") {return true;}
     }
-    return false;
+    // Next, check the flags to see if unnamed keywords are accepted.
+    // This is specified in the flags given to the code object.  Covers functions of the form
+    //    def f3(arg, **kw)
+    // The co_flags attribute is a bitmask; 0x08 indicates keyword arguments.
+    int flags = boost::python::extract<int>(pyFunc.attr("__code__").attr("co_flags"));
+    return (flags & 8) == 8;
 }
 
 
