@@ -47,6 +47,44 @@ ClassAdWrapper *parseFile(FILE *stream)
     return wrapper_result;
 }
 
+// Utility to clarify a couple of evaluations
+static inline bool
+IsStringEnd(const char *str, unsigned off)
+{
+    return (  (str[off] == '\0') || (str[off] == '\n') || (str[off] == '\r')  );
+}
+
+// Convert string escaping from old ClassAd syntax to new ClassAd syntax.
+static void ConvertEscapingOldToNew( const char *str, std::string &buffer )
+{
+        // String escaping is different between new and old ClassAds.
+        // We need to convert the escaping from old to new style before
+        // handing the expression to the new ClassAds parser.
+    while( *str ) {
+        size_t n = strcspn(str,"\\");
+        buffer.append(str,n);
+        str += n;
+        if ( *str == '\\' ) {
+            buffer.append( 1, '\\' );
+            str++;
+            if(  (str[0] != '"') ||
+                 ( /*(str[0] == '"') && */ IsStringEnd(str, 1) )   )
+            {
+                buffer.append( 1, '\\' );
+            }
+        }
+    }
+        // remove trailing whitespace
+    int ix = (int)buffer.size();
+    while (ix > 1) {
+        char ch = buffer[ix-1];
+        if (ch != ' ' && ch != '\t' && ch != '\r' && ch != '\n')
+            break;
+        --ix;
+    }
+    buffer.resize(ix);
+}
+
 ClassAdWrapper *parseOld(boost::python::object input)
 {
     PyErr_Warn(PyExc_DeprecationWarning, "ClassAd Deprecation: parseOld is deprecated; use parseOne, parseNext, or parseAds instead.");
@@ -71,6 +109,14 @@ ClassAdWrapper *parseOld(boost::python::object input)
             continue;
         }
         std::string line_str = boost::python::extract<std::string>(line);
+        // The ClassAd library doesn't understand old ClassAd string
+        // escaping rules. If the expression contains a backslash, we
+        // need to convert to new ClassAd escaping rules.
+        if ( strchr( line_str.c_str(), '\\' ) ) {
+            std::string src = line_str;
+            line_str.clear();
+            ConvertEscapingOldToNew( src.c_str(), line_str );
+        }
         if (!wrapper->Insert(line_str))
         {
             THROW_EX(ValueError, line_str.c_str());
@@ -96,7 +142,20 @@ bool isOldAd(boost::python::object source)
         return false;
     }
     if (!py_hasattr(source, "tell") || !py_hasattr(source, "read") || !py_hasattr(source, "seek")) {THROW_EX(ValueError, "Unable to determine if input is old or new classad");}
-    size_t end_ptr = boost::python::extract<size_t>(source.attr("tell")());
+    size_t end_ptr;
+    try
+    {
+        end_ptr = boost::python::extract<size_t>(source.attr("tell")());
+    }
+    catch (const boost::python::error_already_set&)
+    {
+        if (PyErr_ExceptionMatches(PyExc_IOError))
+        {
+            PyErr_Clear();
+            THROW_EX(ValueError, "Stream cannot rewind; must explicitly chose either old or new ClassAd parser.  Auto-detection not available.");
+        }
+        throw;
+    }
     bool isold = false;
     while (true)
     {
@@ -190,7 +249,22 @@ OldClassAdIterator::next()
 
     bool reset_ptr = py_hasattr(m_source, "tell");
     size_t end_ptr = 0;
-    if (reset_ptr) { end_ptr = boost::python::extract<size_t>(m_source.attr("tell")()); }
+    try
+    {
+        if (reset_ptr) { end_ptr = boost::python::extract<size_t>(m_source.attr("tell")()); }
+    }
+    catch (const boost::python::error_already_set&)
+    {
+        if (PyErr_ExceptionMatches(PyExc_IOError))
+        {
+            PyErr_Clear();
+            reset_ptr = false;
+        }
+        else
+        {
+            throw;
+        }
+    }
 
     while (true)
     {
@@ -269,6 +343,14 @@ OldClassAdIterator::next()
             adchar++;
         }
         if (invalid) {continue;}
+        // The ClassAd library doesn't understand old ClassAd string
+        // escaping rules. If the expression contains a backslash, we
+        // need to convert to new ClassAd escaping rules.
+        if ( strchr( line_str.c_str(), '\\' ) ) {
+            std::string src = line_str;
+            line_str.clear();
+            ConvertEscapingOldToNew( src.c_str(), line_str );
+        }
         if (!m_ad->Insert(line_str))
         {
             THROW_EX(ValueError, line_str.c_str());
