@@ -2,7 +2,6 @@
 
 #include <condor_adtypes.h>
 #include <daemon_types.h>
-#include <boost/python.hpp>
 
 using namespace boost::python;
 
@@ -25,6 +24,7 @@ void export_daemon_and_ad_types()
         .value("Any", ANY_AD)
         .value("Generic", GENERIC_AD)
         .value("Startd", STARTD_AD)
+        .value("StartdPrivate", STARTD_PVT_AD)
         .value("Schedd", SCHEDD_AD)
         .value("Master", MASTER_AD)
         .value("Collector", COLLECTOR_AD)
