@@ -10,4 +10,7 @@ void export_event_reader();
 void export_log_reader();
 void enable_classad_extensions();
 void enable_deprecation_warnings();
+void export_claim();
+void export_startd();
+void export_query_iterator();
 
