@@ -398,12 +398,46 @@ class TestClassad(unittest.TestCase):
         self.assertEquals(classad.ExprTree('size(myIntersect({1, 2}, {2, 3}))').eval(), 1)
         self.assertEquals(classad.ExprTree('myIntersect({1, 2}, {2, 3})[0]').eval(), 2)
 
+    def test_state(self):
+        def myFunc(state): return 1 if state else 0
+        classad.register(myFunc)
+        self.assertEquals(0, classad.ExprTree('myFunc(false)').eval())
+        self.assertEquals(1, classad.ExprTree('myFunc("foo")').eval())
+        ad = classad.ClassAd("""[foo = myFunc(); bar = 2]""")
+        self.assertEquals(1, ad.eval('foo'))
+        ad['foo'] = classad.ExprTree('myFunc(1)')
+        self.assertRaises(TypeError, ad.eval, ('foo',))
+        def myFunc(arg1, **kw): return kw['state']['bar']
+        classad.register(myFunc)
+        self.assertEquals(2, ad.eval('foo'))
+
     def test_refs(self):
         ad = classad.ClassAd({"bar": 2})
         expr = classad.ExprTree("foo =?= bar")
         self.assertEquals(ad.externalRefs(expr), ["foo"])
         self.assertEquals(ad.internalRefs(expr), ["bar"])
 
+    def test_cast(self):
+        self.assertEquals(4, int(classad.ExprTree('1+3')))
+        self.assertEquals(4.5, float(classad.ExprTree('1.0+3.5')))
+        self.assertEquals(34, int(classad.ExprTree('strcat("3", "4")')))
+        self.assertEquals(34.5, float(classad.ExprTree('"34.5"')))
+        self.assertRaises(ValueError, float, classad.ExprTree('"34.foo"'))
+        self.assertRaises(ValueError, int, classad.ExprTree('"12 "'))
+        ad = classad.ClassAd("""[foo = 2+5; bar = foo]""")
+        expr = ad['bar']
+        self.assertEquals(7, int(expr))
+        self.assertEquals(7, int(ad.lookup('bar')))
+        self.assertEquals(0, int(classad.ExprTree('false')))
+        self.assertEquals(0.0, float(classad.ExprTree('false')))
+        self.assertEquals(1, int(classad.ExprTree('true')))
+        self.assertEquals(1.0, float(classad.ExprTree('true')))
+        self.assertEquals(3, int(classad.ExprTree('3.99')))
+        self.assertEquals(3.0, float(classad.ExprTree('1+2')))
+        self.assertRaises(ValueError, int, classad.ExprTree('undefined'))
+        self.assertRaises(ValueError, float, classad.ExprTree('error'))
+        self.assertRaises(ValueError, float, classad.ExprTree('foo'))
+
     def test_pipes(self):
         # One regression we saw in the ClassAd library is the new
         # parsing routines would fail if tell/seek was non-functional.
