@@ -2,6 +2,7 @@
 
 import os
 import re
+import pwd
 import sys
 import time
 import errno
@@ -66,6 +67,7 @@ class WithDaemons(unittest.TestCase):
         os.environ["_condor_STARTER"] = os.path.join(os.getcwd(), "../condor_starter.V6.1/condor_starter")
         os.environ["_condor_NEGOTIATOR"] = os.path.join(os.getcwd(), "../condor_negotiator.V6/condor_negotiator")
         os.environ["_condor_SHADOW"] = os.path.join(os.getcwd(), "../condor_shadow.V6.1/condor_shadow")
+        os.environ["_condor_SHARED_PORT"] = os.path.join(os.getcwd(), "../condor_shared_port/condor_shared_port")
         os.environ["_condor_CONDOR_HOST"] = socket.getfqdn()
         os.environ["_condor_LOCAL_DIR"] = testdir
         os.environ["_condor_LOG"] =  '$(LOCAL_DIR)/log'
@@ -85,6 +87,7 @@ class WithDaemons(unittest.TestCase):
         os.environ["_condor_NEGOTIATOR_INTERVAL"] = "1"
         os.environ["_condor_SCHEDD_INTERVAL"] = "1"
         os.environ["_condor_SCHEDD_MIN_INTERVAL"] = "1"
+        os.environ["_condor_CONDOR_FSYNC"] = "FALSE"
         # Various required attributes for the startd
         os.environ["_condor_START"] = "TRUE"
         os.environ["_condor_SUSPEND"] = "FALSE"
@@ -292,6 +295,103 @@ class TestPythonBindings(WithDaemons):
             time.sleep(1)
         self.assertEquals(open(output_file).read(), "hello world\n");
 
+    def testScheddSubmitMany2(self):
+        self.launch_daemons(["SCHEDD", "COLLECTOR", "STARTD", "NEGOTIATOR"])
+        output_file = os.path.join(testdir, "test.out")
+        if os.path.exists(output_file):
+            os.unlink(output_file)
+        schedd = htcondor.Schedd()
+        ad = classad.parseOne(open("tests/submit.ad"))
+        ads = []
+        cluster = schedd.submitMany(ad, [({'foo': 1}, 5), ({'foo': 2}, 5)], False, ads)
+        #print ads[0]
+        for i in range(60):
+            ads = schedd.xquery("ClusterId == %d" % cluster, ["JobStatus", 'ProcId', 'foo'])
+            ads = list(ads)
+            #print ads
+            for ad in ads:
+                if ad['ProcId'] < 5: self.assertEquals(ad['foo'], 1)
+                else: self.assertEquals(ad['foo'], 2)
+            if len(ads) == 0:
+                break
+            if i % 2 == 0:
+                schedd.reschedule()
+            time.sleep(1)
+        self.assertEquals(open(output_file).read(), "hello world\n");
+
+    def testScheddQueryPoll(self):
+        self.launch_daemons(["SCHEDD", "COLLECTOR", "STARTD", "NEGOTIATOR"])
+        output_file = os.path.join(testdir, "test.out")
+        if os.path.exists(output_file):
+            os.unlink(output_file)
+        schedd = htcondor.Schedd()
+        ad = classad.parseOne(open("tests/submit.ad"))
+        ads = []
+        cluster = schedd.submit(ad, 10, False, ads)
+        for i in range(60):
+            ads_iter = schedd.xquery("ClusterId == %d" % cluster, ["JobStatus"], name="query1")
+            ads_iter2 = schedd.xquery("ClusterId == %d" % cluster, ["JobStatus"], name="query2")
+            ads = []
+            for query in htcondor.poll([ads_iter, ads_iter2]):
+                self.assertTrue(query.tag() in ["query1", "query2"])
+                ads += query.nextAdsNonBlocking()
+            #print ads
+            if len(ads) == 0:
+                break
+            if i % 2 == 0:
+                schedd.reschedule()
+
+    def testNegotiate(self):
+        #htcondor.param['TOOL_DEBUG'] = 'D_FULLDEBUG'
+        #os.environ['_condor_SCHEDD_DEBUG'] = 'D_FULLDEBUG, D_NETWORK'
+        #htcondor.enable_debug()
+
+        self.launch_daemons(["SCHEDD", "COLLECTOR", "STARTD"])
+        output_file = os.path.join(testdir, "test.out")
+        if os.path.exists(output_file):
+            os.unlink(output_file)
+        schedd = htcondor.Schedd()
+
+        schedd.act(htcondor.JobAction.Remove, 'true')
+        ad = classad.parse(open("tests/submit.ad"))
+        ads = []
+        cluster = schedd.submit(ad, 1, False, ads)
+
+        # Get claim for startd
+        claim_ads = []
+        for i in range(10):
+            startd_ads = htcondor.Collector().locateAll(htcondor.DaemonTypes.Startd)
+            private_ads = htcondor.Collector().query(htcondor.AdTypes.StartdPrivate)
+            if (len(startd_ads) != htcondor.param['NUM_CPUS']) or (len(private_ads) != htcondor.param['NUM_CPUS']):
+                time.sleep(1)
+                continue
+            break
+        self.assertEquals(len(startd_ads), len(private_ads))
+        self.assertEquals(len(startd_ads), htcondor.param['NUM_CPUS'])
+        for ad in htcondor.Collector().locateAll(htcondor.DaemonTypes.Startd):
+            for pvt_ad in private_ads:
+                if pvt_ad.get('Name') == ad['Name']:
+                    ad['ClaimId'] = pvt_ad['Capability']
+                    claim_ads.append(ad)
+        self.assertEquals(len(claim_ads), len(startd_ads))
+        claim = claim_ads[0]
+
+        me = "%s@%s" % (pwd.getpwuid(os.geteuid()).pw_name, htcondor.param['UID_DOMAIN'])
+        with schedd.negotiate(me) as session:
+            requests = list(session)
+            self.assertEquals(len(requests), 1)
+            request = requests[0]
+            self.assertTrue(request.symmetricMatch(claim))
+            session.sendClaim(claim['ClaimId'], claim, request)
+
+        for i in range(60):
+            ads = schedd.xquery("ClusterId == %d" % cluster, ["JobStatus"])
+            ads = list(ads)
+            if len(ads) == 0:
+                break
+            time.sleep(1)
+        self.assertEquals(open(output_file).read(), "hello world\n");
+
     def testScheddNonblockingQuery(self):
         self.launch_daemons(["SCHEDD", "COLLECTOR", "STARTD", "NEGOTIATOR"])
         output_file = os.path.join(testdir, "test.out")
@@ -396,7 +496,112 @@ class TestPythonBindings(WithDaemons):
         schedd.act(htcondor.JobAction.Remove, ["%d.0" % cluster])
         ads = schedd.query("ClusterId == %d" % cluster, ["JobStatus"])
         self.assertEquals(len(ads), 0)
-        self.assertEquals(open(output_file).read(), "hello world\n");
+        self.assertEquals(open(output_file).read(), "hello world\n")
+
+
+    def testScheddSubmitFile(self):
+        self.launch_daemons(["SCHEDD", "COLLECTOR", "STARTD", "NEGOTIATOR"])
+        submit_obj = htcondor.Submit()
+
+        submit_obj['foo'] = '$(bar) 1'
+        submit_obj['bar'] = '2'
+
+        self.assertEquals(str(submit_obj), "foo = $(bar) 1\nbar = 2\nqueue")
+        self.assertEquals(submit_obj.expand('foo'), "2 1")
+        self.assertEquals(set(submit_obj.keys()), set(['foo', 'bar']))
+        self.assertEquals(set(submit_obj.values()), set(['$(bar) 1', '2']))
+        self.assertEquals(set(submit_obj.items()), set([('foo', '$(bar) 1'), ('bar', '2')]))
+        d = dict(submit_obj)
+        self.assertEquals(set(d.items()), set([('foo', '$(bar) 1'), ('bar', '2')]))
+
+        self.assertEquals(str(htcondor.Submit(d)), str(submit_obj))
+
+        submit_obj = htcondor.Submit({"executable": "/bin/sh",
+                                      "arguments":  "-c ps faux",
+                                      "output":     "test.out.$(Cluster).$(Process)",
+                                      "+foo":       "true",
+                                      "+baz_bar":   "false",
+                                      "+qux":       "1"})
+        schedd = htcondor.Schedd()
+        ads = []
+        with schedd.transaction() as txn:
+             cluster_id = submit_obj.queue(txn, ad_results=ads)
+
+        self.assertEquals(len(ads), 1)
+        ad = ads[0]
+        self.assertEquals(cluster_id, ad['ClusterId'])
+        self.assertEquals(ad['In'], '/dev/null')
+        self.assertEquals(ad['Args'], "-c ps faux")
+        outfile = "test.out.%d.0" % ad['ClusterId']
+        self.assertEquals(ad['Out'], outfile)
+        self.assertEquals(ad['foo'], True)
+        self.assertEquals(ad['baz_bar'], False)
+        self.assertEquals(ad['qux'], 1)
+
+        finished = False
+        for i in range(60):
+            ads = schedd.query("ClusterId == %d" % cluster_id, ["JobStatus"])
+            if len(ads) == 0:
+                finished = True
+                break
+            if i % 2 == 0:
+                schedd.reschedule()
+            time.sleep(1)
+        self.assertTrue(finished)
+        self.assertTrue(len(open(outfile).read()) > 0)
+
+
+    def testClaim(self):
+        os.environ['_condor_VALID_COD_USERS'] = pwd.getpwuid(os.geteuid()).pw_name
+        self.launch_daemons(["COLLECTOR", "STARTD"])
+        output_file = os.path.abspath(os.path.join(testdir, "test.out"))
+        if os.path.exists(output_file):
+            os.unlink(output_file)
+        coll = htcondor.Collector()
+        for i in range(10):
+            ads = coll.locateAll(htcondor.DaemonTypes.Startd)
+            if len(ads) > 0: break
+            time.sleep(1)
+        job_common = { \
+            'Cmd': '/bin/sh', 
+            'JobUniverse': 5,
+            'Iwd': os.path.abspath(testdir),
+            'Out': 'testclaim.out',
+            'Err': 'testclaim.err',
+            'StarterUserLog': 'testclaim.log',
+        }
+        claim = htcondor.Claim(ads[0])
+        claim.requestCOD()
+        hello_world_job = dict(job_common)
+        hello_world_job['Arguments'] = "-c 'echo hello world > %s'" % output_file
+        claim.activate(hello_world_job)
+        for i in range(10):
+            if os.path.exists(output_file): break
+            time.sleep(1)
+        self.assertEquals(open(output_file).read(), "hello world\n")
+        sleep_job = dict(job_common)
+        sleep_job['Args'] = "-c 'sleep 5m'"
+        claim.activate(sleep_job)
+        claim.suspend()
+        claim.renew()
+        #claim.delegateGSIProxy()
+        claim.resume()
+        claim.deactivate(htcondor.VacateTypes.Fast)
+        claim.release()
+
+    def testDrain(self):
+        self.launch_daemons(["COLLECTOR", "STARTD"])
+        output_file = os.path.abspath(os.path.join(testdir, "test.out"))
+        if os.path.exists(output_file):
+            os.unlink(output_file)
+        coll = htcondor.Collector()
+        for i in range(10):
+            ads = coll.locateAll(htcondor.DaemonTypes.Startd)
+            if len(ads) > 0: break
+            time.sleep(1)
+        startd = htcondor.Startd(ads[0])
+        drain_id = startd.drainJobs(htcondor.DrainTypes.Fast)
+        startd.cancelDrainAllJobs(drain_id)
 
     def testPing(self):
         self.launch_daemons(["COLLECTOR"])
